/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*  26-Dec-04, J Love, GST	Original version.
*  24-Feb-05, J Love, GST	IR display choices.
*  17-Mar-05, J Love, GST	Ground station argument (tmp).
*   8-Feb-06, J Love, GST	Geolocation, etc.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.client;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import javax.swing.*;
import java.awt.*;
import java.awt.image.*;

public abstract class AllBandsViewHandler extends JPanel implements ViewHandler {
  // private int solarCode;
  private String[] tips;
  private int[] bandIndices;
  private boolean isAscending;
  private int nBlockRows;
  private int nBlockCols;
  private ResolutionCombo resolutionCombo;
  private JScrollPane scrollPane;
  private JScrollBar vScrollBar;
  private HPanel handlersPanel;
  private ImageHandler[] imageHandlers;
  private Color[] colors;
  private Color[] grays = GUtil.getGrays();
  /****************************************************************************
  * AllBandsViewHandler.
  ****************************************************************************/
  public AllBandsViewHandler (int platformCode, int solarCode, String[] tips) {
    // this.solarCode = solarCode;
    this.tips = tips;
    switch (solarCode) {
      case Simulcast.DAY_PASS_CODE: {
	bandIndices = Modis.dayBandIndices;
	imageHandlers = new ImageHandler[Modis.DAY_BAND_COUNT];
	break;
      }
      case Simulcast.NIGHT_PASS_CODE: {
	bandIndices = Modis.nightBandIndices;
	imageHandlers = new ImageHandler[Modis.NIGHT_BAND_COUNT];
	break;
      }
    }
    setLayout(new BorderLayout());
    resolutionCombo = new ResolutionCombo(Modis.allBandResolutionChoices,3);
    HPanel controls = new HPanel(new Component[] {new JLabel("Resolution:"),GUtil.HSpace(5),resolutionCombo});
    scrollPane = new JScrollPane(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
    				 JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    vScrollBar = scrollPane.getVerticalScrollBar();
    add(scrollPane,BorderLayout.CENTER);
    add(new MarginPanel(5,1,controls,5,1),BorderLayout.SOUTH);
  }
  /****************************************************************************
  * playStarting.
  ****************************************************************************/
  public void playStarting () {
    GUtil.invokeAndWait(new SetControls(false));
    clear();
  }
  /****************************************************************************
  * clear.
  ****************************************************************************/
  public void clear () {
    GUtil.invokeAndWait(new DoClear());
  }
  private class DoClear implements Runnable {
    public void run () {
      scrollPane.setViewportView(null);
    }
  }
  /****************************************************************************
  * getRequestMessage.
  ****************************************************************************/
  public String getRequestMessage (String passName) {
    StringBuffer msg = new StringBuffer();
    msg.append(Util.XML_HEADER);
    msg.append("<simulcastMessage>");
    msg.append("<playPass>");
    msg.append("<name>" + passName + "</name>");
    msg.append("<partial>");
    msg.append("<send><debowed/></send>");
    msg.append("<bands>");
    for (int x = 0; x < bandIndices.length; x++) {
       msg.append("<band>");
       msg.append("<bandIndex>" + Integer.toString(bandIndices[x]) + "</bandIndex>");
       msg.append("<resolution>" + Modis.allBandResolutionValues[resolutionCombo.getSelectedIndex()] + "</resolution>");
       msg.append("</band>");
    }
    msg.append("</bands>");
    msg.append("</partial>");
    if (Client.optionsScreen.compressCheckBox.isSelected()) {
      msg.append("<compress>" + "gzip" + "</compress>");
    }
    msg.append("</playPass>");
    msg.append("</simulcastMessage>");
    return msg.toString();
  }
  /****************************************************************************
  * headerBlock.
  ****************************************************************************/
  public void headerBlock (HeaderBlock headerBlock) throws Exception {
    // isAscending = Simulcast.isAscending(Client.groundStation,headerBlock);
    isAscending = (headerBlock.getDirectionCode() == Simulcast.ASCENDING_PASS_CODE);
    if (headerBlock.getBandCount() != bandIndices.length) {
      throw new Exception("unexpected number of bands");
    }
    if (headerBlock.getBandIndex(0) != bandIndices[0]) {
      throw new Exception("unexpected band index");
    }
    int resolution = headerBlock.getBandResolution(0);
    nBlockRows = headerBlock.getBlockRowCount(0);
    /*
    if (nBlockRows != 1) {
      throw new Exception("unexpected number of block rows");
    }
    */
    nBlockCols = headerBlock.getBlockColCount(0);
    for (int x = 1; x < bandIndices.length; x++) {
       if (headerBlock.getBandIndex(x) != bandIndices[x]) {
       	 throw new Exception("inconsistent band index");
       }
       if (headerBlock.getBandResolution(x) != resolution) {
       	 throw new Exception("inconsistent resolution");
       }
       if (headerBlock.getBlockRowCount(x) != nBlockRows) {
       	 throw new Exception("inconsistent block rows");
       }
       if (headerBlock.getBlockColCount(x) != nBlockCols) {
       	 throw new Exception("inconsistent block columns");
       }
    }
    for (int x = 0; x < bandIndices.length; x++) {
       imageHandlers[x] = (isAscending ? 
       			   (ImageHandler) new AscendingImageHandler(nBlockCols,true,tips[x]) :
       			   (ImageHandler) new DescendingImageHandler(nBlockCols,true,tips[x]));
    }
    // buffer = new byte[nBlockCols];
    colors = new Color[nBlockCols];
    GUtil.invokeAndWait(new Reload());
  }
  /****************************************************************************
  * scanGroup.
  ****************************************************************************/
  public void scanGroup (ScanGroup scanGroup) throws Exception {
    for (int x = 0; x < bandIndices.length; x++) {
       byte[][] buffer = scanGroup.getDebowedBlock().getData()[x];
       for (int row = 0; row < nBlockRows; row++) {
          for (int col = 0; col < nBlockCols; col++) {
       	     int value = buffer[row][col] & 0xFF;
             if (Modis.isEmissiveBand(bandIndices[x])) {
               value = 255 - Util.adjustValue(value,2.50,50);
             }
             colors[col] = grays[value];
          }
          imageHandlers[x].addRow(colors);
       }
    }
    if (Client.optionsScreen.autoScrollCheckBox.isSelected()) {
      GUtil.invokeAndWait(new DoAutoScroll());
    }
  }
  /****************************************************************************
  * playStopped.
  ****************************************************************************/
  public void playStopped () {
    GUtil.invokeAndWait(new SetControls(true));
  }
  /****************************************************************************
  * getBufferedImage.
  ****************************************************************************/
  public BufferedImage getBufferedImage () {
    Dimension dims = handlersPanel.getSize();
    BufferedImage bi = new BufferedImage(dims.width,dims.height,
			    		 BufferedImage.TYPE_BYTE_GRAY);
    handlersPanel.paint(bi.getGraphics());
    return bi;
  }
  /****************************************************************************
  * Reload.
  ****************************************************************************/
  private class Reload implements Runnable {
    public void run () {
      Component[] components = new Component[1+(bandIndices.length*2)];
      components[0] = GUtil.HSpace(5);
      for (int x = 0, i = 1; x < bandIndices.length; x++) {
         components[i++] = imageHandlers[x];
         components[i++] = GUtil.HSpace(5);
      }
      handlersPanel = new HPanel(components);
      scrollPane.setViewportView(new MarginPanel(0,0,handlersPanel,0,0));
      scrollPane.revalidate();
      scrollPane.repaint();
    }
  }
  /****************************************************************************
  * DoAutoScroll.
  ****************************************************************************/
  private class DoAutoScroll implements Runnable {
    public void run () {
      if (isAscending) {
	vScrollBar.setValue(vScrollBar.getMinimum());
      } else {
	vScrollBar.setValue(vScrollBar.getMaximum());
      }
    }
  }
  /****************************************************************************
  * SetControls.
  ****************************************************************************/
  private class SetControls implements Runnable {
    private boolean enabled;
    public SetControls (boolean enabled) {
      this.enabled = enabled;
    }
    public void run () {
      resolutionCombo.setEnabled(enabled);
    }
  }
}
