/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*  27-Dec-04, J Love, GST	Original version.
*  24-Feb-05, J Love, GST	IR display choices.
*  17-Mar-05, J Love, GST	Ground station argument (tmp).
*   8-Feb-06, J Love, GST	Geolocation, etc.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.client;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import javax.swing.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;

public class ClassicDayViewHandler extends JPanel implements ViewHandler {
  private int nTrueColorBlockRows;
  private int nTrueColorBlockCols;
  private int nBandBlockRows;
  private int nBandBlockCols;
  private JComboBox bandCombo;
  private JPanel bandScrollerPanel;
  private ImageScroller trueColorImageScroller;
  private ImageScroller[] bandImageScrollers = new ImageScroller[Modis.DAY_BAND_COUNT];
  private Color[] bandColors;
  private Color[] trueColorColors;
  private EnhancementAlgorithm algorithm;
  private int[] values = new int[3];
  private static final int trueColorResolution = 1000;
  private static final int bandResolution = 10000;
  private Color[] grays = GUtil.getGrays();
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * ClassicDayViewHandler.
  ****************************************************************************/
  public ClassicDayViewHandler (int platformCode) {
    setLayout(new BorderLayout());
    try {
      algorithm = (EnhancementAlgorithm) Class.forName("gov.nasa.gsfc.aisb.simulcast.util.TrueColorEnhancement2").newInstance();
    } catch (Exception e) {
      algorithm = null;
    }
    bandCombo = new JComboBox(Modis.dayBandChoices);
    bandCombo.setToolTipText("Band to display");
    bandCombo.setSelectedIndex(0);
    bandCombo.addActionListener(new BandSelected());
    trueColorImageScroller = new ImageScroller();
    for (int x = 0; x < Modis.DAY_BAND_COUNT; x++) {
       bandImageScrollers[x] = new ImageScroller();
    }
    bandScrollerPanel = new JPanel(new BorderLayout());
    bandScrollerPanel.add(bandImageScrollers[0]);
    JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
					  new MarginPanel(5,5,new VPanel(new HPanel(new JLabel("Band:"),GUtil.HSpace(5),bandCombo),GUtil.VSpace(5),bandScrollerPanel),5,5),
					  new MarginPanel(5,5,trueColorImageScroller,5,5));
    splitPane.setDividerSize(3);
    splitPane.setResizeWeight(0.1);
    add(new MarginPanel(5,1,splitPane,5,1),BorderLayout.CENTER);
  }
  /****************************************************************************
  * playStarting.
  ****************************************************************************/
  public void playStarting () {
    clear();
  }
  /****************************************************************************
  * clear.
  ****************************************************************************/
  public void clear () {
    trueColorImageScroller.clear();
    for (int x = 0; x < Modis.DAY_BAND_COUNT; x++) {
       bandImageScrollers[x].clear();
    }
  }
  /****************************************************************************
  * getRequestMessage.
  ****************************************************************************/
  public String getRequestMessage (String passName) {
    StringBuffer msg = new StringBuffer();
    msg.append(Util.XML_HEADER);
    msg.append("<simulcastMessage>");
    msg.append("<playPass>");
    msg.append("<name>" + passName + "</name>");
    msg.append("<partial>");
    msg.append("<send><debowed/></send>");
    msg.append("<bands>");
    msg.append("<band>");
    msg.append("<bandIndex>" + Integer.toString(Modis.TRUE_COLOR_RED_BAND_INDEX) + "</bandIndex>");
    msg.append("<resolution>" + Integer.toString(trueColorResolution) + "</resolution>");
    msg.append("</band>");
    msg.append("<band>");
    msg.append("<bandIndex>" + Integer.toString(Modis.TRUE_COLOR_GREEN_BAND_INDEX) + "</bandIndex>");
    msg.append("<resolution>" + Integer.toString(trueColorResolution) + "</resolution>");
    msg.append("</band>");
    msg.append("<band>");
    msg.append("<bandIndex>" + Integer.toString(Modis.TRUE_COLOR_BLUE_BAND_INDEX) + "</bandIndex>");
    msg.append("<resolution>" + Integer.toString(trueColorResolution) + "</resolution>");
    msg.append("</band>");
    for (int x = 0; x < Modis.DAY_BAND_COUNT; x++) {
       msg.append("<band>");
       msg.append("<bandIndex>" + Integer.toString(Modis.dayBandIndices[x]) + "</bandIndex>");
       msg.append("<resolution>" + Integer.toString(bandResolution) + "</resolution>");
       msg.append("</band>");
    }
    msg.append("</bands>");
    msg.append("</partial>");
    if (Client.optionsScreen.compressCheckBox.isSelected()) {
      msg.append("<compress>" + "gzip" + "</compress>");
    }
    msg.append("</playPass>");
    msg.append("</simulcastMessage>");
    return msg.toString();
  }
  /****************************************************************************
  * headerBlock.
  ****************************************************************************/
  public void headerBlock (HeaderBlock headerBlock) throws Exception {
    // boolean isAscending = Simulcast.isAscending(Client.groundStation,headerBlock);
    boolean isAscending = (headerBlock.getDirectionCode() == Simulcast.ASCENDING_PASS_CODE);
    if (headerBlock.getBandCount() != 3 + Modis.DAY_BAND_COUNT) {
      throw new Exception("unexpected number of bands");
    }
    // True color bands...
    if (headerBlock.getBandIndex(0) != Modis.TRUE_COLOR_RED_BAND_INDEX) {
      throw new Exception("unexpected band index");
    }
    int trueColorResolution = headerBlock.getBandResolution(0);
    nTrueColorBlockRows = headerBlock.getBlockRowCount(0);
    nTrueColorBlockCols = headerBlock.getBlockColCount(0);
    if (headerBlock.getBandIndex(1) != Modis.TRUE_COLOR_GREEN_BAND_INDEX) {
      throw new Exception("unexpected band index");
    }
    if (headerBlock.getBandResolution(1) != trueColorResolution) {
      throw new Exception("inconsistent resolution");
    }
    if (headerBlock.getBlockRowCount(1) != nTrueColorBlockRows) {
      throw new Exception("inconsistent block rows");
    }
    if (headerBlock.getBlockColCount(1) != nTrueColorBlockCols) {
      throw new Exception("inconsistent block columns");
    }
    if (headerBlock.getBandIndex(2) != Modis.TRUE_COLOR_BLUE_BAND_INDEX) {
      throw new Exception("unexpected band index");
    }
    if (headerBlock.getBandResolution(2) != trueColorResolution) {
      throw new Exception("inconsistent resolution");
    }
    if (headerBlock.getBlockRowCount(2) != nTrueColorBlockRows) {
      throw new Exception("inconsistent block rows");
    }
    if (headerBlock.getBlockColCount(2) != nTrueColorBlockCols) {
      throw new Exception("inconsistent block columns");
    }
    trueColorImageScroller.reset(nTrueColorBlockCols,true,
    				 isAscending,Modis.trueColorTip,
    				 Client.optionsScreen.limitViewCheckBox.isSelected());
    // redBuffers = new byte[nTrueColorBlockRows][nTrueColorBlockCols];
    // greenBuffers = new byte[nTrueColorBlockRows][nTrueColorBlockCols];
    // blueBuffers = new byte[nTrueColorBlockRows][nTrueColorBlockCols];
    trueColorColors = new Color[nTrueColorBlockCols];
    // All bands...
    if (headerBlock.getBandIndex(3) != Modis.dayBandIndices[0]) {
      throw new Exception("unexpected band index");
    }
    int resolution = headerBlock.getBandResolution(3);
    nBandBlockRows = headerBlock.getBlockRowCount(3);
    nBandBlockCols = headerBlock.getBlockColCount(3);
    bandImageScrollers[0].reset(nBandBlockCols,true,isAscending,
				"Band " + Modis.dayBandChoices[0] + ".",false);
    for (int x = 1; x < Modis.DAY_BAND_COUNT; x++) {
       if (headerBlock.getBandIndex(3+x) != Modis.dayBandIndices[x]) {
       	 throw new Exception("inconsistent band index");
       }
       if (headerBlock.getBandResolution(3+x) != resolution) {
       	 throw new Exception("inconsistent resolution");
       }
       if (headerBlock.getBlockRowCount(3+x) != nBandBlockRows) {
       	 throw new Exception("inconsistent block rows");
       }
       if (headerBlock.getBlockColCount(3+x) != nBandBlockCols) {
       	 throw new Exception("inconsistent block columns");
       }
       bandImageScrollers[x].reset(nBandBlockCols,true,isAscending,
				   "Band " + Modis.dayBandChoices[x] + ".",false);
    }
    // bandBuffer = new byte[nBandBlockCols];
    bandColors = new Color[nBandBlockCols];
  }
  /****************************************************************************
  * scanGroup.
  ****************************************************************************/
  public void scanGroup (ScanGroup scanGroup) throws Exception {
    byte[][] redBuffers = scanGroup.getDebowedBlock().getData()[0];
    byte[][] greenBuffers = scanGroup.getDebowedBlock().getData()[1];
    byte[][] blueBuffers = scanGroup.getDebowedBlock().getData()[2];
    for (int row = 0; row < nTrueColorBlockRows; row++) {
       for (int col = 0; col < nTrueColorBlockCols; col++) {
          values[0] = redBuffers[row][col] & 0xFF;
          values[1] = greenBuffers[row][col] & 0xFF;
          values[2] = blueBuffers[row][col] & 0xFF;
          if (algorithm != null) {
            algorithm.enhance(values);
          }
       	  trueColorColors[col] = new Color(values[0],values[1],values[2]);
       }
       trueColorImageScroller.addRow(trueColorColors);
    }
    for (int x = 0; x < Modis.DAY_BAND_COUNT; x++) {
       byte[][] bandBuffer = scanGroup.getDebowedBlock().getData()[3+x];
       for (int row = 0; row < nBandBlockRows; row++) {
          for (int col = 0; col < nBandBlockCols; col++) {
             int value = bandBuffer[row][col] & 0xFF;
             if (Modis.isEmissiveBand(Modis.dayBandIndices[x])) {
               value = 255 - Util.adjustValue(value,2.50,50);
             }
             bandColors[col] = grays[value];
          }
          bandImageScrollers[x].addRow(bandColors);
       }
    }
  }
  /****************************************************************************
  * playStopped.
  ****************************************************************************/
  public void playStopped () {

  }
  /****************************************************************************
  * getBufferedImage.
  ****************************************************************************/
  public BufferedImage getBufferedImage () {
    return trueColorImageScroller.getBufferedImage();
  }
  /****************************************************************************
  * BandSelected.
  ****************************************************************************/
  private class BandSelected implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      bandScrollerPanel.removeAll();
      bandScrollerPanel.add(bandImageScrollers[bandCombo.getSelectedIndex()]);
      bandScrollerPanel.revalidate();
      bandScrollerPanel.repaint();
    }
  }
}
