/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*  27-Dec-04, J Love, GST	Original version.
*  24-Feb-05, J Love, GST	IR display choices.
*  17-Mar-05, J Love, GST	Ground station argument (tmp).
*   8-Feb-06, J Love, GST	Geolocation.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.client;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import javax.swing.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;

public class ClassicNightViewHandler extends JPanel implements ViewHandler {
  private int nIrBlockRows;
  private int nIrBlockCols;
  private int nBlockRows;
  private int nBlockCols;
  private JComboBox bandCombo;
  private JPanel bandScrollerPanel;
  private ImageScroller irImageScroller;
  private ImageScroller[] bandImageScrollers = new ImageScroller[Modis.NIGHT_BAND_COUNT];
  private Color[] bandRowColors;
  private Color[] irRowColors;
  private static final int irResolution = 1000;
  private static final int bandResolution = 10000;
  private Color[] grays = GUtil.getGrays();
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * ClassicNightViewHandler.
  ****************************************************************************/
  public ClassicNightViewHandler (int platformCode) {
    setLayout(new BorderLayout());
    bandCombo = new JComboBox(Modis.nightBandChoices);
    bandCombo.setSelectedIndex(0);
    bandCombo.addActionListener(new BandSelected());
    irImageScroller = new ImageScroller();
    for (int x = 0; x < Modis.NIGHT_BAND_COUNT; x++) {
       bandImageScrollers[x] = new ImageScroller();
    }
    bandScrollerPanel = new JPanel(new BorderLayout());
    bandScrollerPanel.add(bandImageScrollers[0]);
    JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
					  new MarginPanel(5,5,new VPanel(new HPanel(new JLabel("Band:"),GUtil.HSpace(5),bandCombo),GUtil.VSpace(5),bandScrollerPanel),5,5),
					  new MarginPanel(5,5,irImageScroller,5,5));
    splitPane.setDividerSize(3);
    splitPane.setResizeWeight(0.1);
    add(new MarginPanel(5,1,splitPane,5,1),BorderLayout.CENTER);
  }
  /****************************************************************************
  * playStarting.
  ****************************************************************************/
  public void playStarting () {
    clear();
  }
  /****************************************************************************
  * clear.
  ****************************************************************************/
  public void clear () {
    irImageScroller.clear();
    for (int x = 0; x < Modis.NIGHT_BAND_COUNT; x++) {
       bandImageScrollers[x].clear();
    }
  }
  /****************************************************************************
  * getRequestMessage.
  ****************************************************************************/
  public String getRequestMessage (String passName) {
    StringBuffer msg = new StringBuffer();
    msg.append(Util.XML_HEADER);
    msg.append("<simulcastMessage>");
    msg.append("<playPass>");
    msg.append("<name>" + passName + "</name>");
    msg.append("<partial>");
    msg.append("<send><debowed/></send>");
    msg.append("<bands>");
    msg.append("<band>");
    msg.append("<bandIndex>" + Integer.toString(Modis.INFRARED_BAND_INDEX) + "</bandIndex>");
    msg.append("<resolution>" + Integer.toString(irResolution) + "</resolution>");
    msg.append("</band>");
    for (int x = 0; x < Modis.NIGHT_BAND_COUNT; x++) {
       msg.append("<band>");
       msg.append("<bandIndex>" + Integer.toString(Modis.nightBandIndices[x]) + "</bandIndex>");
       msg.append("<resolution>" + Integer.toString(bandResolution) + "</resolution>");
       msg.append("</band>");
    }
    msg.append("</bands>");
    msg.append("</partial>");
    if (Client.optionsScreen.compressCheckBox.isSelected()) {
      msg.append("<compress>" + "gzip" + "</compress>");
    }
    msg.append("</playPass>");
    msg.append("</simulcastMessage>");
    return msg.toString();
  }
  /****************************************************************************
  * headerBlock.
  ****************************************************************************/
  public void headerBlock (HeaderBlock headerBlock) throws Exception {
    // boolean isAscending = Simulcast.isAscending(Client.groundStation,headerBlock);
    boolean isAscending = (headerBlock.getDirectionCode() == Simulcast.ASCENDING_PASS_CODE);
    if (headerBlock.getBandCount() != 1 + Modis.NIGHT_BAND_COUNT) {
      throw new Exception("unexpected number of bands");
    }
    // Infrared band...
    if (headerBlock.getBandIndex(0) != Modis.INFRARED_BAND_INDEX) {
      throw new Exception("unexpected band index");
    }
    /*int irResolution =*/ headerBlock.getBandResolution(0);
    nIrBlockRows = headerBlock.getBlockRowCount(0);
    nIrBlockCols = headerBlock.getBlockColCount(0);
    irImageScroller.reset(nIrBlockCols,true,
    			  isAscending,Modis.infraredTip,
    			  Client.optionsScreen.limitViewCheckBox.isSelected());
    // irBuffer = new byte[nIrBlockCols];
    irRowColors = new Color[nIrBlockCols];
    // All night bands...
    if (headerBlock.getBandIndex(1) != Modis.nightBandIndices[0]) {
      throw new Exception("unexpected band index");
    }
    int resolution = headerBlock.getBandResolution(1);
    nBlockRows = headerBlock.getBlockRowCount(1);
    nBlockCols = headerBlock.getBlockColCount(1);
    bandImageScrollers[0].reset(nBlockCols,true,isAscending,
    				"Band " + Modis.nightBandChoices[0] + ".",false);
    for (int x = 1; x < Modis.NIGHT_BAND_COUNT; x++) {
       if (headerBlock.getBandIndex(1+x) != Modis.nightBandIndices[x]) {
       	 throw new Exception("inconsistent band index");
       }
       if (headerBlock.getBandResolution(1+x) != resolution) {
       	 throw new Exception("inconsistent resolution");
       }
       if (headerBlock.getBlockRowCount(1+x) != nBlockRows) {
       	 throw new Exception("inconsistent block rows");
       }
       if (headerBlock.getBlockColCount(1+x) != nBlockCols) {
       	 throw new Exception("inconsistent block columns");
       }
       bandImageScrollers[x].reset(nBlockCols,true,isAscending,
       				   "Band " + Modis.nightBandChoices[x] + ".",false);
    }
    // bandBuffer = new byte[nBlockCols];
    bandRowColors = new Color[nBlockCols];
  }
  /****************************************************************************
  * scanGroup.
  ****************************************************************************/
  public void scanGroup (ScanGroup scanGroup) throws Exception {
    byte[][] irBuffer = scanGroup.getDebowedBlock().getData()[0];
    for (int row = 0; row < nIrBlockRows; row++) {
       for (int col = 0; col < nIrBlockCols; col++) {
          int value = irBuffer[row][col] & 0xFF;
          value = 255 - Util.adjustValue(value,2.50,50);
       	  irRowColors[col] = grays[value];
       }
       irImageScroller.addRow(irRowColors);
    }
    for (int x = 0; x < Modis.NIGHT_BAND_COUNT; x++) {
       byte[][] bandBuffer = scanGroup.getDebowedBlock().getData()[1+x];
       for (int row = 0; row < nBlockRows; row++) {
          for (int col = 0; col < nBlockCols; col++) {
             int value = bandBuffer[row][col] & 0xFF;
             if (Modis.isEmissiveBand(Modis.nightBandIndices[x])) {
	       value = 255 - Util.adjustValue(value,2.50,50);
             }
             bandRowColors[col] = grays[value];
          }
          bandImageScrollers[x].addRow(bandRowColors);
       }
    }
  }
  /****************************************************************************
  * playStopped.
  ****************************************************************************/
  public void playStopped () {

  }
  /****************************************************************************
  * getBufferedImage.
  ****************************************************************************/
  public BufferedImage getBufferedImage () {
    return irImageScroller.getBufferedImage();
  }
  /****************************************************************************
  * BandSelected.
  ****************************************************************************/
  private class BandSelected implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      bandScrollerPanel.removeAll();
      bandScrollerPanel.add(bandImageScrollers[bandCombo.getSelectedIndex()]);
      bandScrollerPanel.revalidate();
      bandScrollerPanel.repaint();
    }
  }
}
