/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*  21-Jan-05, J Love, GST	Original version.
*  25-Feb-05, J Love, GST	"Auto View Selection" option.  Added Svalbard
*				to the list.
*  17-Mar-05, J Love, GST	Ground station argument (tmp).
*   8-Feb-06, J Love, GST	Geolocation, etc.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.client;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;

public class Client extends JFrame {
  protected static JFrame frame;
  protected static String clientId = "";
  protected static String serverSimulcastVersion = "?";
  private static JRadioButton passManagerButton;
  private static JRadioButton passViewerButton;
  private static JRadioButton mercatorViewerButton;
  private static JRadioButton polarViewerButton;
  private static JRadioButton optionsScreenButton;
  protected static PassManager passManager;
  protected static PassViewer passViewer;
  protected static MapViewer mercatorViewer;
  protected static MapViewer northPolarViewer;
  protected static MapViewer southPolarViewer;
  protected static MapViewer bothPolarViewer;
  protected static MapViewer polarViewer;
  protected static OptionsScreen optionsScreen;
  protected static ServerHandler serverHandler;
  protected static HostPort serverHostPort = null;
  private static boolean maximize = false;
  private static String autoPlay = "none";
  private static boolean mapAutoZoom = false;
  protected static final Color DARK_GREEN = new Color(0,200,0);
  private JPanel mainPanel;
  protected static JPanel polarPanel;
  private MenuBarClock clock;
  private static String[] serverChoices = new String[] {
    "nisds1.sci.gsfc.nasa.gov:3502",
    "nisfes.sci.gsfc.nasa.gov:3502",
    "nisfes.gsfc.nasa.gov:3502",
    "freddie.coas.oregonstate.edu:3502",
    "localhost:3502"
  };
  private static final String USAGE = "Arguments: [-maximize] [-auto_play {space|mercator|polar}] [-map_auto_zoom] {-server_choice <host:port>}* [<host:port>]";
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * main.
  ****************************************************************************/
  public static void main (String[] args) throws Exception {
    try {
      parseArgs(args);
    } catch (Exception e) {
      System.out.println("Command line error: " + Util.encodeThrowable(e));
      System.out.println(USAGE);
      System.exit(1);
    }
    Client client = new Client();
    client.start();
  }
  /****************************************************************************
  * parseArgs.
  ****************************************************************************/
  private static void parseArgs (String[] args) throws Exception {
    LinkedList<String> servers = new LinkedList<String>();
    int x = 0;
    while (x < args.length) {
      String arg = args[x++];
      if (arg.equals("-maximize")) {
      	maximize = true;
      	continue;
      }
      if (arg.equals("-auto_play")) {
      	autoPlay = args[x++];
      	continue;
      }
      if (arg.equals("-map_auto_zoom")) {
      	mapAutoZoom = true;
      	continue;
      }
      if (arg.equals("-server_choice")) {
      	servers.add(args[x++]);
      	continue;
      }
      if (serverHostPort == null) {
      	serverHostPort = new HostPort(arg);
      	continue;
      }
      throw new Exception("unexpected argument: " + arg);
    }
    if (servers.size() > 0) {
      serverChoices = (String[]) servers.toArray(new String[servers.size()]);
    }
  }
  /****************************************************************************
  * Client (constructor).
  ****************************************************************************/
  public Client () throws Exception {
    frame = this;
    // UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
    UIManager.getDefaults().put("ComboBox.disabledForeground",Color.black);
    setTitle("Simulcast Client");
    getContentPane().setLayout(new BorderLayout());
    JMenuBar menuBar = new JMenuBar();
    JMenu fileMenu = new JMenu("File");
    JMenuItem connectItem = new JMenuItem("Connect...");
    connectItem.setToolTipText("Connect to a Simulcast Server.");
    connectItem.addActionListener(new ConnectButtonClicked());
    fileMenu.add(connectItem);
    JMenuItem disconnectItem = new JMenuItem("Disconnect");
    disconnectItem.setToolTipText("Disconnect from the current Simulcast Server.");
    disconnectItem.addActionListener(new DisconnectButtonClicked());
    fileMenu.add(disconnectItem);
    JMenuItem exitItem = new JMenuItem("Exit");
    exitItem.setToolTipText("Exit the Simulcast Client.");
    exitItem.addActionListener(new ExitButtonClicked());
    fileMenu.add(exitItem);
    menuBar.add(fileMenu);

    JMenu helpMenu = new JMenu("Help");
    JMenuItem aboutItem = new JMenuItem("About Simulcast...");
    aboutItem.setToolTipText("Displays Simulcast version information.");
    aboutItem.addActionListener(new AboutButtonClicked(this));
    helpMenu.add(aboutItem);
    menuBar.add(helpMenu);
    menuBar.add(Box.createHorizontalGlue());
    clock = new MenuBarClock(true);
    menuBar.add(clock);
    getContentPane().add(menuBar,BorderLayout.NORTH);

    mainPanel = new JPanel(new BorderLayout());
    serverHandler = new ServerHandler();
    passManager = new PassManager();
    passViewer = new PassViewer();
    mercatorViewer = new MapViewer(MapHandler.MERCATOR);
    northPolarViewer = new MapViewer(MapHandler.NORTH_POLAR);
    southPolarViewer = new MapViewer(MapHandler.SOUTH_POLAR);
    bothPolarViewer = new MapViewer(MapHandler.BOTH_POLAR);
    polarViewer = northPolarViewer;
    polarPanel = new JPanel(new BorderLayout());
    polarPanel.add(polarViewer,BorderLayout.CENTER);
    optionsScreen = new OptionsScreen(autoPlay,mapAutoZoom);
    passManagerButton = new JRadioButton("Pass Manager",false);
    passManagerButton.addActionListener(new SwitchMainPanel(passManager));
    passViewerButton = new JRadioButton("Space View",false);
    passViewerButton.addActionListener(new SwitchMainPanel(passViewer));
    mercatorViewerButton = new JRadioButton("Mercator Map View",false);
    mercatorViewerButton.addActionListener(new SwitchMainPanel(mercatorViewer));
    polarViewerButton = new JRadioButton("Polar Map View",false);
    polarViewerButton.addActionListener(new SwitchMainPanel(polarPanel));
    optionsScreenButton = new JRadioButton("Options",false);
    optionsScreenButton.addActionListener(new SwitchMainPanel(optionsScreen));
    GUtil.buttonGroup(passManagerButton,passViewerButton,mercatorViewerButton,polarViewerButton,optionsScreenButton);

    HPanel buttonPanel = new HPanel(GUtil.HGlue(),passManagerButton,GUtil.HSpace(40),passViewerButton,GUtil.HSpace(40),mercatorViewerButton,GUtil.HSpace(40),polarViewerButton,GUtil.HSpace(40),optionsScreenButton,GUtil.HGlue());
    getContentPane().add(new MarginPanel(3,3,new VPanel(buttonPanel,GUtil.VSpace(5),mainPanel),3,3),BorderLayout.CENTER);
    pack();
    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    if (maximize) {
      setExtendedState(MAXIMIZED_BOTH);
    } else {
      GUtil.configureWindow(this,0.95,0.85);
    }
    
    /*    
    setUndecorated(true);
    GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice().setFullScreenWindow(frame);
    */
    
    setVisible(true);
    passManagerButton.doClick();
  }
  /****************************************************************************
  * start.
  ****************************************************************************/
  public void start () throws Exception {
    clock.start();
    serverHandler.start();
    Util.sleep(0.5);
    if (serverHostPort == null) {
      SwingUtilities.invokeLater(new PromptForServer());
    }
  }
  /****************************************************************************
  * SwitchMainPanel.
  ****************************************************************************/
  private class SwitchMainPanel implements ActionListener {
    private Component component;
    public SwitchMainPanel (Component component) {
      this.component = component;
    }
    public void actionPerformed (ActionEvent ae) {
      mainPanel.removeAll();
      mainPanel.add(component);
      mainPanel.revalidate();
      mainPanel.repaint();
    }
  }
  /****************************************************************************
  * switchToPassManager.
  ****************************************************************************/
  public static void switchToPassManager () {
    passManagerButton.doClick();
  }
  /****************************************************************************
  * switchToPassViewer.
  ****************************************************************************/
  public static void switchToPassViewer () {
    passViewerButton.doClick();
  }
  /****************************************************************************
  * switchToMercatorViewer.
  ****************************************************************************/
  public static void switchToMercatorViewer () {
    mercatorViewerButton.doClick();
  }
  /****************************************************************************
  * switchToPolarViewer.
  ****************************************************************************/
  public static void switchToPolarViewer () {
    polarViewerButton.doClick();
  }
  /****************************************************************************
  * PromptForServer.
  ****************************************************************************/
  private class PromptForServer implements Runnable {
    private JDialog dialog;
    private JComboBox combo;
    private boolean done = false;
    public void run () {
      dialog = new JDialog(frame,"Simulcast Server Selection",true);
      dialog.setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
      dialog.setResizable(false);
      Container content = dialog.getContentPane();
      combo = new JComboBox(serverChoices);
      combo.setToolTipText("Enter the Simulcast Server (in the form host:port) or select one from the list.");
      combo.setEditable(true);
      combo.setMaximumSize(combo.getPreferredSize());
      JButton connectButton = new JButton("Connect");
      connectButton.setToolTipText("Connects to the specified Simulcast Server.");
      connectButton.addActionListener(new SelectButtonClicked());
      JButton cancelButton = new JButton("Cancel");
      cancelButton.setToolTipText("Cancels without connecting.");
      cancelButton.addActionListener(new CancelButtonClicked());
      content.add(new MarginPanel(10,10,new VPanel(
        new HPanel(GUtil.HGlue(),new JLabel("Select the Simulcast Server host:port..."),GUtil.HGlue()),
        GUtil.VSpace(10),
        new HPanel(GUtil.HGlue(),combo,GUtil.HGlue()),
        GUtil.VSpace(10),
        new HPanel(GUtil.HGlue(),connectButton,GUtil.HSpace(10),cancelButton,GUtil.HGlue()),
        GUtil.VGlue()
      ),10,10));
      dialog.pack();
      GUtil.centerWindow(dialog);
      dialog.setVisible(true);
      while (!done) {
      	Util.sleep(0.25);
      }
    }
    private class SelectButtonClicked implements ActionListener {
      public void actionPerformed (ActionEvent ae) {
        try {
          serverHostPort = new HostPort((String)combo.getSelectedItem());
          serverHandler.disconnect();
          dialog.dispose();
          done = true;
        } catch (Exception e) {
          dialog.dispose();
          JOptionPane.showMessageDialog(frame,e.toString(),"Error",
            				  JOptionPane.ERROR_MESSAGE);
          dialog.setVisible(true);
        }
      }
    }
    private class CancelButtonClicked implements ActionListener {
      public void actionPerformed (ActionEvent ae) {
        dialog.dispose();
        done = true;
      }
    }
  }
  /****************************************************************************
  * ConnectButtonClicked.
  ****************************************************************************/
  private class ConnectButtonClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      SwingUtilities.invokeLater(new PromptForServer());
    }
  }
  /****************************************************************************
  * DisconnectButtonClicked.
  ****************************************************************************/
  private class DisconnectButtonClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      serverHostPort = null;
      serverHandler.disconnect();
    }
  }
  /****************************************************************************
  * ExitButtonClicked.
  ****************************************************************************/
  private class ExitButtonClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      System.exit(0);
    }
  }
  /****************************************************************************
  * MapOptionChanged.
  ****************************************************************************/
  /*
  private class MapOptionChanged implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      mercatorViewer.redraw();
      polarViewer.redraw();
    }
  }
  */
  /****************************************************************************
  * AboutButtonClicked.
  ****************************************************************************/
  private class AboutButtonClicked implements ActionListener {
    private JFrame frame;
    public AboutButtonClicked (JFrame frame) {
      this.frame = frame;
    }
    public void actionPerformed (ActionEvent ae) {
      String text = "Simulcast Client Version " + Simulcast.VERSION +
		    ", Simulcast Server Version " +
		    serverSimulcastVersion;
      JOptionPane.showMessageDialog(frame,text,"About Simulcast...",
				    JOptionPane.INFORMATION_MESSAGE);
    }
  }
}
