/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*  20-Feb-06, J Love, GST	Original version.
*   2-May-06, J Love, GST	The right-click menu is back.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.client;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import gov.nasa.gsfc.drl.geo.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.*;

public class MapHandler extends JPanel implements OrbitsOverlayListener {
  private int projection;
  private MapViewer mapViewer;
  private MapImage image = null;
  private MapParms mapParms = null;
  private GroundStationOverlay gsOverlay = null;
  private OrbitsOverlay orbitsOverlay = null;
  private ShpOverlayLoader shpLoader = null;
  private GroundStations groundStations = null;
  private Object[][] orbitParms = null;
  private int[] firstCorner = null;
  private int[] dragCorner = null;
  private int[] secondCorner = null;
  public static final String[] shpNames = new String[] {
    // "Countries",
    "Countries",		// ESRI
    "US States",
    // "US States",		// ESRI
    "US Urban Areas",		// ESRI
    "US Cities",
    "Canadian Territories",
    "Mexican States",
    "Caribbean Islands"
  };
  public static final boolean[] shpDefaultStates = new boolean[] {
    true,
    true,
    false,
    false,
    true,
    false,
    false
  };
  private static final String[] shpFiles = new String[] {
    "shapes/world-esri/world.shp",
    "shapes/states/s_12my03.shp",
    "shapes/urban-esri/USurban.shp",
    "shapes/cities/cities.shp",
    "shapes/canada/PROVINCE.SHP",
    "shapes/mexico/states.shp",
    "shapes/caribbean/carib_wvs_geo_wgs84.shp"
  };
  private static final Color[] shpColors = new Color[] {
    new Color(0,120,0),
    new Color(0,120,0),
    new Color(0,100,0),
    new Color(0,80,0),
    new Color(0,120,0),
    new Color(0,120,0),
    new Color(0,120,0)
  };
  private ShpOverlay[] shpOverlays = new ShpOverlay[] {
    null,
    null,
    null,
    null,
    null,
    null,
    null
  };
  public static final int MERCATOR = 1;
  public static final int BOTH_POLAR = 2;
  public static final int NORTH_POLAR = 3;
  public static final int SOUTH_POLAR = 4;
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * MapHandler.
  ****************************************************************************/
  public MapHandler (MapViewer mapViewer, int projection) {
    this.mapViewer = mapViewer;
    this.projection = projection;
  }
  /****************************************************************************
  * MapListener.
  ****************************************************************************/
  private class MapListener extends ComponentAdapter {
    public void componentResized (ComponentEvent ce) {
      resized();
    }
  }
  private synchronized void resized () {
    Dimension dims = getSize();
    prereconfig();
    mapParms.resize(dims.width,dims.height);
    reconfig();
  }
  /****************************************************************************
  * prereconfig.
  ****************************************************************************/
  private void prereconfig () {
    if (shpLoader != null) {
      shpLoader.stop();
    }
    if (orbitsOverlay != null) {
      orbitsOverlay.stop();
    }
  }
  /****************************************************************************
  * reconfig.
  ****************************************************************************/
  private void reconfig () {
    config();
    revalidate();
    repaint();
  }
  /****************************************************************************
  * config.
  ****************************************************************************/
  private void config () {
    image = new MapImage(mapParms);
    if (groundStations != null) {
      gsOverlay = new GroundStationOverlay(groundStations,mapParms);
    } else {
      gsOverlay = null;
    }
    if (orbitParms != null) {
      try {
	orbitsOverlay = new OrbitsOverlay(orbitParms,mapParms,this);
      } catch (Exception e) {
      	e.printStackTrace();
      	orbitsOverlay = null;
      }
    } else {
      orbitsOverlay = null;
    }
    for (int i = 0; i < shpOverlays.length; i++) {
       shpOverlays[i] = null;
    }
    shpLoader = new ShpOverlayLoader();
    Util.startThread(shpLoader);
  }
  /****************************************************************************
  * redraw.
  ****************************************************************************/
  public void redraw () {
    repaint();
  }
  /****************************************************************************
  * setGroundStations.
  ****************************************************************************/
  public void setGroundStations (GroundStations groundStations) {
    this.groundStations = groundStations;
    if (mapParms != null) {
      gsOverlay = new GroundStationOverlay(groundStations,mapParms);
    }
    repaint();
  }
  /****************************************************************************
  * setTLEs.
  ****************************************************************************/
  public void setTLEs (String[] aquaTLE, String[] terraTLE) {
    orbitParms = new Object[][] {{new Color(160,160,255),"AQUA",aquaTLE[0],aquaTLE[1]},{Color.RED,"TERRA",terraTLE[0],terraTLE[1]}};
    if (mapParms != null) {
      try {
	orbitsOverlay = new OrbitsOverlay(orbitParms,mapParms,this);
      } catch (Exception e) {
      	e.printStackTrace();
      	orbitsOverlay = null;
      }
    }
    repaint();
  }
  /****************************************************************************
  * getAreaFilter.
  ****************************************************************************/
  public String getAreaFilter () {
    if (Client.optionsScreen.autoZoomCheckBox.isSelected()) {
      return "";
    }
    return mapParms.getAreaFilter();
  }
  /****************************************************************************
  * playStarting.
  ****************************************************************************/
  public void playStarting () {
    if (Client.optionsScreen.autoEraseCheckBox.isSelected()) {
      clear();
    }
    mapViewer.playStarting();
  }
  /****************************************************************************
  * clear.
  ****************************************************************************/
  public void clear () {
    if (image != null) {
      image.clear();
    }
    repaint();
  }
  /****************************************************************************
  * headerBlock.
  ****************************************************************************/
  public void headerBlock (HeaderBlock headerBlock) throws Exception {
    if (Client.optionsScreen.autoZoomCheckBox.isSelected()) {
      SwingUtilities.invokeLater(new DoAutoZoom(headerBlock));
    }
  }
  /****************************************************************************
  * dataBlock.
  ****************************************************************************/
  public void dataBlock (int nGeoDataRows, int nGeoDataCols, int geoBinSize,
  			 double[][] geoLats, double[][] geoLons, Color[][] colors) {
    image.dataBlock(nGeoDataRows,nGeoDataCols,geoBinSize,geoLats,geoLons,colors);
    repaint();
  }
  /****************************************************************************
  * playStopped.
  ****************************************************************************/
  public void playStopped () {
    mapViewer.playStopped();
  }
  /****************************************************************************
  * getBufferedImage.
  ****************************************************************************/
  public BufferedImage getBufferedImage () {
    BufferedImage bi = new BufferedImage(mapParms.getWidth(),
            				 mapParms.getHeight(),
            				 BufferedImage.TYPE_INT_RGB);
    Graphics g = bi.getGraphics();
    paint(g);
    return bi;
  }
  /****************************************************************************
  * orbitsUpdated.
  ****************************************************************************/
  public synchronized void orbitsUpdated () {
    repaint();
  }
  /****************************************************************************
  * ShpOverlayLoader.
  ****************************************************************************/
  private class ShpOverlayLoader implements Runnable {
    private ShpOverlay overlay = null;
    private boolean stop = false;
    private boolean stopped = false;
    public void run () {
      for (int i = 0; i < shpFiles.length; i++) {
         try {
           overlay = new ShpOverlay(mapParms);
	   overlay.addShpFile(shpFiles[i],shpColors[i]);
	   shpOverlays[i] = overlay;
	   repaint();
         } catch (InterruptedException e) {
           break;
         } catch (Exception e) {
           e.printStackTrace();
         }
         if (stop) {
           break;
         }
      }
      stopped = true;
    }
    public void stop () {
      stop = true;
      if (overlay != null) {
	overlay.abort();
      }
      while (!stopped) {
      	Util.sleep(1);
      }
    }
  }
  /****************************************************************************
  * ActionMouse.
  ****************************************************************************/
  private class ActionMouse extends MouseAdapter {
    public void mousePressed (MouseEvent me) {
      switch (me.getButton()) {
        case MouseEvent.BUTTON1: {
	  firstCorner = new int[] {me.getX(),me.getY()};
	  break;
        }
        case MouseEvent.BUTTON3: {
          JPopupMenu popupMenu = new JPopupMenu();
          JMenuItem zoomInItem = new JMenuItem("Zoom In (2x)");
          zoomInItem.addActionListener(new ZoomInClicked(me.getX(),me.getY()));
          popupMenu.add(zoomInItem);
          JMenuItem zoomOutItem = new JMenuItem("Zoom Out (2x)");
          zoomOutItem.addActionListener(new ZoomOutClicked(me.getX(),me.getY()));
          popupMenu.add(zoomOutItem);
          JMenuItem zoomOutFullItem = new JMenuItem("Zoom Out (Full)");
          zoomOutFullItem.addActionListener(new ZoomOutFullClicked());
          popupMenu.add(zoomOutFullItem);
          popupMenu.addSeparator();
          JMenuItem showGridItem = new JMenuItem(Client.optionsScreen.showGridCheckBox.isSelected() ? "Hide Grid" : "Show Grid");
          showGridItem.addActionListener(new ShowGridClicked());
          popupMenu.add(showGridItem);
          JMenuItem showFeaturesItem = new JMenuItem(Client.optionsScreen.showFeaturesCheckBox.isSelected() ? "Hide Features" : "Show Features");
          showFeaturesItem.addActionListener(new ShowFeaturesClicked());
          popupMenu.add(showFeaturesItem);
          JMenuItem showOrbitsItem = new JMenuItem(Client.optionsScreen.showOrbitsCheckBox.isSelected() ? "Hide Orbits" : "Show Orbits");
          showOrbitsItem.addActionListener(new ShowOrbitsClicked());
          popupMenu.add(showOrbitsItem);
          JMenuItem showStationsItem = new JMenuItem(Client.optionsScreen.showGroundStationCheckBox.isSelected() ? "Hide Stations" : "Show Stations");
          showStationsItem.addActionListener(new ShowStationsClicked());
          popupMenu.add(showStationsItem);
          popupMenu.addSeparator();
          JMenuItem clearItem = new JMenuItem("Clear Swaths");
          clearItem.addActionListener(new ClearClicked());
          popupMenu.add(clearItem);
          JMenuItem saveAsItem = new JMenuItem("Save As...");
          saveAsItem.addActionListener(new SaveAsClicked());
          popupMenu.add(saveAsItem);
          popupMenu.pack();
          popupMenu.show(me.getComponent(),me.getX(),me.getY());
	  break;
        }
      }
    }
    public void mouseReleased (MouseEvent me) {
      switch (me.getButton()) {
        case MouseEvent.BUTTON1: {
          secondCorner = new int[] {me.getX(),me.getY()};
          boolean zoomExtents = false;
          if (firstCorner != null && secondCorner != null) {
            if (firstCorner[0] != secondCorner[0] || firstCorner[1] != secondCorner[1]) {
      	      zoomExtents = true;
      	    }
          }
          if (zoomExtents) {
      	    if (Math.abs(firstCorner[0]-secondCorner[0]) > 10 && Math.abs(firstCorner[1]-secondCorner[1]) > 10) {
              prereconfig();
              mapParms.zoom(Util.min(firstCorner[0],secondCorner[0]),
      		            Util.min(firstCorner[1],secondCorner[1]),
		            Util.max(firstCorner[0],secondCorner[0]),
      		            Util.max(firstCorner[1],secondCorner[1]));
              reconfig();
      	    }
          } else {
	    prereconfig();
	    mapParms.zoom(2.0,me.getX(),me.getY());
	    reconfig();
          }
	  firstCorner = null;
	  dragCorner = null;
	  secondCorner = null;
          break;
        }
        case MouseEvent.BUTTON3: {
          break;
        }
      }
      repaint();
    }
  }
  private class ZoomInClicked implements ActionListener {
    private int x, y;
    public ZoomInClicked (int x, int y) {
      this.x = x;
      this.y = y;
    }
    public void actionPerformed (ActionEvent ae) {
      prereconfig();
      mapParms.zoom(2.0,x,y);
      reconfig();
    }
  }
  private class ZoomOutClicked implements ActionListener {
    private int x, y;
    public ZoomOutClicked (int x, int y) {
      this.x = x;
      this.y = y;
    }
    public void actionPerformed (ActionEvent ae) {
      prereconfig();
      mapParms.zoom(1/2.0,x,y);
      reconfig();
    }
  }
  private class ZoomOutFullClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      prereconfig();
      mapParms.zoom(1/1000000000.0,0,0);
      reconfig();
    }
  }
  private class ShowGridClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      Client.optionsScreen.showGridCheckBox.setSelected(!Client.optionsScreen.showGridCheckBox.isSelected());
      repaint();
    }
  }
  private class ShowFeaturesClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      Client.optionsScreen.showFeaturesCheckBox.setSelected(!Client.optionsScreen.showFeaturesCheckBox.isSelected());
      repaint();
    }
  }
  private class ShowOrbitsClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      Client.optionsScreen.showOrbitsCheckBox.setSelected(!Client.optionsScreen.showOrbitsCheckBox.isSelected());
      repaint();
    }
  }
  private class ShowStationsClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      Client.optionsScreen.showGroundStationCheckBox.setSelected(!Client.optionsScreen.showGroundStationCheckBox.isSelected());
      repaint();
    }
  }
  private class ClearClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      clear();
    }
  }
  private class SaveAsClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      String passName = mapViewer.getPassName();
      if (passName != null) {
        BufferedImage bi = getBufferedImage();
        SaveAs saveAs = new SaveAs(passName,bi);
        saveAs.pack();
        GUtil.centerWindow(saveAs);
        saveAs.setVisible(true);
      }
    }
  }
  /****************************************************************************
  * ActionMouseMotion.
  ****************************************************************************/
  private class ActionMouseMotion extends MouseMotionAdapter {
    public void mouseMoved (MouseEvent me) {
      // double[] loc = mapParms.getLatLon(me.getX(),me.getY());
      // System.out.println(loc[0] + " " + loc[1]);
    }
    public void mouseDragged (MouseEvent me) {
      if (firstCorner != null) {
	dragCorner = new int[] {me.getX(),me.getY()};
	repaint();
      }
    }
  }
  /****************************************************************************
  * DoAutoZoom.
  ****************************************************************************/
  private class DoAutoZoom implements Runnable {
    private HeaderBlock headerBlock;
    public DoAutoZoom (HeaderBlock headerBlock) {
      this.headerBlock = headerBlock;
    }
    public void run () {
      Geo.Extents extents = headerBlock.getEstExtents();
      prereconfig();
      mapParms.zoom(extents.getBeginRight().getLat(),extents.getBeginRight().getLon(),
      		    extents.getBeginLeft().getLat(),extents.getBeginLeft().getLon(),
		    extents.getEndRight().getLat(),extents.getEndRight().getLon(),
		    extents.getEndLeft().getLat(),extents.getEndLeft().getLon());
      reconfig();
    }
  }
  /****************************************************************************
  * paint.
  ****************************************************************************/
  public synchronized void paint (Graphics g) {
    if (mapParms == null) {
      Dimension dims = getSize();
      switch (projection) {
        case MERCATOR: {
	  mapParms = new MercatorMapParms(dims.width,dims.height);
	  break;
        }
        case BOTH_POLAR: {
	  mapParms = new BothPolesMapParms(dims.width,dims.height);
	  break;
        }
        case NORTH_POLAR: {
	  mapParms = new SinglePoleMapParms(true,dims.width,dims.height);
	  break;
        }
        case SOUTH_POLAR: {
	  mapParms = new SinglePoleMapParms(false,dims.width,dims.height);
	  break;
        }
      }
      config();
      if (mapParms.isZoomable()) {
        addMouseListener(new ActionMouse());
        addMouseMotionListener(new ActionMouseMotion());
      }
      addComponentListener(new MapListener());
    }
    // Ignore paint request if the map parameters don't agree with the size
    // of this JPanel.  This happens during a resizing of the JFrame.
    if (getWidth() != mapParms.getWidth() || getHeight() != mapParms.getHeight()) {
      g.setColor(Color.black);
      g.fillRect(0,0,getWidth(),getHeight());
      return;
    }
    // Draw background.
    g.setColor(Color.black);
    g.fillRect(0,0,mapParms.getWidth(),mapParms.getHeight());
    // Draw imagery.
    mapParms.drawImage(g,image);
    // Draw features.
    if (Client.optionsScreen.showFeaturesCheckBox.isSelected()) {
      for (int i = 0; i < shpOverlays.length; i++) {
         if (shpOverlays[i] != null) {
           if (Client.optionsScreen.showFeatureCheckBox[i].isSelected()) {
             mapParms.drawImage(g,shpOverlays[i]);
           }
         }
      }
    }
    // Draw orbits.
    if (Client.optionsScreen.showOrbitsCheckBox.isSelected()) {
      if (orbitsOverlay != null) {
      	orbitsOverlay.draw(g);
      }
    }
    // Draw ground station locations.
    if (Client.optionsScreen.showGroundStationCheckBox.isSelected()) {
      if (gsOverlay != null) {
      	gsOverlay.draw(g);
      }
    }
    // Draw grid.
    if (Client.optionsScreen.showGridCheckBox.isSelected()) {
      mapParms.drawGrid(g);
    }
    // Draw border.
    g.setColor(Color.gray.brighter());
    mapParms.drawBorder(g);
    // Zoom Extents drag.
    if (dragCorner != null) {
      g.setColor(Color.white);
      g.drawRect(Util.min(firstCorner[0],dragCorner[0]),
                 Util.min(firstCorner[1],dragCorner[1]),
                 Math.abs(firstCorner[0]-dragCorner[0]),
                 Math.abs(firstCorner[1]-dragCorner[1]));
    }
  }
}
