/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*   8-Feb-06, J Love, GST	Original version.
*  18-Apr-06, J Love, GST	Multiple pass players.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.client;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import javax.swing.*;

public class MapViewer extends JPanel {
  private int projection;
  private MapHandler mapHandler;
  private PassPlayer passPlayer = null;
  private HPanel playerPanel;
  private JComboBox platformCombo;
  private JComboBox[] viewCombos;
  private JComboBox viewCombo;
  private HPanel viewComboPanel;
  private JButton eraseButton;
  private JButton saveAsButton;
  private static final String[] viewChoices = new String[] {
    "MODIS Auto Day/Night","MODIS True Color","MODIS Infrared","MODIS Single Band","MODIS False Color"
  };
  private ViewHandler[][] viewHandlers;
  private ViewHandler viewHandler;
  private HPanel viewHandlerPanel;
  private HPanel noPassPanel = new HPanel(new VPanel(new CustomLabel("No pass selected."),new CustomLabel(" ")),GUtil.HGlue());
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * MapViewer.
  ****************************************************************************/
  public MapViewer (int projection) {
    this.projection = projection;
    setLayout(new BorderLayout());
    mapHandler = new MapHandler(this,projection);
    viewHandlers = new ViewHandler[][] {
      new ViewHandler[] {
	new AutoDayNightMapViewHandler(mapHandler),
	new TrueColorMapViewHandler(mapHandler),
	new InfraredMapViewHandler(mapHandler),
	new SingleBandMapViewHandler(mapHandler),
	new FalseColorMapViewHandler(mapHandler)
      },
      new ViewHandler[] {
	new AutoDayNightMapViewHandler(mapHandler),
	new TrueColorMapViewHandler(mapHandler),
	new InfraredMapViewHandler(mapHandler),
	new SingleBandMapViewHandler(mapHandler),
	new FalseColorMapViewHandler(mapHandler)
      }
    };
    eraseButton = new IconButton(GUtil.eraseIcon);
    eraseButton.addActionListener(new EraseButtonClicked());
    eraseButton.setToolTipText("Clear Swaths");
    eraseButton.setVisible(false);
    saveAsButton = new IconButton(GUtil.saveIcon);
    saveAsButton.addActionListener(new SaveAsButtonClicked());
    saveAsButton.setToolTipText("Save As...");
    saveAsButton.setVisible(false);
    platformCombo = new JComboBox(ViewHandler.platformChoices);
    platformCombo.setToolTipText("Satellite being viewed (changes automatically if necessary).");
    platformCombo.setSelectedIndex(0);
    platformCombo.setMaximumSize(new Dimension(platformCombo.getPreferredSize().width,saveAsButton.getPreferredSize().height));
    platformCombo.addActionListener(new PlatformSelected());
    viewCombos = new JComboBox[2/*ViewHandler.platformChoices.length*/];	// No NPP yet...
    viewCombos[ViewHandler.TERRA_PLATFORM] = new JComboBox(viewChoices);
    viewCombos[ViewHandler.TERRA_PLATFORM].setToolTipText("Instrument data view for TERRA.");
    viewCombos[ViewHandler.TERRA_PLATFORM].setSelectedIndex(0);
    viewCombos[ViewHandler.TERRA_PLATFORM].setMaximumSize(new Dimension(viewCombos[ViewHandler.TERRA_PLATFORM].getPreferredSize().width,saveAsButton.getPreferredSize().height));
    viewCombos[ViewHandler.TERRA_PLATFORM].addActionListener(new ViewSelected());
    viewCombos[ViewHandler.AQUA_PLATFORM] = new JComboBox(viewChoices);
    viewCombos[ViewHandler.AQUA_PLATFORM].setToolTipText("Instrument data view for AQUA.");
    viewCombos[ViewHandler.AQUA_PLATFORM].setSelectedIndex(0);
    viewCombos[ViewHandler.AQUA_PLATFORM].setMaximumSize(new Dimension(viewCombos[ViewHandler.AQUA_PLATFORM].getPreferredSize().width,saveAsButton.getPreferredSize().height));
    viewCombos[ViewHandler.AQUA_PLATFORM].addActionListener(new ViewSelected());
    viewHandler = viewHandlers[0][0];
    viewHandlerPanel = new HPanel((Component)viewHandler);
    playerPanel = new HPanel(noPassPanel);
    viewCombo = viewCombos[0];
    viewComboPanel = new HPanel(viewCombo);
    HPanel h2 = new HPanel(playerPanel,GUtil.HSpace(5),platformCombo,GUtil.HSpace(5),viewComboPanel,GUtil.HSpace(5),eraseButton,GUtil.HSpace(2),saveAsButton);
    add(new MarginPanel(0,0,mapHandler,5,0),BorderLayout.CENTER);
    add(new VPanel(viewHandlerPanel,GUtil.VSpace(5),new HLinePanel(1,Color.BLACK),GUtil.VSpace(5),h2),BorderLayout.SOUTH);
  }
  /****************************************************************************
  * redraw.
  ****************************************************************************/
  public void redraw () {
    mapHandler.redraw();
  }
  /****************************************************************************
  * getPassName.
  ****************************************************************************/
  public String getPassName () {
    if (passPlayer != null) {
      return passPlayer.getPassName();
    }
    return null;
  }
  /****************************************************************************
  * loadPass.
  ****************************************************************************/
  public void loadPass (String passName, int platformCode,
  		        int instrumentCode, int startSolarCode, boolean play) {
    if (passPlayer != null) {
      if (passPlayer.isPlaying()) {
      	return;
      }
    }
    switch (platformCode) {
      case Simulcast.TERRA_PLATFORM_CODE: {
	platformCombo.setSelectedIndex(ViewHandler.TERRA_PLATFORM);
	break;
      }
      case Simulcast.AQUA_PLATFORM_CODE: {
	platformCombo.setSelectedIndex(ViewHandler.AQUA_PLATFORM);
	break;
      }
      case Simulcast.NPP_PLATFORM_CODE: {
	platformCombo.setSelectedIndex(ViewHandler.NPP_PLATFORM);
	break;
      }
    }
    viewComboPanel.removeAll();
    viewCombo = viewCombos[platformCombo.getSelectedIndex()];
    viewComboPanel.add(viewCombo);
    viewHandlerPanel.removeAll();
    viewHandler = viewHandlers[platformCombo.getSelectedIndex()][viewCombo.getSelectedIndex()];
    viewHandlerPanel.add((Component)viewHandler);
    playerPanel.removeAll();
    passPlayer = new PassPlayer(passName,viewHandler);
    playerPanel.add(passPlayer);
    platformCombo.setEnabled(false);
    eraseButton.setVisible(true);
    saveAsButton.setVisible(true);
    revalidate();
    repaint();
    if (play) {
      passPlayer.play();
    }
  }
  /****************************************************************************
  * playStarting.
  ****************************************************************************/
  public void playStarting () {
    viewCombo.setEnabled(false);
  }
  /****************************************************************************
  * clear.
  ****************************************************************************/
  public void clear () {
    mapHandler.clear();
    playerPanel.removeAll();
    passPlayer = null;
    playerPanel.add(noPassPanel);
    platformCombo.setEnabled(true);
    viewCombo.setEnabled(true);
    eraseButton.setVisible(false);
    saveAsButton.setVisible(false);
    revalidate();
    repaint();
  }
  /****************************************************************************
  * playStopped.
  ****************************************************************************/
  public void playStopped () {
    viewCombo.setEnabled(true);
    if (Client.optionsScreen.loopCheckBox.isSelected()) {
      playNext();
    }
  }
  private void playNext () {
    String passName = passPlayer.getPassName();
    while (true) {
      PassHandler handler = Client.passManager.getNextPassHandler(passName);
      switch (projection) {
	case MapHandler.MERCATOR: {
	  if (handler.loadMercator(true)) {
	    return;
	  }
	  break;
	}
	case MapHandler.BOTH_POLAR:
	case MapHandler.NORTH_POLAR:
	case MapHandler.SOUTH_POLAR: {
	  if (handler.loadPolar(true)) {
	    return;
	  }
	  break;
	}
      }
      passName = handler.getPassName();
    }
  }
  /****************************************************************************
  * setTLEs.
  ****************************************************************************/
  public void setTLEs (String[] aquaTLE, String[] terraTLE) throws Exception {
    mapHandler.setTLEs(aquaTLE,terraTLE);
  }
  /****************************************************************************
  * setGroundStations.
  ****************************************************************************/
  public void setGroundStations (GroundStations groundStations) {
    mapHandler.setGroundStations(groundStations);
  }
  /****************************************************************************
  * PlatformSelected.
  ****************************************************************************/
  private class PlatformSelected implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      viewComboPanel.removeAll();
      viewCombo = viewCombos[platformCombo.getSelectedIndex()];
      viewComboPanel.add(viewCombo);
      viewComboPanel.revalidate();
      viewComboPanel.repaint();
      viewHandlerPanel.removeAll();
      viewHandler = viewHandlers[platformCombo.getSelectedIndex()][viewCombo.getSelectedIndex()];
      viewHandlerPanel.add((Component)viewHandler);
      viewHandlerPanel.revalidate();
      viewHandlerPanel.repaint();
    }
  }
  /****************************************************************************
  * ViewSelected.
  ****************************************************************************/
  private class ViewSelected implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      viewHandlerPanel.removeAll();
      viewHandler = viewHandlers[platformCombo.getSelectedIndex()][viewCombo.getSelectedIndex()];
      viewHandlerPanel.add((Component)viewHandler);
      viewHandlerPanel.revalidate();
      viewHandlerPanel.repaint();
      if (passPlayer != null) {
	passPlayer.setViewHandler(viewHandler);
      }
    }
  }
  /****************************************************************************
  * EraseButtonClicked.
  ****************************************************************************/
  private class EraseButtonClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      mapHandler.clear();
    }
  }
  /****************************************************************************
  * SaveAsButtonClicked.
  ****************************************************************************/
  private class SaveAsButtonClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      String passName = passPlayer.getPassName();
      if (passName != null) {
        BufferedImage bi = mapHandler.getBufferedImage();
        SaveAs saveAs = new SaveAs(passName,bi);
        saveAs.pack();
        GUtil.centerWindow(saveAs);
        saveAs.setVisible(true);
      }
    }
  }
}
