/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*  27-Dec-04, J Love, GST	Original version.
*   8-Feb-06, J Love, GST	Geolocation, etc.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.client;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.io.*;
import java.util.zip.*;

public class PassPlayer extends HPanel {
  private String passName;
  private ViewHandler viewHandler;
  private HPanel contentPanel;
  private HPanel playPanel;
  private IconButton playButton;
  private IconButton pauseButton;
  private IconButton stopButton;
  private CustomLabel passNameField;
  private CustomLabel dateTimeField;
  private CustomLabel rateField;
  private CustomLabel nScansField;
  private boolean playing = false;
  private boolean pause = false;
  private boolean stop = false;
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * PassPlayer.
  ****************************************************************************/
  public PassPlayer (String passName, ViewHandler viewHandler) {
    this.passName = passName;
    this.viewHandler = viewHandler;
    contentPanel = new HPanel();
    passNameField = new CustomLabel(passName);
    passNameField.setToolTipText("Name of the pass.");
    dateTimeField = new CustomLabel(" ");
    dateTimeField.setToolTipText("Time of the last scan (GMT) received.");
    nScansField = new CustomLabel(" ");
    nScansField.setToolTipText("Number of scans received since the start of the pass.");
    rateField = new CustomLabel(" ");
    rateField.setToolTipText("Rate at which scans are being received (typically 100% except over slow connections).");
    HPanel h1 = new HPanel(passNameField,GUtil.HGlue(),nScansField);
    HPanel h2 = new HPanel(dateTimeField,GUtil.HGlue(),GUtil.HGlue(),rateField);
    playButton = new IconButton(GUtil.playIcon);
    playButton.addActionListener(new PlayButtonClicked());
    pauseButton = new IconButton(GUtil.pauseIcon);
    pauseButton.addActionListener(new PauseButtonClicked());
    stopButton = new IconButton(GUtil.stopIcon);
    stopButton.addActionListener(new StopButtonClicked());
    playPanel = new HPanel(new HPanel(new VPanel(h1,h2),GUtil.HSpace(5),playButton,GUtil.HSpace(2),pauseButton,GUtil.HSpace(2),stopButton));
    contentPanel.add(playPanel);
    add(contentPanel);
  }
  /****************************************************************************
  * setViewHandler.
  ****************************************************************************/
  public void setViewHandler (ViewHandler viewHandler) {
    this.viewHandler = viewHandler;
  }
  /****************************************************************************
  * isPlaying.
  ****************************************************************************/
  public boolean isPlaying () {
    return playing;
  }
  /****************************************************************************
  * getPassName.
  ****************************************************************************/
  public String getPassName () {
    return passName;
  }
  /****************************************************************************
  * PlayButtonClicked.
  ****************************************************************************/
  private class PlayButtonClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      play();
    }
  }
  /****************************************************************************
  * PauseButtonClicked.
  ****************************************************************************/
  private class PauseButtonClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      pause = !pause;
    }
  }
  /****************************************************************************
  * StopButtonClicked.
  ****************************************************************************/
  private class StopButtonClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      stop = true;
    }
  }
  /****************************************************************************
  * play.
  ****************************************************************************/
  public void play () {
    if (!playing) {
      playing = true;
      Util.startThread(new PlayPass());
    }
  }
  /****************************************************************************
  * PlayPass.
  ****************************************************************************/
  private class PlayPass implements Runnable {
    private Socket s = null;
    private DataInputStream in = null;
    private DataOutputStream out = null;
    public PlayPass () {

    }
    public void run () {
      viewHandler.playStarting();
      try {
      	playPass();
      } catch (Exception e) {
      	// e.printStackTrace();
	error(Util.encodeException(e));
      } finally {
      	try {out.close();} catch (Exception ex) {}
      	try {in.close();} catch (Exception ex) {}
      	try {s.close();} catch (Exception ex) {}
      }
      playing = false;
      viewHandler.playStopped();
      repaint();
    }
    private void playPass () throws Exception {
      GUtil.invokeAndWait(new UpdateLabel(dateTimeField,"Requesting pass data..."));
      GUtil.invokeAndWait(new UpdateLabel(nScansField," "));
      GUtil.invokeAndWait(new UpdateLabel(rateField," "));
      Util.sleep(0.5);
      s = new Socket(Client.serverHostPort.getHost(),Client.serverHostPort.getPort());
      out = new DataOutputStream(new BufferedOutputStream(s.getOutputStream()));
      send(Util.XML_HEADER + "<simulcastMessage>" + "<playMode clientId=\"" +
      	   Client.clientId + "\"/>" + "</simulcastMessage>");
      send(viewHandler.getRequestMessage(passName));
      if (Client.optionsScreen.compressCheckBox.isSelected()) {
	in = new DataInputStream(new BufferedInputStream(new GZIPInputStream(s.getInputStream())));
      } else {
	in = new DataInputStream(new BufferedInputStream(s.getInputStream()));
      }
      readPass(in);
      in.close();
      out.close();
      s.close();
    }
    private void send (String msg) throws Exception {
      out.writeUTF(msg);
      out.flush();
    }
    private void readPass (DataInputStream in) throws Exception {
      RateHandler rateHandler = new RateHandler();
      pause = false;
      stop = false;
      HeaderBlock headerBlock = null;
      /*
      ScanBlock scanBlock = null;
      GeoBlock geoBlock = null;
      DataBlock bowedBlock = null;
      DataBlock debowedBlock = null;
      */
      ClassList blocks = new ClassList();
      while (true) {
        if (stop) {
	  update(rateHandler.toText(true));
          return;
        }
        if (pause) {
          Util.sleep(0.1);
          continue;
        }
        Object block = Simulcast.readBlock(in,blocks);
        if (block instanceof HeaderBlock) {
	  headerBlock = (HeaderBlock) block;
	  viewHandler.headerBlock(headerBlock);
          continue;
        }
        if (block instanceof ScanBlock) {
          ScanBlock scanBlock = (ScanBlock) block;
          if (!scanBlock.isNullScan()) {
            GeoBlock geoBlock = (GeoBlock) (headerBlock.isGeolocated() ? Simulcast.readBlock(in,blocks) : null);
            BowedDataBlock bowedBlock = (BowedDataBlock) (headerBlock.isGeolocated() ? Simulcast.readBlock(in,blocks) : null);
            DebowedDataBlock debowedBlock = (DebowedDataBlock) (headerBlock.isDebowed() ? Simulcast.readBlock(in,blocks) : null);
            ScanGroup scanGroup = new ScanGroup(scanBlock,geoBlock,bowedBlock,debowedBlock);
	    rateHandler.scanReceived(scanGroup);
	    viewHandler.scanGroup(scanGroup);
          }
	  update(scanBlock.getScanTime(),rateHandler.toText(false),scanBlock.getScanNumber());
	  continue;
        }
        if (block instanceof FooterBlock) {
          // FooterBlock footerBlock = (FooterBlock) block;
	  update(rateHandler.toText(true));
	  return;
        }
        if (block instanceof ErrorBlock) {
          ErrorBlock errorBlock = (ErrorBlock) block;
	  throw new Exception(errorBlock.getText());
        }
        throw new Exception("unknown type of block");
      }
    }
  }
  /****************************************************************************
  * error.
  ****************************************************************************/
  public void error (String text) {
    GUtil.invokeAndWait(new DoError(text));
  }
  private class DoError implements Runnable {
    private String text;
    public DoError (String text) {
      this.text = text;
    }
    public void run () {
      contentPanel.removeAll();
      JButton okButton = new JButton("Ok");
      okButton.setForeground(Color.red);
      okButton.addActionListener(new OkButtonClicked());
      contentPanel.add(new HPanel(new VPanel(new CustomLabel("ERROR..."+text,Color.red),new CustomLabel(" ")),GUtil.HGlue(),okButton));
      contentPanel.revalidate();
      contentPanel.repaint();
    }
    private class OkButtonClicked implements ActionListener {
      public void actionPerformed (ActionEvent ae) {
        contentPanel.removeAll();
	dateTimeField.setText(" ");
	nScansField.setText(" ");
	rateField.setText(" ");
        contentPanel.add(playPanel);
        contentPanel.revalidate();
        contentPanel.repaint();
      }
    }
  }
  /****************************************************************************
  * update.
  ****************************************************************************/
  public void update (String rate) {
    GUtil.invokeAndWait(new UpdateLabel(rateField,rate));
  }
  public void update (long dateTime, String rate, int nScans) {
    GUtil.invokeAndWait(new UpdateLabel(dateTimeField,Util.encodeDateTime(dateTime)));
    GUtil.invokeAndWait(new UpdateLabel(nScansField,"Scan" + " " + Integer.toString(nScans)));
    rate = (rate != null ? rate : "");
    if (playing) {
      if (pause) {
        rate = (!rate.equals("") ? (rate+", ") : "") + "PAUSED";
      } else {
        rate = (!rate.equals("") ? (rate+", ") : "") + "PLAYING";
      }
    }
    GUtil.invokeAndWait(new UpdateLabel(rateField,rate));
  }
}
