/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*  21-Jan-05, J Love, GST	Original version.
*   8-Feb-06, J Love, GST	Geolocation, etc.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.client;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;

public class PassViewer extends JPanel implements ViewHandler {
  private JComboBox platformCombo;
  private JComboBox viewCombo;
  private JComboBox aquaViewCombo;
  private JComboBox terraViewCombo;
  private JComboBox nppViewCombo;
  private HPanel viewComboHolder;
  private ViewHandler viewHandler;
  private PassPlayer passPlayer = null;
  private HPanel playerPanel;
  private IconButton saveAsButton;
  private ClassList terraViewHandlers = new ClassList();
  private ClassList aquaViewHandlers = new ClassList();
  private ClassList nppViewHandlers = new ClassList();
  private HPanel noPassPanel = new HPanel(new VPanel(new CustomLabel("No pass selected."),new CustomLabel(" ")),GUtil.HGlue());
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * PassViewer.
  ****************************************************************************/
  public PassViewer () throws Exception {
    setLayout(new BorderLayout());
    saveAsButton = new IconButton(GUtil.saveIcon);
    saveAsButton.addActionListener(new SaveAsButtonClicked());
    saveAsButton.setVisible(false);
    saveAsButton.setToolTipText("Save As...");
    platformCombo = new JComboBox(platformChoices);
    platformCombo.setToolTipText("Satellite being viewed (changes automatically if necessary).");
    platformCombo.setSelectedIndex(TERRA_PLATFORM);
    platformCombo.setMaximumSize(new Dimension(platformCombo.getPreferredSize().width,saveAsButton.getPreferredSize().height));
    platformCombo.addActionListener(new PlatformSelected());
    aquaViewCombo = new JComboBox(terraAquaViewChoices);
    aquaViewCombo.setToolTipText("Instrument data view (changes automatically if necessary depending on day/night pass).");
    aquaViewCombo.setSelectedIndex(MODIS_TRUE_COLOR_DAY);
    aquaViewCombo.setMaximumSize(new Dimension(aquaViewCombo.getPreferredSize().width,saveAsButton.getPreferredSize().height));
    aquaViewCombo.addActionListener(new ViewSelected());
    terraViewCombo = new JComboBox(terraAquaViewChoices);
    terraViewCombo.setToolTipText("Instrument data view (changes automatically if necessary depending on day/night pass).");
    terraViewCombo.setSelectedIndex(MODIS_TRUE_COLOR_DAY);
    terraViewCombo.setMaximumSize(new Dimension(terraViewCombo.getPreferredSize().width,saveAsButton.getPreferredSize().height));
    terraViewCombo.addActionListener(new ViewSelected());
    nppViewCombo = new JComboBox(nppViewChoices);
    nppViewCombo.setToolTipText("Instrument data view (changes automatically if necessary depending on day/night pass).");
    nppViewCombo.setSelectedIndex(VIIRS_TRUE_COLOR_DAY);
    nppViewCombo.setMaximumSize(new Dimension(nppViewCombo.getPreferredSize().width,saveAsButton.getPreferredSize().height));
    nppViewCombo.addActionListener(new ViewSelected());
    viewCombo = terraViewCombo;
    viewComboHolder = new HPanel(viewCombo);
    HPanel h1 = new HPanel(platformCombo,GUtil.HSpace(5),viewComboHolder);
    viewHandler = new TrueColorViewHandler(Simulcast.TERRA_PLATFORM_CODE);
    add((Component)viewHandler,BorderLayout.CENTER);
    playerPanel = new HPanel(noPassPanel);
    HPanel h2 = new HPanel(playerPanel,GUtil.HSpace(5),h1,GUtil.HSpace(5),saveAsButton);
    add(new VPanel(new HLinePanel(1,Color.BLACK),GUtil.VSpace(5),h2),BorderLayout.SOUTH);
    terraViewHandlers.addFirst(viewHandler);
  }
  /****************************************************************************
  * loadPass.
  ****************************************************************************/
  public void loadPass (String passName, int platformCode,
  		        int instrumentCode, int startSolarCode, boolean play) {
    if (passPlayer != null) {
      if (passPlayer.isPlaying()) {
      	return;
      }
    }
    playerPanel.removeAll();
    passPlayer = new PassPlayer(passName,this);
    playerPanel.add(passPlayer);
    loadAppropriateViewHandler(platformCode,instrumentCode,startSolarCode);
    GUtil.invokeAndWait(new SetControls(false,true));
    saveAsButton.setVisible(true);
    if (play) {
      passPlayer.play();
    }
  }
  /****************************************************************************
  * playStarting.
  ****************************************************************************/
  public void playStarting () {
    GUtil.invokeAndWait(new SetControls(false,false));
    viewHandler.playStarting();
  }
  /****************************************************************************
  * clear.
  ****************************************************************************/
  public void clear () {
    viewHandler.clear();
    playerPanel.removeAll();
    passPlayer = null;
    playerPanel.add(noPassPanel);
    GUtil.invokeAndWait(new SetControls(true,true));
    saveAsButton.setVisible(false);
    revalidate();
    repaint();
  }
  /****************************************************************************
  * getRequestMessage.
  ****************************************************************************/
  public String getRequestMessage (String passName) {
    return viewHandler.getRequestMessage(passName);
  }
  /****************************************************************************
  * headerBlock.
  ****************************************************************************/
  public void headerBlock (HeaderBlock headerBlock) throws Exception {
    viewHandler.headerBlock(headerBlock);
  }
  /****************************************************************************
  * scanGroup.
  ****************************************************************************/
  public void scanGroup (ScanGroup scanGroup) throws Exception {
    viewHandler.scanGroup(scanGroup);
  }
  /****************************************************************************
  * playStopped.
  ****************************************************************************/
  public void playStopped () {
    GUtil.invokeAndWait(new SetControls(false,true));
    viewHandler.playStopped();
    if (Client.optionsScreen.loopCheckBox.isSelected()) {
      playNext();
    }
  }
  private void playNext () {
    String passName = passPlayer.getPassName();
    while (true) {
      PassHandler handler = Client.passManager.getNextPassHandler(passName);
      if (handler.loadView(true)) {
	return;
      }
      passName = handler.getPassName();
    }
  }
  /****************************************************************************
  * getBufferedImage.
  ****************************************************************************/
  public BufferedImage getBufferedImage () {
    return viewHandler.getBufferedImage();
  }
  /****************************************************************************
  * PlatformSelected.
  ****************************************************************************/
  private class PlatformSelected implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      viewComboHolder.removeAll();
      switch (platformCombo.getSelectedIndex()) {
	case AQUA_PLATFORM: {
	  viewCombo = aquaViewCombo;
	  break;
	}
	case TERRA_PLATFORM: {
	  viewCombo = terraViewCombo;
	  break;
	}
	case NPP_PLATFORM: {
	  viewCombo = nppViewCombo;
	  break;
	}
      }
      viewComboHolder.add(viewCombo);
      viewComboHolder.revalidate();
      viewComboHolder.repaint();
      loadViewHandler(platformCombo.getSelectedIndex(),
      		      viewCombo.getSelectedIndex());
    }
  }
  /****************************************************************************
  * ViewSelected.
  ****************************************************************************/
  private class ViewSelected implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      loadViewHandler(platformCombo.getSelectedIndex(),
      		      viewCombo.getSelectedIndex());
    }
  }
  /****************************************************************************
  * SaveAsButtonClicked.
  ****************************************************************************/
  private class SaveAsButtonClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      String passName = passPlayer.getPassName();
      if (passName != null) {
        BufferedImage bi = viewHandler.getBufferedImage();
        SaveAs saveAs = new SaveAs(passName,bi);
        saveAs.pack();
        GUtil.centerWindow(saveAs);
        saveAs.setVisible(true);
      }
    }
  }
  /****************************************************************************
  * SetControls.
  ****************************************************************************/
  private class SetControls implements Runnable {
    private boolean enablePlatform;
    private boolean enableView;
    public SetControls (boolean enablePlatform, boolean enableView) {
      this.enablePlatform = enablePlatform;
      this.enableView = enableView;
    }
    public void run () {
      platformCombo.setEnabled(enablePlatform);
      viewCombo.setEnabled(enableView);
    }
  }
  /****************************************************************************
  * loadAppropriateViewHandler.
  ****************************************************************************/
  private void loadAppropriateViewHandler (int platformCode,
  					   int instrumentCode,
  					   int startSolarCode) {
    int platformIndex = getPlatformIndex(platformCode);
    int viewIndex = -1;
    switch (platformCode) {
      case Simulcast.TERRA_PLATFORM_CODE: {
      	viewIndex = Client.optionsScreen.autoViewSelectionCheckBox.isSelected() ?
      		    getAppropriateTerraAquaViewIndex(terraViewHandlers,instrumentCode,startSolarCode) :
      		    getMostRecentTerraAquaViewIndex(terraViewHandlers,instrumentCode);
      	break;
      }
      case Simulcast.AQUA_PLATFORM_CODE: {
      	viewIndex = Client.optionsScreen.autoViewSelectionCheckBox.isSelected() ?
      		    getAppropriateTerraAquaViewIndex(aquaViewHandlers,instrumentCode,startSolarCode) :
      		    getMostRecentTerraAquaViewIndex(aquaViewHandlers,instrumentCode);
      	break;
      }
      case Simulcast.NPP_PLATFORM_CODE: {
      	viewIndex = Client.optionsScreen.autoViewSelectionCheckBox.isSelected() ?
      		    getAppropriateNppViewIndex(nppViewHandlers,instrumentCode,startSolarCode) :
      		    getMostRecentNppViewIndex(nppViewHandlers,instrumentCode);
      	break;
      }
    }
    if (platformIndex != platformCombo.getSelectedIndex() || viewIndex != viewCombo.getSelectedIndex()) {
      platformCombo.setSelectedIndex(platformIndex);
      viewCombo.setSelectedIndex(viewIndex);
    }
  }
  /****************************************************************************
  * getAppropriateTerraAquaViewIndex.
  ****************************************************************************/
  private int getAppropriateTerraAquaViewIndex (ClassList handlers, int instrumentCode, int startSolarCode) {
    switch (instrumentCode) {
      case Simulcast.MODIS_INSTRUMENT_CODE: {
      	switch (startSolarCode) {
      	  case Simulcast.DAY_PASS_CODE: {
      	    return getAppropriateTerraAquaDayViewIndex(handlers);
      	  }
      	  case Simulcast.NIGHT_PASS_CODE: {
      	    return getAppropriateTerraAquaNightViewIndex(handlers);
      	  }
      	}
      	break;
      }
    }
    return -1;
  }
  /****************************************************************************
  * getAppropriateTerraAquaDayViewIndex.
  ****************************************************************************/
  private int getAppropriateTerraAquaDayViewIndex (ClassList handlers) {
    Iterator it = handlers.iterator();
    while (it.hasNext()) {
      ViewHandler vh = (ViewHandler) it.next();
      if (vh instanceof ClassicDayViewHandler) {
      	return MODIS_CLASSIC_DAY;
      }
      if (vh instanceof TrueColorViewHandler) {
      	return MODIS_TRUE_COLOR_DAY;
      }
      if (vh instanceof AllBandsDayViewHandler) {
      	return ALL_MODIS_BANDS_DAY;
      }
      if (vh instanceof FalseColorViewHandler) {
      	return MODIS_FALSE_COLOR;
      }
      if (vh instanceof SingleBandViewHandler) {
      	return SINGLE_MODIS_BAND;
      }
    }
    return MODIS_TRUE_COLOR_DAY;
  }
  /****************************************************************************
  * getAppropriateTerraAquaNightViewIndex.
  ****************************************************************************/
  private int getAppropriateTerraAquaNightViewIndex (ClassList handlers) {
    Iterator it = handlers.iterator();
    while (it.hasNext()) {
      ViewHandler vh = (ViewHandler) it.next();
      if (vh instanceof ClassicNightViewHandler) {
      	return MODIS_CLASSIC_NIGHT;
      }
      if (vh instanceof InfraredViewHandler) {
      	return MODIS_INFRARED_NIGHT;
      }
      if (vh instanceof AllBandsNightViewHandler) {
      	return ALL_MODIS_BANDS_NIGHT;
      }
      if (vh instanceof FalseColorViewHandler) {
      	return MODIS_FALSE_COLOR;
      }
      if (vh instanceof SingleBandViewHandler) {
      	return SINGLE_MODIS_BAND;
      }
    }
    return MODIS_INFRARED_NIGHT;
  }
  /****************************************************************************
  * getMostRecentTerraAquaViewIndex.
  ****************************************************************************/
  private int getMostRecentTerraAquaViewIndex (ClassList handlers, int instrumentCode) {
    if (handlers.size() < 1) {
      return MODIS_TRUE_COLOR_DAY;
    }
    ViewHandler vh = (ViewHandler) handlers.getFirst();
    if (vh instanceof ClassicDayViewHandler) {
      return MODIS_CLASSIC_DAY;
    }
    if (vh instanceof ClassicNightViewHandler) {
      return MODIS_CLASSIC_NIGHT;
    }
    if (vh instanceof TrueColorViewHandler) {
      return MODIS_TRUE_COLOR_DAY;
    }
    if (vh instanceof InfraredViewHandler) {
      return MODIS_INFRARED_NIGHT;
    }
    if (vh instanceof AllBandsDayViewHandler) {
      return ALL_MODIS_BANDS_DAY;
    }
    if (vh instanceof AllBandsNightViewHandler) {
      return ALL_MODIS_BANDS_NIGHT;
    }
    if (vh instanceof FalseColorViewHandler) {
      return MODIS_FALSE_COLOR;
    }
    if (vh instanceof SingleBandViewHandler) {
      return SINGLE_MODIS_BAND;
    }
    return -1;
  }
  /****************************************************************************
  * getAppropriateNppViewIndex.
  ****************************************************************************/
  private int getAppropriateNppViewIndex (ClassList handlers, int instrumentCode, int startSolarCode) {
    switch (instrumentCode) {
      case Simulcast.VIIRS_INSTRUMENT_CODE: {
      	switch (startSolarCode) {
      	  case Simulcast.DAY_PASS_CODE: {
      	    return getAppropriateNppDayViewIndex(handlers);
      	  }
      	  case Simulcast.NIGHT_PASS_CODE: {
      	    return getAppropriateNppNightViewIndex(handlers);
      	  }
      	}
      	break;
      }
    }
    return -1;
  }
  /****************************************************************************
  * getAppropriateNppDayViewIndex.
  ****************************************************************************/
  private int getAppropriateNppDayViewIndex (ClassList handlers) {
    Iterator it = handlers.iterator();
    while (it.hasNext()) {
      ViewHandler vh = (ViewHandler) it.next();
      if (vh instanceof ClassicDayViewHandler) {
      	return VIIRS_CLASSIC_DAY;
      }
      if (vh instanceof TrueColorViewHandler) {
      	return VIIRS_TRUE_COLOR_DAY;
      }
      if (vh instanceof AllBandsDayViewHandler) {
      	return ALL_VIIRS_BANDS_DAY;
      }
      if (vh instanceof FalseColorViewHandler) {
      	return VIIRS_FALSE_COLOR;
      }
      if (vh instanceof SingleBandViewHandler) {
      	return SINGLE_VIIRS_BAND;
      }
    }
    return VIIRS_TRUE_COLOR_DAY;
  }
  /****************************************************************************
  * getAppropriateNppNightViewIndex.
  ****************************************************************************/
  private int getAppropriateNppNightViewIndex (ClassList handlers) {
    Iterator it = handlers.iterator();
    while (it.hasNext()) {
      ViewHandler vh = (ViewHandler) it.next();
      if (vh instanceof ClassicNightViewHandler) {
      	return VIIRS_CLASSIC_NIGHT;
      }
      if (vh instanceof InfraredViewHandler) {
      	return VIIRS_INFRARED_NIGHT;
      }
      if (vh instanceof AllBandsNightViewHandler) {
      	return ALL_VIIRS_BANDS_NIGHT;
      }
      if (vh instanceof FalseColorViewHandler) {
      	return VIIRS_FALSE_COLOR;
      }
      if (vh instanceof SingleBandViewHandler) {
      	return SINGLE_VIIRS_BAND;
      }
    }
    return VIIRS_INFRARED_NIGHT;
  }
  /****************************************************************************
  * getMostRecentNppViewIndex.
  ****************************************************************************/
  private int getMostRecentNppViewIndex (ClassList handlers, int instrumentCode) {
    if (handlers.size() < 1) {
      return VIIRS_TRUE_COLOR_DAY;
    }
    ViewHandler vh = (ViewHandler) handlers.getFirst();
    if (vh instanceof ClassicDayViewHandler) {
      return VIIRS_CLASSIC_DAY;
    }
    if (vh instanceof ClassicNightViewHandler) {
      return VIIRS_CLASSIC_NIGHT;
    }
    if (vh instanceof TrueColorViewHandler) {
      return VIIRS_TRUE_COLOR_DAY;
    }
    if (vh instanceof InfraredViewHandler) {
      return VIIRS_INFRARED_NIGHT;
    }
    if (vh instanceof AllBandsDayViewHandler) {
      return ALL_VIIRS_BANDS_DAY;
    }
    if (vh instanceof AllBandsNightViewHandler) {
      return ALL_VIIRS_BANDS_NIGHT;
    }
    if (vh instanceof FalseColorViewHandler) {
      return VIIRS_FALSE_COLOR;
    }
    if (vh instanceof SingleBandViewHandler) {
      return SINGLE_VIIRS_BAND;
    }
    return -1;
  }
  /****************************************************************************
  * loadViewHandler.
  ****************************************************************************/
  private void loadViewHandler (int platformIndex, int viewIndex) {
    viewHandler.clear();
    remove((Component)viewHandler);
    switch (platformIndex) {
      case TERRA_PLATFORM: {
	switch (viewIndex) {
	  case MODIS_CLASSIC_DAY: {
	    viewHandler = (ViewHandler) terraViewHandlers.remove(ClassicDayViewHandler.class);
	    if (viewHandler == null) {
	      viewHandler = new ClassicDayViewHandler(Simulcast.TERRA_PLATFORM_CODE);
	    }
            break;
          }
	  case MODIS_CLASSIC_NIGHT: {
	    viewHandler = (ViewHandler) terraViewHandlers.remove(ClassicNightViewHandler.class);
	    if (viewHandler == null) {
	      viewHandler = new ClassicNightViewHandler(Simulcast.TERRA_PLATFORM_CODE);
	    }
            break;
          }
	  case MODIS_TRUE_COLOR_DAY: {
	    viewHandler = (ViewHandler) terraViewHandlers.remove(TrueColorViewHandler.class);
	    if (viewHandler == null) {
	      viewHandler = new TrueColorViewHandler(Simulcast.TERRA_PLATFORM_CODE);
	    }
            break;
          }
	  case MODIS_INFRARED_NIGHT: {
	    viewHandler = (ViewHandler) terraViewHandlers.remove(InfraredViewHandler.class);
	    if (viewHandler == null) {
	      viewHandler = new InfraredViewHandler(Simulcast.TERRA_PLATFORM_CODE);
	    }
            break;
          }
	  case ALL_MODIS_BANDS_DAY: {
	    viewHandler = (ViewHandler) terraViewHandlers.remove(AllBandsDayViewHandler.class);
	    if (viewHandler == null) {
	      viewHandler = new AllBandsDayViewHandler(Simulcast.TERRA_PLATFORM_CODE);
	    }
            break;
          }
	  case ALL_MODIS_BANDS_NIGHT: {
	    viewHandler = (ViewHandler) terraViewHandlers.remove(AllBandsNightViewHandler.class);
	    if (viewHandler == null) {
	      viewHandler = new AllBandsNightViewHandler(Simulcast.TERRA_PLATFORM_CODE);
	    }
            break;
          }
	  case MODIS_FALSE_COLOR: {
	    viewHandler = (ViewHandler) terraViewHandlers.remove(FalseColorViewHandler.class);
	    if (viewHandler == null) {
	      viewHandler = new FalseColorViewHandler(Simulcast.TERRA_PLATFORM_CODE);
	    }
            break;
          }
	  case SINGLE_MODIS_BAND: {
	    viewHandler = (ViewHandler) terraViewHandlers.remove(SingleBandViewHandler.class);
	    if (viewHandler == null) {
	      viewHandler = new SingleBandViewHandler(Simulcast.TERRA_PLATFORM_CODE);
	    }
            break;
          }
        }
        terraViewHandlers.addFirst(viewHandler);
        break;
      }
      case AQUA_PLATFORM: {
	switch (viewIndex) {
	  case MODIS_CLASSIC_DAY: {
	    viewHandler = (ViewHandler) aquaViewHandlers.remove(ClassicDayViewHandler.class);
	    if (viewHandler == null) {
	      viewHandler = new ClassicDayViewHandler(Simulcast.AQUA_PLATFORM_CODE);
	    }
            break;
          }
	  case MODIS_CLASSIC_NIGHT: {
	    viewHandler = (ViewHandler) aquaViewHandlers.remove(ClassicNightViewHandler.class);
	    if (viewHandler == null) {
	      viewHandler = new ClassicNightViewHandler(Simulcast.AQUA_PLATFORM_CODE);
	    }
            break;
          }
	  case MODIS_TRUE_COLOR_DAY: {
	    viewHandler = (ViewHandler) aquaViewHandlers.remove(TrueColorViewHandler.class);
	    if (viewHandler == null) {
	      viewHandler = new TrueColorViewHandler(Simulcast.AQUA_PLATFORM_CODE);
	    }
            break;
          }
	  case MODIS_INFRARED_NIGHT: {
	    viewHandler = (ViewHandler) aquaViewHandlers.remove(InfraredViewHandler.class);
	    if (viewHandler == null) {
	      viewHandler = new InfraredViewHandler(Simulcast.AQUA_PLATFORM_CODE);
	    }
            break;
          }
	  case ALL_MODIS_BANDS_DAY: {
	    viewHandler = (ViewHandler) aquaViewHandlers.remove(AllBandsDayViewHandler.class);
	    if (viewHandler == null) {
	      viewHandler = new AllBandsDayViewHandler(Simulcast.AQUA_PLATFORM_CODE);
	    }
            break;
          }
	  case ALL_MODIS_BANDS_NIGHT: {
	    viewHandler = (ViewHandler) aquaViewHandlers.remove(AllBandsNightViewHandler.class);
	    if (viewHandler == null) {
	      viewHandler = new AllBandsNightViewHandler(Simulcast.AQUA_PLATFORM_CODE);
	    }
            break;
          }
	  case MODIS_FALSE_COLOR: {
	    viewHandler = (ViewHandler) aquaViewHandlers.remove(FalseColorViewHandler.class);
	    if (viewHandler == null) {
	      viewHandler = new FalseColorViewHandler(Simulcast.AQUA_PLATFORM_CODE);
	    }
            break;
          }
	  case SINGLE_MODIS_BAND: {
	    viewHandler = (ViewHandler) aquaViewHandlers.remove(SingleBandViewHandler.class);
	    if (viewHandler == null) {
	      viewHandler = new SingleBandViewHandler(Simulcast.AQUA_PLATFORM_CODE);
	    }
            break;
          }
        }
        aquaViewHandlers.addFirst(viewHandler);
        break;
      }
      case NPP_PLATFORM: {
	switch (viewIndex) {
	  case VIIRS_CLASSIC_DAY: {
	    viewHandler = (ViewHandler) nppViewHandlers.remove(ClassicDayViewHandler.class);
	    if (viewHandler == null) {
	      viewHandler = new ClassicDayViewHandler(Simulcast.NPP_PLATFORM_CODE);
	    }
            break;
          }
	  case VIIRS_CLASSIC_NIGHT: {
	    viewHandler = (ViewHandler) nppViewHandlers.remove(ClassicNightViewHandler.class);
	    if (viewHandler == null) {
	      viewHandler = new ClassicNightViewHandler(Simulcast.NPP_PLATFORM_CODE);
	    }
            break;
          }
	  case VIIRS_TRUE_COLOR_DAY: {
	    viewHandler = (ViewHandler) nppViewHandlers.remove(TrueColorViewHandler.class);
	    if (viewHandler == null) {
	      viewHandler = new TrueColorViewHandler(Simulcast.NPP_PLATFORM_CODE);
	    }
            break;
          }
	  case VIIRS_INFRARED_NIGHT: {
	    viewHandler = (ViewHandler) nppViewHandlers.remove(InfraredViewHandler.class);
	    if (viewHandler == null) {
	      viewHandler = new InfraredViewHandler(Simulcast.NPP_PLATFORM_CODE);
	    }
            break;
          }
	  case ALL_VIIRS_BANDS_DAY: {
	    viewHandler = (ViewHandler) nppViewHandlers.remove(AllBandsDayViewHandler.class);
	    if (viewHandler == null) {
	      viewHandler = new AllBandsDayViewHandler(Simulcast.NPP_PLATFORM_CODE);
	    }
            break;
          }
	  case ALL_VIIRS_BANDS_NIGHT: {
	    viewHandler = (ViewHandler) nppViewHandlers.remove(AllBandsNightViewHandler.class);
	    if (viewHandler == null) {
	      viewHandler = new AllBandsNightViewHandler(Simulcast.NPP_PLATFORM_CODE);
	    }
            break;
          }
	  case VIIRS_FALSE_COLOR: {
	    viewHandler = (ViewHandler) nppViewHandlers.remove(FalseColorViewHandler.class);
	    if (viewHandler == null) {
	      viewHandler = new FalseColorViewHandler(Simulcast.NPP_PLATFORM_CODE);
	    }
            break;
          }
	  case SINGLE_VIIRS_BAND: {
	    viewHandler = (ViewHandler) nppViewHandlers.remove(SingleBandViewHandler.class);
	    if (viewHandler == null) {
	      viewHandler = new SingleBandViewHandler(Simulcast.NPP_PLATFORM_CODE);
	    }
            break;
          }
          case ATMS: {
	    viewHandler = (ViewHandler) nppViewHandlers.remove(AtmsViewHandler.class);
	    if (viewHandler == null) {
	      viewHandler = new AtmsViewHandler(Simulcast.NPP_PLATFORM_CODE);
	    }
            break;
          }
          case CrIS: {
	    viewHandler = (ViewHandler) nppViewHandlers.remove(CrisViewHandler.class);
	    if (viewHandler == null) {
	      viewHandler = new CrisViewHandler(Simulcast.NPP_PLATFORM_CODE);
	    }
            break;
          }
          case OMPS: {
	    viewHandler = (ViewHandler) nppViewHandlers.remove(OmpsViewHandler.class);
	    if (viewHandler == null) {
	      viewHandler = new OmpsViewHandler(Simulcast.NPP_PLATFORM_CODE);
	    }
            break;
          }
        }
        nppViewHandlers.addFirst(viewHandler);
        break;
      }
    }
    add((Component)viewHandler,BorderLayout.CENTER);
    revalidate();
    repaint();
  }
  /****************************************************************************
  * getPlatformIndex.
  ****************************************************************************/
  private int getPlatformIndex (int platformCode) {
    switch (platformCode) {
      case Simulcast.TERRA_PLATFORM_CODE: {
      	return TERRA_PLATFORM;
      }
      case Simulcast.AQUA_PLATFORM_CODE: {
      	return AQUA_PLATFORM;
      }
      case Simulcast.NPP_PLATFORM_CODE: {
      	return NPP_PLATFORM;
      }
    }
    return -1;
  }
}
