/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*  27-Dec-04, J Love, GST	Original version.
*  10-Feb-06, J Love, GST	Geolocation, etc.
*  12-Mar-06, J Love, GST	More pass details.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.client;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.net.*;
import java.io.*;
import javax.swing.*;
import java.util.*;
import java.awt.*;

public class ServerHandler extends JPanel implements Runnable {
  private Socket s = null;
  private DataInputStream in = null;
  private DataOutputStream out = null;
  private CustomLabel status;
  private java.util.Timer timer;
  private static final long TIMEOUT_INTERVAL_MS = 2 * Util.SERVER_READY_INTERVAL_SEC * 1000;
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * ServerHandler (constructor).
  ****************************************************************************/
  public ServerHandler () {
    setLayout(new BoxLayout(this,BoxLayout.X_AXIS));
    status = new CustomLabel("");
    add(status);
    add(GUtil.HGlue());
  }
  /****************************************************************************
  * start.
  ****************************************************************************/
  public void start () {
    Util.startThread(this);
  }
  /****************************************************************************
  * run.
  ****************************************************************************/
  public void run () {
    while (true) {
      try {
      	if (Client.serverHostPort != null) {
	  doConnection();
      	} else {
	  GUtil.invokeAndWait(new UpdateLabel(status,""));
      	}
      } catch (Exception e) {
      	GUtil.invokeAndWait(new DoDisconnected(Util.encodeException(e)));
      } finally {
      	try {timer.cancel();} catch (Exception ex) {}
      	shutdown();
      }
      Util.sleep(1.0);
    }
  }
  /****************************************************************************
  * doConnection.
  ****************************************************************************/
  private void doConnection () throws Exception {
    GUtil.invokeAndWait(new UpdateStatus("Connecting to server " + Client.serverHostPort.toString() + "..."));
    Util.sleep(0.5);
    s = new Socket(Client.serverHostPort.getHost(),Client.serverHostPort.getPort());
    in = new DataInputStream(new BufferedInputStream(s.getInputStream()));
    out = new DataOutputStream(new BufferedOutputStream(s.getOutputStream()));
    startTimer();
    GUtil.invokeAndWait(new UpdateStatus("Connected to Simulcast server on " + Client.serverHostPort.toString()));
    send(Util.XML_HEADER + "<simulcastMessage>" + "<clientMode/>" + "</simulcastMessage>");
    while (true) {
      String msg = in.readUTF();
      // System.out.println("rcvd: " + msg);
      restartTimer();
      GUtil.invokeAndWait(new DoMessage(msg));
    }
  }
  /****************************************************************************
  * DoMessage.
  ****************************************************************************/
  private class DoMessage implements Runnable {
    private String msg;
    public DoMessage (String msg) {
      this.msg = msg;
    }
    public void run () {
      try {
	doMessage(msg);
      } catch (Exception e) {
      	e.printStackTrace();
      }
    }
    private void doMessage (String msg) throws Exception {
      // System.out.println("ServerHandler.DoMessage.doMessage, msg: " + msg);
      XmlParser parser = new XmlParser(msg,"simulcastMessage");
      if (parser.hasElement("serverReady")) {

        return;
      }
      if (parser.hasElement("passListing")) {
        Client.passManager.handlePassListing(parser.getElement("passListing"));
        return;
      }
      if (parser.hasElement("passDetails")) {
        Client.passManager.handlePassDetails(parser.getElement("passDetails"));
        return;
      }
      if (parser.hasElement("passStarted")) {
        Client.passManager.handlePassStarted(parser.getElement("passStarted").getElement("passDetails"));
        return;
      }
      if (parser.hasElement("passUpdated")) {
        Client.passManager.handlePassUpdated(parser.getElement("passUpdated").getElement("passDetails"));
        return;
      }
      if (parser.hasElement("passEnded")) {
        Client.passManager.handlePassEnded(parser.getElement("passEnded").getElement("passDetails"));
        return;
      }
      if (parser.hasElement("passRemoved")) {
        Client.passManager.handlePassRemoved(parser.getElement("passRemoved"));
        return;
      }
      if (parser.hasElement("clientId")) {
        Client.clientId = parser.getElementValue("clientId");
        Client.serverSimulcastVersion = parser.getElementValue("simulcastVersion");
        return;
      }
      if (parser.hasElement("mostRecentTLEs")) {
        String[] aquaTLE = new String[] {
          parser.getElement("mostRecentTLEs").getElement("aqua").getElement("tle").getElementValue("line1"),
          parser.getElement("mostRecentTLEs").getElement("aqua").getElement("tle").getElementValue("line2")
        };
        String[] terraTLE = new String[] {
          parser.getElement("mostRecentTLEs").getElement("terra").getElement("tle").getElementValue("line1"),
          parser.getElement("mostRecentTLEs").getElement("terra").getElement("tle").getElementValue("line2")
        };
        Client.mercatorViewer.setTLEs(aquaTLE,terraTLE);
        Client.bothPolarViewer.setTLEs(aquaTLE,terraTLE);
        Client.northPolarViewer.setTLEs(aquaTLE,terraTLE);
        Client.southPolarViewer.setTLEs(aquaTLE,terraTLE);
        return;
      }
      if (parser.hasElement("groundStations")) {
      	Client.mercatorViewer.setGroundStations(new GroundStations(parser.getElement("groundStations")));
      	Client.bothPolarViewer.setGroundStations(new GroundStations(parser.getElement("groundStations")));
      	Client.northPolarViewer.setGroundStations(new GroundStations(parser.getElement("groundStations")));
      	Client.southPolarViewer.setGroundStations(new GroundStations(parser.getElement("groundStations")));
      	return;
      }
      throw new Exception("unknown message from server");
    }
  }
  /****************************************************************************
  * DoDisconnected.
  ****************************************************************************/
  private class DoDisconnected implements Runnable {
    private String reason;
    public DoDisconnected (String reason) {
      this.reason = reason;
    }
    public void run () {
      Client.passManager.clear();
      Client.passViewer.clear();
      Client.mercatorViewer.clear();
      Client.polarViewer.clear();
      Client.serverSimulcastVersion = "?";
      String text = "Server unavailable (" + reason + ")";
      status.update(text,Color.red);
      Client.switchToPassManager();
    }
  }
  /****************************************************************************
  * UpdateStatus.
  ****************************************************************************/
  private class UpdateStatus implements Runnable {
    private String text;
    public UpdateStatus (String text) {
      this.text = text;
    }
    public void run () {
      status.update(text,Color.black);
    }
  }
  /****************************************************************************
  * send.
  ****************************************************************************/
  public synchronized void send (String msg) {
    try {
      out.writeUTF(msg);
      out.flush();
      // System.out.println("ServerHandler.sent: " + msg);
    } catch (Exception e) {
      e.printStackTrace();
    }
  }
  /****************************************************************************
  * restartTimer.
  ****************************************************************************/
  private void restartTimer () {
    timer.cancel();
    startTimer();
  }
  /****************************************************************************
  * startTimer.
  ****************************************************************************/
  private void startTimer () {
    timer = new java.util.Timer();
    timer.schedule(new TimeOut(),TIMEOUT_INTERVAL_MS);
  }
  /****************************************************************************
  * TimeOut.
  ****************************************************************************/
  private class TimeOut extends TimerTask {
    public void run () {
      shutdown();
    }
  }
  /****************************************************************************
  * disconnect.
  ****************************************************************************/
  public void disconnect () {
    shutdown();
  }
  /****************************************************************************
  * shutdown.
  ****************************************************************************/
  public void shutdown () {
    // try {s.shutdownInput();} catch (Exception ex) {}
    // try {s.shutdownOutput();} catch (Exception ex) {}
    try {in.close();} catch (Exception ex) {}
    try {out.close();} catch (Exception ex) {}
    try {s.close();} catch (Exception ex) {}
  }
}
