/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*  10-Feb-06, J Love, GST	Original version.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.client;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import javax.swing.*;
import java.awt.*;
import java.awt.image.*;

public class TrueColorMapViewHandler extends JPanel implements ViewHandler {
  private MapHandler mapHandler;
  private int nBlockRows;
  private int nBlockCols;
  private int nGeoDataRows;
  private int nGeoDataCols;
  private int geoBinSize;
  private ResolutionCombo resolutionCombo;
  private JComboBox enhancementCombo;
  private PictureControls pictureControls;
  private EnhancementAlgorithm algorithm = null;
  private Color[][] colors;
  private int[] values = new int[3];
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * TrueColorMapViewHandler.
  ****************************************************************************/
  public TrueColorMapViewHandler (MapHandler mapHandler) {
    this.mapHandler = mapHandler;
    setLayout(new BorderLayout());
    resolutionCombo = new ResolutionCombo(Modis.resolutionChoices,2);
    enhancementCombo = new JComboBox(Modis.dayEnhancementChoices);
    enhancementCombo.setToolTipText("Enhancement algorithm to be applied (before contrast and brightness adjustments).");
    enhancementCombo.setMaximumSize(enhancementCombo.getPreferredSize());
    enhancementCombo.setSelectedIndex(Modis.DAY_ENHANCEMENT_2);
    pictureControls = new PictureControls(1.00,0);
    HPanel controls = new HPanel(new JLabel("Resolution:"),GUtil.HSpace(5),resolutionCombo,GUtil.HSpace(10),new JLabel("Enhancement:"),GUtil.HSpace(5),enhancementCombo,GUtil.HSpace(10),pictureControls);
    add(new MarginPanel(5,1,controls,5,1),BorderLayout.SOUTH);
  }
  /****************************************************************************
  * playStarting.
  ****************************************************************************/
  public void playStarting () {
    GUtil.invokeAndWait(new PlayStarting());
  }
  private class PlayStarting implements Runnable {
    public void run () {
      resolutionCombo.setEnabled(false);
      enhancementCombo.setEnabled(false);
      setEnhancement();
      mapHandler.playStarting();
    }
    private void setEnhancement () {
      String className = Modis.dayEnhancementClassNames[enhancementCombo.getSelectedIndex()];
      if (className == null) {
        algorithm = null;
      } else {
        try {
	  algorithm = (EnhancementAlgorithm) Class.forName(className).newInstance();
        } catch (Exception e) {
      	  algorithm = null;
        }
      }
    }
  }
  /****************************************************************************
  * clear.
  ****************************************************************************/
  public void clear () {

  }
  /****************************************************************************
  * getRequestMessage.
  ****************************************************************************/
  public String getRequestMessage (String passName) {
    StringBuffer msg = new StringBuffer();
    msg.append(Util.XML_HEADER);
    msg.append("<simulcastMessage>");
    msg.append("<playPass>");
    msg.append("<name>" + passName + "</name>");
    msg.append("<partial>");
    msg.append(mapHandler.getAreaFilter());
    msg.append("<send><geolocated/></send>");
    msg.append("<bands>");
    msg.append("<band>");
    msg.append("<bandIndex>" + Integer.toString(Modis.TRUE_COLOR_RED_BAND_INDEX) + "</bandIndex>");
    msg.append("<resolution>" + Modis.resolutionValues[resolutionCombo.getSelectedIndex()] + "</resolution>");
    msg.append("</band>");
    msg.append("<band>");
    msg.append("<bandIndex>" + Integer.toString(Modis.TRUE_COLOR_GREEN_BAND_INDEX) + "</bandIndex>");
    msg.append("<resolution>" + Modis.resolutionValues[resolutionCombo.getSelectedIndex()] + "</resolution>");
    msg.append("</band>");
    msg.append("<band>");
    msg.append("<bandIndex>" + Integer.toString(Modis.TRUE_COLOR_BLUE_BAND_INDEX) + "</bandIndex>");
    msg.append("<resolution>" + Modis.resolutionValues[resolutionCombo.getSelectedIndex()] + "</resolution>");
    msg.append("</band>");
    msg.append("</bands>");
    msg.append("</partial>");
    if (Client.optionsScreen.compressCheckBox.isSelected()) {
      msg.append("<compress>" + "gzip" + "</compress>");
    }
    msg.append("</playPass>");
    msg.append("</simulcastMessage>");
    return msg.toString();
  }
  /****************************************************************************
  * headerBlock.
  ****************************************************************************/
  public void headerBlock (HeaderBlock headerBlock) throws Exception {
    if (headerBlock.getBandCount() != 3) {
      throw new Exception("unexpected number of bands");
    }
    if (headerBlock.getBandIndex(0) != Modis.TRUE_COLOR_RED_BAND_INDEX) {
      throw new Exception("unexpected red band index");
    }
    int resolution = headerBlock.getBandResolution(0);
    nBlockRows = headerBlock.getBlockRowCount(0);
    nBlockCols = headerBlock.getBlockColCount(0);
    if (headerBlock.getBandIndex(1) != Modis.TRUE_COLOR_GREEN_BAND_INDEX) {
      throw new Exception("unexpected green band index");
    }
    if (headerBlock.getBandResolution(1) != resolution) {
      throw new Exception("inconsistent resolutions");
    }
    if (headerBlock.getBlockRowCount(1) != nBlockRows) {
      throw new Exception("inconsistent block rows");
    }
    if (headerBlock.getBlockColCount(1) != nBlockCols) {
      throw new Exception("inconsistent block columns");
    }
    if (headerBlock.getBandIndex(2) != Modis.TRUE_COLOR_BLUE_BAND_INDEX) {
      throw new Exception("unexpected blue band index");
    }
    if (headerBlock.getBandResolution(2) != resolution) {
      throw new Exception("inconsistent resolutions");
    }
    if (headerBlock.getBlockRowCount(2) != nBlockRows) {
      throw new Exception("inconsistent block rows");
    }
    if (headerBlock.getBlockColCount(2) != nBlockCols) {
      throw new Exception("inconsistent block columns");
    }
    geoBinSize = headerBlock.getGeoResolution() / resolution;
    nGeoDataRows = headerBlock.getGeoBlockRowCount();
    if ((nGeoDataRows * geoBinSize) != nBlockRows) {
      throw new Exception("inconsistent geolocation block rows");
    }
    nGeoDataCols = headerBlock.getGeoBlockColCount();
    if ((nGeoDataCols * geoBinSize) != nBlockCols) {
      throw new Exception("inconsistent geolocation block columns");
    }
    colors = new Color[nBlockRows][nBlockCols];
    mapHandler.headerBlock(headerBlock);
  }
  /****************************************************************************
  * scanGroup.
  ****************************************************************************/
  public void scanGroup (ScanGroup scanGroup) throws Exception {
    byte[][] redBuffer = scanGroup.getBowedBlock().getData()[0];
    byte[][] greenBuffer = scanGroup.getBowedBlock().getData()[1];
    byte[][] blueBuffer = scanGroup.getBowedBlock().getData()[2];
    for (int row = 0; row < nBlockRows; row++) {
       for (int col = 0; col < nBlockCols; col++) {
          values[0] = redBuffer[row][col] & 0xFF;
          values[1] = greenBuffer[row][col] & 0xFF;
          values[2] = blueBuffer[row][col] & 0xFF;
          if (algorithm != null) {
            algorithm.enhance(values);
          }
          values[0] = pictureControls.adjustValue(values[0]);
          values[1] = pictureControls.adjustValue(values[1]);
          values[2] = pictureControls.adjustValue(values[2]);
          colors[row][col] = new Color(values[0],values[1],values[2]);
       }
    }
    mapHandler.dataBlock(nGeoDataRows,nGeoDataCols,geoBinSize,
    			 scanGroup.getGeoBlock().getLats(),
			 scanGroup.getGeoBlock().getLons(),colors);
  }
  /****************************************************************************
  * playStopped.
  ****************************************************************************/
  public void playStopped () {
    GUtil.invokeAndWait(new PlayStopped());
  }
  private class PlayStopped implements Runnable {
    public void run () {
      resolutionCombo.setEnabled(true);
      // portionControls.setEnabled(true);
      enhancementCombo.setEnabled(true);
      mapHandler.playStopped();
    }
  }
  /****************************************************************************
  * getBufferedImage.
  ****************************************************************************/
  public BufferedImage getBufferedImage () {
  	
    return null;
  }
  /****************************************************************************
  * ResolutionSelected.
  ****************************************************************************/
  /*
  private class ResolutionSelected implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      portionControls.adjustWidthForResolution(Integer.parseInt(Modis.resolutionValues[resolutionCombo.getSelectedIndex()]));
    }
  }
  */
}
