/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*  27-Dec-04, J Love, GST	Original version.
*  19-Mar-05, J Love, GST	Ground station argument (tmp).
*  10-Feb-05, J Love, GST	Geolocation, etc.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.client;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import javax.swing.*;
import java.awt.*;
import java.awt.image.*;

public class TrueColorViewHandler extends JPanel implements ViewHandler {
  private int nBlockRows;
  private int nBlockCols;
  private ResolutionCombo resolutionCombo;
  private JComboBox enhancementCombo;
  private PictureControls pictureControls;
  private ImageScroller imageScroller;
  private EnhancementAlgorithm algorithm = null;
  private Color[] colors;
  private int[] values = new int[3];
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * TrueColorViewHandler.
  ****************************************************************************/
  public TrueColorViewHandler (int platformCode) {
    setLayout(new BorderLayout());
    resolutionCombo = new ResolutionCombo(Modis.resolutionChoices,2);
    enhancementCombo = new JComboBox(Modis.dayEnhancementChoices);
    enhancementCombo.setToolTipText("Enhancement algorithm to be applied (before contrast and brightness adjustments).");
    enhancementCombo.setMaximumSize(enhancementCombo.getPreferredSize());
    enhancementCombo.setSelectedIndex(Modis.DAY_ENHANCEMENT_2);
    pictureControls = new PictureControls(1.00,0);
    imageScroller = new ImageScroller();
    HPanel controls = new HPanel(new JLabel("Resolution:"),GUtil.HSpace(5),resolutionCombo,GUtil.HSpace(10),new JLabel("Enhancement:"),GUtil.HSpace(5),enhancementCombo,GUtil.HSpace(10),pictureControls);
    add(imageScroller,BorderLayout.CENTER);
    add(new MarginPanel(5,1,controls,5,1),BorderLayout.SOUTH);
  }
  /****************************************************************************
  * playStarting.
  ****************************************************************************/
  public void playStarting () {
    GUtil.invokeAndWait(new SetControls(false));
    String className = Modis.dayEnhancementClassNames[enhancementCombo.getSelectedIndex()];
    if (className == null) {
      algorithm = null;
    } else {
      try {
	algorithm = (EnhancementAlgorithm) Class.forName(className).newInstance();
      } catch (Exception e) {
      	algorithm = null;
      }
    }
    clear();
  }
  /****************************************************************************
  * clear.
  ****************************************************************************/
  public void clear () {
    imageScroller.clear();
  }
  /****************************************************************************
  * getRequestMessage.
  ****************************************************************************/
  public String getRequestMessage (String passName) {
    StringBuffer msg = new StringBuffer();
    msg.append(Util.XML_HEADER);
    msg.append("<simulcastMessage>");
    msg.append("<playPass>");
    msg.append("<name>" + passName + "</name>");
    msg.append("<partial>");
    msg.append("<send><debowed/></send>");
    msg.append("<bands>");
    msg.append("<band>");
    msg.append("<bandIndex>" + Integer.toString(Modis.TRUE_COLOR_RED_BAND_INDEX) + "</bandIndex>");
    msg.append("<resolution>" + Modis.resolutionValues[resolutionCombo.getSelectedIndex()] + "</resolution>");
    msg.append("</band>");
    msg.append("<band>");
    msg.append("<bandIndex>" + Integer.toString(Modis.TRUE_COLOR_GREEN_BAND_INDEX) + "</bandIndex>");
    msg.append("<resolution>" + Modis.resolutionValues[resolutionCombo.getSelectedIndex()] + "</resolution>");
    msg.append("</band>");
    msg.append("<band>");
    msg.append("<bandIndex>" + Integer.toString(Modis.TRUE_COLOR_BLUE_BAND_INDEX) + "</bandIndex>");
    msg.append("<resolution>" + Modis.resolutionValues[resolutionCombo.getSelectedIndex()] + "</resolution>");
    msg.append("</band>");
    msg.append("</bands>");
    msg.append("</partial>");
    if (Client.optionsScreen.compressCheckBox.isSelected()) {
      msg.append("<compress>" + "gzip" + "</compress>");
    }
    msg.append("</playPass>");
    msg.append("</simulcastMessage>");
    return msg.toString();
  }
  /****************************************************************************
  * headerBlock.
  ****************************************************************************/
  public void headerBlock (HeaderBlock headerBlock) throws Exception {
    // boolean isAscending = Simulcast.isAscending(Client.groundStation,headerBlock);
    boolean isAscending = (headerBlock.getDirectionCode() == Simulcast.ASCENDING_PASS_CODE);
    if (headerBlock.getBandCount() != 3) {
      throw new Exception("unexpected number of bands");
    }
    if (headerBlock.getBandIndex(0) != Modis.TRUE_COLOR_RED_BAND_INDEX) {
      throw new Exception("unexpected red band index");
    }
    int resolution = headerBlock.getBandResolution(0);
    nBlockRows = headerBlock.getBlockRowCount(0);
    nBlockCols = headerBlock.getBlockColCount(0);
    if (headerBlock.getBandIndex(1) != Modis.TRUE_COLOR_GREEN_BAND_INDEX) {
      throw new Exception("unexpected green band index");
    }
    if (headerBlock.getBandResolution(1) != resolution) {
      throw new Exception("inconsistent resolutions");
    }
    if (headerBlock.getBlockRowCount(1) != nBlockRows) {
      throw new Exception("inconsistent block rows");
    }
    if (headerBlock.getBlockColCount(1) != nBlockCols) {
      throw new Exception("inconsistent block columns");
    }
    if (headerBlock.getBandIndex(2) != Modis.TRUE_COLOR_BLUE_BAND_INDEX) {
      throw new Exception("unexpected blue band index");
    }
    if (headerBlock.getBandResolution(2) != resolution) {
      throw new Exception("inconsistent resolutions");
    }
    if (headerBlock.getBlockRowCount(2) != nBlockRows) {
      throw new Exception("inconsistent block rows");
    }
    if (headerBlock.getBlockColCount(2) != nBlockCols) {
      throw new Exception("inconsistent block columns");
    }
    imageScroller.reset(nBlockCols,true,isAscending,Modis.trueColorTip,
    			Client.optionsScreen.limitViewCheckBox.isSelected());
    colors = new Color[nBlockCols];
  }
  /****************************************************************************
  * scanGroup.
  ****************************************************************************/
  public void scanGroup (ScanGroup scanGroup) throws Exception {
    byte[][] redBuffer = scanGroup.getDebowedBlock().getData()[0];
    byte[][] greenBuffer = scanGroup.getDebowedBlock().getData()[1];
    byte[][] blueBuffer = scanGroup.getDebowedBlock().getData()[2];
    for (int row = 0; row < nBlockRows; row++) {
       for (int col = 0; col < nBlockCols; col++) {
          values[0] = redBuffer[row][col] & 0xFF;
          values[1] = greenBuffer[row][col] & 0xFF;
          values[2] = blueBuffer[row][col] & 0xFF;
          if (algorithm != null) {
            algorithm.enhance(values);
          }
          values[0] = pictureControls.adjustValue(values[0]);
          values[1] = pictureControls.adjustValue(values[1]);
          values[2] = pictureControls.adjustValue(values[2]);
          colors[col] = new Color(values[0],values[1],values[2]);
       }
       imageScroller.addRow(colors);
    }
  }
  /****************************************************************************
  * playStopped.
  ****************************************************************************/
  public void playStopped () {
    GUtil.invokeAndWait(new SetControls(true));
  }
  /****************************************************************************
  * getBufferedImage.
  ****************************************************************************/
  public BufferedImage getBufferedImage () {
    return imageScroller.getBufferedImage();
  }
  /****************************************************************************
  * SetControls.
  ****************************************************************************/
  private class SetControls implements Runnable {
    private boolean enabled;
    public SetControls (boolean enabled) {
      this.enabled = enabled;
    }
    public void run () {
      resolutionCombo.setEnabled(enabled);
      enhancementCombo.setEnabled(enabled);
    }
  }
}
