/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CONSOLE
*
*  History:
*
*  21-Jan-04, J Love, GST	Original version.
*  25-Feb-05, J Love, GST	Added Svalbard to the list.
*  10-Feb-06, J Love, GST	New port numbers for V3.0, etc.
*   2-Jan-07, J Love, GST	Tweaked initial window size.
*  11-Feb-07, J Love, GST	Better command line handling.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.console;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;

public class Console extends JFrame {
  protected static JFrame frame;
  private JPanel mainPanel;
  private MenuBarClock clock;
  protected static ServerManager serverManager;
  private static String[] serverChoices = new String[] {
    "nisds1.sci.gsfc.nasa.gov:3502",
    "nisfes.sci.gsfc.nasa.gov:3502",
    "nisfes.gsfc.nasa.gov:3502",
    "freddie.coas.oregonstate.edu:3502",
    "springbok.ssec.wisc.edu:3502",
    "simulcast.tss.no:3502",
    "localhost:3502"
  };
  private static final String USAGE = "Arguments: {-server_choice <host:port>}* [<host:port>]";
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * main.
  ****************************************************************************/
  public static void main (String[] args) throws Exception {
    HostPort serverHostPort = null;
    try {
      serverHostPort = parseArgs(args);
    } catch (Exception e) {
      System.out.println("Error: " + Util.encodeThrowable(e));
      System.out.println(USAGE);
      System.exit(1);
    }
    Console console = new Console();
    console.start(serverHostPort);
  }
  /****************************************************************************
  * parseArgs.
  ****************************************************************************/
  private static HostPort parseArgs (String[] args) throws Exception {
    HostPort serverHostPort = null;
    LinkedList<String> servers = new LinkedList<String>();
    int x = 0;
    while (x < args.length) {
      String arg = args[x++];
      if (arg.equals("-server_choice")) {
      	servers.add(args[x++]);
      	continue;
      }
      if (serverHostPort == null) {
      	serverHostPort = new HostPort(arg);
      	continue;
      }
      throw new Exception("unexpected argument: " + arg);
    }
    if (servers.size() > 0) {
      serverChoices = (String[]) servers.toArray(new String[servers.size()]);
    }
    return serverHostPort;
  }
  /****************************************************************************
  * Console.
  ****************************************************************************/
  public Console () throws Exception {
    frame = this;
    // UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
    UIManager.getDefaults().put("ComboBox.disabledForeground",Color.black);
    setTitle("Simulcast Console");
    mainPanel = new JPanel(new BorderLayout());
    getContentPane().setLayout(new BorderLayout());
    getContentPane().add(mainPanel,BorderLayout.CENTER);
    JMenuBar menuBar = new JMenuBar();
    JMenu fileMenu = new JMenu("File");
    JMenuItem connectItem = new JMenuItem("Connect...");
    connectItem.addActionListener(new ConnectButtonClicked());
    fileMenu.add(connectItem);
    JMenuItem exitItem = new JMenuItem("Exit");
    exitItem.addActionListener(new ExitButtonClicked());
    fileMenu.add(exitItem);
    menuBar.add(fileMenu);
    JMenu helpMenu = new JMenu("Help");
    JMenuItem aboutItem = new JMenuItem("About Simulcast...");
    aboutItem.addActionListener(new AboutButtonClicked(this));
    helpMenu.add(aboutItem);
    menuBar.add(helpMenu);
    menuBar.add(Box.createHorizontalGlue());
    clock = new MenuBarClock(true);
    menuBar.add(clock);
    mainPanel.add(menuBar,BorderLayout.NORTH);
    serverManager = new ServerManager();
    mainPanel.add(new MarginPanel(5,5,new JScrollPane(serverManager),5,5),BorderLayout.CENTER);
    pack();
    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    // GUtil.configureWindow(this,0.90,0.85);
    GUtil.configureWindow(this,1000,700);
    setVisible(true);
  }
  /****************************************************************************
  * start.
  ****************************************************************************/
  public void start (HostPort serverAddr) throws Exception {
    clock.start();
    if (serverAddr != null) {
      serverManager.addServer(new ServerHandler(serverAddr));
    } else {
      SwingUtilities.invokeLater(new PromptForServer());
    }
  }
  /****************************************************************************
  * PromptForServer.
  ****************************************************************************/
  private class PromptForServer implements Runnable {
    private JDialog dialog;
    private JComboBox combo;
    public void run () {
      dialog = new JDialog(frame,"Server Selection",true);
      dialog.setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
      dialog.setResizable(false);
      Container content = dialog.getContentPane();
      combo = new JComboBox(serverChoices);
      combo.setEditable(true);
      combo.setMaximumSize(combo.getPreferredSize());
      JButton selectButton = new JButton("Select");
      selectButton.addActionListener(new SelectButtonClicked());
      JButton cancelButton = new JButton("Cancel");
      cancelButton.addActionListener(new CancelButtonClicked());
      content.add(new MarginPanel(10,10,new VPanel(
        new HPanel(GUtil.HGlue(),new JLabel("Select the Simulcast Server host:port..."),GUtil.HGlue()),
        GUtil.VSpace(10),
        new HPanel(GUtil.HGlue(),combo,GUtil.HGlue()),
        GUtil.VSpace(10),
        new HPanel(GUtil.HGlue(),selectButton,GUtil.HSpace(10),cancelButton,GUtil.HGlue()),
        GUtil.VGlue()
      ),10,10));
      dialog.pack();
      GUtil.centerWindow(dialog);
      dialog.setVisible(true);
    }
    private class SelectButtonClicked implements ActionListener {
      public void actionPerformed (ActionEvent ae) {
	try {
          HostPort serverHostPort = new HostPort((String)combo.getSelectedItem());
          ServerHandler handler = new ServerHandler(serverHostPort);
          serverManager.addServer(handler);
	} catch (Exception e) {
	  e.printStackTrace();
          JOptionPane.showMessageDialog(frame,e.toString(),"Error",
            				JOptionPane.ERROR_MESSAGE);
      	}
        dialog.dispose();
      }
    }
    private class CancelButtonClicked implements ActionListener {
      public void actionPerformed (ActionEvent ae) {
        dialog.dispose();
      }
    }
  }
  /****************************************************************************
  * ConnectButtonClicked.
  ****************************************************************************/
  private class ConnectButtonClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      SwingUtilities.invokeLater(new PromptForServer());
    }
  }
  /****************************************************************************
  * ExitButtonClicked.
  ****************************************************************************/
  private class ExitButtonClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      System.exit(0);
    }
  }
  /****************************************************************************
  * AboutButtonClicked.
  ****************************************************************************/
  private class AboutButtonClicked implements ActionListener {
    private JFrame frame;
    public AboutButtonClicked (JFrame frame) {
      this.frame = frame;
    }
    public void actionPerformed (ActionEvent ae) {
      JOptionPane.showMessageDialog(frame,"Simulcast Console Version " +
      				    Simulcast.VERSION,"About Simulcast...",
				    JOptionPane.INFORMATION_MESSAGE);
    }
  }
}
