/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CONSOLE
*
*  History:
*
*  23-Nov-04, J Love, GST	Original version.
*  10-Feb-06, J Love, GST	Geolocation, etc.
*  10-Jan-07, J Love, GST	Added PassArchiveManager (and renamed
*   				PassManager to PassHistoryManager).  Added
*   				DEBUG option.  Dealt with IPv6.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.console;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.net.*;
import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

public class ServerHandler extends HPanel implements Runnable {
  private HostPort serverHostPort;
  private Socket s = null;
  private DataInputStream in = null;
  private DataOutputStream out = null;
  private java.util.Timer timer;
  private ArrowPanel arrowPanel;
  private ConsoleManager consoleManager;
  private ClientManager clientManager;
  private PingManager pingManager;
  private PassHistoryManager passHistoryManager;
  private PassArchiveManager passArchiveManager;
  private static final long TIMEOUT_INTERVAL_MS = 2 * Util.SERVER_READY_INTERVAL_SEC * 1000;
  private static final boolean DEBUG = false;
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * ServerHandler.
  ****************************************************************************/
  public ServerHandler (HostPort serverHostPort) {
    this.serverHostPort = serverHostPort;
    consoleManager = new ConsoleManager();
    clientManager = new ClientManager();
    pingManager = new PingManager();
    passHistoryManager = new PassHistoryManager();
    passArchiveManager = new PassArchiveManager(this);
    VPanel categoryPanel = new VPanel(consoleManager,clientManager,pingManager,passHistoryManager,passArchiveManager);
    JButton disconnectButton = new JButton("Disconnect");
    disconnectButton.addActionListener(new DisconnectButtonClicked());
    MarginPanel buttonPanel = new MarginPanel(5,5,disconnectButton,5,5);
    buttonPanel.setAlignmentY(TOP_ALIGNMENT);
    arrowPanel = new ArrowPanel(false,ArrowButton.DOWN,"",15,categoryPanel);
    arrowPanel.setRightClickRunnable(new RightClick());
    arrowPanel.setAlignmentY(TOP_ALIGNMENT);
    HPanel p1 = new HPanel(arrowPanel,GUtil.HGlue(),buttonPanel);
    HPanel p2 = new HPanel(p1);
    p2.setBorder(new LineBorder(Color.black,1));
    add(new MarginPanel(5,5,p2,0,5));
  }
  private class RightClick implements Runnable {
    public void run () {
      System.out.println("ServerHandler, right click");
    }
  }
  /****************************************************************************
  * run.
  ****************************************************************************/
  public void run () {
    try {
      doConnection();
    } catch (Exception e) {
      String text = "Error on connection to " + serverHostPort.toString() +
      		    ", " + Util.encodeException(e);
      GUtil.invokeAndWait(new UpdateTitle(arrowPanel,text,Color.red));
      Util.sleep(2.0);
      Console.serverManager.removeServer(this);
    } finally {
      try {timer.cancel();} catch (Exception ex) {}
      try {in.close();} catch (Exception ex) {}
      try {out.close();} catch (Exception ex) {}
      try {s.close();} catch (Exception ex) {}
    }
  }
  /****************************************************************************
  * doConnection.
  ****************************************************************************/
  private void doConnection () throws Exception {
    String text = "Connecting to " + serverHostPort.toString();
    GUtil.invokeAndWait(new UpdateTitle(arrowPanel,text,Color.black));
    Util.sleep(1.0);
    s = new Socket(serverHostPort.getHost(),serverHostPort.getPort());
    in = new DataInputStream(new BufferedInputStream(s.getInputStream()));
    out = new DataOutputStream(new BufferedOutputStream(s.getOutputStream()));
    startTimer();
    text = "SERVER, " + serverHostPort.toString();
    GUtil.invokeAndWait(new UpdateTitle(arrowPanel,text,Color.black));
    send(Util.XML_HEADER + "<simulcastMessage>" + "<consoleMode/>" +
	 "</simulcastMessage>");
    while (true) {
      String msg = in.readUTF();
      if (DEBUG) {
	System.out.println("ServerHandler.doConnection, msg: " + msg);
      }
      restartTimer();
      GUtil.invokeAndWait(new DoMessage(msg));
    }
  }
  /****************************************************************************
  * DoMessage.
  ****************************************************************************/
  private class DoMessage implements Runnable {
    private String msg;
    public DoMessage (String msg) {
      this.msg = msg;
    }
    public void run () {
      try {
        doMessage(msg);
      } catch (Exception e) {
      	e.printStackTrace();
      }
    }
  }
  private void doMessage (String msg) throws Exception {
    XmlParser parser = new XmlParser(msg,"simulcastMessage");
    if (parser.hasElement("startOfPassPing")) {
      pingManager.pingReceived(parser.getElement("startOfPassPing"));
      return;
    }
    if (parser.hasElement("startOfPass")) {
      passHistoryManager.startOfPass(parser.getElement("startOfPass"));
      return;
    }
    if (parser.hasElement("endOfPass")) {
      passHistoryManager.endOfPass(parser.getElement("endOfPass"));
      return;
    }
    if (parser.hasElement("errorOnPass")) {
      passHistoryManager.errorOnPass(parser.getElement("errorOnPass"));
      return;
    }
    if (parser.hasElement("serverReady")) {

      return;
    }
    if (parser.hasElement("startOfPassPing")) {
      pingManager.pingReceived(parser.getElement("startOfPassPing"));
      return;
    }
    if (parser.hasElement("startOfPass")) {
      passHistoryManager.startOfPass(parser.getElement("startOfPass"));
      return;
    }
    if (parser.hasElement("endOfPass")) {
      passHistoryManager.endOfPass(parser.getElement("endOfPass"));
      return;
    }
    if (parser.hasElement("errorOnPass")) {
      passHistoryManager.errorOnPass(parser.getElement("errorOnPass"));
      return;
    }
    if (parser.hasElement("passListing")) {
      passArchiveManager.passListing(parser.getElement("passListing").getElements("passDetails"));
      return;
    }
    if (parser.hasElement("passDetails")) {
      passArchiveManager.passDetails(parser.getElement("passDetails"));
      return;
    }
    if (parser.hasElement("passStarted")) {
      passArchiveManager.passStarted(parser.getElement("passStarted").getElement("passDetails"));
      return;
    }
    if (parser.hasElement("passUpdated")) {
      passArchiveManager.passUpdated(parser.getElement("passUpdated").getElement("passDetails"));
      return;
    }
    if (parser.hasElement("passEnded")) {
      passArchiveManager.passEnded(parser.getElement("passEnded").getElement("passDetails"));
      return;
    }
    if (parser.hasElement("passRemoved")) {
      passArchiveManager.passRemoved(parser.getElement("passRemoved"));
      return;
    }
    if (parser.hasElement("consoleListing")) {
      consoleManager.consoleListing(parser.getElement("consoleListing").getElements("console"));
      return;
    }
    if (parser.hasElement("consoleAdded")) {
      consoleManager.consoleAdded(parser.getElement("consoleAdded").getElement("console"));
      return;
    }
    if (parser.hasElement("consoleRemoved")) {
      consoleManager.consoleRemoved(parser.getElement("consoleRemoved").getElement("console"));
      return;
    }
    if (parser.hasElement("clientListing")) {
      clientManager.clientListing(parser.getElement("clientListing").getElements("client"));
      return;
    }
    if (parser.hasElement("clientAdded")) {
      clientManager.clientAdded(parser.getElement("clientAdded").getElement("client"));
      return;
    }
    if (parser.hasElement("clientRemoved")) {
      clientManager.clientRemoved(parser.getElement("clientRemoved").getElement("client"));
      return;
    }
    if (parser.hasElement("playerListing")) {
      clientManager.playerListing(parser.getElement("playerListing").getElements("player"));
      return;
    }
    if (parser.hasElement("playerAdded")) {
      clientManager.playerAdded(parser.getElement("playerAdded").getElement("player"));
      return;
    }
    if (parser.hasElement("playerUpdate")) {
      clientManager.playerUpdate(parser.getElement("playerUpdate").getElement("player"));
      return;
    }
    if (parser.hasElement("playerRemoved")) {
      clientManager.playerRemoved(parser.getElement("playerRemoved").getElement("player"));
      return;
    }
    if (parser.hasElement("consoleId")) {

      return;
    }
    throw new Exception("unknown message from server");
  }
  /****************************************************************************
  * send.
  ****************************************************************************/
  public void send (String msg) {
    try {
      out.writeUTF(msg);
      out.flush();
      if (DEBUG) {
	System.out.println("ServerHandler.send, msg: " + msg);
      }
    } catch (Exception e) {
      e.printStackTrace();
    }
  }
  /****************************************************************************
  * restartTimer.
  ****************************************************************************/
  private void restartTimer () {
    timer.cancel();
    startTimer();
  }
  /****************************************************************************
  * startTimer.
  ****************************************************************************/
  private void startTimer () {
    timer = new java.util.Timer();
    timer.schedule(new TimeOut(),TIMEOUT_INTERVAL_MS);
  }
  /****************************************************************************
  * TimeOut.
  ****************************************************************************/
  private class TimeOut extends TimerTask {
    public void run () {
      // try {s.shutdownInput();} catch (Exception ex) {}
      // try {s.shutdownOutput();} catch (Exception ex) {}
      try {in.close();} catch (Exception ex) {}
      try {out.close();} catch (Exception ex) {}
      try {s.close();} catch (Exception ex) {}
    }
  }
  /****************************************************************************
  * DisconnectButtonClicked.
  ****************************************************************************/
  private class DisconnectButtonClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      // try {s.shutdownInput();} catch (Exception ex) {}
      // try {s.shutdownOutput();} catch (Exception ex) {}
      try {in.close();} catch (Exception ex) {}
      try {out.close();} catch (Exception ex) {}
      try {s.close();} catch (Exception ex) {}
    }
  }
}
