/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/IMAGES
*
*  History:
*
*  16-Nov-04, J Love, GST	Original version.
*  22-Mar-05, J Love, GST	NSLS logging.
*  19-Dec-05, J Love, GST	Geolocation, etc.
*   1-Feb-07, J Love, GST	Daemon threads for Windows service.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.images;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.net.*;
import java.io.*;

public class ServerHandler implements Runnable {
  private Socket s = null;
  private DataInputStream in = null;
  private DataOutputStream out = null;
  /****************************************************************************
  * ServerHandler.
  ****************************************************************************/
  public ServerHandler () {

  }
  /****************************************************************************
  * run.
  ****************************************************************************/
  public void run () {
    Images.log.info("ServerHandler: running","v");
    while (true) {
      try {
        doConnection();
      } catch (Throwable t) {
	Images.log.warning("Not able to connect to server.","n");
	Images.log.warning("ServerHandler: error trying to connect to server",t,"v");
      } finally {
        try {in.close();} catch (Exception ex) {}
        try {out.close();} catch (Exception ex) {}
        try {s.close();} catch (Exception ex) {}
      }
      Util.sleep(5.0);
    }
  }
  /****************************************************************************
  * doConnection.
  ****************************************************************************/
  private void doConnection () throws Exception {
    s = new Socket(Images.serverHostPort.getHost(),Images.serverHostPort.getPort());
    in = new DataInputStream(new BufferedInputStream(s.getInputStream()));
    out = new DataOutputStream(new BufferedOutputStream(s.getOutputStream()));
    Images.log.info("ServerHandler: connected, s: " +
      			  s.getInetAddress().getHostAddress() + ":" +
		          Integer.toString(s.getPort()),"v");
    send(Util.XML_HEADER + "<simulcastMessage>" + "<clientMode/>" + "<imageMode/>" + "</simulcastMessage>");
    while (true) {
      String msg = in.readUTF();
      // System.out.println("rcvd: " + msg);
      try {
	doMessage(msg);
      } catch (Exception e) {
        Images.log.warning("Message from server is not corrected format.","n");
	Images.log.warning("ServerHandler: parsing message from server",e,"v");
      }
    }
  }
  /****************************************************************************
  * doMessage.
  ****************************************************************************/
  private void doMessage (String msg) throws Exception {
    XmlParser parser = new XmlParser(msg,"simulcastMessage");
    if (parser.hasElement("passListing")) {
      if (Images.imageExistingPasses.equals("no")) {
        // Ignore (the pass listing was sent automatically after the connection
      	// was made).
	return;
      }
      if (Images.imageExistingPasses.equals("all")) {
	Util.startThread(new ImageExistingPasses(parser.getElement("passListing").getElements("passDetails")),true);
	return;
      }
      if (Images.imageExistingPasses.equals("last")) {
      	XmlParser[] parsers = parser.getElement("passListing").getElements("passDetails");
      	if (parsers.length > 0) {
	  Util.startThread(new ImageExistingPasses(new XmlParser[] {parsers[parsers.length-1]}),true);
      	}
	return;
      }
      XmlParser[] parsers = parser.getElement("passListing").getElements("passDetails");
      for (int i = 0; i < parsers.length; i++) {
	 String passName = parsers[i].getElementValue("name");
	 if (passName.equals(Images.imageExistingPasses)) {
	   Util.startThread(new ImageExistingPasses(new XmlParser[] {parsers[i]}),true);
	   break;
	 }
      }
      return;
    }
    if (parser.hasElement("passStarted")) {
      // Ignore this - it doesn't have the starting solar code.
      return;
    }
    if (parser.hasElement("passUpdated")) {
      if (Images.imageNewPasses) {
	String passName = parser.getElement("passUpdated").getElement("passDetails").getElementValue("name");
	String startSolarCode = parser.getElement("passUpdated").getElement("passDetails").getElementValue("startSolarCode");
	Util.startThread(new ImageNewPass(passName,startSolarCode),true);
      }
      return;
    }
    if (parser.hasElement("passEnded")) {
      return;
    }
    // Ignore everything else.
    return;
  }
  /****************************************************************************
  * send.
  ****************************************************************************/
  public void send (String msg) {
    try {
      out.writeUTF(msg);
      out.flush();
      // System.out.println("send: " + msg);
    } catch (Exception e) {
      Images.log.warning("Not able to send message to server","n");
      Images.log.warning("ServerHandler: sending message to server",e,"v");
    }
  }
  /****************************************************************************
  * ImageNewPass.
  ****************************************************************************/
  private class ImageNewPass implements Runnable {
    private String passName;
    private String startSolarCode;
    public ImageNewPass (String passName, String startSolarCode) {
      this.passName = passName;
      this.startSolarCode = startSolarCode;
    }
    public void run () {
      try {
	doImages(passName,startSolarCode);
      } catch (Exception e) {
	Images.log.warning("New image processing error for " + passName,"n");
	Images.log.warning("ServerHandlerImageNewPass: processing images",e,"v");
      }
    }
  }
  /****************************************************************************
  * ImageExistingPasses.
  ****************************************************************************/
  private class ImageExistingPasses implements Runnable {
    private XmlParser[] parsers;
    public ImageExistingPasses (XmlParser[] parsers) {
      this.parsers = parsers;
    }
    public void run () {
      try {
      	image();
      } catch (Exception e) {
	Images.log.warning("Existing pass image processing error.","n");
	Images.log.warning("ServerHandler.ImageExistingPasses: processing images",e,"v");
      }
    }
    private void image () throws Exception {
      for (int i = 0; i < parsers.length; i++) {
	 String passName = parsers[i].getElementValue("name");
	 String startSolarCode = parsers[i].getElementValue("startSolarCode");
	 try {
	   doImages(passName,startSolarCode);
	 } catch (Exception e) {
	   Images.log.warning("Existing pass image processing error for " + passName,"n");
	   Images.log.warning("ServerHandler.ImageExistingPasses: processing images",e,"v");
	 }
      }
    }
  }
  /****************************************************************************
  * doImages.
  ****************************************************************************/
  private void doImages (String passName,
  			 String startSolarCode) throws Exception {
    //Images.log.info("Creating image for " + passName,"n");
    Images.log.info("ServerHandler: creating images: " + passName,"v");
    BufferedReader in = null;
    BufferedReader err = null;
    try {
      String cmd = Images.handlerScriptFile.getAbsolutePath() + " " +
		   passName + " " + startSolarCode + " " + Images.configFile;
      Images.log.info("ServerHandler: executing PassHandler: " + cmd,"v");
      Process process = Runtime.getRuntime().exec(cmd);
      in = new BufferedReader(new InputStreamReader(process.getInputStream()));
      while (true) {
        String line = in.readLine();
        if (line == null) {
      	  break;
        }
        if (line.startsWith("INFO: ")) {
  	  //Images.log.info("Info from image processing: " + line.substring(6),"n");
	  Images.log.info("ServerHandler: from PassHandler: " + line.substring(6),"v");
	  continue;
        }
        if (line.startsWith("WARNING: ")) {
	  Images.log.warning("Warning from image processing: " + line.substring(9), "n");
	  Images.log.warning("ServerHandler: from PassHandler: " + line.substring(9),"v");
	  continue;
        }
        if (line.startsWith("ERROR: ")) {
  	  Images.log.warning("Error from image processing: " + line.substring(7),"n");
	  Images.log.warning("ServerHandler: from PassHandler: " + line.substring(7),"v");
	  continue;
        }
        // Another kind of errors.
	//Images.log.warning("Info from image processing: " + line,"n");
	Images.log.warning("ServerHandler: from PassHandler: " + line,"v");
      }
      int exitValue = process.waitFor();
      if (exitValue != 0) {
        StringBuffer text = new StringBuffer();
        err = new BufferedReader(new InputStreamReader(process.getErrorStream()));
        while (true) {
          String line = err.readLine();
          if (line == null) {
      	    break;
          }
          text.append(" " + line);
        }
        throw new Exception("PassHandler failed, exit: " + exitValue + ", text: " + text.toString());
      }
      //Images.log.info("Simulcast completes creating image for " + passName + ".","n");
      Images.log.info("ServerHandler: completed, exit: " + Integer.toString(exitValue),"v");
    } finally {
      try {in.close();} catch (Exception e) {}
      try {err.close();} catch (Exception e) {}
    }
  }
}
