/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*   8-Feb-06, J Love, GST	Original version.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.images.handler;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.awt.*;

public class AutoDayNightMapViewHandler extends ViewHandler {
  private MapHandler mapHandler;
  private int resolution;
  private boolean compress;
  // private ShpOverlay features = null;
  private int nBlockRows;
  private int nBlockCols;
  private int nGeoDataRows;
  private int nGeoDataCols;
  private int geoBinSize;
  private EnhancementAlgorithm dayAlgorithm;
  private double dayContrast;
  private int dayBrightness;
  private EnhancementAlgorithm nightAlgorithm;
  private double nightContrast;
  private int nightBrightness;
  private int[] values = new int[3];
  private Color[][] colors;
  private Color[] grays = GUtil.getGrays();
  // private static final String WIDTH_PCT = "100";
  // private static final String OFF_CENTER_PCT = "0";
  /****************************************************************************
  * AutoDayNightMapViewHandler.
  ****************************************************************************/
  public AutoDayNightMapViewHandler (MapHandler mapHandler,
  				     boolean compress, int resolution,
  				     String dayAlgorithmClassName,
		      		     double dayContrast, int dayBrightness,
				     String nightAlgorithmClassName,
		      		     double nightContrast, int nightBrightness) throws Exception {
    this.mapHandler = mapHandler;
    this.resolution = resolution;
    this.compress = compress;
    this.dayAlgorithm = ((!dayAlgorithmClassName.equals("")) ? (EnhancementAlgorithm) Class.forName(dayAlgorithmClassName).newInstance() : null);
    this.dayContrast = dayContrast;
    this.dayBrightness = dayBrightness;
    this.nightAlgorithm = ((!nightAlgorithmClassName.equals("")) ? (EnhancementAlgorithm) Class.forName(nightAlgorithmClassName).newInstance() : null);
    this.nightContrast = nightContrast;
    this.nightBrightness = nightBrightness;
  }
  /****************************************************************************
  * getRequestMessage.
  ****************************************************************************/
  public String getRequestMessage (String passName) {
    StringBuffer msg = new StringBuffer();
    msg.append(Util.XML_HEADER);
    msg.append("<simulcastMessage>");
    msg.append("<playPass>");
    msg.append("<name>" + passName + "</name>");
    msg.append("<partial>");
    msg.append("<send><geolocated/></send>");
    msg.append("<bands>");
    msg.append("<band>");
    msg.append("<bandIndex>" + Integer.toString(Modis.TRUE_COLOR_RED_BAND_INDEX) + "</bandIndex>");
    msg.append("<resolution>" + Integer.toString(resolution) + "</resolution>");
    msg.append("</band>");
    msg.append("<band>");
    msg.append("<bandIndex>" + Integer.toString(Modis.TRUE_COLOR_GREEN_BAND_INDEX) + "</bandIndex>");
    msg.append("<resolution>" + Integer.toString(resolution) + "</resolution>");
    msg.append("</band>");
    msg.append("<band>");
    msg.append("<bandIndex>" + Integer.toString(Modis.TRUE_COLOR_BLUE_BAND_INDEX) + "</bandIndex>");
    msg.append("<resolution>" + Integer.toString(resolution) + "</resolution>");
    msg.append("</band>");
    msg.append("<band>");
    msg.append("<bandIndex>" + Integer.toString(Modis.INFRARED_BAND_INDEX) + "</bandIndex>");
    msg.append("<resolution>" + Integer.toString(resolution) + "</resolution>");
    msg.append("</band>");
    msg.append("</bands>");
    msg.append("</partial>");
    if (compress) {
      msg.append("<compress>" + "gzip" + "</compress>");
    }
    msg.append("</playPass>");
    msg.append("</simulcastMessage>");
    return msg.toString();
  }
  /****************************************************************************
  * headerBlock.
  ****************************************************************************/
  public void headerBlock (HeaderBlock headerBlock) throws Exception {
    if (headerBlock.getBandCount() != 4) {
      throw new Exception("unexpected number of bands");
    }
    if (headerBlock.getBandIndex(0) != Modis.TRUE_COLOR_RED_BAND_INDEX) {
      throw new Exception("unexpected red band index");
    }
    int resolution = headerBlock.getBandResolution(0);
    nBlockRows = headerBlock.getBlockRowCount(0);
    nBlockCols = headerBlock.getBlockColCount(0);
    if (headerBlock.getBandIndex(1) != Modis.TRUE_COLOR_GREEN_BAND_INDEX) {
      throw new Exception("unexpected green band index");
    }
    if (headerBlock.getBandResolution(1) != resolution) {
      throw new Exception("inconsistent resolutions");
    }
    if (headerBlock.getBlockRowCount(1) != nBlockRows) {
      throw new Exception("inconsistent block rows");
    }
    if (headerBlock.getBlockColCount(1) != nBlockCols) {
      throw new Exception("inconsistent block columns (" + headerBlock.getBlockColCount(1) + " vs " + nBlockCols + ")");
    }
    if (headerBlock.getBandIndex(2) != Modis.TRUE_COLOR_BLUE_BAND_INDEX) {
      throw new Exception("unexpected blue band index");
    }
    if (headerBlock.getBandResolution(2) != resolution) {
      throw new Exception("inconsistent resolutions");
    }
    if (headerBlock.getBlockRowCount(2) != nBlockRows) {
      throw new Exception("inconsistent block rows");
    }
    if (headerBlock.getBlockColCount(2) != nBlockCols) {
      throw new Exception("inconsistent block columns (" + headerBlock.getBlockColCount(2) + " vs " + nBlockCols + ")");
    }
    if (headerBlock.getBandIndex(3) != Modis.INFRARED_BAND_INDEX) {
      throw new Exception("unexpected infrared band index");
    }
    if (headerBlock.getBandResolution(3) != resolution) {
      throw new Exception("inconsistent resolutions");
    }
    if (headerBlock.getBlockRowCount(3) != nBlockRows) {
      throw new Exception("inconsistent block rows");
    }
    if (headerBlock.getBlockColCount(3) != nBlockCols) {
      throw new Exception("inconsistent block columns (" + headerBlock.getBlockColCount(2) + " vs " + nBlockCols + ")");
    }
    geoBinSize = headerBlock.getGeoResolution() / resolution;
    nGeoDataRows = headerBlock.getGeoBlockRowCount();
    if ((nGeoDataRows * geoBinSize) != nBlockRows) {
      throw new Exception("inconsistent geolocation block rows");
    }
    nGeoDataCols = headerBlock.getGeoBlockColCount();
    if ((nGeoDataCols * geoBinSize) != nBlockCols) {
      throw new Exception("inconsistent geolocation block columns");
    }
    colors = new Color[nBlockRows][nBlockCols];
    mapHandler.headerBlock(headerBlock);
  }
  /****************************************************************************
  * scanGroup.
  ****************************************************************************/
  public void scanGroup (ScanGroup scanGroup) throws Exception {
    switch (scanGroup.getScanBlock().getSolarCode()) {
      case Simulcast.DAY_PASS_CODE: {
      	dayScan(scanGroup);
      	break;
      }
      case Simulcast.NIGHT_PASS_CODE: {
      	nightScan(scanGroup);
      	break;
      }
    }
  }
  private void dayScan (ScanGroup scanGroup) throws Exception {
    byte[][] redBuffer = scanGroup.getBowedBlock().getData()[0];
    byte[][] greenBuffer = scanGroup.getBowedBlock().getData()[1];
    byte[][] blueBuffer = scanGroup.getBowedBlock().getData()[2];
    for (int row = 0; row < nBlockRows; row++) {
       for (int col = 0; col < nBlockCols; col++) {
          values[0] = redBuffer[row][col] & 0xFF;
          values[1] = greenBuffer[row][col] & 0xFF;
          values[2] = blueBuffer[row][col] & 0xFF;
          if (dayAlgorithm != null) {
            dayAlgorithm.enhance(values);
          }
	  values[0] = Util.adjustValue(values[0],dayContrast,dayBrightness);
	  values[1] = Util.adjustValue(values[1],dayContrast,dayBrightness);
	  values[2] = Util.adjustValue(values[2],dayContrast,dayBrightness);
          colors[row][col] = new Color(values[0],values[1],values[2]);
       }
    }
    mapHandler.dataBlock(nGeoDataRows,nGeoDataCols,geoBinSize,
    			 scanGroup.getGeoBlock().getLats(),
			 scanGroup.getGeoBlock().getLons(),colors);
  }
  private void nightScan (ScanGroup scanGroup) throws Exception {
    byte[][] buffer = scanGroup.getBowedBlock().getData()[3];
    for (int row = 0; row < nBlockRows; row++) {
       for (int col = 0; col < nBlockCols; col++) {
          int value = buffer[row][col] & 0xFF;
          if (nightAlgorithm != null) {
            nightAlgorithm.enhance(values);
          }
          value = 255 - Util.adjustValue(value,nightContrast,nightBrightness);
	  colors[row][col] = grays[value];
       }
    }
    mapHandler.dataBlock(nGeoDataRows,nGeoDataCols,geoBinSize,
    			 scanGroup.getGeoBlock().getLats(),
			 scanGroup.getGeoBlock().getLons(),colors);
  }
  /****************************************************************************
  * footerBlock.
  ****************************************************************************/
  public void footerBlock (FooterBlock footerBlock) throws Exception {
  	
  }
}
