/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/IMAGES
*
*  History:
*
*  16-Nov-04, J Love, GST	Original version.
*  17-Mar-05, J Love, GST	Ground station argument (tmp).
*   8-Feb-06, J Love, GST	Geolocation, etc.
*  20-Feb-07, J Love, GST	Use of sc.home property.  Specify more in the
*   				configuration file.  Tweaked email subject.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.images.handler;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.io.*;
import java.net.*;
import java.util.zip.*;
import java.awt.*;
import java.awt.image.*;
import java.text.*;
import java.util.*;
import javax.mail.*;
import javax.mail.internet.*;
import javax.activation.*;
import javax.media.jai.*;
import com.sun.media.jai.codec.*;

public class PassHandler {
  private String passName;
  private boolean mailImage;
  private boolean scpImage;
  private MapHandler mapHandler;
  private ViewHandler viewHandler;
  private int resolution;
  private Socket s;
  private DataInputStream in;
  private DataOutputStream out;
  private File file;
  private FileOutputStream mapOut;
  private HeaderBlock headerBlock;
  private HostPort serverHostPort;
  private File scratchDir;
  private MailDestination[] mailDestinations;
  private ScpDestination[] scpDestinations;
  private SimpleDateFormat dateTimeFormat;
  private static final boolean COMPRESS = false;
  private static final boolean DEBUG = false;
  /****************************************************************************
  * main.
  ****************************************************************************/
  public static final void main (String[] args) {
    try {
      PassHandler handler = new PassHandler(args,"lowResolution");
      handler.go();
    } catch (Exception e) {
      e.printStackTrace();
      System.err.println(e.toString());
      System.exit(1);
    }
    try {
      PassHandler handler = new PassHandler(args,"highResolution");
      handler.go();
    } catch (Exception e) {
      e.printStackTrace();
      System.err.println(e.toString());
      System.exit(1);
    }
    System.exit(0);
  }
  /****************************************************************************
  * PassHandler.
  ****************************************************************************/
  public PassHandler (String[] args, String tag) throws Exception {
    passName = args[0];
    XmlParser parser = new XmlParser(Util.getFile(args[2]),"imagesConfig");
    int width = Integer.parseInt(parser.getElement(tag).getElementValue("width"));
    int height = Integer.parseInt(parser.getElement(tag).getElementValue("height"));
    resolution = Integer.parseInt(parser.getElement(tag).getElementValue("resolution"));
    mailImage = parser.getElement(tag).hasElement("mail");
    scpImage = parser.getElement(tag).hasElement("scp");
    scratchDir = Util.getFile(parser.getElementValue("scratchDirPath"));
    serverHostPort = new HostPort(parser.getElementValue("serverHostPort"));
    String trueColorAlgorithmClassName = parser.getElement("trueColorImageEnhancement").getElementValue("algorithmClassName");
    double trueColorContrast = Double.parseDouble(parser.getElement("trueColorImageEnhancement").getElementValue("contrast"));
    int trueColorBrightness = Integer.parseInt(parser.getElement("trueColorImageEnhancement").getElementValue("brightness"));
    String infraredAlgorithmClassName = parser.getElement("infraredImageEnhancement").getElementValue("algorithmClassName");
    double infraredContrast = Double.parseDouble(parser.getElement("infraredImageEnhancement").getElementValue("contrast"));
    int infraredBrightness = Integer.parseInt(parser.getElement("infraredImageEnhancement").getElementValue("brightness"));
    mailDestinations = parseMailDestinations(parser.getElement("mailDestinations").getElements("mailDestination"));
    scpDestinations = parseScpDestinations(parser.getElement("scpDestinations").getElements("scpDestination"));
    dateTimeFormat = new SimpleDateFormat("yyyyMMMdd_HHmm");
    dateTimeFormat.setTimeZone(TimeZone.getTimeZone("GMT"));
    mapHandler = new MapHandler(width,height);
    viewHandler = new AutoDayNightMapViewHandler(mapHandler,COMPRESS,resolution,
    						 trueColorAlgorithmClassName,
      				     	         trueColorContrast,trueColorBrightness,
						 infraredAlgorithmClassName,
      				     		 infraredContrast,infraredBrightness);
  }
  private static MailDestination[] parseMailDestinations (XmlParser[] parsers) throws Exception {
    LinkedList<MailDestination> list = new LinkedList<MailDestination>();
    for (int i = 0; i < parsers.length; i++) {
       list.add(new MailDestination(parsers[i]));
    }
    return (MailDestination[]) list.toArray(new MailDestination[list.size()]);
  }
  private static ScpDestination[] parseScpDestinations (XmlParser[] parsers) throws Exception {
    LinkedList<ScpDestination> list = new LinkedList<ScpDestination>();
    for (int i = 0; i < parsers.length; i++) {
       list.add(new ScpDestination(parsers[i]));
    }
    return (ScpDestination[]) list.toArray(new ScpDestination[list.size()]);
  }
  /****************************************************************************
  * go.
  ****************************************************************************/
  public void go () throws Exception {
    try {
      playPass();
    } finally {
      try {out.close();} catch (Exception ex) {}
      try {in.close();} catch (Exception ex) {}
      try {s.close();} catch (Exception ex) {}
      try {mapOut.close();} catch (Exception ex) {}
      if (!DEBUG) {
        try {file.delete();} catch (Exception ex) {}
      }
    }
  }
  /****************************************************************************
  * playPass.
  ****************************************************************************/
  private void playPass () throws Exception {
    System.out.println("INFO: connecting to server");
    s = new Socket(serverHostPort.getHost(),serverHostPort.getPort());
    out = new DataOutputStream(new BufferedOutputStream(s.getOutputStream()));
    System.out.println("INFO: requesting pass");
    send(Util.XML_HEADER + "<simulcastMessage>" + "<imagePlayMode/>" + "<playMode/>" +
	 "</simulcastMessage>");
    String requestMessage = viewHandler.getRequestMessage(passName);
    send(requestMessage);
    if (COMPRESS) {
      in = new DataInputStream(new BufferedInputStream(new GZIPInputStream(s.getInputStream())));
    } else {
      in = new DataInputStream(new BufferedInputStream(s.getInputStream()));
    }
    System.out.println("INFO: reading pass");
    readPass();
    in.close();
    out.close();
    s.close();
    System.out.println("INFO: creating image");
    createImage();
    if (mailImage) {
      mailImage();
    }
    if (scpImage) {
      scpImage();
    }
    System.out.println("INFO: deleting image");
    if (!DEBUG) {
      file.delete();
    }
  }
  /****************************************************************************
  * send.
  ****************************************************************************/
  private void send (String msg) throws Exception {
    out.writeUTF(msg);
    out.flush();
  }
  /****************************************************************************
  * readPass.
  ****************************************************************************/
  private void readPass () throws Exception {
    /*
    ScanBlock scanBlock = null;
    GeoBlock geoBlock = null;
    DataBlock bowedBlock = null;
    DataBlock debowedBlock = null;
    */
    ClassList blocks = new ClassList();
    while (true) {
      Object block = Simulcast.readBlock(in,blocks);
      if (block instanceof HeaderBlock) {
        headerBlock = (HeaderBlock) block;
	viewHandler.headerBlock(headerBlock);
        continue;
      }
      if (block instanceof ScanBlock) {
        ScanBlock scanBlock = (ScanBlock) block;
        GeoBlock geoBlock = (GeoBlock) (headerBlock.isGeolocated() ? Simulcast.readBlock(in,blocks) : null);
        BowedDataBlock bowedBlock = (BowedDataBlock) (headerBlock.isGeolocated() ? Simulcast.readBlock(in,blocks) : null);
        DebowedDataBlock debowedBlock = (DebowedDataBlock) (headerBlock.isDebowed() ? Simulcast.readBlock(in,blocks) : null);
	viewHandler.scanGroup(new ScanGroup(scanBlock,geoBlock,bowedBlock,debowedBlock));
	continue;
      }
      if (block instanceof FooterBlock) {
        FooterBlock footerBlock = (FooterBlock) block;
        viewHandler.footerBlock(footerBlock);
	return;
      }
      if (block instanceof ErrorBlock) {
        ErrorBlock errorBlock = (ErrorBlock) block;
	throw new Exception(errorBlock.getText());
      }
      throw new Exception("unknown type of block");
    }
  }
  /****************************************************************************
  * createImage.
  ****************************************************************************/
  private void createImage () throws Exception {
    String baseName = Simulcast.getPlatformName(headerBlock.getPlatformCode()) + "_" + dateTimeFormat.format(new Date(headerBlock.getStartedAt()));
    BufferedImage bi = mapHandler.getBufferedImage();
    
    BufferedImage bi2 = new BufferedImage(10+bi.getWidth()+10,10+bi.getHeight()+10,bi.getType());
    Graphics g = bi2.getGraphics();
    g.setColor(Color.black);
    g.drawImage(bi,10,10,null);
    
    file = new File(scratchDir,baseName + "_" + "R" + Integer.toString(resolution/1000) + "km" + ".jpg");
    JPEGEncodeParam param = new JPEGEncodeParam();
    param.setQuality((float)0.95);
    mapOut = new FileOutputStream(file);
    JAI.create("encode",bi2,mapOut,"JPEG",param);
    mapOut.close();
  }
  /****************************************************************************
  * scpImage.
  ****************************************************************************/
  private void scpImage () {
    if (scpDestinations.length > 0) {
      try {
        System.out.println("INFO: transferring image (scp)");
        int nFailed = doScp(file);
        if (nFailed > 0) {
          System.out.println("WARNING: transferred image (scp), " + Integer.toString(nFailed) + " " + (nFailed != 1 ? "retries" : "retry"));
        }
      } catch (Exception e) {
      	System.out.println("ERROR: transferring image (scp) failed: " + e.toString());
      }
    }
  }
  private int doScp (File file) throws Exception {
    int nFailed = 0;
    for (int i = 0; i < scpDestinations.length; i++) {
       nFailed += scpToDestination(scpDestinations[i],file);
    }
    return nFailed;
  }
  private int scpToDestination (ScpDestination scpDestination, File file) throws Exception {
    int exitValue = 0;
    int nFailed = 0;
    String cmd = scpDestination.getCmd(file);
    for (int i = 0; i < 36; i++) {
       Process process = Runtime.getRuntime().exec(cmd);
       exitValue = process.waitFor();
       if (exitValue == 0) {
       	 return nFailed;
       }
    }
    throw new Exception("scp failed, exit code: " + exitValue);
  }
  /****************************************************************************
  * mailImage.
  ****************************************************************************/
  private void mailImage () {
    if (mailDestinations.length > 0) {
      String subject = "Simulcast V" + Simulcast.VERSION + ", " +
      		       headerBlock.getGroundStation().getName() + ", " +
      		       Simulcast.getPlatformName(headerBlock.getPlatformCode()) + ", " +
		       Simulcast.getInstrumentName(headerBlock.getInstrumentCode()) + ", " +
		       Util.encodeDateTime(headerBlock.getStartedAt()) + ", " +
		       Integer.toString(resolution) + "m" + ", " +
		       passName;
      String body = "JPEG attached.";
      for (int i = 0; i < mailDestinations.length; i++) {
      	 try {
           System.out.println("INFO: mailing image");
           mailToDestination(mailDestinations[i],subject,body);
      	 } catch (Exception e) {
           System.out.println("ERROR: mailing image failed: " + e.toString());
      	 }
      }
    }
  }
  private void mailToDestination (MailDestination mailDestination, String subject, String body) throws Exception {
    Properties props = System.getProperties();
    props.put("mail.smtp.host",mailDestination.getSmtpHost());
    Session session = Session.getDefaultInstance(props,null);
    Message msg = new MimeMessage(session);
    mailDestination.loadMessage(msg);
    msg.setSentDate(new Date());
    msg.setSubject(subject);
    BodyPart messageBodyPart = new MimeBodyPart();
    messageBodyPart.setText(body);
    Multipart multipart = new MimeMultipart();
    multipart.addBodyPart(messageBodyPart);
    messageBodyPart = new MimeBodyPart();
    DataSource source = new FileDataSource(file);
    messageBodyPart.setDataHandler(new DataHandler(source));
    messageBodyPart.setFileName(file.getName());
    multipart.addBodyPart(messageBodyPart);
    msg.setContent(multipart);
    Transport.send(msg);
  }
}
