/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/PROCESSOR
*
*  History:
*
*   6-Jul-05, J Love, GST	Original version.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.processor;

public class DeBow {
  private int[][] ix1000;
  private int[][] ix500;
  private int[][] ix250;
  private static final double scanAngle1000 = 110.0 / 1354;
  private static final double trackAngle1000 = 0.812352557 / 10;
  private static final double earthRadius = 1000 * 6378.17;
  private static final double spacecraftAltitude = 1000 * 705.295;
  /****************************************************************************
  * NoBow.
  ****************************************************************************/
  public DeBow () {
    ix1000 = calc(10,1354,1000,scanAngle1000,trackAngle1000);
    /*
    for (int rx = 0; rx < 10; rx++) {
       for (int cx = 1353; cx <= 1353; cx++) {
       	  System.out.println("rx: " + rx + ", cx: " + cx + ", ix: " + ix1000[rx][cx]);
       }
    }
    */
    ix500 = calc(2*10,2*1354,1000/2,scanAngle1000/2,trackAngle1000/2);
    ix250 = calc(4*10,4*1354,1000/4,scanAngle1000/4,trackAngle1000/4);
  }
  /****************************************************************************
  * load.
  ****************************************************************************/
  public void load (byte[][][] values, byte[][][] valuesNoBow) {
    for (int bandIndex = 0; bandIndex <= 1; bandIndex++) {
       load(4*10,4*1354,values[bandIndex],valuesNoBow[bandIndex],ix250);
    }
    for (int bandIndex = 2; bandIndex <= 6; bandIndex++) {
       load(2*10,2*1354,values[bandIndex],valuesNoBow[bandIndex],ix500);
    }
    for (int bandIndex = 7; bandIndex <= 37; bandIndex++) {
       load(10,1354,values[bandIndex],valuesNoBow[bandIndex],ix1000);
    }
  }
  private void load (int nRows, int nCols, byte[][] values, byte[][] valuesNoBow, int[][] ix) {
    for (int rx = 0; rx < nRows; rx++) {
       for (int cx = 0; cx < nCols; cx++) {
          // System.out.println("rx: " + rx + ", cx: " + cx + ", ix: " + ix[rx][cx]);
          valuesNoBow[rx][cx] = values[ix[rx][cx]][cx];
       }
    }
  }
  /****************************************************************************
  * calc.
  ****************************************************************************/
  private int[][] calc (int nRows, int nCols, int resolution, double scanAngleIncr, double trackAngleIncr) {
    int nRowsHalf = nRows / 2;
    int nColsHalf = nCols / 2;
    int[][] z = new int[nRowsHalf+1][nColsHalf+1];
    for (int row = 0; row <= nRowsHalf; row++) {
       for (int col = 0; col <= nColsHalf; col++) {
          double scanAngle = col * scanAngleIncr;
          double trackAngle = row * trackAngleIncr;
          z[row][col] = (int) Math.round(calcZ(scanAngle,trackAngle) / resolution);
       }
    }
    int[][] ix4th = new int[nRowsHalf+1][nColsHalf+1];
    for (int col = 0; col <= nColsHalf; col++) {
       int row = 0;
       int binCount = z[row+1][col] - z[row][col];
       int binValue = 0;
       for (int ixRow = 0; ixRow < nRowsHalf; ixRow++) {
          if (binCount == 0) {
            row++;
	    binCount = z[row+1][col] - z[row][col];
            binValue++;
          }
          ix4th[ixRow][col] = binValue;
          binCount--;
       }
    }
    /*
    for (int col = 0; col < nColsHalf; col++) {
       for (int row = 0; row < nRowsHalf; row++) {
          System.out.print(ix[row][col] + " ");
       }
       System.out.println("");
    }
    */
    int[][] ix = new int[nRows][nCols];
    for (int rx = 0; rx < nRowsHalf; rx++) {
       for (int cx = 0; cx < nColsHalf; cx++) {
          ix[rx][(nColsHalf-1)-cx] = (nRowsHalf-1) - ix4th[(nRowsHalf-1)-rx][cx];
          ix[rx][nColsHalf+cx] = (nRowsHalf-1) - ix4th[(nRowsHalf-1)-rx][cx];
          ix[nRowsHalf+rx][(nColsHalf-1)-cx] = nRowsHalf + ix4th[rx][cx];
          ix[nRowsHalf+rx][nColsHalf+cx] = nRowsHalf + ix4th[rx][cx];
       }
    }
    return ix;
  }
  /****************************************************************************
  * calcZ.
  ****************************************************************************/
  private double calcZ (double scanAngle, double trackAngle) {
    double d1;
    double scD = spacecraftAltitude + earthRadius;
    if (Math.abs(scanAngle) > 0.0) {
      double aB = 180.0 - asin((scD * sin(Math.abs(scanAngle))) / earthRadius);
      double aA = 180.0 - aB - Math.abs(scanAngle);
      d1 = (scD * sin(aA)) / sin(aB);
    } else {
      d1 = spacecraftAltitude;
    }
    return (d1 * sin(trackAngle));
  }
  /****************************************************************************
  * sin, asin, etc.
  ****************************************************************************/
  private double sin (double degrees) {
    return Math.sin(Math.toRadians(degrees));
  }
  private double asin (double value) {
    return Math.toDegrees(Math.asin(value));
  }
}
