/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/PROCESSOR
*
*  History:
*
*  19-Nov-04, J Love, GST	Original version.
*  22-Mar-05, J Love, GST	NSLS logging.
*   8-Feb-06, J Love, GST	Geolocation, etc.
*  16-Mar-06, J Love, GST	Fixed bug where first packets are after the
*				earth view portion of a scan.
*  12-Jan-07, J Love, GST	Added input timeout.
*   1-Feb-07, J Love, GST	Use of sc.home property.  Daemon threads for
*   				Windows service.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.processor;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import gov.nasa.gsfc.drl.geo.*;
import java.io.*;
import java.net.*;
// import java.text.*;

public class ModisHandler implements PassHandler, TimeoutMonitorListener, Runnable {
  private int platformCode;
  private Socket sRouter;
  private DataInputStream inRouter;
  private String lutBinPath;
  private LUTs luts;
  private long timeTagAdjustment;
  private HostPort serverAddr;
  private StationPassParameters passParameters;
  private Socket sServer;
  private DataInputStream inServer;
  private DataOutputStream outServer;
  private byte[][][] values = new byte[][][] {	// [BAND][ROW][COL]
    new byte[4*10][4*1354], new byte[4*10][4*1354],
    new byte[2*10][2*1354], new byte[2*10][2*1354], new byte[2*10][2*1354],
    new byte[2*10][2*1354], new byte[2*10][2*1354],
    new byte[10][1354], new byte[10][1354], new byte[10][1354],
    new byte[10][1354], new byte[10][1354], new byte[10][1354],
    new byte[10][1354], new byte[10][1354], new byte[10][1354],
    new byte[10][1354], new byte[10][1354], new byte[10][1354],
    new byte[10][1354], new byte[10][1354], new byte[10][1354],
    new byte[10][1354], new byte[10][1354], new byte[10][1354],
    new byte[10][1354], new byte[10][1354], new byte[10][1354],
    new byte[10][1354], new byte[10][1354], new byte[10][1354],
    new byte[10][1354], new byte[10][1354], new byte[10][1354],
    new byte[10][1354], new byte[10][1354], new byte[10][1354],
    new byte[10][1354]
  };
  private byte[][][] valuesNoBow = new byte[][][] {	// [BAND][ROW][COL]
    new byte[4*10][4*1354], new byte[4*10][4*1354],
    new byte[2*10][2*1354], new byte[2*10][2*1354], new byte[2*10][2*1354],
    new byte[2*10][2*1354], new byte[2*10][2*1354],
    new byte[10][1354], new byte[10][1354], new byte[10][1354],
    new byte[10][1354], new byte[10][1354], new byte[10][1354],
    new byte[10][1354], new byte[10][1354], new byte[10][1354],
    new byte[10][1354], new byte[10][1354], new byte[10][1354],
    new byte[10][1354], new byte[10][1354], new byte[10][1354],
    new byte[10][1354], new byte[10][1354], new byte[10][1354],
    new byte[10][1354], new byte[10][1354], new byte[10][1354],
    new byte[10][1354], new byte[10][1354], new byte[10][1354],
    new byte[10][1354], new byte[10][1354], new byte[10][1354],
    new byte[10][1354], new byte[10][1354], new byte[10][1354],
    new byte[10][1354]
  };
  private int[][][] svSums = new int[][][] {	// [BAND][FOV.DETECTOR][SAMPLE]
    new int[40][4], new int[40][4],
    new int[20][2], new int[20][2], new int[20][2], new int[20][2], new int[20][2],
    new int[10][1], new int[10][1], new int[10][1], new int[10][1], new int[10][1],
    new int[10][1], new int[10][1], new int[10][1], new int[10][1], new int[10][1],
    new int[10][1], new int[10][1], new int[10][1], new int[10][1], new int[10][1],
    new int[10][1], new int[10][1], new int[10][1], new int[10][1], new int[10][1],
    new int[10][1], new int[10][1], new int[10][1], new int[10][1], new int[10][1],
    new int[10][1], new int[10][1], new int[10][1], new int[10][1], new int[10][1],
    new int[10][1]
  };
  private Geo geo;
  private Geo.Extents estExtents;
  private Geo.Extents scanExtents;
  private ScanBlock scanBlock;
  private GeoBlock geoBlock;
  private DataBlock bowedBlock;
  private DataBlock debowedBlock;
  private int startSolarCode;
  private int solarCode;
  private int directionCode;
  private int nScans;
  private boolean firstScan;
  private int nDayScans;
  private int nNightScans;
  private int nDayEvGroups;
  private int nDaySvCalGroups;
  private int nDaySdCalGroups;
  private int nDaySrcaCalGroups;
  private int nDayBbCalGroups;
  private int nNightEvGroups;
  private int nNightSvCalGroups;
  private int nNightSdCalGroups;
  private int nNightSrcaCalGroups;
  private int nNightBbCalGroups;
  private int nEng1Groups;
  private int nEng2Groups;
  private static final int instrumentCode = Simulcast.MODIS_INSTRUMENT_CODE;
  // private static final int EV_FRAMES_PER_SCAN = 1354;
  private static final int SV_FRAMES_PER_SCAN = 50;
  // private static final int FOVS_PER_SCAN = 10;
  private static final boolean DO_CALIBRATION = true;
  private static final int ERF_CONTRAST = 256;
  private static final int ERF_BRIGHTNESS = 0;
  private static final int[] dayBandIndices = new int[] {
    0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,
    27,28,29,30,31,32,33,34,35,36,37
  };
  private static final int[] dayBandResolutions = new int[] {
    250,250,500,500,500,500,500,1000,1000,1000,1000,1000,1000,1000,1000,1000,
    1000,1000,1000,1000,1000,1000,1000,1000,1000,1000,1000,1000,1000,1000,1000,
    1000,1000,1000,1000,1000,1000,1000
  };
  private static final int[] dayBlockRowCounts = new int[] {
    40,40,20,20,20,20,20,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,
    10,10,10,10,10,10,10,10,10,10,10,10,10,10,10
  };
  private static final int[] dayBlockColCounts = new int[] {
    4*1354,4*1354,2*1354,2*1354,2*1354,2*1354,2*1354,1*1354,1*1354,1*1354,
    1*1354,1*1354,1*1354,1*1354,1*1354,1*1354,1*1354,1*1354,1*1354,1*1354,
    1*1354,1*1354,1*1354,1*1354,1*1354,1*1354,1*1354,1*1354,1*1354,1*1354,
    1*1354,1*1354,1*1354,1*1354,1*1354,1*1354,1*1354,1*1354
  };
  // private static final DecimalFormat pt7 = new DecimalFormat("0.0000000");
  private static final DeBow noBow = new DeBow();
  /****************************************************************************
  * start.
  ****************************************************************************/
  public Thread start (Socket sRouter, DataInputStream inRouter,
  		       String[] parameters, HostPort serverAddr,
  		       StationPassParameters passParameters) throws Exception {
    this.sRouter = sRouter;
    this.inRouter = inRouter;
    this.lutBinPath = parameters[1];
    this.timeTagAdjustment = Long.parseLong(parameters[2]);
    this.serverAddr = serverAddr;
    this.platformCode = Simulcast.getPlatformCode(parameters[0]);
    this.passParameters = passParameters;
    scanExtents = new Geo.Extents();
    scanBlock = new ScanBlock(scanExtents);
    geoBlock = new GeoBlock(/*Double.parseDouble(parameters[3]),
			    Double.parseDouble(parameters[4])*/);
    bowedBlock = new BowedDataBlock(values);
    debowedBlock = new DebowedDataBlock(valuesNoBow);
    return Util.startThread(this,true);
  }
  /****************************************************************************
  * run.
  ****************************************************************************/
  public void run () {
    sServer = null;
    inServer = null;
    outServer = null;
    try {
      if (sRouter == null) {
	doPing();
      } else {
      	doPass();
      }
    } catch (Exception e) {
      Processor.log.warning("Not able to connect server.","n");
      Processor.log.warning("ModisHandler: handling " + (sRouter == null ? "PING" : "PASS"),e,"v");
    } finally {
      // try {sRouter.shutdownInput();} catch (Exception ex) {}
      try {inRouter.close();} catch (Exception ex) {}
      try {sRouter.close();} catch (Exception ex) {}
      // try {sServer.shutdownInput();} catch (Exception ex) {}
      // try {sServer.shutdownOutput();} catch (Exception ex) {}
      try {inServer.close();} catch (Exception ex) {}
      try {outServer.close();} catch (Exception ex) {}
      try {sServer.close();} catch (Exception ex) {}
    }
    Processor.handlerCache.put(this);
  }
  /****************************************************************************
  * doPing.
  ****************************************************************************/
  private void doPing () throws Exception {
    Processor.log.info("ModisHandler: running (PING)" +
    			     ", " + passParameters.toText() +
			     ", platformCode: " + Integer.toString(platformCode) +
			     ", instrumentCode: " + Integer.toString(instrumentCode) +
			     ", serverAddr: " + serverAddr.toString(),"v");
    sServer = new Socket(serverAddr.getHost(),serverAddr.getPort());
    outServer = new DataOutputStream(new BufferedOutputStream(sServer.getOutputStream()));
    String msg = Util.XML_HEADER + "<simulcastMessage>" +
		 "<startOfPassPing>" + passParameters.toXml() +
		 "</startOfPassPing>" + "</simulcastMessage>";
    outServer.writeUTF(msg);
    outServer.flush();
    outServer.close();
    // sServer.shutdownOutput();
    sServer.close();
    Processor.log.info("ModisHandler: completed (PING)","v");
  }
  /****************************************************************************
  * doPass.
  ****************************************************************************/
  private void doPass () throws Exception {
    Processor.log.info("ModisHandler: running (PASS), sRouter: " +
      		       sRouter.getInetAddress().getHostAddress() + ":" +
		       Integer.toString(sRouter.getPort()) +
		       ", " + passParameters.toText() +
		       ", platformCode: " + Integer.toString(platformCode) +
		       ", instrumentCode: " + Integer.toString(instrumentCode) +
		       ", serverAddr: " + serverAddr.toString(),"v");
    sServer = new Socket(serverAddr.getHost(),serverAddr.getPort());
    inServer = new DataInputStream(new BufferedInputStream(sServer.getInputStream()));
    outServer = new DataOutputStream(new BufferedOutputStream(sServer.getOutputStream()));
    String msg = Util.XML_HEADER + "<simulcastMessage>" +
		 "<startOfPass>" + passParameters.toXml() +
		 "</startOfPass>" + "</simulcastMessage>";
    outServer.writeUTF(msg);
    outServer.flush();
    String summary = null;
    TimeoutMonitor monitor = new TimeoutMonitor(Processor.inputTimeOut,this);
    monitor.start();
    try {
      init();
      summary = process(monitor);
    } catch (Exception e) {
      if (monitor.isTimedOut()) {
	e = new Exception("timeout while processing pass ("+Util.encodeTimeInterval(Processor.inputTimeOut)+")");
      }
      ErrorBlock errorBlock = new ErrorBlock(Util.encodeException(e));
      errorBlock.write(outServer);
      outServer.flush();
      throw e;
    }
    monitor.stop();
    // sRouter.shutdownInput();
    inRouter.close();
    sRouter.close();
    // sServer.shutdownInput();
    // sServer.shutdownOutput();
    inServer.close();
    outServer.close();
    sServer.close();
    Processor.log.info("ModisHandler: completed (PASS), " + summary,"v");
  }
  /****************************************************************************
  * init.
  ****************************************************************************/
  private void init () throws Exception {
    firstScan = true;
    nScans = 0;
    nDayScans = 0;
    nNightScans = 0;
    nDayEvGroups = 0;
    nDaySvCalGroups = 0;
    nDaySdCalGroups = 0;
    nDaySrcaCalGroups = 0;
    nDayBbCalGroups = 0;
    nNightEvGroups = 0;
    nNightSvCalGroups = 0;
    nNightSdCalGroups = 0;
    nNightSrcaCalGroups = 0;
    nNightBbCalGroups = 0;
    nEng1Groups = 0;
    nEng2Groups = 0;
    startSolarCode = Simulcast.UNKNOWN;
    luts = Processor.lutsManager.get(lutBinPath);
    Util.fill(svSums,0);
  }
  /***************************************************************************
  * process.
  ***************************************************************************/
  private String process (TimeoutMonitor monitor) throws Exception {
    byte[] buffer = new byte[624];
    long timeTag;		// Milliseconds since 00:00:00.000 1/1/1970 UTC.
    long currentTimeTag = -1;
    int currentScan = -1;
    while (true) {
      if (!Util.fillBuffer(inRouter,buffer,6,true)) {
	break;
      }
      monitor.inputReceived();
      // int version = ((buffer[0] & 0xE0) >>> 5);
      // int type = ((buffer[0] & 0x10) >>> 4);
      // int secHdrFlag = ((buffer[0] & 0x08) >>> 3);
      int apid = (((buffer[0] & 0x07) << 8) | (buffer[1] & 0xFF));
      if (apid != Modis.APID) {
      	throw new Exception("unknown apid: " + apid);
      }
      int groupSequence = (buffer[2] & 0xC0) >>> 6;
      // int pktCount = (((buffer[2] & 0x3F) << 8) | (buffer[3] & 0xFF));
      // int pktLength = (((buffer[4] & 0xFF) << 8) | (buffer[5] & 0xFF));

      Util.fillBuffer(inRouter,buffer,9);
      int dayCount = ((buffer[0] & 0xFF) << 8) | (buffer[1] & 0xFF);
      int coarseTime = ((buffer[2] & 0xFF) << 24) | ((buffer[3] & 0xFF) << 16) | ((buffer[4] & 0xFF) << 8) | (buffer[5] & 0xFF);
      // int fineTime = ((buffer[6] & 0xFF) << 8) | (buffer[7] & 0xFF);
      // long tiaSeconds = ((dayCount - 12784) * 86400) + (coarseTime / 1000);	// 12784 is days from 1/1/1958 to 1/1/1993
      timeTag = ((((long)dayCount) - 4383) * 86400 * 1000) + coarseTime;	// 4383 is days from 1/1/1958 to 1/1/1970
      
      timeTag += timeTagAdjustment;

      // int quickLook = ((buffer[8] & 0x80) >>> 7);
      int packetType = (buffer[8] & 0x70) >>> 4;
      int mirrorSide = buffer[8] & 0x01;
      int scanCount = ((buffer[8] & 0x0E) >>> 1);
      
      int frameCount = 0;
      int calType = 0;
      // int calMode = 0;
      // int calFrameCount = 0;
	  
      Util.fillBuffer(inRouter,buffer,3);
      int sourceType = (buffer[0] & 0x80) >>> 7;
      switch (sourceType) {
      	case 0: {
	  frameCount = (((buffer[0] & 0x7F) << 4) | ((buffer[1] & 0xF0) >>> 4));
	  break;
      	}
      	case 1: {
	  calType = (buffer[0] & 0x60) >>> 5;
	  // calMode = (buffer[0] & 0x18) >>> 3;
	  // calFrameCount = ((buffer[0] & 0x03) << 4) | ((buffer[1] & 0xF0) >>> 4);
	  break;
	}
      }
      // int sciState = ((buffer[2] & 0x02) >>> 1);
      // int sciAbnorm = (buffer[2] & 0x01);

      // Check if a new scan has started.  Send the scan only if this isn't
      // the first packet and the current time tag has been set (which won't
      // be the case if the first packets received for the first scan were
      // after the earth view portion).
      if (scanCount != currentScan) {
        if (currentScan != -1 && currentTimeTag != -1) {
          writeScan(currentTimeTag);
	  Util.fill(values,(byte)0);
	  Util.fill(valuesNoBow,(byte)0);
	  Util.fill(svSums,0);
        }
        currentScan = scanCount;
      }

      switch (packetType) {
	case 0: {
	  // Day Group
	  Util.fillBuffer(inRouter,buffer,624);
	  switch (sourceType) {
	    case 0: {
	      // Earth View
	      nDayEvGroups++;
	      solarCode = Simulcast.DAY_PASS_CODE;
	      currentTimeTag = timeTag;
	      // System.out.println("timeTag: " + timeTag + ", frameCount: " + frameCount + ", scanCount: " + scanCount);
	      switch (groupSequence) {
	      	case 1: {
	      	  loadDayValues(buffer,frameCount-1,0,mirrorSide);
	      	  break;
	      	}
	      	case 2: {
	      	  loadDayValues(buffer,frameCount-1,5,mirrorSide);
	      	  break;
	      	}
	      	default: {
	      	  throw new Exception("unexpected group sequence: " +
	      	  		      Integer.toString(groupSequence));
	      	}
	      }
	      break;
	    }
	    case 1: {
	      // Calibration
	      switch (calType) {
	        case 0: {
	          // Solar Diffuser
	      	  nDaySdCalGroups++;
	      	  break;
	      	}
	      	case 1: {
	      	  // SRCA
	      	  nDaySrcaCalGroups++;
		  break;
	      	}
	      	case 2: {
	      	  nDayBbCalGroups++;
	      	  // Black Body
	      	  break;
	      	}
	      	case 3: {
	      	  // Space View
	      	  nDaySvCalGroups++;
	          switch (groupSequence) {
	            case 1: {
	      	      sumDayValues(buffer,0,svSums);
	      	      break;
	      	    }
	      	    case 2: {
	      	      sumDayValues(buffer,5,svSums);
	      	      break;
	            }
		    default: {
	      	      throw new Exception("unexpected group sequence: " +
	      	      			  Integer.toString(groupSequence));
	      	    }
	          }
	      	  break;
	        }
	      }
	      break;
	    }
	  }
	  break;
	}
	case 1: {
	  // NightGroup
	  Util.fillBuffer(inRouter,buffer,258);
	  switch (sourceType) {
	    case 0: {
	      // Earth View
	      nNightEvGroups++;
	      solarCode = Simulcast.NIGHT_PASS_CODE;
	      currentTimeTag = timeTag;
	      switch (groupSequence) {
	      	case 3: {
	      	  loadNightValues(buffer,frameCount-1,mirrorSide);
	      	  break;
	      	}
	      	default: {
	      	  throw new Exception("unexpected group sequence: " +
	      	  		      Integer.toString(groupSequence));
	      	}
	      }
	      break;
	    }
	    case 1: {
	      // Calibration
	      switch (calType) {
	        case 0: {
	          // Solar Diffuser
	      	  nNightSdCalGroups++;
	      	  break;
	      	}
	      	case 1: {
	      	  // SRCA
	      	  nNightSrcaCalGroups++;
		  break;
	      	}
	      	case 2: {
	      	  // Black Body
	      	  nNightBbCalGroups++;
	      	  break;
	      	}
	      	case 3: {
	      	  // Space View
	      	  nNightSvCalGroups++;
	          switch (groupSequence) {
	            case 3: {
	      	      sumNightValues(buffer,svSums);
	      	      break;
	      	    }
	      	    default: {
	      	      throw new Exception("unexpected group sequence: " +
	      	      			  Integer.toString(groupSequence));
	      	    }
	          }
	      	  break;
	        }
	      }
	      break;
	    }
	  }
	  break;
	}
	case 2: {
	  // Engineering 1
	  nEng1Groups++;
	  Util.fillBuffer(inRouter,buffer,624);
	  break;
	}
	case 4: {
	  // Engineering 2
	  nEng2Groups++;
	  Util.fillBuffer(inRouter,buffer,624);
	  switch (groupSequence) {
	    case 1: {
	      // S/C ancillary data (start at byte offset 128).
	      /*
	       * packet header at 128-133
	       * secondary header id & timestamp at 134-141
	       * flag byte at 142
	       * time conversion at 143-145
	       */
	      /*
	      int scPosX = extract32_2C(buffer,146,-3);
	      int scPosY = extract32_2C(buffer,150,-3);
	      int scPosZ = extract32_2C(buffer,154,-3);
	      System.out.println("pos: " + scPosX + " " + scPosY + " " + scPosZ);
	      int scVelX = extract32_2C(buffer,158,-12);
	      int scVelY = extract32_2C(buffer,162,-12);
	      int scVelZ = extract32_2C(buffer,166,-12);
	      System.out.println("vel: " + scVelX + " " + scVelY + " " + scVelZ);
	      int scRoll = extract12_2C(buffer,170,0);
	      int scPitch = extract12_2C(buffer,172,0);
	      int scYaw = extract12_2C(buffer,174,0);
	      System.out.println("att: " + scRoll + " " + scPitch + " " + scYaw);
	      */
	      break;
	    }
	    case 2: {
	      break;
	    }
	  }
	  break;
	}
	default: {
	  throw new Exception("unexpected packet type: " +
	  		      Integer.toString(packetType));
	}
      }
    }
    
    if (nScans < 1) {
      throw new Exception("no scans");
    }
    
    //
    // Write last scan.
    //
    writeScan(currentTimeTag);
    
    //
    // Write footer block.
    //
    writeFooterBlock(currentTimeTag);
    
    // System.out.println(nDayEvGroups + " " + nNightEvGroups);
    
    long totalGroups = nDayEvGroups + nDaySvCalGroups + nDaySdCalGroups +
    		       nDaySrcaCalGroups + nDayBbCalGroups + nNightEvGroups +
    		       nNightSvCalGroups + nNightSdCalGroups + nNightSrcaCalGroups +
		       nNightBbCalGroups + nEng1Groups + nEng2Groups;
    return Long.toString(totalGroups) + " groups processed";
  }
  /****************************************************************************
  * extract32_2C.
  ****************************************************************************/
  /* ONLY NEEDED IF ENGINEERING GROUPS ARE BEING READ
  private int extract32_2C (byte[] buffer, int offset, int scale) {
    // System.out.println(buffer[offset] + " " + buffer[offset+1] + " " + buffer[offset+2] + " " + buffer[offset+3]);
    int value = ((buffer[offset] & 0xFF) << 24) |
		((buffer[offset+1] & 0xFF) << 16) |
		((buffer[offset+2] & 0xFF) << 8) |
		(buffer[offset+3] & 0xFF);
    return value;
  }
  */
  /****************************************************************************
  * extract12_2C.
  ****************************************************************************/
  /* ONLY NEEDED IF ENGINEERING GROUPS ARE BEING READ
  private int extract12_2C (byte[] buffer, int offset, int scale) {
    // System.out.println(buffer[offset] + " " + buffer[offset+1]);
    int value = ((buffer[offset] & 0x0F) << 8) |
		(buffer[offset+1] & 0xFF);
    if ((value & 0x0800) != 0) {
      value = value | 0xFFFFF000;
    }
    return value;
  }
  */
  /****************************************************************************
  * loadDayValues.
  ****************************************************************************/
  private void loadDayValues (byte[] buffer, int frameIndex,
  			      int startFovIndex, int mirrorSide) {
    int value;
    int offset = 0;
    boolean onByte = true;
    int stopFovIndex = startFovIndex + 4;
    for (int fovIndex = startFovIndex; fovIndex <= stopFovIndex; fovIndex++) {
       for (int bandIndex = 0; bandIndex <= 1; bandIndex++) {
       	  for (int sampleIndex = 0; sampleIndex < 4; sampleIndex++) {
	     int col = (4 * frameIndex) + sampleIndex;
       	     for (int detectorIndex = 0; detectorIndex < 4; detectorIndex++) {
       	     	if (onByte) {
		  value = ((buffer[offset] & 0xFF) << 4) | ((buffer[offset+1] & 0xF0) >>> 4);
		  offset += 1;
		  onByte = false;
       	     	} else {
		  value = ((buffer[offset] & 0x0F) << 8) | (buffer[offset+1] & 0xFF);
		  offset += 2;
		  onByte = true;
       	     	}
		if (DO_CALIBRATION) {
		  int reflectiveBandIndex = Modis.getReflectiveBandIndex(bandIndex);
		  int svAverage = svSums[bandIndex][(4*fovIndex)+detectorIndex][sampleIndex] / SV_FRAMES_PER_SCAN;
       	     	  double dnEVstar = (value - svAverage) / luts.getRVSrefLUT(reflectiveBandIndex)[detectorIndex/4][mirrorSide][frameIndex];
       	     	  double erf = luts.getM1LUT(reflectiveBandIndex)[detectorIndex/4][detectorIndex%4][mirrorSide] * dnEVstar;
       	     	  value = Util.toRange(0,(int)((erf*ERF_CONTRAST)+ERF_BRIGHTNESS),255);
		} else {
		  value = value >>> 4;
		}
       	     	int row = (4 * fovIndex) + detectorIndex;
       	     	row = 39 - row;
		values[bandIndex][row][col] = (byte) value;
       	     }
       	  }
       }
       for (int bandIndex = 2; bandIndex <= 6; bandIndex++) {
       	  for (int sampleIndex = 0; sampleIndex < 2; sampleIndex++) {
	     int col = (2 * frameIndex) + sampleIndex;
       	     for (int detectorIndex = 0; detectorIndex < 2; detectorIndex++) {
       	     	if (onByte) {
		  value = ((buffer[offset] & 0xFF) << 4) | ((buffer[offset+1] & 0xF0) >>> 4);
		  offset += 1;
		  onByte = false;
       	     	} else {
		  value  = ((buffer[offset] & 0x0F) << 8) | (buffer[offset+1] & 0xFF);
		  offset += 2;
		  onByte = true;
       	     	}
		if (DO_CALIBRATION) {
		  int reflectiveBandIndex = Modis.getReflectiveBandIndex(bandIndex);
		  int svAverage = svSums[bandIndex][(2*fovIndex)+detectorIndex][sampleIndex] / SV_FRAMES_PER_SCAN;
       	     	  double dnEVstar = (value - svAverage) / luts.getRVSrefLUT(reflectiveBandIndex)[detectorIndex/4][mirrorSide][frameIndex];
       	     	  double erf = luts.getM1LUT(reflectiveBandIndex)[detectorIndex/4][detectorIndex%4][mirrorSide] * dnEVstar;
       	     	  value = Util.toRange(0,(int)((erf*ERF_CONTRAST)+ERF_BRIGHTNESS),255);
		} else {
		  value = value >>> 4;
		}
       	     	int row = (2 * fovIndex) + detectorIndex;
       	     	row = 19 - row;
		values[bandIndex][row][col] = (byte) value;
       	     }
       	  }
       }
       for (int bandIndex = 7; bandIndex <= 37; bandIndex++) {
       	  for (int sampleIndex = 0; sampleIndex < 1; sampleIndex++) {
	     int col = frameIndex + sampleIndex;
       	     for (int detectorIndex = 0; detectorIndex < 1; detectorIndex++) {
       	     	if (onByte) {
		  value = ((buffer[offset] & 0xFF) << 4) | ((buffer[offset+1] & 0xF0) >>> 4);
		  offset += 1;
		  onByte = false;
       	     	} else {
		  value = ((buffer[offset] & 0x0F) << 8) | (buffer[offset+1] & 0xFF);
		  offset += 2;
		  onByte = true;
       	     	}
		if (DO_CALIBRATION) {
		  int svAverage = svSums[bandIndex][fovIndex+detectorIndex][sampleIndex] / SV_FRAMES_PER_SCAN;
             	  if (Modis.isReflectiveBand(bandIndex)) {
             	    int reflectiveBandIndex = Modis.getReflectiveBandIndex(bandIndex);
       	     	    double dnEVstar = (value - svAverage) / luts.getRVSrefLUT(reflectiveBandIndex)[detectorIndex/4][mirrorSide][frameIndex];
       	     	    double erf = luts.getM1LUT(reflectiveBandIndex)[detectorIndex/4][detectorIndex%4][mirrorSide] * dnEVstar;
       	     	    value = Util.toRange(0,(int)((erf*ERF_CONTRAST)+ERF_BRIGHTNESS),255);
             	  } else {
             	    // int emissiveBandIndex = Modis.getEmissiveBandIndex(bandIndex);
       	     	    int dnEV = (value - svAverage);
       	     	    value = dnEV >>> 4;
             	  }
		} else {
		  value = value >>> 4;
		}
       	     	int row = fovIndex + detectorIndex;
       	     	row = 9 - row;
		values[bandIndex][row][col] = (byte) value;
       	     }
       	  }
       }
    }
  }
  /****************************************************************************
  * loadNightValues.
  * Do day-only bands really need to be set to 0 each time?
  ****************************************************************************/
  private void loadNightValues (byte[] buffer, int frameIndex, int mirrorSide) {
    int value;
    int offset = 0;
    boolean onByte = true;
    for (int fovIndex = 0; fovIndex <= 9; fovIndex++) {
       for (int bandIndex = 0; bandIndex <= 1; bandIndex++) {
       	  for (int sampleIndex = 0; sampleIndex < 4; sampleIndex++) {
	     int col = (4 * frameIndex) + sampleIndex;
       	     for (int detectorIndex = 0; detectorIndex < 4; detectorIndex++) {
       	     	int row = (4 * fovIndex) + detectorIndex;
       	     	row = 39 - row;
		values[bandIndex][row][col] = (byte) 0;
       	     }
       	  }
       }
       for (int bandIndex = 2; bandIndex <= 6; bandIndex++) {
       	  for (int sampleIndex = 0; sampleIndex < 2; sampleIndex++) {
	     int col = (2 * frameIndex) + sampleIndex;
       	     for (int detectorIndex = 0; detectorIndex < 2; detectorIndex++) {
       	     	int row = (2 * fovIndex) + detectorIndex;
       	     	row = 19 - row;
		values[bandIndex][row][col] = (byte) 0;
       	     }
       	  }
       }
       for (int bandIndex = 7; bandIndex <= 20; bandIndex++) {
       	  for (int sampleIndex = 0; sampleIndex < 1; sampleIndex++) {
	     int col = frameIndex + sampleIndex;
       	     for (int detectorIndex = 0; detectorIndex < 1; detectorIndex++) {
       	     	int row = fovIndex + detectorIndex;
       	     	row = 9 - row;
		values[bandIndex][row][col] = (byte) 0;
       	     }
       	  }
       }
       for (int bandIndex = 21; bandIndex <= 37; bandIndex++) {
       	  for (int sampleIndex = 0; sampleIndex < 1; sampleIndex++) {
	     int col = frameIndex + sampleIndex;
       	     for (int detectorIndex = 0; detectorIndex < 1; detectorIndex++) {
       	     	if (onByte) {
		  value = ((buffer[offset] & 0xFF) << 4) | ((buffer[offset+1] & 0xF0) >>> 4);
		  offset += 1;
		  onByte = false;
       	     	} else {
		  value = ((buffer[offset] & 0x0F) << 8) | (buffer[offset+1] & 0xFF);
		  offset += 2;
		  onByte = true;
       	     	}
		if (DO_CALIBRATION) {
		  int svAverage = svSums[bandIndex][fovIndex+detectorIndex][sampleIndex] / SV_FRAMES_PER_SCAN;
             	  if (Modis.isReflectiveBand(bandIndex)) {
             	    int reflectiveBandIndex = Modis.getReflectiveBandIndex(bandIndex);
       	     	    double dnEVstar = (value - svAverage) / luts.getRVSrefLUT(reflectiveBandIndex)[detectorIndex/4][mirrorSide][frameIndex];
       	     	    double erf = luts.getM1LUT(reflectiveBandIndex)[detectorIndex/4][detectorIndex%4][mirrorSide] * dnEVstar;
       	     	    value = Util.toRange(0,(int)((erf*ERF_CONTRAST)+ERF_BRIGHTNESS),255);
             	  } else {
             	    // int emissiveBandIndex = Modis.getEmissiveBandIndex(bandIndex);
       	     	    int dnEV = (value - svAverage);
       	     	    value = dnEV >>> 4;
             	  }
		} else {
		  value = value >>> 4;
		}
       	     	int row = fovIndex + detectorIndex;
       	     	row = 9 - row;
		values[bandIndex][row][col] = (byte) value;
       	     }
       	  }
       }
    }
  }
  /****************************************************************************
  * sumDayValues.
  ****************************************************************************/
  private void sumDayValues (byte[] buffer, int startFovIndex, int[][][] sums) {
    int value;
    int offset = 0;
    boolean onByte = true;
    int stopFovIndex = startFovIndex + 4;
    for (int fovIndex = startFovIndex; fovIndex <= stopFovIndex; fovIndex++) {
       for (int bandIndex = 0; bandIndex <= 1; bandIndex++) {
       	  for (int sampleIndex = 0; sampleIndex < 4; sampleIndex++) {
       	     for (int detectorIndex = 0; detectorIndex < 4; detectorIndex++) {
       	     	if (onByte) {
		  value = ((buffer[offset] & 0xFF) << 4) | ((buffer[offset+1] & 0xF0) >>> 4);
		  offset += 1;
		  onByte = false;
       	     	} else {
		  value = ((buffer[offset] & 0x0F) << 8) | (buffer[offset+1] & 0xFF);
		  offset += 2;
		  onByte = true;
       	     	}
		sums[bandIndex][(4*fovIndex)+detectorIndex][sampleIndex] += value;
       	     }
       	  }
       }
       for (int bandIndex = 2; bandIndex <= 6; bandIndex++) {
       	  for (int sampleIndex = 0; sampleIndex < 2; sampleIndex++) {
       	     for (int detectorIndex = 0; detectorIndex < 2; detectorIndex++) {
       	     	if (onByte) {
		  value = ((buffer[offset] & 0xFF) << 4) | ((buffer[offset+1] & 0xF0) >>> 4);
		  offset += 1;
		  onByte = false;
       	     	} else {
		  value  = ((buffer[offset] & 0x0F) << 8) | (buffer[offset+1] & 0xFF);
		  offset += 2;
		  onByte = true;
       	     	}
		sums[bandIndex][(2*fovIndex)+detectorIndex][sampleIndex] += value;
       	     }
       	  }
       }
       for (int bandIndex = 7; bandIndex <= 37; bandIndex++) {
       	  for (int sampleIndex = 0; sampleIndex < 1; sampleIndex++) {
       	     for (int detectorIndex = 0; detectorIndex < 1; detectorIndex++) {
       	     	if (onByte) {
		  value = ((buffer[offset] & 0xFF) << 4) | ((buffer[offset+1] & 0xF0) >>> 4);
		  offset += 1;
		  onByte = false;
       	     	} else {
		  value = ((buffer[offset] & 0x0F) << 8) | (buffer[offset+1] & 0xFF);
		  offset += 2;
		  onByte = true;
       	     	}
		sums[bandIndex][fovIndex+detectorIndex][sampleIndex] += value;
       	     }
       	  }
       }
    }
  }
  /****************************************************************************
  * sumNightValues.
  ****************************************************************************/
  private void sumNightValues (byte[] buffer, int[][][] sums) {
    int value;
    int offset = 0;
    boolean onByte = true;
    for (int fovIndex = 0; fovIndex <= 9; fovIndex++) {
       for (int bandIndex = 21; bandIndex <= 37; bandIndex++) {
       	  for (int sampleIndex = 0; sampleIndex < 1; sampleIndex++) {
       	     for (int detectorIndex = 0; detectorIndex < 1; detectorIndex++) {
       	     	if (onByte) {
		  value = ((buffer[offset] & 0xFF) << 4) | ((buffer[offset+1] & 0xF0) >>> 4);
		  offset += 1;
		  onByte = false;
       	     	} else {
		  value = ((buffer[offset] & 0x0F) << 8) | (buffer[offset+1] & 0xFF);
		  offset += 2;
		  onByte = true;
       	     	}
		sums[bandIndex][fovIndex+detectorIndex][sampleIndex] += value;
       	     }
       	  }
       }
    }
  }
  /***************************************************************************
  * writeScan.
  ***************************************************************************/
  private void writeScan (long scanTimeTag) throws Exception {
    nScans++;
    if (solarCode == Simulcast.DAY_PASS_CODE) {
      nDayScans++;
      if (startSolarCode == Simulcast.UNKNOWN) {
        startSolarCode = Simulcast.DAY_PASS_CODE;
      }
    } else {
      nNightScans++;
      if (startSolarCode == Simulcast.UNKNOWN) {
        startSolarCode = Simulcast.NIGHT_PASS_CODE;
      }
    }
    if (firstScan) {
      setOrbit(scanTimeTag);
      Geo.Location loc1 = geo.calcLoc(scanTimeTag);
      Geo.Location loc2 = geo.calcLoc(scanTimeTag+1000); // 1 second later.
      directionCode = (loc2.getLat() > loc1.getLat() ?
      		       Simulcast.ASCENDING_PASS_CODE :
      		       Simulcast.DESCENDING_PASS_CODE);
      writeHeaderBlock(scanTimeTag);
      firstScan = false;
    }
    Geo.Location loc = geo.calcLoc(scanTimeTag);
    double[][] geoLats = geoBlock.getLats();
    double[][] geoLons = geoBlock.getLons();
    geo.calcScan(scanTimeTag,geoLats,geoLons);
    scanExtents.getBeginLeft().set(geoLats[0][1354],geoLons[0][1354]);
    scanExtents.getBeginRight().set(geoLats[0][0],geoLons[0][0]);
    scanExtents.getEndLeft().set(geoLats[10][1354],geoLons[10][1354]);
    scanExtents.getEndRight().set(geoLats[10][0],geoLons[10][0]);
    scanBlock.set(scanTimeTag,solarCode,nScans,loc.getLat(),loc.getLon(),false);
    scanBlock.write(outServer);
    outServer.flush();
    geoBlock.write(outServer);
    outServer.flush();
    bowedBlock.write(outServer);
    outServer.flush();
    noBow.load(values,valuesNoBow);
    debowedBlock.write(outServer);
    outServer.flush();
  }
  /***************************************************************************
  * writeHeaderBlock.
  ***************************************************************************/
  private void writeHeaderBlock (long startTimeTag) throws Exception {
    HeaderBlock header = new HeaderBlock(passParameters.getGroundStation(),
    					 platformCode,instrumentCode,
            				 directionCode,startSolarCode,startTimeTag,
            				 estExtents,true,true,
					 1000,10,1354,38,
    				         dayBandIndices,dayBandResolutions,
    				         dayBlockRowCounts,dayBlockColCounts,
    				         1000,10,1354);
    header.write(outServer);
    outServer.flush();
  }
  /***************************************************************************
  * writeFooterBlock.
  ***************************************************************************/
  private void writeFooterBlock (long endTimeTag) throws Exception {
    FooterBlock footer = new FooterBlock(solarCode,endTimeTag,nDayScans,
    				         nNightScans,nDayEvGroups,
    				         nDaySvCalGroups,nDaySdCalGroups,
    				         nDaySrcaCalGroups,nDayBbCalGroups,
    				         nNightEvGroups,nNightSvCalGroups,
    				         nNightSdCalGroups,nNightSrcaCalGroups,
    				         nNightBbCalGroups,nEng1Groups,
    				         nEng2Groups);
    footer.write(outServer);
    outServer.flush();
  }
  /***************************************************************************
  * setOrbit.
  ***************************************************************************/
  private void setOrbit (long scanTimeTag) throws Exception {
    String msg = Util.XML_HEADER + "<simulcastMessage>" +
		 "<sendClosestTLE>" + "<satellite>" +
		 Simulcast.getPlatformName(platformCode) + "</satellite>" +
		 "<dateTime>" + Long.toString(scanTimeTag) + "</dateTime>" +
		 "</sendClosestTLE>" + "</simulcastMessage>";
    XmlBlock requestBlock = new XmlBlock(msg);
    requestBlock.write(outServer);
    outServer.flush();
    XmlBlock replyBlock = (XmlBlock) Simulcast.readBlock(inServer);
    String reply = replyBlock.getMsg();
    XmlParser parser = new XmlParser(reply,"simulcastMessage");
    if (parser.hasElement("closestTLE")) {
      String line1 = parser.getElement("closestTLE").getElement("tle").getElementValue("line1");
      String line2 = parser.getElement("closestTLE").getElement("tle").getElementValue("line2");
      geo = new Geo(new String[] {Simulcast.getPlatformName(platformCode),line1,line2});
      GroundStation groundStation = passParameters.getGroundStation();
      estExtents = geo.getEstExtents(scanTimeTag,groundStation.getLat(),groundStation.getLon());
    } else {
      throw new Exception("failed to retrieve TLE from server (unexpected reply)");
    }
  }        
  /****************************************************************************
  * timedOut.
  ****************************************************************************/
  public void timedOut () {
    // Don't shutdown the input.  It puts the input stream at EOF which is expected...
    // try {sRouter.shutdownInput();} catch (Exception ex) {}
    try {inRouter.close();} catch (Exception ex) {}
    try {sRouter.close();} catch (Exception ex) {}
  }
}
