/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/PROCESSOR
*
*  History:
*
*   6-Dec-04, J Love, GST	Original version.
*  30-Jun-05, J Love, GST	NSLS logging, etc.
*  19-Dec-05, J Love, GST	Geolocation and LUTsManager.
*  10-Jan-07, J Love, GST	Log environment at startup.  Added input
*   				timeout.  Dealt with IPv6.
*   1-Feb-07, J Love, GST	Use of sc.home property.  Daemon threads for
*   				Windows service.  Added "stop" for use by
*   				Advanced Installer service installation on
*   				Windows.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.processor;
import gov.nasa.gsfc.aisb.simulcast.*;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import gov.nasa.gsfc.nisgs.nsls.*;
import java.net.*;
import java.io.*;

public class Processor implements SimulcastService {
  protected static Log log;
  protected static HandlerCache handlerCache = new HandlerCache();
  protected static LUTsManager lutsManager = new LUTsManager();
  private static StringMap configs = new StringMap();
  private static HostPort inputAddr;
  protected static long inputTimeOut;
  private static boolean stop = false;
  /****************************************************************************
  * main.
  ****************************************************************************/
  public static void main (String[] args) throws Exception {
    if (args.length != 1) {
      System.out.println("Usage: Processor <config-file>");
      return;
    }
    Processor processor = new Processor();
    processor.start(args);
    // On Windows, loop until "stop" is invoked by the executable that was
    // installed as a Windows service.  This should be the only non-daemon
    // thread running so when this thread ends the JVM should exit.
    // On Linux, not used by the JSW but doesn't hurt too bad.
    while (!stop) {
      Util.sleep(250);
    }
  }
  /****************************************************************************
  * start.
  ****************************************************************************/
  public void start (String[] args) throws Exception {
    XmlParser parser = new XmlParser(Util.getFile(args[0]),"processorConfig");
    log = Util.initLog(parser.getElement("log"));
    log.setDefaultSource(new NSLS.SC("Processor"));
    log.info("Processor: STARTING"+", "+Util.getEnv(),"v");
    parseProcesses(parser.getElement("processes").getElements("process"));
    int inputPort = Integer.parseInt(parser.getElementValue("inputPort"));
    inputTimeOut = Util.decodeTimeInterval(parser.getElementValue("inputTimeOut"));
    Util.startThread(new ListenForPass(inputPort),true);
  }
  private void parseProcesses (XmlParser[] parsers) throws Exception {
    for (int i = 0; i < parsers.length; i ++) {
       String passType = parsers[i].getAttributeValue("passType");
       String handlerClassName = parsers[i].getElementValue("handlerClassName");
       String[] handlerParameters = parsers[i].getElement("handlerParameters").getElementValues("parameter");
       HostPort serverAddr = new HostPort(parsers[i].getElementValue("serverHostPort"));
       ProcessConfig config = new ProcessConfig(passType,handlerClassName,handlerParameters,serverAddr);
       configs.put(passType,config);
    }
  }
  /****************************************************************************
  * ListenForPass.
  ****************************************************************************/
  private class ListenForPass implements Runnable {
    private int inputPort;
    public ListenForPass (int inputPort) {
      this.inputPort = inputPort;
    }
    public void run () {
      try {
      	listen();
      } catch (Exception e) {
	log.warning("Not able to listen to processor input port.","n");
	log.warning("ListenForPass: listening for pass",e,"v");
      }
    }
    private void listen () throws Exception {
      ServerSocket ss = new ServerSocket(inputPort);
      inputAddr = new HostPort(ss);
      log.info("ListenForPass: listening on " + inputAddr.toString(),"v");
      while (true) {
	Socket s = ss.accept();
	log.info("ListenForPass:" + " " + Util.getAcceptedLogEntry(s),"v");
	Thread thread = Util.startThread(new ModeReader(s),true);
	log.info("ListenForPass: created/started a ModeReader, " + thread.getName(),"v");
      }
    }
  }
  /****************************************************************************
  * ModeReader.
  ****************************************************************************/
  private static class ModeReader implements Runnable, TimeoutMonitorListener {
    private Socket s;
    private DataInputStream in = null;
    public ModeReader (Socket s) {
      this.s = s;
    }
    public void run () {
      TimeoutMonitor monitor = new TimeoutMonitor(Util.MS_PER_MIN,this);
      monitor.start();
      try {
	in = new DataInputStream(new BufferedInputStream(s.getInputStream()));
	String msg = in.readUTF();
	XmlParser parser = new XmlParser(msg,"simulcastMessage");
	doMessage(parser);
      } catch (Exception e) {
	if (monitor.isTimedOut()) {
	  log.warning("Time out while process input data.","n");
	  log.warning("ModeReader: TIMEOUT while handling mode","v");
	} else {
	  log.warning("Data error while process input data.","n");
	  log.warning("ModeReader: handling mode",e,"v");
	}
      	try {in.close();} catch (Exception ex) {}
      	try {s.close();} catch (Exception ex) {}
      }
      monitor.stop();
    }
    private void doMessage (XmlParser parser) throws Exception {
      if (parser.hasElement("startOfPass")) {
	StationPassParameters passParameters = new StationPassParameters(parser.getElement("startOfPass").getElement("stationPassParameters"));
	passParameters.routerOutputAddr = s.getInetAddress().getHostAddress();
	passParameters.processorInputAddr = inputAddr.toString();
	ProcessConfig config = (ProcessConfig) configs.get(passParameters.passType);
	if (config == null) {
	  throw new Exception("unknown pass type: " + passParameters.passType);
	}
	Class handlerClass = Class.forName(config.handlerClassName);
	PassHandler handler = handlerCache.get(handlerClass);
	passParameters.processorHandlerClassName = handler.getClass().getName();
	Thread thread = handler.start(s,in,config.handlerParameters,
				      config.serverAddr,passParameters);
	log.info("ModeReader: created/started a PassHandler (PASS), " + thread.getName(),"v");
	return;
      }
      if (parser.hasElement("startOfPassPing")) {
	in.close();
	s.close();
	StationPassParameters passParameters = new StationPassParameters(parser.getElement("startOfPassPing").getElement("stationPassParameters"));
	passParameters.routerOutputAddr = s.getInetAddress().getHostAddress();
	passParameters.processorInputAddr = inputAddr.toString();
	ProcessConfig config = (ProcessConfig) configs.get(passParameters.passType);
	if (config == null) {
	  throw new Exception("unknown pass type: " + passParameters.passType);
	}
	Class handlerClass = Class.forName(config.handlerClassName);
	PassHandler handler = handlerCache.get(handlerClass);
	passParameters.processorHandlerClassName = handler.getClass().getName();
	Thread thread = handler.start(null,null,config.handlerParameters,
	  			      config.serverAddr,passParameters);
	log.info("ModeReader: created/started a PassHandler (PING), " + thread.getName(),"v");
	return;
      }
      throw new Exception("unknown mode");
    }
    public void timedOut () {
      try {in.close();} catch (Exception ex) {}
      try {s.close();} catch (Exception ex) {}
    }
  }
  /****************************************************************************
  * stop.
  * On Windows, invoked when the service is told to stop.
  ****************************************************************************/
  public static void stop () {
    stop = true;
  }
}
