/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/ROUTER
*
*  History:
*
*  18-Nov-04, J Love, GST	Original version.
*  22-Mar-05, J Love, GST	NSLS logging.
*  19-Apr-05, J Love, GST	Routes to multiple processors.
*  28-Nov-05, J Love, GST	Geolocation and handle non-MODIS APIDs
* 				properly.
*  12-Jan-07, J Love, GST	Added input timeout.
*   1-Feb-07, J Love, GST	Daemon threads for Windows service.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.router;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.io.*;
import java.net.*;

public class ModisHandler implements Runnable, TimeoutMonitorListener {
  private Socket inSocket;
  private DataInputStream in;
  private HostPort[] processorAddrs;
  private StationPassParameters passParameters;
  private static final int MODIS_APID = 64;
  private static final int MAX_MODIS_PACKET_LENGTH = 642;
  /****************************************************************************
  * ModisHandler.
  ****************************************************************************/
  public ModisHandler (Socket inSocket, 
  		       HostPort[] processorAddrs,
		       StationPassParameters passParameters) {
    this.inSocket = inSocket;
    this.processorAddrs = processorAddrs;
    this.passParameters = passParameters;
  }
  /****************************************************************************
  * run.
  ****************************************************************************/
  public void run () {
    if (inSocket == null) {
      doPing();
    } else {
      doPass();
    }
  }
  /****************************************************************************
  * doPing.
  ****************************************************************************/
  private void doPing () {
    for (int i = 0; i < processorAddrs.length; i++) {
       doPing(processorAddrs[i]);
    }
  }
  private void doPing (HostPort processorAddr) {
    Socket outSocket = null;
    DataOutputStream out = null;
    Router.log.info("ModisHandler: running (PING)" +
		    ", " + passParameters.toText() +
		    ", processorAddr: " + processorAddr.toString(),"v");
    try {
      outSocket = new Socket(processorAddr.getHost(),processorAddr.getPort());
      out = new DataOutputStream(new BufferedOutputStream(outSocket.getOutputStream()));
      String msg = Util.XML_HEADER + "<simulcastMessage>" +
		   "<startOfPassPing>" + passParameters.toXml() +
		   "</startOfPassPing>" + "</simulcastMessage>";
      out.writeUTF(msg);
      out.flush();
      out.close();
      outSocket.close();
      Router.log.info("ModisHandler: completed (PING)","v");
    } catch (Exception ex) {
      ex.printStackTrace();
      Router.log.warning("Router not able to connect Processor.",ex,"n");
      Router.log.warning("ModisHandler: handling (PING)",ex,"v");
      try {out.close();} catch (Exception e) {}
      try {outSocket.close();} catch (Exception e) {}
    }
  }
  /****************************************************************************
  * doPass.
  ****************************************************************************/
  private void doPass () {
    Router.log.info("ModisHandler: running (PASS input), inSocket: " +
      		    inSocket.getInetAddress().getHostAddress() + ":" +
		    Integer.toString(inSocket.getPort()) +
		    ", " + passParameters.toText(),"v");
    Outgoing[] outgoing = new Outgoing[processorAddrs.length];
    for (int i = 0; i < processorAddrs.length; i++) {
       outgoing[i] = new Outgoing(processorAddrs[i]);
       Util.startThread(outgoing[i],true);
    }
    TimeoutMonitor monitor = new TimeoutMonitor(Router.inputTimeOut,this);
    monitor.start();
    try {
      in = new DataInputStream(new BufferedInputStream(inSocket.getInputStream()));
      String summary = filterStream(in,outgoing,monitor);
      in.close();
      inSocket.close();
      for (int i = 0; i < outgoing.length; i++) {
         outgoing[i].send(new byte[0]);
      }
      Router.log.info("completed (PASS input), " + summary,"v");
    } catch (Exception ex) {
      if (monitor.isTimedOut()) {
	Router.log.warning("Time out when router connects processor.","n");
	Router.log.warning("ModisHandler: handling (PASS input), TIMEOUT ("+Util.encodeTimeInterval(Router.inputTimeOut)+")","v");
      } else {
	Router.log.warning("Router not able to connect processor.","n");
	Router.log.warning("ModisHandler: handling (PASS input)",ex,"v");
      }
      for (int i = 0; i < outgoing.length; i++) {
         outgoing[i].send(null);
      }
      try {in.close();} catch (Exception e) {}
      try {inSocket.close();} catch (Exception e) {}
    }
    monitor.stop();
  }
  /***************************************************************************
  * filterStream.
  ***************************************************************************/
  private String filterStream (DataInputStream in,
  			       Outgoing[] outgoing,
  			       TimeoutMonitor monitor) throws Exception {
    byte[] buffer1 = new byte[6];
    long nReceived = 0;
    long nTransferred = 0;
    long nDiscarded = 0;
    while (true) {
      if (!Util.fillBuffer(in,buffer1,6,true)) {
	break;
      }
      monitor.inputReceived();
      int apid = (((buffer1[0] & 0x07) << 8) | (buffer1[1] & 0xFF));
      int pktLength = (((buffer1[4] & 0xFF) << 8) | (buffer1[5] & 0xFF));
      int nRemaining = pktLength + 1;
      if (apid == MODIS_APID) {
        if (6 + nRemaining > MAX_MODIS_PACKET_LENGTH) {
	  throw new Exception("illegal packet length: " + Integer.toString(6+nRemaining));
        }
        byte[] buffer2 = new byte[6+nRemaining];
        System.arraycopy(buffer1,0,buffer2,0,6);
        Util.fillBuffer(in,buffer2,6,nRemaining);
        boolean more = false;
        for (int i = 0; i < outgoing.length; i++) {
           more |= outgoing[i].send(buffer2);
        }
        if (!more) {
          throw new Exception("all destinations have failed");
        }
        nTransferred++;
      } else {
      	in.skip(nRemaining);
      	nDiscarded++;
      }
      nReceived++;
    }
    return Long.toString(nReceived) + " packets received, " +
    	   Long.toString(nTransferred) + " packets transferred, " +
    	   Long.toString(nDiscarded) + " packets discarded";
  }
  /****************************************************************************
  * timedOut.
  ****************************************************************************/
  public void timedOut () {
    try {in.close();} catch (Exception e) {}
    try {inSocket.close();} catch (Exception e) {}
  }
  /***************************************************************************
  * Outgoing.
  ***************************************************************************/
  private class Outgoing implements Runnable {
    private HostPort processorAddr;
    private Socket outSocket = null;
    private DataOutputStream out = null;
    private static final int CONNECTING = 1;
    private static final int CONNECTED = 2;
    private static final int CLOSED = 3;
    private static final int ERROR = 4;
    private int state = CONNECTING;
    public Outgoing (HostPort processorAddr) {
      this.processorAddr = processorAddr;
    }
    public void run () {
      Router.log.info("ModisHandler.Outgoing: running (PASS output), " + passParameters.toText() +
		      ", processorAddr: " + processorAddr.toString(),"v");
      try {
	outSocket = new Socket(processorAddr.getHost(),processorAddr.getPort());
	out = new DataOutputStream(new BufferedOutputStream(outSocket.getOutputStream()));
	String msg = Util.XML_HEADER + "<simulcastMessage>" +
		     "<startOfPass>" + passParameters.toXml() +
		     "</startOfPass>" + "</simulcastMessage>";
	out.writeUTF(msg);
	out.flush();
	state = CONNECTED;
      } catch (Exception ex) {
      	state = ERROR;
	Router.log.warning("Router not able to write data to processor.","n");
	Router.log.warning("ModisHandler.Outgoing: handling (PASS output), processorAddr: " + processorAddr.toString(),ex,"v");
	try {out.close();} catch (Exception e) {}
	try {outSocket.close();} catch (Exception e) {}
      }
    }
    public boolean send (byte[] buffer) {
      switch (state) {
	case CONNECTING: {
	  return true;
	}
	case CONNECTED: {
	  return connected(buffer);
	}
	case ERROR: {
	  return false;
	}
	case CLOSED: {
	  return false;
	}
      }
      return false;
    }
    private boolean connected (byte[] buffer) {
      try {
	if (buffer == null) {
	  out.close();
	  outSocket.close();
	  Router.log.warning("Router aborted when write data to processor.","n");
	  Router.log.warning("ModisHandler.Outgoing: aborted (PASS output) error on source, processorAddr: " + processorAddr.toString(),"v");
	  state = CLOSED;
	  return false;
	}
	if (buffer.length == 0) {
	  out.close();
	  outSocket.close();
	  Router.log.info("ModisHandler.Outgoing: completed (PASS output), processorAddr: " + processorAddr.toString(),"v");
	  state = CLOSED;
	  return false;
	}
	out.write(buffer);
      	out.flush();
      	return true;
      } catch (Exception ex) {
      	state = ERROR;
	try {out.close();} catch (Exception e) {}
	try {outSocket.close();} catch (Exception e) {}
	Router.log.warning("Error occured when router writes data to processor.","n");
	Router.log.warning("ModisHandler.Outgoing: handling (PASS output), processorAddr: " + processorAddr.toString(),ex,"v");
	return false;
      }
    }
  }
}
