/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/ROUTER
*
*  History:
*
*  18-Nov-04, J Love, GST	Original version.
*  22-Mar-05, J Love, GST	NSLS logging.
*  18-Apr-05, J Love, GST	Routes to multiple processors.
*   8-Feb-06, J Love, GST	Geolocation, etc.
*   4-Jan-07, J Love, GST	Log environment at startup.  Added input
*   				timeout.
*   1-Feb-07, J Love, GST	Ping interval format.  Use of sc.home property.
*   				Daemon threads for Windows service.  Added
*   				"stop" for use by Advanced Installer service
*   				installation on Windows.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.router;
import gov.nasa.gsfc.aisb.simulcast.*;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import gov.nasa.gsfc.nisgs.nsls.*;
import java.util.*;
import java.net.*;

public class Router implements SimulcastService {
  protected static Log log;
  protected static long inputTimeOut;
  private static LinkedList<ListenForPass> listeners = new LinkedList<ListenForPass>();
  private static boolean stop = false;
  /****************************************************************************
  * main.
  ****************************************************************************/
  public static void main (String[] args) throws Exception {
    if (args.length != 1) {
      System.out.println("Usage: Router <config-file>");
      return;
    }
    Router router = new Router();
    router.start(args);
    // On Windows, loop until "stop" is invoked by the executable that was
    // installed as a Windows service.  This should be the only non-daemon
    // thread running so when this thread ends the JVM should exit.
    // On Linux, not used by the JSW but doesn't hurt too bad.
    while (!stop) {
      Util.sleep(250);
    }
  }
  /****************************************************************************
  * start.
  ****************************************************************************/
  public void start (String[] args) throws Exception {
    XmlParser parser = new XmlParser(Util.getFile(args[0]),"routerConfig");
    log = Util.initLog(parser.getElement("log"));
    log.setDefaultSource(new NSLS.SC("Router"));
    log.info("Router: STARTING"+", "+Util.getEnv(),"v");
    inputTimeOut = Util.decodeTimeInterval(parser.getElementValue("inputTimeOut"));
    parseRoutes(parser.getElement("routes").getElements("route"));
    Util.startThread(new DoPings(Util.decodeTimeInterval(parser.getElementValue("pingInterval"))),true);
    Iterator it = listeners.iterator();
    while (it.hasNext()) {
      ListenForPass listener = (ListenForPass) it.next();
      Util.startThread(listener,true);
    }
  }
  private void parseRoutes (XmlParser[] parsers) throws Exception {
    for (int i = 0; i < parsers.length; i ++) {
       int inputPort = Integer.parseInt(parsers[i].getElementValue("inputPort"));
       String handlerClassName = parsers[i].getElementValue("handlerClassName");
       GroundStation groundStation = new GroundStation(parsers[i].getElement("groundStation"));
       HostPort[] processorAddrs = parseProcessorAddresses(parsers[i].getElement("processors"));
       ListenForPass listener = new ListenForPass(inputPort,handlerClassName,groundStation,processorAddrs);
       listeners.add(listener);
    }
  }
  private HostPort[] parseProcessorAddresses (XmlParser parser) throws Exception {
    XmlParser[] parsers = parser.getElements("hostPort");
    HostPort[] addrs = new HostPort[parsers.length];
    for (int i = 0; i < parsers.length; i++) {
       addrs[i] = new HostPort(parsers[i].getElementValue());
    }
    return addrs;
  }
  /****************************************************************************
  * ListenForPass.
  ****************************************************************************/
  private class ListenForPass implements Runnable {
    private int inputPort;
    private String handlerClassName;
    private GroundStation groundStation;
    private HostPort[] processorAddrs;
    private HostPort inputAddr;
    public ListenForPass (int inputPort, String handlerClassName, GroundStation groundStation, HostPort[] processorAddrs) {
      this.inputPort = inputPort;
      this.handlerClassName = handlerClassName;
      this.groundStation = groundStation;
      this.processorAddrs = processorAddrs;
    }
    public void run () {
      try {
      	listen();
      } catch (Exception e) {
	log.warning("Error occured when router listens to input port.","n");
	log.warning("ListenForPass: listening for pass",e,"v");
      }
    }
    private void listen () throws Exception {
      ServerSocket ss = new ServerSocket(inputPort);
      inputAddr = new HostPort(ss);
      log.info("ListenForPass: listening on " + inputAddr.toString(),"v");
      while (true) {
	Socket s = ss.accept();
	log.info("ListenForPass:" + " " + Util.getAcceptedLogEntry(s),"v");
	Class handlerClass = Class.forName(handlerClassName);
	PassHandler handler = (PassHandler) handlerClass.newInstance();
	Thread thread = handler.start(s,groundStation,processorAddrs,inputAddr);
	log.info("ListenForPass: created/started a PassHandler (PASS), " + thread.getName(),"v");
      }
    }
    public void ping () {
      try {
        Class handlerClass = Class.forName(handlerClassName);
        PassHandler handler = (PassHandler) handlerClass.newInstance();
        Thread thread = handler.start(null,groundStation,processorAddrs,inputAddr);
	log.info("ListenForPass: created/started a PassHandler (PING), " + thread.getName(),"v");
      } catch (Exception e) {
	log.warning("Error occured when router connects processor.","n");
	log.warning("ListenForPass: creating PassHandler (PING)",e,"v");
      }
    }
  }
  /****************************************************************************
  * DoPings.
  ****************************************************************************/
  private class DoPings implements Runnable {
    private long msInterval;
    public DoPings (long msInterval) {
      this.msInterval = msInterval;
    }
    public void run () {
      long now = System.currentTimeMillis();
      long pingAt = now + (msInterval - (now % msInterval));
      while (true) {
      	long sleepFor = pingAt - System.currentTimeMillis();
        Util.sleep(sleepFor);
	Iterator it = listeners.iterator();
	while (it.hasNext()) {
	  ListenForPass listener = (ListenForPass) it.next();
	  listener.ping();
	}
        pingAt += msInterval;
      }
    }
  }
  /****************************************************************************
  * stop.
  * On Windows, invoked when the service is told to stop.
  ****************************************************************************/
  public static void stop () {
    stop = true;
  }
}
