/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/SERVER
*
*  History:
*
*   9-Dec-04, J Love, GST	Original version.
*  22-Mar-05, J Love, GST	NSLS logging.
*   8-Feb-06, J Love, GST	Geolocation, etc.
*  12-Mar-06, J Love, GST	More pass details.
*   9-Sep-06, J Love, GST	Try to catch various ways to disconnect without
*   				reporting an error.
*   5-Feb-07, J Love, GST	Daemon threads for Windows service.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.server;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.net.*;
import java.io.*;

public class ClientHandler implements Runnable {
  private long connectedAt;
  private Socket s;
  private DataInputStream in;
  private DataOutputStream out = null;
  private ObjectQueue toSend = new ObjectQueue();
  private String clientId;
  private boolean downstreamServer;
  private boolean loopMode;
  private boolean imageMode;
  /****************************************************************************
  * ClientHandler (constructor).
  ****************************************************************************/
  public ClientHandler (Socket s, DataInputStream in,
  			String clientId, boolean downstreamServer, boolean loopMode, boolean imageMode) throws Exception {
    connectedAt = System.currentTimeMillis();
    this.s = s;
    this.in = in;
    this.out = new DataOutputStream(new BufferedOutputStream(s.getOutputStream()));
    this.clientId = clientId;
    this.downstreamServer = downstreamServer;
    this.loopMode=loopMode;
    this.imageMode=imageMode;
    send(Util.XML_HEADER + "<simulcastMessage>" +
	 "<clientId>" + clientId + "</clientId>" +
	 "<simulcastVersion>" + Simulcast.VERSION + "</simulcastVersion>" +
	 "</simulcastMessage>");
    send(Server.tleManager.getMostRecentTLEs());
    send(Util.XML_HEADER + "<simulcastMessage>" +
         Server.groundStations.toXml() + "</simulcastMessage>");
    // Send the pass listing last because the Loopback test waits on it and
    // then immediately disconnects.  If there were other messages in the
    // Outgoing queue a write error might occur.
    send(Server.passManager.getPassListing());
  }
  /****************************************************************************
  * run.
  ****************************************************************************/
  public void run () {
    if(!loopMode && !imageMode)
    {
      Server.log.info("Simulcast client from: " +
		      s.getInetAddress().getHostAddress() + ":" +
		      Integer.toString(s.getPort()),"n");
    }
    Server.log.info("ClientHandler: running, s: " +
		    s.getInetAddress().getHostAddress() + ":" +
		    Integer.toString(s.getPort()),"v");
    try {
      Util.startThread(new HandleOutgoing(),true);
      doRun();
    } catch (EOFException e) {
      if(!loopMode && !imageMode)
      {
        Server.log.info("Simulcast client from: " +
		        s.getInetAddress().getHostAddress() + ":" +
		        Integer.toString(s.getPort()) + " disconnected.","n");
      }
      Server.log.info("ClientHandler.run/EOFException: " + e.toString() + " (client disconnected)","v");
    } catch (SocketException e) {
      if(!loopMode && !imageMode)
      {
        Server.log.info("Simulcast client from: " +
		        s.getInetAddress().getHostAddress() + ":" +
		        Integer.toString(s.getPort()) + " disconnected.","n");
      }
      Server.log.info("ClientHandler.run/SocketException: " + e.toString() + " (client disconnected)","v");
    } catch (Exception e) {
      if(!loopMode && !imageMode)
      {
        Server.log.warning("Client disconnected.","n");
      }
      Server.log.warning("ClientHandler.run/Exception",e,"v");
    } finally {
      try {in.close();} catch (Exception ex) {}
      try {out.close();} catch (Exception ex) {}
      try {s.close();} catch (Exception ex) {}
    }
    Server.clientManager.removeClient(this);
    Server.log.info("ClientHandler.run: stopping","v");
    toSend.put(null);
  }
  private void doRun () throws Exception {
    while (true) {
      String msg = in.readUTF();
      Server.log.info("ClientHandler.doRun, msg: " + msg,"v");
      try {
	doMessage(msg);
      } catch (Exception e) {
	Server.log.warning("Client message has error.","n");
	Server.log.warning("ClientHandler.doRun/Exception",e,"v");
      }
    }
  }
  private void doMessage (String msg) throws Exception {
    XmlParser parser = new XmlParser(msg,"simulcastMessage");
    if (parser.hasElement("sendPassDetails")) {
      send(Server.passManager.getPassDetails(parser.getElement("sendPassDetails").getElementValue("passName")));
      return;
    }
    if (parser.hasElement("visualBand"))
    {
       String passName=parser.getElement("visualBand").getElementValue("name");
       msg = Util.XML_HEADER + "<simulcastMessage>" + "<visualBand>" +
       Server.passManager.getPass(passName).getDetails(true) + "</visualBand>" + "</simulcastMessage>";
       send(msg);
       return;
    }
    if (parser.hasElement("infraredBand"))
    {
       String passName=parser.getElement("infraredBand").getElementValue("name");
       msg = Util.XML_HEADER + "<simulcastMessage>" + "<infraredBand>" +
       Server.passManager.getPass(passName).getDetails(true) + "</infraredBand>" + "</simulcastMessage>";
       send(msg);
       return;
    }
    if (parser.hasElement("grayBand"))
    {
       String passName=parser.getElement("grayBand").getElementValue("name");
       msg = Util.XML_HEADER + "<simulcastMessage>" + "<grayBand>" +
       Server.passManager.getPass(passName).getDetails(true) + "</grayBand>" + "</simulcastMessage>";
       send(msg);
       return;
    }
    if (parser.hasElement("redBand"))
    {
       String passName=parser.getElement("redBand").getElementValue("name");
       msg = Util.XML_HEADER + "<simulcastMessage>" + "<redBand>" +
       Server.passManager.getPass(passName).getDetails(true) + "</redBand>" + "</simulcastMessage>";
       send(msg);
       return;
    }
    if (parser.hasElement("greenBand"))
    {
       String passName=parser.getElement("greenBand").getElementValue("name");
       msg = Util.XML_HEADER + "<simulcastMessage>" + "<greenBand>" +
       Server.passManager.getPass(passName).getDetails(true) + "</greenBand>" + "</simulcastMessage>";
       send(msg);
       return;
    }
    if (parser.hasElement("blueBand"))
    {
       String passName=parser.getElement("blueBand").getElementValue("name");
       msg = Util.XML_HEADER + "<simulcastMessage>" + "<blueBand>" +
       Server.passManager.getPass(passName).getDetails(true) + "</blueBand>" + "</simulcastMessage>";
       send(msg);
       return;
    }
    if (parser.hasElement("mapInstrumentBand"))
    {
       String passName=parser.getElement("mapInstrumentBand").getElementValue("name");
       msg = Util.XML_HEADER + "<simulcastMessage>" + "<mapInstrumentBand>" +
       Server.passManager.getPass(passName).getDetails(true) + "</mapInstrumentBand>" + "</simulcastMessage>";
       send(msg);
       return;
    }

    throw new Exception("unknown message from client");
  }
  /****************************************************************************
  * send.
  ****************************************************************************/
  public void send (String msg) {
    toSend.put(msg);
  }
  /****************************************************************************
  * HandleOutgoing.
  ****************************************************************************/
  private class HandleOutgoing implements Runnable {
    public void run () {
      Server.log.info("ClientHandler.HandleOutgoing.run: starting","v");
      try {
        doRun();
      } catch (SocketException e) {
	Server.log.info("Simulcast client from: " +
			s.getInetAddress().getHostAddress() + ":" +
			Integer.toString(s.getPort()) + " disconnected.","n");
	Server.log.info("ClientHandler.HandleOutgoing.run/Exception: " + e.toString() + " (client disconnected)","v");
      } catch (Exception e) {
	Server.log.warning("Error when server write data to client.","n");
	Server.log.warning("ClientHandler.HandleOutgoing.run/Exception",e,"v");
      } finally {
      	
      }
      Server.log.info("ClientHandler.HandleOutgoing.run: stopping","v");
    }
    private void doRun () throws Exception {
      while (true) {
      	Object o = toSend.get();
      	if (o == null) {
      	  return;
      	}
      	if (o instanceof String) {
      	  String msg = (String) o;
      	  out.writeUTF(msg);
      	  out.flush();
      	  // System.out.println("sent: " + msg);
      	  continue;
      	}
      }
    }
  }
  /****************************************************************************
  * toXml.
  ****************************************************************************/
  public String toXml () {
    return "<client id=\"" + clientId + "\">" +
	   (downstreamServer ? "<downstreamServer/>" : "") +
    	   "<connectedAt>" + Long.toString(connectedAt) + "</connectedAt>" +
	   "<remoteHostPort>" + s.getInetAddress().getHostAddress() + ":" +
	   Integer.toString(s.getPort()) + "</remoteHostPort>" + "</client>";
  }
}
