/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/SERVER
*
*  History:
*
*  22-Nov-04, J Love, GST	Original version.
*  22-Mar-05, J Love, GST	NSLS logging.
*  20-Dec-05, J Love, GST	Better log messages, etc.
*   5-Jan-07, J Love, GST	More sent after connection occurs.  Better
*   				handling of console disconnects.
*   5-Feb-07, J Love, GST	Daemon threads for Windows service.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.server;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.net.*;
import java.io.*;

public class ConsoleHandler implements Runnable {
  private long connectedAt;
  private Socket s;
  private DataInputStream in;
  private DataOutputStream out = null;
  private ObjectQueue toSend = new ObjectQueue();
  private String consoleId;
  /****************************************************************************
  * ConsoleHandler.
  ****************************************************************************/
  public ConsoleHandler (Socket s, DataInputStream in, String consoleId) throws Exception {
    connectedAt = System.currentTimeMillis();
    this.s = s;
    this.in = in;
    this.out = new DataOutputStream(s.getOutputStream());
    this.consoleId = consoleId;
    send(Util.XML_HEADER + "<simulcastMessage>" +
	 "<consoleId>" + consoleId + "</consoleId>" +
	 "<simulcastVersion>" + Simulcast.VERSION + "</simulcastVersion>" +
	 "</simulcastMessage>");
    send(Server.consoleManager.getConsoleListing());
    send(Server.clientManager.getClientListing());
    send(Server.playerManager.getPlayerListing());
    send(Server.passManager.getPassListing());
  }
  /****************************************************************************
  * run.
  ****************************************************************************/
  public void run () {
    Server.log.info("ConsoleHandler: running, s: " +
      			  s.getInetAddress().getHostAddress() + ":" +
		          Integer.toString(s.getPort()));
    try {
      Util.startThread(new HandleOutgoing(),true);
      doRun();
    } catch (EOFException e) {
      Server.log.info("Simulcast client from: " +
		      s.getInetAddress().getHostAddress() + ":" +
		      Integer.toString(s.getPort()) + " disconnected.","n");
      Server.log.info("ConsoleHandler.run/EOFException: " + e.toString() + " (client disconnected)","v");
    } catch (SocketException e) {
      Server.log.info("Simulcast client from: " +
		      s.getInetAddress().getHostAddress() + ":" +
		      Integer.toString(s.getPort()) + " disconnected.","n");
      Server.log.info("ConsoleHandler.run/SocketException: " + e.toString() + " (client disconnected)","v");
    } catch (Exception e) {
      Server.log.warning("Server disconnected console.","n");
      Server.log.warning("ConsoleHandler.run/Exception",e,"v");
    } finally {
      try {in.close();} catch (Exception ex) {}
      try {out.close();} catch (Exception ex) {}
      try {s.close();} catch (Exception ex) {}
    }
    Server.consoleManager.removeConsole(this);
    Server.log.info("ConsoleHandler: stopping","v");
    toSend.put(null);
  }
  private void doRun () throws Exception {
    while (true) {
      String msg = in.readUTF();
      Server.log.info("ConsoleHandler: from console: " + msg,"v");
      try {
	doMessage(msg);
      } catch (Exception e) {
	Server.log.warning("Error when parse console message.","n");
	Server.log.warning("ConsoleHandler: parsing message from console",e,"v");
      }
    }
  }
  private void doMessage (String msg) throws Exception {
    XmlParser parser = new XmlParser(msg,"simulcastMessage");
    if (parser.hasElement("purgePasses")) {
      Server.passManager.purgePasses();
      return;
    }
    if (parser.hasElement("deletePass")) {
      Server.passManager.deletePass(parser.getElement("deletePass").getElementValue("name"));
      return;
    }
    throw new Exception("unknown message from console");
  }
  /****************************************************************************
  * send.
  ****************************************************************************/
  public void send (String msg) {
    toSend.put(msg);
  }
  /****************************************************************************
  * HandleOutgoing.
  ****************************************************************************/
  private class HandleOutgoing implements Runnable {
    public void run () {
      Server.log.info("ConsoleHandler.HandleOutgoing: running","v");
      try {
        doRun();
      } catch (SocketException e) {
        Server.log.info("Simulcast client from: " +
		        s.getInetAddress().getHostAddress() + ":" +
		        Integer.toString(s.getPort()) + " disconnected.","n");
	Server.log.info("ConsoleHandler.HandleOutgoing.run/Exception: " + e.toString() + " (client disconnected)","v");
      } catch (Exception e) {
	Server.log.warning("Server has error when write data to console.","n");
	Server.log.warning("ConsoleHandler.HandleOutgoing.run/Exception",e,"v");
      } finally {
      	
      }
      Server.log.info("ConsoleHandler.HandleOutgoing: stopping","v");
    }
    private void doRun () throws Exception {
      while (true) {
      	Object o = toSend.get();
      	if (o == null) {
      	  return;
      	}
      	if (o instanceof String) {
      	  String msg = (String) o;
      	  out.writeUTF(msg);
      	  out.flush();
      	  continue;
      	}
      }
    }
  }
  /****************************************************************************
  * toXml.
  ****************************************************************************/
  public String toXml () {
    return "<console id=\"" + consoleId + "\">" +
    	   "<connectedAt>" + Long.toString(connectedAt) + "</connectedAt>" +
	   "<remoteHostPort>" + s.getInetAddress().getHostAddress() + ":" + Integer.toString(s.getPort()) + "</remoteHostPort>" +
	   "</console>";
  }
}
