/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/SERVER
*
*  History:
*
*   6-Dec-04, J Love, GST	Original version.
*  22-Mar-05, J Love, GST	NSLS logging.
*  19-Dec-05, J Love, GST	Better log messages, etc.
*  12-Jan-07, J Love, GST	Dealt with IPv6.
*   1-Feb-07, J Love, GST	Daemon threads for Windows service.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.server;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.net.*;
import java.io.*;

public class ListenForProcessors implements Runnable {
  private int inputPort;
  private ServerSocket ss = null;
  private HostPort inputAddr;
  /****************************************************************************
  * ListenForProcessors.
  ****************************************************************************/
  public ListenForProcessors (int inputPort) {
    this.inputPort = inputPort;
  }
  /****************************************************************************
  * run.
  ****************************************************************************/
  public void run () {
    try {
      listen();
    } catch (Exception e) {
      Server.log.warning("Error when server linstens to processor","n");
      Server.log.warning("ListenForProcessors: listening for processor",e,"v");
    } finally {
      try {ss.close();} catch (Exception e) {}
    }
  }
  /****************************************************************************
  * listen.
  ****************************************************************************/
  private void listen () throws Exception {
    ss = new ServerSocket(inputPort);
    inputAddr = new HostPort(ss);
    Server.log.info("ListenForProcessors: listening on " + inputAddr.toString(),"v");
    while (true) {
      Socket s = ss.accept();
      Server.log.info("ListenForProcessors:" + " " + Util.getAcceptedLogEntry(s),"v");
      Thread thread = Util.startThread(new ModeReader(s),true);
      Server.log.info("ListenForProcessors: created/started a ModeReader, " + thread.getName(),"v");
    }
  }
  /****************************************************************************
  * ModeReader.
  ****************************************************************************/
  private class ModeReader implements Runnable, TimeoutMonitorListener {
    private Socket s;
    private DataInputStream in = null;
    private DataOutputStream out = null;
    public ModeReader (Socket s) {
      this.s = s;
    }
    public void run () {
      TimeoutMonitor monitor = new TimeoutMonitor(Util.MS_PER_MIN,this);
      monitor.start();
      try {
	in = new DataInputStream(new BufferedInputStream(s.getInputStream()));
        out = new DataOutputStream(new BufferedOutputStream(s.getOutputStream()));
	String msg = in.readUTF();
	XmlParser parser = new XmlParser(msg,"simulcastMessage");
	doMessage(parser);
      } catch (Exception e) {
	if (monitor.isTimedOut()) {
	  Server.log.warning("Time out when listening to processor.","n");
	  Server.log.warning("ListenForProcessors.ModeReader: TIMEOUT while handling mode","v");
	} else {
	  Server.log.warning("Not able to parse processor message.","n");
	  Server.log.warning("ListenForProcessors.ModeReader: handling mode",e,"v");
	}
      	try {out.close();} catch (Exception ex) {}
      	try {in.close();} catch (Exception ex) {}
      	try {s.close();} catch (Exception ex) {}
      }
      monitor.stop();
    }
    private void doMessage (XmlParser parser) throws Exception {
      if (parser.hasElement("startOfPass")) {
	StationPassParameters passParameters = new StationPassParameters(parser.getElement("startOfPass").getElement("stationPassParameters"));
	passParameters.processorOutputAddr = s.getInetAddress().getHostAddress();
	passParameters.serverInputAddr = inputAddr.toString();
	Thread thread = Util.startThread(new PassSpooler(s,in,out,passParameters,null),true);
	Server.log.info("ListenForProcessors.ModeReader: created/started a PassSpooler, " + thread.getName(),"v");
	return;
      }
      if (parser.hasElement("startOfPassPing")) {
	out.close();
	in.close();
	s.close();
	StationPassParameters passParameters = new StationPassParameters(parser.getElement("startOfPassPing").getElement("stationPassParameters"));
	passParameters.processorOutputAddr = s.getInetAddress().getHostAddress();
	passParameters.serverInputAddr = inputAddr.toString();
	Server.log.info("ListenForProcessors.ModeReader: received pass PING" + ", " + passParameters.toText(),"v");
	Server.consoleManager.broadcast(Util.XML_HEADER + "<simulcastMessage>" +
					"<startOfPassPing>" + passParameters.toXml() +
					"</startOfPassPing>" + "</simulcastMessage>");
	return;
      }
      throw new Exception("unknown mode");
    }
    public void timedOut () {
      try {out.close();} catch (Exception ex) {}
      try {in.close();} catch (Exception ex) {}
      try {s.close();} catch (Exception ex) {}
    }
  }
}
