/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/SERVER
*
*  History:
*
*  23-Nov-04, J Love, GST	Original version.
*   3-Nov-05, J Love, GST	Geolocation, etc.
*  11-Mar-06, J Love, GST	Downstream server resolution handling.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.server;
import gov.nasa.gsfc.aisb.simulcast.util.*;

class OutputBand {
  private int bandOffset;	// Position within data block.
  private int resolution;
  private InputBand inputBand;
  private int nBlockRows;
  private int nBlockCols;
  private int jump;
  // private int iColStart;
  // private int colStop;
  private int oRow = 0;
  private static final int[] supportedResolutions = new int[] {
    250,500,1000,2000,5000,10000,20000,30000,50000
  };
  /****************************************************************************
  * OutputBand (constructor).
  ****************************************************************************/
  public OutputBand (int bandOffset, int resolution, InputBand inputBand) throws Exception {
    if (!Util.oneOf(resolution,supportedResolutions)) {
      throw new Exception("illegal resolution");
    }
    if (resolution < inputBand.getBandResolution()) {
      throw new Exception("output resolution finer than input resolution");
    }
    if (resolution % inputBand.getBandResolution() != 0) {
      throw new Exception("output resolution not divisible by input resolution");
    }
    this.bandOffset = bandOffset;
    this.resolution = resolution;
    this.inputBand = inputBand;
  }
  /****************************************************************************
  * calc.
  ****************************************************************************/
  public void calc (HeaderBlock outputHeader,
  		    int nInputBlocksPerOutputBlock) throws Exception {
    if (outputHeader.getCoarsestResolution() % resolution != 0) {
      throw new Exception("not an evenly divisible output resolution");
    }
    jump = resolution / inputBand.getBandResolution();
    nBlockRows = (inputBand.getBlockRowCount() * nInputBlocksPerOutputBlock) / jump;
    nBlockCols = ((int) (inputBand.getBlockColCount())) / jump;
    if (resolution == outputHeader.getCoarsestResolution()) {
      outputHeader.setCoarsestBlockRowCount(nBlockRows);
      outputHeader.setCoarsestBlockColCount(nBlockCols);
    }
    outputHeader.setBandIndex(bandOffset,getBandIndex());
    outputHeader.setBandResolution(bandOffset,getBandResolution());
    outputHeader.setBlockRowCount(bandOffset,getBlockRowCount());
    outputHeader.setBlockColCount(bandOffset,getBlockColCount());
  }
  /****************************************************************************
  * getBandOffset.
  ****************************************************************************/
  public int getBandOffset () {
    return bandOffset;
  }
  /****************************************************************************
  * getBandIndex.
  ****************************************************************************/
  public int getBandIndex () {
    return inputBand.getBandIndex();
  }
  /****************************************************************************
  * getBandResolution.
  ****************************************************************************/
  public int getBandResolution () {
    return resolution;
  }
  /****************************************************************************
  * getBlockRowCount.
  ****************************************************************************/
  public int getBlockRowCount () {
    return nBlockRows;
  }
  /****************************************************************************
  * getBlockColCount.
  ****************************************************************************/
  public int getBlockColCount () {
    return nBlockCols;
  }
  /****************************************************************************
  * loadDataBlock.
  ****************************************************************************/
  public void loadDataBlock (boolean reset, DataBlock inputDataBlock, DataBlock outputDataBlock) {
    if (reset) {
      oRow = 0;
    }
    if (oRow < nBlockRows) {
      for (int row = 0; row < inputBand.getBlockRowCount(); row += jump) {
      	 byte[] oData = outputDataBlock.getData()[bandOffset][oRow];
      	 byte[] iData = inputDataBlock.getData()[inputBand.getBandOffset()][row];
      	 for (int oCol = 0, iCol = 0; oCol < nBlockCols; oCol++, iCol += jump) {
      	    oData[oCol] = iData[iCol];
      	 }
      	 oRow++;
      }
    }
  }
  /****************************************************************************
  * toXml.
  ****************************************************************************/
  public String toXml () {
    StringBuffer xml = new StringBuffer();
    xml.append("<outputBand>");
    xml.append("<bandIndex>" + Integer.toString(inputBand.getBandIndex()) + "</bandIndex>");
    xml.append("<resolution>" + Integer.toString(resolution) + "</resolution>");
    xml.append("</outputBand>");
    return xml.toString();
  }
}
