/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/SERVER
*
*  History:
*
*  16-Nov-04, J Love, GST	Original version.
*  28-Nov-05, J Love, GST	Geolocation, etc.
*  12-Mar-06, J Love, GST	More details.
*  15-Feb-07, J Love, GST	Better handling of error conditions.  Attempt
*  				to fix passes with errors at startup.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.server;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.util.*;
import java.io.*;

public class Pass {
  private File file;
  private String passName;
  private String stationId;
  private int seqNumber;
  private HeaderBlock headerBlock = null;
  private FooterBlock footerBlock = null;
  private String error = null;
  private DataOutputStream out;
  private PassReaders readers = new PassReaders();
  private boolean deletePending = false;
  public long nBlocks;
  /****************************************************************************
  * Pass.
  ****************************************************************************/
  public Pass (String passName) throws Exception {
    this.passName = passName;
    file = new File(Server.spoolDir,passName);
    try {
      out = new DataOutputStream(new BufferedOutputStream(new FileOutputStream(file)));
      nBlocks = 0;
    } catch (Exception e) {
      try {out.close();} catch (Exception ex) {}
      throw e;
    }
    parseName();
  }
  public Pass (File file) {
    this.file = file;
    passName = file.getName();
    out = null;
    nBlocks = 0;
    RandomAccessFile io = null;
    try {
      io = new RandomAccessFile(file,"r");
      scan(io);
      io.close();
      Server.log.info("Loading existing pass: " + passName,"n");
      Server.log.info("Pass: scanned " + passName + " (" +
	  	      Long.toString(file.length()) + " bytes)","v");
    } catch (Exception e) {
      if (nBlocks<=1)
      {
        Server.log.warning("No data pass: " + passName);
	// Delete in PassManager.
      }
      else
      {
      Server.log.warning("Pass: " + file.getName() + " has error.","n");
      Server.log.warning("Pass: error scanning file " + file.getName() + ": " +
	  	       Util.encodeException(e),"v");
      if (Server.Fix.equals("yes"))
      {
      try {
	Server.log.info("Attempting to fix " + passName,"n");
	Server.log.info("Pass: attempting to fix " + passName,"v");
	error = null;
	io = new RandomAccessFile(file,"rw");
	fix(io);
	io.close();
	Server.log.info("Successfully fixed " + passName,"n");
	Server.log.info("Pass: successfully fixed " + passName,"v");
      } catch (Exception ex) {
	error = Util.encodeException(e);
	Server.log.warning("Not able to fix " + passName,"n");
	Server.log.warning("Pass: failed to fix " + passName + ": " + error,"v");
	try {
	  Server.log.info("Attempting to write error block to " + passName,"n");
	  Server.log.info("Pass: attempting to write error block to " + passName,"v");
	  io.seek(0);
	  ErrorBlock errorBlock = new ErrorBlock(error);
	  errorBlock.write(io);
	  io.close();
	  nBlocks = 1;
	  Server.log.info("Successfully wrote error block to " + passName,"n");
	  Server.log.info("Pass: successfully wrote error block to " + passName,"v");
	} catch (Exception exc) {
	  Server.log.warning("Not able to write error block to " + passName,"n");
	  Server.log.warning("Pass: failed to write error block to " + passName + ": " + error,"v");
	}
      }
      }
      }
    } finally {
      try {io.close();} catch (Exception e) {}
    }
    parseName();
  }
  /****************************************************************************
  * scan.
  ****************************************************************************/
  private void scan (RandomAccessFile io) throws Exception {
    scanHeader(io);
  }
  /****************************************************************************
  * scanHeader.
  ****************************************************************************/
  private void scanHeader (RandomAccessFile io) throws Exception {
    Object block = Simulcast.readBlock(io);
    if (block instanceof HeaderBlock) {
      headerBlock = (HeaderBlock) block;
      nBlocks = headerBlock.getBlockCount();
      if (nBlocks == 0) {
	throw new Exception("illegal block count in header");
      }
      long offset = headerBlock.getOffsetOfLastBlock();
      if (offset == 0) {
	throw new Exception("illegal last block offset in header");
      }
      scanFooter(io,offset);
      return;
    }
    if (block instanceof ErrorBlock) {
      ErrorBlock errorBlock = (ErrorBlock) block;
      error = errorBlock.getText();
      nBlocks = 1;
      throw new Exception(error);
    }
    throw new Exception("first block has an unexpected type: " + block.getClass().getName());
  }
  /****************************************************************************
  * scanFooter.
  ****************************************************************************/
  private void scanFooter (RandomAccessFile io, long offsetOfLastBlock) throws Exception {
    io.seek(offsetOfLastBlock);
    Object block = Simulcast.readBlock(io);
    if (block instanceof FooterBlock) {
      footerBlock = (FooterBlock) block;
      return;
    }
    if (block instanceof ErrorBlock) {
      ErrorBlock errorBlock = (ErrorBlock) block;
      error = errorBlock.getText();
      throw new Exception(error);
    }
    throw new Exception("last block has an unexpected type: " + block.getClass().getName());
  }
  /****************************************************************************
  * fix.
  ****************************************************************************/
  private void fix (RandomAccessFile io) throws Exception {
    ClassList iBlocks = new ClassList();
    Object block = Simulcast.readBlock(io,iBlocks);
    if (block instanceof HeaderBlock) {
      headerBlock = (HeaderBlock) block;
      try {
	fix(io,iBlocks);
      } catch (Exception e) {
	//Server.log.warning("Error whren fix " + passName,"n");
	Server.log.warning("Pass.fix: error scanning " + passName + ", " + Util.encodeException(e),"v");
	io.seek(headerBlock.getOffsetOfLastBlock());
	if (headerBlock.getBlockCount() > 1) {
	  //Server.log.info("Writing new footer block to " + passName,"n");
	  Server.log.info("Pass.fix: writing new footer block to " + passName,"v");
	  footerBlock.write(io);
	} else {
	  Server.log.info("Writing error block to " + passName + " (no scans)","n");
	  Server.log.info("Pass.fix: writing error block to " + passName + " (no scans)","v");
	  ErrorBlock errorBlock = new ErrorBlock("no scans");
	  errorBlock.write(io);
	}
        headerBlock.incrBlockCount();
	nBlocks = headerBlock.getBlockCount();
        //Server.log.info("Writing updated header to " + passName,"n");
        Server.log.info("Pass.fix: writing updated header to " + passName,"v");
	long length = io.getFilePointer();
	io.seek(0);
	headerBlock.write(io);
	io.setLength(length);
      }
      return;
    }
    if (block instanceof ErrorBlock) {
      ErrorBlock errorBlock = (ErrorBlock) block;
      error = errorBlock.getText();
      nBlocks = 1;
      return;
    }
    throw new Exception("first block has an unexpected type: " + block.getClass().getName());
  }
  private void fix (RandomAccessFile io, ClassList iBlocks) throws Exception {
    headerBlock.setBlockCount(1);
    headerBlock.setOffsetOfLastBlock(io.getFilePointer());
    footerBlock = new FooterBlock();
    while (true) {
      Object block = Simulcast.readBlock(io,iBlocks);
      if (block instanceof ScanBlock) {
	ScanBlock scanBlock = (ScanBlock) block;
	block = Simulcast.readBlock(io,iBlocks);
	if (!(block instanceof GeoBlock)) {
	  throw new Exception("unexpected block type in scan group: " + block.getClass().getName());
	}
	block = Simulcast.readBlock(io,iBlocks);
	if (!(block instanceof BowedDataBlock)) {
	  throw new Exception("unexpected block type in scan group: " + block.getClass().getName());
	}
	block = Simulcast.readBlock(io,iBlocks);
	if (!(block instanceof DebowedDataBlock)) {
	  throw new Exception("unexpected block type in scan group: " + block.getClass().getName());
	}
	headerBlock.incrBlockCount(4);
	headerBlock.setOffsetOfLastBlock(io.getFilePointer());
	switch (scanBlock.getSolarCode()) {
	  case Simulcast.DAY_PASS_CODE: {
	    footerBlock.incrDayScanCount();
	    footerBlock.setEndSolarCode(Simulcast.DAY_PASS_CODE);
	    break;
	  }
	  case Simulcast.NIGHT_PASS_CODE: {
	    footerBlock.incrNightScanCount();
	    footerBlock.setEndSolarCode(Simulcast.NIGHT_PASS_CODE);
	    break;
	  }
	}
	footerBlock.setEndedAt(scanBlock.getScanTime());
	continue;
      }
      if (block instanceof FooterBlock) {
	// Assume it is corrupted...
	throw new Exception("possible corrupted footer");
      }
      if (block instanceof ErrorBlock) {
	ErrorBlock errorBlock = (ErrorBlock) block;
	throw new Exception(errorBlock.getText());
      }
      throw new Exception("unexpected block type: " + block.getClass().getName());
    }
  }
  /****************************************************************************
  * parseName.
  ****************************************************************************/
  private void parseName () {
    StringTokenizer byDot = new StringTokenizer(passName,".");
    stationId = byDot.nextToken();
    seqNumber = Integer.parseInt(byDot.nextToken());
  }
  /****************************************************************************
  * doBlock.
  ****************************************************************************/
  public synchronized void doBlock (Object o) throws Exception {
    if (o instanceof HeaderBlock) {
      headerBlock = (HeaderBlock) o;
      headerBlock.write(out);
      out.flush();
      nBlocks++;
      // size = out.size();
      Server.passManager.passUpdated(this);
      return;
    }
    if (o instanceof ScanBlock) {
      ScanBlock scanBlock = (ScanBlock) o;
      scanBlock.write(out);
      out.flush();
      nBlocks++;
      // size = out.size();
      return;
    }
    if (o instanceof GeoBlock) {
      GeoBlock geoBlock = (GeoBlock) o;
      geoBlock.write(out);
      out.flush();
      nBlocks++;
      // size = out.size();
      return;
    }
    if (o instanceof DataBlock) {
      DataBlock dataBlock = (DataBlock) o;
      dataBlock.write(out);
      out.flush();
      nBlocks++;
      // size = out.size();
      return;
    }
    if (o instanceof FooterBlock) {
      handleLastBlock(o);
      return;
    }
    if (o instanceof ErrorBlock) {
      handleLastBlock(o);
      return;
    }
    throw new Exception("unknown type of block");
  }
  /****************************************************************************
  * handleLastBlock.
  ****************************************************************************/
  private void handleLastBlock (Object o) throws Exception {
    // If this is the first block (the header block is null), just write
    // the ErrorBlock (it has to be, right?) and close the file.
    if (headerBlock == null) {
      try {
        ErrorBlock errorBlock = (ErrorBlock) o;
        error = errorBlock.getText();
        errorBlock.write(out);
	out.flush();
	nBlocks++;		// set number of blocks for readers
	// size = out.size();
        out.close();
	out = null;
	return;
      } catch (Exception e) {
        try {out.close();} catch (Exception ex) {}
        out = null;
        throw e;
      }
    }
    // Otherwise, do the normal last block processing...
    try {
      // Set header field(s).
      headerBlock.setOffsetOfLastBlock(out.size());
      // Write last block.
      if (o instanceof FooterBlock) {
        footerBlock = (FooterBlock) o;
        footerBlock.write(out);
      }
      if (o instanceof ErrorBlock) {
        ErrorBlock errorBlock = (ErrorBlock) o;
        error = errorBlock.getText();
        errorBlock.write(out);
      }
      out.flush();
      // Set number of blocks for readers.
      nBlocks++;
      // size = out.size();
      // And then it can be closed.
      out.close();
      out = null;
    } catch (Exception e) {
      try {out.close();} catch (Exception ex) {}
      out = null;
      throw e;
    }
    // Update header.
    headerBlock.setBlockCount(nBlocks);
    updateHeader();
  }
  /****************************************************************************
  * updateHeader.
  ****************************************************************************/
  private void updateHeader () throws Exception {
    RandomAccessFile io = null;
    try {
      io = new RandomAccessFile(file,"rw");
      headerBlock.write(io);
      io.close();
    } catch (Exception e) {
      error = Util.encodeException(e);
      try {io.close();} catch (Exception ex) {}
      throw e;
    }
  }
  /****************************************************************************
  * setError.
  ****************************************************************************/
  public synchronized void setError (String text) throws Exception {
    error = text;
    if (notDone()) {
      try {
        handleLastBlock(new ErrorBlock(text));
      } catch (Exception e) {
        try {out.close();} catch (Exception ex) {}
        out = null;
        throw e;
      }
    }
  }
  /****************************************************************************
  * notDone.
  ****************************************************************************/
  public synchronized boolean notDone () {
    return (out != null);
  }
  /****************************************************************************
  * open.
  * Invoked by a reader to start reading a pass.
  ****************************************************************************/
  public synchronized void open () throws Exception {
    String id = Thread.currentThread().getName();
    readers.put(id,new PassReader());
  }
  /****************************************************************************
  * readBlock.
  * DON'T make this 'synchronized' or it could block the writer.
  ****************************************************************************/
  public Object readBlock (ClassList blocks) throws Exception {
    String id = Thread.currentThread().getName();
    PassReader reader = readers.get(id);
    return reader.readBlock(blocks);
  }
  /****************************************************************************
  * skipBlock.
  * DON'T make this 'synchronized' or it could block the writer.
  ****************************************************************************/
  public void skipBlock () throws Exception {
    String id = Thread.currentThread().getName();
    PassReader reader = readers.get(id);
    reader.skipBlock();
  }
  /****************************************************************************
  * close.
  * Invoked by a reader when done reading a pass.
  ****************************************************************************/
  public synchronized void close () throws Exception {
    String id = Thread.currentThread().getName();
    PassReader reader = readers.get(id);
    readers.remove(id);
    reader.close();
    if (deletePending) {
      if (out == null && readers.size() == 0) {
      	if (file.delete()) {
  	  Server.log.info("Pass deleted: " + getPassName(),"n");
	  Server.log.info("Pass.close: pass deleted, name: " + getPassName(),"v");
      	} else {
	  Server.log.warning("Error deleting file " + file.getAbsolutePath(),"n");
	  Server.log.warning("Pass.close: error deleting file " + file.getAbsolutePath(),"v");
      	}
      }
    }
  }
  /****************************************************************************
  * delete.
  ****************************************************************************/
  public synchronized void delete () {
    if (out == null && readers.size() == 0) {
      if (file.delete()) {
	Server.log.info("Pass deleted: " + getPassName(),"n");
        Server.log.info("Pass.delete: pass deleted, name: " + getPassName(),"v");
      } else {
	Server.log.warning("Error deleting file " + file.getAbsolutePath(),"n");
	Server.log.warning("Pass.delete: error deleting file " + file.getAbsolutePath(),"v");
      }
      return;
    }
    deletePending = true;
    return;
  }
  /****************************************************************************
  * getPassName.
  ****************************************************************************/
  public String getPassName () {
    return passName;
  }
  /****************************************************************************
  * getStationId.
  ****************************************************************************/
  public String getStationId () {
    return stationId;
  }
  /****************************************************************************
  * getSeqNumber.
  ****************************************************************************/
  public int getSeqNumber () {
    return seqNumber;
  }
  /****************************************************************************
  * getModTime.
  ****************************************************************************/
  public long getModTime () {
    return file.lastModified();
  }
  /****************************************************************************
  * getSize.
  ****************************************************************************/
  /*
  public synchronized long getSize () {
    return size;
  }
  */
  /****************************************************************************
  * isError.
  ****************************************************************************/
  public boolean isError () {
    return (error != null);
  }
  /****************************************************************************
  * getError.
  ****************************************************************************/
  public String getError () {
    return error;
  }
  /****************************************************************************
  * getDetails.
  ****************************************************************************/
  public synchronized String getDetails () {
    return getDetails(false);
  }
  public synchronized String getDetails (boolean all) {
    StringBuffer xml = new StringBuffer();
    xml.append("<passDetails>");
    xml.append("<name>" + passName + "</name>");
    if (headerBlock != null) {
      xml.append(headerBlock.getGroundStation().toXml());
      xml.append("<platformCode>" + headerBlock.getPlatformCode() + "</platformCode>");
      xml.append("<instrumentCode>" + headerBlock.getInstrumentCode() + "</instrumentCode>");
      xml.append("<directionCode>" + headerBlock.getDirectionCode() + "</directionCode>");
      xml.append("<startSolarCode>" + headerBlock.getStartSolarCode() + "</startSolarCode>");
      xml.append("<startedAt>" + Long.toString(headerBlock.getStartedAt()) + "</startedAt>");
      if (all) {
	xml.append("<more>");
        // xml.append(headerBlock.getEstExtents().toXml());
        if (headerBlock.isGeolocated()) {
	  xml.append("<geolocated/>");
        }
        if (headerBlock.isDebowed()) {
	  xml.append("<debowed/>");
        }
        xml.append("<coarsestResolution>" + Integer.toString(headerBlock.getCoarsestResolution()) + "</coarsestResolution>");
        xml.append("<coarsestBlockRowCount>" + Integer.toString(headerBlock.getCoarsestBlockRowCount()) + "</coarsestBlockRowCount>");
        xml.append("<coarsestBlockColCount>" + Integer.toString(headerBlock.getCoarsestBlockColCount()) + "</coarsestBlockColCount>");
        xml.append("<bands>");
        for (int b = 0; b < headerBlock.getBandCount(); b++) {
	   xml.append("<band>");
	   xml.append("<bandIndex>"+Integer.toString(headerBlock.getBandIndex(b))+"</bandIndex>");
	   xml.append("<bandResolution>"+Integer.toString(headerBlock.getBandResolution(b))+"</bandResolution>");
	   xml.append("<blockRowCount>"+Integer.toString(headerBlock.getBlockRowCount(b))+"</blockRowCount>");
	   xml.append("<blockColCount>"+Integer.toString(headerBlock.getBlockColCount(b))+"</blockColCount>");
	   xml.append("</band>");
        }
        xml.append("</bands>");
        xml.append("<geoResolution>"+Integer.toString(headerBlock.getGeoResolution())+"</geoResolution>");
        xml.append("<geoBlockRowCount>"+Integer.toString(headerBlock.getGeoBlockRowCount())+"</geoBlockRowCount>");
        xml.append("<geoBlockColCount>"+Integer.toString(headerBlock.getGeoBlockColCount())+"</geoBlockColCount>");
	xml.append("</more>");
      }
    }
    if (footerBlock != null) {
      xml.append("<endSolarCode>" + footerBlock.getEndSolarCode() + "</endSolarCode>");
      xml.append("<endedAt>" + Long.toString(footerBlock.getEndedAt()) + "</endedAt>");
      xml.append("<dayScanCount>" + Integer.toString(footerBlock.getDayScanCount()) + "</dayScanCount>");
      xml.append("<nightScanCount>" + Integer.toString(footerBlock.getNightScanCount()) + "</nightScanCount>");
    }
    if (error != null) {
      xml.append("<error>" + Util.encodeContent(error) + "</error>");
    }
    xml.append("</passDetails>");
    return xml.toString();
  }
  /****************************************************************************
  * PassReaders.
  ****************************************************************************/
  private class PassReaders {
    private StringMap readerMap = new StringMap();
    public synchronized void put (String id, PassReader reader) {
      readerMap.put(id,reader);
    }
    public synchronized PassReader get (String id) throws Exception {
      return (PassReader) readerMap.get(id);
    }
    public synchronized void remove (String id) {
      readerMap.remove(id);
    }
    public synchronized int size () {
      return readerMap.size();
    }
  }
  /****************************************************************************
  * PassReader.
  ****************************************************************************/
  private class PassReader {
    private DataInputStream in;
    private int nBlocksRead = 0;
    public PassReader () throws Exception {
      in = new DataInputStream(new BufferedInputStream(new FileInputStream(file)));
    }
    public Object readBlock (ClassList blocks) throws Exception {
      while (true) {
        if (nBlocksRead < nBlocks) {
      	  Object block = Simulcast.readBlock(in,blocks);
      	  nBlocksRead++;
      	  return block;
        }
      	if (out == null) {
      	  throw new EOFException();
      	}
      	if (in == null) {
      	  throw new Exception("file closed");
      	}
        Util.sleep(50);
      }
    }
    public void skipBlock () throws Exception {
      while (true) {
        if (nBlocksRead < nBlocks) {
      	  Simulcast.skipBlock(in);
      	  nBlocksRead++;
      	  return;
        }
      	if (out == null) {
      	  throw new EOFException();
      	}
      	if (in == null) {
      	  throw new Exception("file closed");
      	}
        Util.sleep(50);
      }
    }
    public void close () throws Exception {
      in.close();
      in = null;
    }
  }
}
