/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/SERVER
*
*  History:
*
*  16-Nov-04, J Love, GST	Original version.
*  22-Mar-05, J Love, GST	NSLS logging.
*   8-Feb-06, J Love, GST	Geolocation, etc.
*  18-Apr-06, J Love, GST	Handle (ignore) .svn directory.
*   1-Aug-06, J Love, GST	Maintain pass listings by station.
*   5-Jul-07, J Love, GST	Send pass status messages to consoles.
*  19-Jan-07, J Love, GST	Use of sc.home property.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.server;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.util.*;
import java.io.*;

public class PassManager {
  private StringMap2<SeqNumber> seqNumberById = new StringMap2<SeqNumber>();
  private StringMap2<LinkedList<Pass>> passesById = new StringMap2<LinkedList<Pass>>();
  /****************************************************************************
  * PassManager.
  ****************************************************************************/
  public PassManager () throws Exception {
    // File dir = new File(Server.spoolDirPath);
    File[] files = Server.spoolDir.listFiles();
    if (files == null) {
      throw new Exception("failed to get directory listing");
    }
    for (int i = 0; i < files.length; i++) {
       if (files[i].isFile()) {
         Pass pass = new Pass(files[i]);
         SeqNumber seq = (SeqNumber) seqNumberById.get(pass.getStationId());
         if (seq != null) {
           seq.set(Util.max(seq.get(),pass.getSeqNumber()));
         } else {
           seqNumberById.put(pass.getStationId(),new SeqNumber(pass.getSeqNumber()));
         }
         insertByPassName(pass);

         if (pass.nBlocks<=1)
         {
           if (Server.Fix.equals("yes"))
           {
             deletePassAtStart(pass.getPassName());
           }
         }
       }
    }
    Server.log.info("Simulcast server is ready.");
  }
  public void insertByPassName (Pass newPass) {
    LinkedList<Pass> passes = (LinkedList<Pass>) passesById.get(newPass.getStationId());
    if (passes == null) {
      passes = new LinkedList<Pass>();
      passesById.put(newPass.getStationId(),passes);
    }
    for (int i = 0; i < passes.size(); i++) {
       Pass pass = (Pass) passes.get(i);
       if (newPass.getPassName().compareTo(pass.getPassName()) < 0) {
       	 passes.add(i,newPass);
       	 return;
       }
    }
    passes.add(newPass);
  }
  /****************************************************************************
  * getNextPassName.
  ****************************************************************************/
  public synchronized String getNextPassName (String stationId) {
    SeqNumber seq = (SeqNumber) seqNumberById.get(stationId);
    if (seq == null) {
      seq = new SeqNumber(0);
      seqNumberById.put(stationId,seq);
    }
    seq.incr();
    return stationId + "." + Util.lz6.format(seq.get()) + ".SC";
  }
  /****************************************************************************
  * passStarted.
  ****************************************************************************/
  public synchronized void passStarted (Pass newPass) {
    LinkedList<Pass> passes = (LinkedList<Pass>) passesById.get(newPass.getStationId());
    if (passes == null) {
      passes = new LinkedList<Pass>();
      passesById.put(newPass.getStationId(),passes);
    }
    while (passes.size() >= Server.maxSavedPasses) {
      Pass pass = (Pass) passes.removeFirst();
      String msg = Util.XML_HEADER + "<simulcastMessage>" + "<passRemoved>" +
		   "<name>" + pass.getPassName() + "</name>" +
		   "</passRemoved>" + "</simulcastMessage>";
      Server.clientManager.broadcast(msg);
      Server.consoleManager.broadcast(msg);
      pass.delete();
    }
    passes.addLast(newPass);
    String msg = Util.XML_HEADER + "<simulcastMessage>" + "<passStarted>" +
		 newPass.getDetails() + "</passStarted>" + "</simulcastMessage>";
    Server.clientManager.broadcast(msg);
    Server.consoleManager.broadcast(msg);
  }
  /****************************************************************************
  * passUpdated.
  ****************************************************************************/
  public synchronized void passUpdated (Pass pass) {
    String msg = Util.XML_HEADER + "<simulcastMessage>" + "<passUpdated>" +
    		 pass.getDetails() + "</passUpdated>" + "</simulcastMessage>";
    Server.clientManager.broadcast(msg);
    Server.consoleManager.broadcast(msg);
  }
  /****************************************************************************
  * passEnded.
  ****************************************************************************/
  public synchronized void passEnded (Pass pass) {
    String msg = Util.XML_HEADER + "<simulcastMessage>" + "<passEnded>" +
    		 pass.getDetails() + "</passEnded>" + "</simulcastMessage>";
    Server.clientManager.broadcast(msg);
    Server.consoleManager.broadcast(msg);
  }
  /****************************************************************************
  * passExists.
  ****************************************************************************/
  public synchronized boolean passExists (String passName) throws Exception {
    Iterator<LinkedList<Pass>> it = passesById.iterator();
    while (it.hasNext()) {
      LinkedList<Pass> passes = (LinkedList<Pass>) it.next();
      for (Pass pass : passes) {
         if (pass.getPassName().equals(passName)) {
	   return true;
         }
      }
    }
    return false;
  }
  /****************************************************************************
  * getPass.
  ****************************************************************************/
  public synchronized Pass getPass (String passName) throws Exception {
    Iterator<LinkedList<Pass>> it = passesById.iterator();
    while (it.hasNext()) {
      LinkedList<Pass> passes = (LinkedList<Pass>) it.next();
      for (Pass pass : passes) {
         if (pass.getPassName().equals(passName)) {
	   return pass;
         }
      }
    }
    throw new Exception("no such pass: " + passName);
  }
  /****************************************************************************
  * purgePasses.
  ****************************************************************************/
  public synchronized void purgePasses () {
    Iterator<LinkedList<Pass>> it = passesById.iterator();
    while (it.hasNext()) {
      LinkedList<Pass> passes = (LinkedList<Pass>) it.next();
      Iterator<Pass> it1 = passes.iterator();
      while (it1.hasNext()) {
        Pass pass = it1.next();
        if (pass.isError()) {
          pass.delete();
          it1.remove();
          String msg = Util.XML_HEADER + "<simulcastMessage>" +
          	       "<passRemoved>" + "<name>" + pass.getPassName() +
          	       "</name>" + "</passRemoved>" + "</simulcastMessage>";
	  Server.clientManager.broadcast(msg);
	  Server.consoleManager.broadcast(msg);
        }
      }
    }
  }
  /****************************************************************************
  * deletePass.
  ****************************************************************************/
  public synchronized void deletePass (String passName) throws Exception {
    Iterator<LinkedList<Pass>> it = passesById.iterator();
    while (it.hasNext()) {
      LinkedList<Pass> passes = (LinkedList<Pass>) it.next();
      for (Pass pass : passes) {
         if (pass.getPassName().equals(passName)) {
           if (pass.notDone()) {
             throw new Exception("cannot delete an active pass");
           }
	   pass.delete();
	   passes.remove(pass);
	   String msg = Util.XML_HEADER + "<simulcastMessage>" +
	   		"<passRemoved>" + "<name>" + passName + "</name>" +
	   		"</passRemoved>" + "</simulcastMessage>";
	   Server.clientManager.broadcast(msg);
	   Server.consoleManager.broadcast(msg);
	   return;
         }
      }
    }
    throw new Exception("no such pass: " + passName);
  }
  /****************************************************************************
  * deletePassAtStart.
  ****************************************************************************/
  public synchronized void deletePassAtStart (String passName) throws Exception {
    Iterator<LinkedList<Pass>> it = passesById.iterator();
    while (it.hasNext()) {
      LinkedList<Pass> passes = (LinkedList<Pass>) it.next();
      for (Pass pass : passes) {
         if (pass.getPassName().equals(passName)) {
           if (pass.notDone()) {
             throw new Exception("cannot delete an active pass");
           }
           pass.delete();
  	   passes.remove(pass);
  	   return;
         }
      }
    }
    throw new Exception("no such pass: " + passName);
  }
  /****************************************************************************
  * deletePassDetail.
  ****************************************************************************/
  public synchronized void deletePassDetail (String passName) throws Exception
  {
    Iterator<LinkedList<Pass>> it = passesById.iterator();
    while (it.hasNext())
    {
      LinkedList<Pass> passes = (LinkedList<Pass>) it.next();
      for (Pass pass : passes)
      {
        if (pass.getPassName().equals(passName))
        {
          if (pass.notDone())
          {
             throw new Exception("cannot delete an active pass");
          }
          passes.remove(pass);
          return;
        }
      }
    }
    throw new Exception("no such pass: " + passName);
  }
  /****************************************************************************
  * getPassListing.
  ****************************************************************************/
  public synchronized String getPassListing () {
    StringBuffer xml = new StringBuffer(Util.XML_HEADER);
    xml.append("<simulcastMessage>");
    xml.append("<passListing>");
    Iterator<LinkedList<Pass>> it = passesById.iterator();
    while (it.hasNext()) {
      LinkedList<Pass> passes = (LinkedList<Pass>) it.next();
      for (Pass pass : passes) {
         xml.append(pass.getDetails());
      }
    }
    xml.append("</passListing>");
    xml.append("</simulcastMessage>");
    return xml.toString();
  }
  /****************************************************************************
  * getPassDetails.
  ****************************************************************************/
  public synchronized String getPassDetails (String passName) throws Exception {
    Pass pass = getPass(passName);
    StringBuffer xml = new StringBuffer(Util.XML_HEADER);
    xml.append("<simulcastMessage>");
    xml.append(pass.getDetails(true));
    xml.append("</simulcastMessage>");
    return xml.toString();
  }
}
