/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/SERVER
*
*  History:
*
*  15-Dec-04, J Love, GST	Original version.
*  22-Mar-05, J Love, GST	NSLS logging.
*   8-Feb-06, J Love, GST	Geolocation.
*   9-Sep-06, J Love, GST	Tweaked log messages.
*  14-Oct-06, J Love, GST	Catch playing a pass containing only an
*				ErrorBlock (no HeaderBlock).  Clients probably
*				wouldn't do this but an UpstreamServer might.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.server;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import gov.nasa.gsfc.drl.geo.*;
import java.util.*;
import java.util.zip.*;
import java.net.*;
import java.io.*;

public class PassPlayer implements Runnable {
  private long connectedAt;
  private Socket s;
  private DataInputStream in;
  private DataOutputStream out = null;
  private String playerId;
  private String clientId;
  private Pass pass = null;
  private String passName = null;
  private boolean imagePlayMode;
  private interface Player {
    public void play () throws Exception;
  }
  /****************************************************************************
  * PassPlayer (constructor).
  ****************************************************************************/
  public PassPlayer (Socket s, DataInputStream in,
  		     String playerId, String clientId, boolean imagePlayMode) throws Exception {
    connectedAt = System.currentTimeMillis();
    this.s = s;
    this.in = in;
    this.playerId = playerId;
    this.clientId = clientId;
    this.imagePlayMode = imagePlayMode;
  }
  /****************************************************************************
  * run.
  ****************************************************************************/
  public void run () {
    Server.log.info("PassPlayer.run: starting, s: " + s.getInetAddress().getHostAddress() + ":" + Integer.toString(s.getPort()),"v");
    try {
      play();
    } catch (EOFException e) {
      Server.log.info("Stopping playing " + passName,"n");
      Server.log.info("PassPlayer.run/EOFException: " + e.toString() + " (client disconnected)","v");
    } catch (SocketException e) {
      Server.log.info("Stopping playing " + passName,"n");
      Server.log.info("PassPlayer.run/SocketException: " + e.toString() + " (client disconnected)","v");
    } catch (Exception e) {
      // e.printStackTrace();
      Server.log.warning("Error when playing pass.","n");
      Server.log.warning("PassPlayer.run/Exception",e,"v");
      try {
      	ErrorBlock errorBlock = new ErrorBlock(Util.encodeException(e));
      	errorBlock.write(out);
      	out.flush();
      } catch (Exception ex) {
      	
      }
    } finally {
      try {in.close();} catch (Exception ex) {}
      try {out.close();} catch (Exception ex) {}
      try {s.close();} catch (Exception ex) {}
      try {pass.close();} catch (Exception ex) {}
    }
    Server.playerManager.removePlayer(this);
    Server.log.info("PassPlayer.run: stopping","v");
  }
  /****************************************************************************
  * play.
  ****************************************************************************/
  private void play () throws Exception {
    String msg = in.readUTF();
    // System.out.println("from client: " + msg);
    XmlParser parser = new XmlParser(msg,"simulcastMessage");
    passName = parser.getElement("playPass").getElementValue("name");
    if (!imagePlayMode)
    {
      Server.log.info("Request pass for play: " + passName,"n");
    }
    Server.log.info("PassPlayer.play, msg: " + msg,"v");
    Player player = load(parser.getElement("playPass"));
    Server.consoleManager.broadcast(Util.XML_HEADER + "<simulcastMessage>" +
				    "<playerUpdate>" + toXml() +
				    "</playerUpdate>" + "</simulcastMessage>");
    Server.log.info("PassPlayer.play: playing pass","v");
    player.play();
    out.close();
    in.close();
    s.close();
    pass.close();
    if (!imagePlayMode)
    {
      Server.log.info("Completed playing pass: " + passName,"n");
    }
    Server.log.info("PassPlayer.play: playing pass, completed","v");
  }
  /****************************************************************************
  * load.
  ****************************************************************************/
  private Player load (XmlParser parser) throws Exception {
    passName = parser.getElementValue("name");
    if (!imagePlayMode)
    {
      Server.log.info("Starting playing " + passName,"n");
    }
    Server.log.info("PassPlayer.load, loading: " + passName,"v");
    pass = Server.passManager.getPass(passName);
    pass.open();
    if (parser.hasElement("compress")) {
      if (parser.getElementValue("compress").equals("gzip")) {
	out = new DataOutputStream(new BufferedOutputStream(new GZIPOutputStream(s.getOutputStream())));
      } else {
      	throw new Exception("unknown compression");
      }
    } else {
      out = new DataOutputStream(new BufferedOutputStream(s.getOutputStream()));
    }
    if (parser.hasElement("partial")) {
      return new PartialPlayer(parser.getElement("partial"));
    }
    if (parser.hasElement("full")) {
      return new FullPlayer(parser.getElement("full"));
    }
    throw new Exception("unknown play type");
  }
  /****************************************************************************
  * PartialPlayer.
  ****************************************************************************/
  private class PartialPlayer implements Player {
    private boolean sendGeolocated = false;
    private boolean sendDebowed = false;
    private HeaderBlock inputHeader;
    private HeaderBlock outputHeader;
    private IntegerMap inputBands = new IntegerMap();
    private LinkedList<OutputBand> outputBands = new LinkedList<OutputBand>();
    // private InputGeo inputGeo;
    private OutputGeo outputGeo;
    private int nInputBlocksPerOutputBlock;
    private MapParms mapParms = null;
    private ClassList iBlocks = new ClassList();
    public PartialPlayer (XmlParser parser) throws Exception {
      if (parser.hasElement("areaFilter")) {
      	if (parser.getElement("areaFilter").hasElement("mercator")) {
      	  mapParms = new MercatorMapParms(parser.getElement("areaFilter").getElement("mercator"));
      	}
      	if (parser.getElement("areaFilter").hasElement("singlePole")) {
      	  mapParms = new SinglePoleMapParms(parser.getElement("areaFilter").getElement("singlePole"));
      	}
      }
      sendGeolocated = parser.getElement("send").hasElement("geolocated");
      sendDebowed = parser.getElement("send").hasElement("debowed");
      Object block = pass.readBlock(iBlocks);
      if (block instanceof HeaderBlock) {
	inputHeader = (HeaderBlock) block;
      } else {
	if (block instanceof ErrorBlock) {
	  ErrorBlock errorBlock = (ErrorBlock) block;
	  throw new Exception(errorBlock.getText());
	} else {
	  throw new Exception("unknown type of block: " + block.getClass().getName());
	}
      }
      outputHeader = new HeaderBlock(inputHeader,sendGeolocated,sendDebowed);
      for (int bandOffset = 0; bandOffset < inputHeader.getBandCount(); bandOffset++) {
         InputBand inputBand = new InputBand(bandOffset,
       					     inputHeader.getBandIndex(bandOffset),
					     inputHeader.getBandResolution(bandOffset),
					     inputHeader.getBlockRowCount(bandOffset),
					     inputHeader.getBlockColCount(bandOffset));
         inputBands.put(inputBand.getBandIndex(),inputBand);
      }
      outputHeader.setCoarsestResolution(0);
      XmlParser[] parsers = parser.getElement("bands").getElements("band");
      for (int bandOffset = 0; bandOffset < parsers.length; bandOffset++) {
         int bandIndex = Integer.parseInt(parsers[bandOffset].getElementValue("bandIndex"));
         InputBand inputBand = (InputBand) inputBands.get(bandIndex);
         if (inputBand == null) {
           throw new Exception("unavailable band (index " + Integer.toString(bandIndex) + ")");
         }
         int outputResolution = Integer.parseInt(parsers[bandOffset].getElementValue("resolution"));
         outputBands.add(new OutputBand(bandOffset,outputResolution,inputBand));
         if (outputResolution > outputHeader.getCoarsestResolution()) {
       	   outputHeader.setCoarsestResolution(outputResolution);
         }
      }
      nInputBlocksPerOutputBlock = outputHeader.getCoarsestResolution() /
				      (inputHeader.getCoarsestBlockRowCount() *
				       inputHeader.getCoarsestResolution());
      if (nInputBlocksPerOutputBlock < 1) {
        nInputBlocksPerOutputBlock = 1;
      }
      outputHeader.setBandCount(outputBands.size());
      for (OutputBand outputBand : outputBands) {
        outputBand.calc(outputHeader,nInputBlocksPerOutputBlock);
      }
      if (sendGeolocated) {
        outputGeo = new OutputGeo(inputHeader,outputHeader);
      }
    }
    public void play () throws Exception {
      outputHeader.write(out);
      out.flush();
      Geo.Extents outputScanExtents = new Geo.Extents();
      ScanBlock outputScanBlock = new ScanBlock(outputScanExtents);
      GeoBlock outputGeoBlock = null;
      BowedDataBlock outputBowedBlock = null;
      byte[][][] bowed = null;
      if (sendGeolocated) {
        outputGeoBlock = new GeoBlock(outputGeo.getBlockRowCount(),outputGeo.getBlockColCount());
        bowed = new byte[outputBands.size()][][];
        for (int bandOffset = 0; bandOffset < outputBands.size(); bandOffset++) {
           OutputBand outputBand = outputBands.get(bandOffset);
           bowed[bandOffset] = new byte[outputBand.getBlockRowCount()][outputBand.getBlockColCount()];
        }
        outputBowedBlock = new BowedDataBlock(bowed);
      }
      DebowedDataBlock outputDebowedBlock = null;
      byte[][][] debowed = null;
      if (sendDebowed) {
        debowed = new byte[outputBands.size()][][];
        for (int bandOffset = 0; bandOffset < outputBands.size(); bandOffset++) {
           OutputBand outputBand = outputBands.get(bandOffset);
           debowed[bandOffset] = new byte[outputBand.getBlockRowCount()][outputBand.getBlockColCount()];
        }
        outputDebowedBlock = new DebowedDataBlock(debowed);
      }
      int counter = 0;
      while (true) {
        Object block = pass.readBlock(iBlocks);
        if (block instanceof ScanBlock) {
          ScanBlock inputScanBlock = (ScanBlock) block;
	  GeoBlock inputGeoBlock = null;
          if (counter == 0) {
	    if (mapParms != null) {
      	      if (!mapParms.isVisible(inputScanBlock.getExtents())) {
	        outputScanBlock.set(inputScanBlock.getScanTime(),inputScanBlock.getSolarCode(),
	     		            inputScanBlock.getScanNumber(),inputScanBlock.getSpacecraftLat(),
			            inputScanBlock.getSpacecraftLon(),true);
 	        outputScanBlock.write(out);
 	        out.flush();
	        pass.skipBlock();	// GEO block.
	        pass.skipBlock();	// DATA block (bowed).
	        pass.skipBlock();	// DATA block (debowed).
      	        continue;
              }
	      inputGeoBlock = (GeoBlock) pass.readBlock(iBlocks);
      	      if (!mapParms.isVisible(inputGeoBlock)) {
	        outputScanBlock.set(inputScanBlock.getScanTime(),inputScanBlock.getSolarCode(),
	     		            inputScanBlock.getScanNumber(),inputScanBlock.getSpacecraftLat(),
			            inputScanBlock.getSpacecraftLon(),true);
 	        outputScanBlock.write(out);
 	        out.flush();
	        pass.skipBlock();	// DATA block (bowed).
	        pass.skipBlock();	// DATA block (debowed).
      	        continue;
              }
            }
	    counter = 1;
          } else {
            counter++;
          }
	  boolean isFirstBlock = (counter == 1);
	  boolean isLastBlock = (counter == nInputBlocksPerOutputBlock);
	  if (isFirstBlock) {
	    outputScanBlock.set(inputScanBlock.getScanTime(),inputScanBlock.getSolarCode(),
	     		        inputScanBlock.getScanNumber(),inputScanBlock.getSpacecraftLat(),
			        inputScanBlock.getSpacecraftLon(),false);
	  }
	  if (sendGeolocated) {
	    if (inputGeoBlock == null) {
	      inputGeoBlock = (GeoBlock) pass.readBlock(iBlocks);
	    }
	    outputGeo.loadGeoData(isFirstBlock,isLastBlock,inputGeoBlock,outputGeoBlock);
	    BowedDataBlock inputBowedBlock = (BowedDataBlock) pass.readBlock(iBlocks);
	    Iterator it = outputBands.iterator();
	    while (it.hasNext()) {
	      OutputBand outputBand = (OutputBand) it.next();
	      outputBand.loadDataBlock(isFirstBlock,inputBowedBlock,outputBowedBlock);
	    }
	  } else {
	    pass.skipBlock();	// GEO block.
	    pass.skipBlock();	// BOWED DATA block.
	  }
	  if (sendDebowed) {
	    DebowedDataBlock inputDebowedBlock = (DebowedDataBlock) pass.readBlock(iBlocks);
	    Iterator it = outputBands.iterator();
	    while (it.hasNext()) {
	      OutputBand outputBand = (OutputBand) it.next();
	      outputBand.loadDataBlock(isFirstBlock,inputDebowedBlock,outputDebowedBlock);
	    }
	  } else {
	    pass.skipBlock();	// DEBOWED DATA block.
	  }
	  if (isLastBlock) {
	    if (sendGeolocated) {
	      outputScanExtents.getBeginLeft().set(outputGeoBlock.getLats()[0][outputGeo.getBlockColCount()],outputGeoBlock.getLons()[0][outputGeo.getBlockColCount()]);
	      outputScanExtents.getBeginRight().set(outputGeoBlock.getLats()[0][0],outputGeoBlock.getLons()[0][0]);
	      outputScanExtents.getEndLeft().set(outputGeoBlock.getLats()[outputGeo.getBlockRowCount()][outputGeo.getBlockColCount()],outputGeoBlock.getLons()[outputGeo.getBlockRowCount()][outputGeo.getBlockColCount()]);
	      outputScanExtents.getEndRight().set(outputGeoBlock.getLats()[outputGeo.getBlockRowCount()][0],outputGeoBlock.getLons()[outputGeo.getBlockRowCount()][0]);
            }
            outputScanBlock.write(out);
            if (sendGeolocated) {
              outputGeoBlock.write(out);
              outputBowedBlock.write(out);
            }
            if (sendDebowed) {
              outputDebowedBlock.write(out);
            }
            out.flush();
            counter = 0;
	  }
	  continue;
        }
        if (block instanceof FooterBlock) {
          FooterBlock footerBlock = (FooterBlock) block;
      	  footerBlock.write(out);
      	  return;
        }
        if (block instanceof ErrorBlock) {
          ErrorBlock errorBlock = (ErrorBlock) block;
      	  errorBlock.write(out);
      	  return;
        }
        throw new Exception("unknown type of block");
      }
    }
  }
  /****************************************************************************
  * FullPlayer.
  ****************************************************************************/
  private class FullPlayer implements Player {
    private ClassList iBlocks = new ClassList();
    public FullPlayer (XmlParser parser) throws Exception {
    	
    }
    public void play () throws Exception {
      while (true) {
      	try {
          Block block = (Block) pass.readBlock(iBlocks);
          block.write(out);
          out.flush();
      	} catch (EOFException e) {
      	  break;
      	}
      }
    }
  }
  /****************************************************************************
  * toXml.
  ****************************************************************************/
  public String toXml () {
    StringBuffer xml = new StringBuffer();
    xml.append("<player id=\"" + playerId + "\">");
    xml.append("<clientId>" + clientId + "</clientId>");
    xml.append("<connectedAt>" + Long.toString(connectedAt) + "</connectedAt>");
    xml.append("<remoteHostPort>" + s.getInetAddress().getHostAddress() + ":" + Integer.toString(s.getPort()) + "</remoteHostPort>");
    if (passName != null) {
      xml.append("<passName>" + passName + "</passName>");
    }
    xml.append("</player>");
    return xml.toString();
  }
}
