/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/SERVER
*
*  History:
*
*   6-Dec-04, J Love, GST	Original version.
*  22-Mar-05, J Love, GST	NSLS logging.
*  19-Dec-05, J Love, GST	Geolocation, etc.
*  28-Jul-06, J Love, GST	ServerHandler/Listener.
*  17-Jul-07, J Love, GST	Added timeout.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.server;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.io.*;
import java.net.*;
import java.util.*;

public class PassSpooler implements Runnable, TimeoutMonitorListener {
  private Socket s;
  private DataInputStream in;
  private DataOutputStream out;
  private PassParameters passParameters;
  private UpstreamServerListener usl;
  private String passName = null;
  private Pass pass = null;
  private ClassList blocks = new ClassList();
  /****************************************************************************
  * PassSpooler.
  ****************************************************************************/
  public PassSpooler (Socket s, DataInputStream in, DataOutputStream out,
  		      PassParameters passParameters, UpstreamServerListener usl) throws Exception {
    this.s = s;
    this.in = in;
    this.out = out;
    this.passParameters = passParameters;
    this.usl = usl;
  }
  /****************************************************************************
  * run.
  ****************************************************************************/
  public void run () {
    Server.log.info("PassSpooler: running, s: " +
		    s.getInetAddress().getHostAddress() + ":" +
		    Integer.toString(s.getPort()) + ", " +
		    passParameters.toText(),"v");
    try {
      spool();
    } catch (Exception e) {
      Server.log.warning("Error when writing pass.","n");
      Server.log.warning("PassSpooler: spooling pass",e,"v");
    } finally {
      try {out.close();} catch (Exception e) {}
      try {in.close();} catch (Exception e) {}
      try {s.close();} catch (Exception e) {}
    }
  }
  /****************************************************************************
  * spool.
  ****************************************************************************/
  private void spool () throws Exception {
    if (passParameters instanceof StationPassParameters) {
      StationPassParameters spp = (StationPassParameters) passParameters;
      passName = Server.passManager.getNextPassName(spp.getGroundStation().getId());
    }
    if (passParameters instanceof UpstreamPassParameters) {
      UpstreamPassParameters upp = (UpstreamPassParameters) passParameters;
      if (Server.passManager.passExists(upp.getPassName())) {
      	throw new Exception("pass alreay exists: " + upp.getPassName());
      }
      passName = upp.getPassName();
    }
    pass = new Pass(passName);
    Server.passManager.passStarted(pass);
    Server.consoleManager.broadcast(Util.XML_HEADER + "<simulcastMessage>" + "<startOfPass name=\"" + passName + "\">" + passParameters.toXml() + "</startOfPass>" + "</simulcastMessage>");
    Server.log.info("Start to write new pass: " + passName,"n");
    Server.log.info("PassSpooler: pass started, name: " + passName,"v");
    TimeoutMonitor monitor = new TimeoutMonitor(Server.inputTimeOut,this);
    monitor.start();
    try {
      spool2(monitor);
    } catch (Exception ex) {
      try {
	if (monitor.isTimedOut()) {
	  pass.setError("timeout while spooling pass (" +
	    		Util.encodeTimeInterval(Server.inputTimeOut) + ")");
	} else {
	  pass.setError(ex.toString());
	}
      } catch (Exception e) {
	Server.log.warning("Failed to set error in pass file.","n");
	Server.log.warning("PassSpooler: pass ERROR, name: " + passName + ", failed to set error in pass file: " + e.toString(),"v");
      }
    }
    monitor.stop();
    if (pass.isError()) {
      String msg = Util.XML_HEADER + "<simulcastMessage>" +
		   "<errorOnPass name=\"" + passName + "\">" +
		   Util.encodeContent(pass.getError()) + "</errorOnPass>" +
		   "</simulcastMessage>";
      Server.consoleManager.broadcast(msg);
      Server.log.warning("Error when writing pass: " + passName + ".","n");
      Server.log.warning("PassSpooler: pass ERROR, name: " + passName + ", " + pass.getError(),"v");
    } else {
      String msg = Util.XML_HEADER + "<simulcastMessage>" +
		   "<endOfPass name=\"" + passName + "\">" +
		   "</endOfPass>" + "</simulcastMessage>";
      Server.consoleManager.broadcast(msg);
      Server.log.info("End of writing new pass: " + passName,"n");
      Server.log.info("PassSpooler: pass ended, name: " + passName,"v");
    }
    Server.passManager.passEnded(pass);
    if (usl != null) {
      usl.requestCompleted();
    }
    // If error, delete or fix it after finish spool.
    if (pass.isError())
    {
      //if (pass.getError().equals("no scans"))
      if (pass.nBlocks<=1)
      {
        Server.log.warning("No data pass: " + passName);
        if (Server.Fix.equals("yes"))
        {
          Server.passManager.deletePass(passName);
        }
      }
      else if (pass.getError().startsWith("timeout while") ||
               pass.getError().startsWith("java.net.SocketException")||
               pass.getError().startsWith("java.io.EOFException"))
      {

        String aName=Server.spoolDir.getPath()+"/"+passName;
      
        Server.passManager.deletePassDetail(passName);
      
        // Pass will decide to fix or not.
        Pass pass=new Pass(new File(aName));

        pass.setError(null);
      
        Server.passManager.insertByPassName(pass);
      
        Server.passManager.passUpdated(pass);
      }
    }
  }
  /****************************************************************************
  * spool2.
  ****************************************************************************/
  private void spool2 (TimeoutMonitor monitor) throws Exception {
    // If a processor, handle the TLE request.    
    if (usl == null) {
      handleTLErequest();
    }
    // Handle pass data.
    while (pass.notDone()) {
      Object block = Simulcast.readBlock(in,blocks);
      monitor.inputReceived();
      pass.doBlock(block);
    }
    out.close();
    in.close();
    s.close();
  }
  /****************************************************************************
  * handleTLErequest.
  ****************************************************************************/
  private void handleTLErequest () throws Exception {
    Object block = Simulcast.readBlock(in,blocks);
    if (block instanceof XmlBlock) {
      XmlBlock requestBlock = (XmlBlock) block;
      XmlParser parser = new XmlParser(requestBlock.getMsg(),"simulcastMessage");
      if (parser.hasElement("sendClosestTLE")) {
        String satellite = parser.getElement("sendClosestTLE").getElementValue("satellite");
        Date date = new Date(Long.parseLong(parser.getElement("sendClosestTLE").getElementValue("dateTime")));
        String reply = Server.tleManager.getClosestTLE(satellite,date);
	XmlBlock replyBlock = new XmlBlock(reply);
	out = new DataOutputStream(new BufferedOutputStream(s.getOutputStream()));
        replyBlock.write(out);
        out.flush();
      } else {
        throw new Exception("unexpected request from Processor (not for closest TLE)");
      }
      return;
    }
    pass.doBlock(block);
    return;
  }
  /****************************************************************************
  * timedOut.
  ****************************************************************************/
  public void timedOut () {
    // try {s.shutdownInput();} catch (Exception e) {}
    // try {s.shutdownOutput();} catch (Exception e) {}
    try {out.close();} catch (Exception e) {}
    try {in.close();} catch (Exception e) {}
    try {s.close();} catch (Exception e) {}
  }
}
