/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/SERVER
*
*  History:
*
*   8-Feb-06, J Love, GST	Original version.
*  30-Jan-07, J Love, GST	Added log events.  Update at startup and at
*				the top of the hour.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.server;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.io.*;
import java.net.*;
import java.util.*;

public class TLEArchive implements Runnable {
  private String spacecraft;
  private File file;
  private URL url;
  private LinkedList<TLE> list;
  /****************************************************************************
  * TLEArchive.
  ****************************************************************************/
  public TLEArchive (String spacecraft, XmlParser parser) throws Exception {
    this.spacecraft = spacecraft;
    file = Util.getFile(parser.getElementValue("archive"));
    url = new URL(parser.getElementValue("source"));
    read();
  }
  /****************************************************************************
  * read.
  ****************************************************************************/
  private void read () throws Exception {
    list = new LinkedList<TLE>();
    BufferedReader in = null;
    try {
      in = new BufferedReader(new FileReader(file));
      while (true) {
        String[] lines = new String[2];
        for (int i = 0; i < lines.length; i++) {
	   lines[i] = in.readLine();
	   if (lines[i] == null) {
	     throw new EOFException();
	   }
        }
        list.addLast(new TLE(lines));
      }
    } catch (EOFException eof) {
      in.close();
    } catch (Exception e) {
      throw e;
    } finally {
      try {in.close();} catch (Exception e) {}
    }
  }
  /****************************************************************************
  * run.
  ****************************************************************************/
  public void run () {
    update();
    long msInterval = 60 * 60 * 1000;
    long now = System.currentTimeMillis();
    long updateAt = now + (msInterval - (now % msInterval));
    while (true) {
      long sleepFor = updateAt - System.currentTimeMillis();
      Util.sleep(sleepFor);
      update();
      updateAt += msInterval;
    }
  }
  /****************************************************************************
  * update.
  ****************************************************************************/
  public void update () {
    BufferedReader in = null;
    try {
      Server.log.info("TLEArchive.update: updating TLE for " + spacecraft +
	  	      " (URL: " + url.toString() + ", file: " +
	  	      file.getAbsolutePath() + ")","v");
      in = new BufferedReader(new InputStreamReader(url.openStream()));
      while (true) {
        String[] lines = new String[3];
        for (int i = 0; i < lines.length; i++) {
	   lines[i] = in.readLine();
	   if (lines[i] == null) {
	     throw new EOFException();
	   }
        }
        if (lines[0].startsWith(spacecraft)) {
          TLE tle = new TLE(new String[] {lines[1],lines[2]});
          synchronized (this) {
	    insert(tle);
          }
          break;
        }
      }
      in.close();
      Server.log.info("Successfully updated TLE for " +
	  	      spacecraft);
    } catch (EOFException eof) {
      try {
        in.close();
	Server.log.warning("TLEArchive.update: failed to update TLE for " +
	    		 spacecraft + ", TLE not found in URL file","n");
	Server.log.warning("TLEArchive.update: failed to update TLE for " +
	    		 spacecraft + ", TLE not found in URL file","v");
      } catch (Exception e) {
	Server.log.warning("TLEArchive.update: failed to update TLE for " +
		    		 spacecraft,"n");
	Server.log.warning("TLEArchive.update: failed to update TLE for " +
	    		 spacecraft,e,"v");
      }
    } catch (Exception e) {
      Server.log.warning("TLEArchive.update: failed to update TLE for " +
   		       spacecraft,"n");
      Server.log.warning("TLEArchive.update: failed to update TLE for " +
	 	       spacecraft,e,"v");
    } finally {
      try {in.close();} catch (Exception e) {}
    }
  }
  /****************************************************************************
  * insert.
  ****************************************************************************/
  private void insert (TLE newTle) throws Exception {
    for (int i = 0; i < list.size(); i++) {
       TLE tle = list.get(i);
       if (newTle.getDate().before(tle.getDate())) {
         list.add(i,newTle);
         write();
         return;
       }
       if (newTle.getDate().equals(tle.getDate())) {
         return;
       }
    }
    list.add(newTle);
    write();
  }
  /****************************************************************************
  * write.
  ****************************************************************************/
  private void write () throws Exception {
    PrintWriter out = null;
    try {
      out = new PrintWriter(file);
      for (TLE tle : list) {
        String[] lines = tle.getLines();
        for (int i = 0; i < lines.length; i++) {
           out.println(lines[i]);
        }
      }
      out.close();
    } catch (Exception e) {
      throw e;
    } finally {
      try {out.close();} catch (Exception e) {}
    }
  }
  /****************************************************************************
  * closestTLEtoXml.
  ****************************************************************************/
  public synchronized String closestTLEtoXml (Date date) {
    // System.out.println(date.toString());
    TLE closestTLE = null;
    long closestDiff = Long.MAX_VALUE;
    for (TLE tle : list) {
      long diff = Math.abs(tle.getDate().getTime() - date.getTime());
      if (diff > closestDiff) {
        break;
      }
      closestDiff = diff;
      closestTLE = tle;
    }
    return closestTLE.toXml();
  }
  /****************************************************************************
  * mostRecentTLEtoXml.
  ****************************************************************************/
  public synchronized String mostRecentTLEtoXml () {
    TLE tle = (TLE) list.getLast();
    return tle.toXml();
  }
}
