/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*  28-Jul-06, J Love, GST	Original version.
*   1-Feb-07, J Love, GST	Daemon threads for Windows service.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.server;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.util.*;
import java.net.*;
import java.io.*;

public class UpstreamServerListener implements ServerHandlerListener {
  private ServerHandler serverHandler;
  private String portion;
  private int stationI;
  private LinkedList<String> requests = null;
  private String clientId;
  /****************************************************************************
  * ServerListener.
  ****************************************************************************/
  public UpstreamServerListener (ServerHandler serverHandler, String portion, int stationI) {
    this.serverHandler = serverHandler;
    this.portion = portion;
    this.stationI = stationI;
  }
  /****************************************************************************
  * info.
  ****************************************************************************/
  public void info (String text) {
    Server.log.info("Start to connect: " + serverHandler.getHostPort(), "n");
    Server.log.info("UpstreamServerListener: " + text + " (" + serverHandler.getHostPort() + ")", "v");
  }
  /****************************************************************************
  * error.
  ****************************************************************************/
  public void error (String text, boolean wasConnected) {
    Server.log.warning("Unable to connect: " + serverHandler.getHostPort(), "n");
    Server.log.warning("UpstreamServerListener: " + text + " (" + serverHandler.getHostPort() + ")", "v");
  }
  /****************************************************************************
  * connected.
  ****************************************************************************/
  public void connected (String hostPort) {
    Server.log.info("Connected upstream server: " + serverHandler.getHostPort(),"n");
    Server.log.info("UpstreamServerListener: " + "connected"  + " (" + serverHandler.getHostPort() + ")","v");
    serverHandler.send(Util.XML_HEADER + "<simulcastMessage>" + "<clientMode>" +
		       "<downstreamServer/>" + "</clientMode>" + "</simulcastMessage>");
    requests = new LinkedList<String>();
  }
  /****************************************************************************
  * received.
  ****************************************************************************/
  public void received (String msg) {
    // System.out.println("UpstreamServerListener.received: " + msg);
    try {
      doReceived(msg);
    } catch (Exception e) {
      Server.log.warning("Error handling received message: " + msg + " (" + serverHandler.getHostPort() + ")","n");
      Server.log.warning("UpstreamServerListener: " + "error handling received message: " + msg + " (" + serverHandler.getHostPort() + ")",e,"v");
    }
  }
  private void doReceived (String msg) throws Exception
  {
    GroundStation[] stations=Server.groundStations.getStations();
	
    XmlParser parser = new XmlParser(msg,"simulcastMessage");
    if (parser.hasElement("passListing"))
    {
      XmlParser[] parsers = parser.getElement("passListing").getElements("passDetails");
  		
      LinkedList<String> list = new LinkedList<String>();
      
      for (int i = 0; i < parsers.length; i++)
      {
	    list.addLast(parsers[i].getElementValue("name"));
      }
      String[] passNames = list.toArray(new String[list.size()]);
      Arrays.sort(passNames);

      LinkedList<String> stationList = new LinkedList<String>();
      for (int j = 0; j < passNames.length; j++)
      {
        if (passNames[j].startsWith(stations[stationI].getId()+"."))
      	{
      	  stationList.addLast(passNames[j]);
      	}
      }

      String[] stationPassNames = stationList.toArray(new String[stationList.size()]);
      Arrays.sort(stationPassNames);
        
      LinkedList<String> requestPassNames = new LinkedList<String>();
	  
      for (int k = Util.max(0,stationPassNames.length-Server.maxSavedPasses); k < stationPassNames.length; k++)
      {
        if (!Server.passManager.passExists(stationPassNames[k]))
        {
          requestPassNames.addLast(stationPassNames[k]);
        }
      }
      for (String passName : requestPassNames)
      {
        newRequest(passName);
      }
      return;
    }
    if (parser.hasElement("passStarted")) {
      newRequest(parser.getElement("passStarted").getElement("passDetails").getElementValue("name"));
      return;
    }
    if (parser.hasElement("clientId")) {
      clientId = parser.getElementValue("clientId");
      return;
    }
    // Ignore the rest.
    return;
  }
  /****************************************************************************
  * disconnected.
  ****************************************************************************/
  public void disconnected () {
    Server.log.warning("Disconnected to " + serverHandler.getHostPort(), "n");
    Server.log.warning("UpstreamServerListener: " + "disconnected"  + " (" + serverHandler.getHostPort() + ")", "v");
  }
  /****************************************************************************
  * newRequest.
  ****************************************************************************/
  private synchronized void newRequest (String passName) {
    requests.addLast(passName);
    if (requests.size() == 1) {
      Thread thread = Util.startThread(new RequestPass(passName),true);
      Server.log.info("UpstreamServerListener: created/started a RequestPass, " + thread.getName() + " (" + serverHandler.getHostPort() + ")","v");
    }
  }
  /****************************************************************************
  * requestCompleted.
  ****************************************************************************/
  public synchronized void requestCompleted () {
    requests.removeFirst();
    if (requests.size() > 0) {
      Thread thread = Util.startThread(new RequestPass(requests.getFirst()),true);
      Server.log.info("UpstreamServerListener: created/started a RequestPass, " + thread.getName() + " (" + serverHandler.getHostPort() + ")","v");
    }
  }
  /****************************************************************************
  * RequestPass.
  ****************************************************************************/
  private class RequestPass implements Runnable {
    private String passName;
    public RequestPass (String passName) {
      this.passName = passName;
    }
    public void run () {
      requestPass(passName);
    }
  }
  private void requestPass (String passName) {
    Socket s = null;
    DataInputStream in = null;
    DataOutputStream out = null;
    try {
      HostPort upstreamServerAddr = serverHandler.getHostPort();
      s = new Socket(upstreamServerAddr.getHost(),upstreamServerAddr.getPort());
      in = new DataInputStream(new BufferedInputStream(s.getInputStream()));
      out = new DataOutputStream(new BufferedOutputStream(s.getOutputStream()));
      out.writeUTF(Util.XML_HEADER + "<simulcastMessage>" +
		   "<playMode clientId=\"" + clientId + "\"/>" +
		   "</simulcastMessage>");
      out.writeUTF(Util.XML_HEADER + "<simulcastMessage>" + "<playPass>" +
		   "<name>" + passName + "</name>" + portion +
		   "</playPass>" + "</simulcastMessage>" + "\r\n");
      out.flush();
      Thread thread = Util.startThread(new PassSpooler(s,in,out,new UpstreamPassParameters(upstreamServerAddr,passName),this),true);
      Server.log.info("UpstreamServerListener.RequestPass: created/started a PassSpooler for upstream pass, " + thread.getName() + " (" + serverHandler.getHostPort() + ")","v");
    } catch (Exception e) {
      e.printStackTrace();
      Server.log.warning("Failed to start request for upstream pass." + " (" + serverHandler.getHostPort() + ")","n");
      Server.log.warning("UpstreamServerListener.RequestPass: failed to start PassSpooler for upstream pass" + " (" + serverHandler.getHostPort() + ")",e,"v");
      try {out.close();} catch (Exception ex) {}
      try {in.close();} catch (Exception ex) {}
      try {s.close();} catch (Exception ex) {}
    }
  }
}
