/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/TEST
*
*  History:
*
*  21-Jan-05, J Love, GST	Original version.
*  16-Dec-05, J Love, GST	Clean up, etc.
*   5-Jan-07, J Love, GST	Changed how EOF is simulated.
*  11-Jan-07, J Love, GST	Changed how "no scans" is simulated.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.test;
import gov.nasa.gsfc.aisb.simulcast.*;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.io.*;
import java.net.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class RtSTPS implements SimulcastService {
  private HostPort aquaHostPort;
  private HostPort terraHostPort;
  private JPanel mainPanel;
  private JRadioButton aquaButton;
  private JRadioButton terraButton;
  private JFileChooser chooser;
  private JButton selectButton;
  private JButton stopButton;
  private JCheckBox pauseCheckBox;
  private File pdsFile = null;
  private LabeledField statusField = null;
  private Socket s = null;
  private OutputStream out = null;
  private boolean stop;
  /****************************************************************************
  * main.
  ****************************************************************************/
  public static void main (String[] args) throws Exception {
    if (args.length == 0) {
      System.out.println("Usage: RtSTPS <aqua-host:port> <terra-host:port> [<pds-file> <aqua|terra>]");
      return;
    }
    RtSTPS rtStps = new RtSTPS();
    rtStps.start(args);
  }
  /****************************************************************************
  * start.
  ****************************************************************************/
  public void start (String[] args) throws Exception {
    aquaHostPort = new HostPort(args[0]);
    terraHostPort = new HostPort(args[1]);
    if (args.length < 3) {
      JFrame frame = new JFrame();
      frame.setTitle("RT-STPS Simulator");
      mainPanel = new JPanel(new BorderLayout());
      frame.getContentPane().setLayout(new BorderLayout());
      frame.getContentPane().add(mainPanel,BorderLayout.CENTER);
      aquaButton = new JRadioButton("Aqua",true);
      terraButton = new JRadioButton("Terra",false);
      ButtonGroup group = new ButtonGroup();
      group.add(aquaButton);
      group.add(terraButton);
      loadChooser();
      frame.pack();
      frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      GUtil.configureWindow(frame,0.60,0.50);
      frame.setVisible(true);
    }
    else 
    {
      //Util.startThread(new SendStream(new File(args[2]), (args[3].equals("aqua") ? aquaHostPort : terraHostPort)));
      pdsFile = new File(args[2]); 
      if (args[3].equals("terra"))
      {
        //System.out.println("terra path: " + pdsFile.getPath() + "\n");
        HostPort hostPort = terraHostPort;
        Util.startThread(new SendStream(pdsFile,hostPort));
      }
      if (args[3].equals("aqua"))
      {
        //System.out.println("aqua path: " + pdsFile.getPath() + "\n");
        HostPort hostPort = aquaHostPort;
        Util.startThread(new SendStream(pdsFile,hostPort));
      }
    }
  }
  /****************************************************************************
  * ChooserAction.
  ****************************************************************************/
  private class ChooserAction implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      if (ae.getActionCommand().equals(JFileChooser.APPROVE_SELECTION)) {
	pdsFile = chooser.getSelectedFile();
	mainPanel.removeAll();
	statusField = new LabeledField("Status:");
	stopButton = new JButton("Stop");
	stopButton.addActionListener(new CancelButtonClicked());
	pauseCheckBox = new JCheckBox("Pause",true);
	mainPanel.add(new MarginPanel(10,10,new HPanel(statusField,GUtil.HSpace(10),stopButton,GUtil.HSpace(10),pauseCheckBox),10,10),BorderLayout.NORTH);
	selectButton = new JButton("Select PDS File");
	selectButton.setEnabled(false);
	selectButton.addActionListener(new SelectButtonClicked());
	mainPanel.add(new MarginPanel(10,0,selectButton,10,10),BorderLayout.SOUTH);
	mainPanel.revalidate();
	mainPanel.repaint();
	HostPort hostPort = (aquaButton.isSelected() ? aquaHostPort : terraHostPort);
        //System.out.println("path: " + pdsFile.getPath() + "\n");
	Util.startThread(new SendStream(pdsFile,hostPort));
	return;
      }
      if (ae.getActionCommand().equals(JFileChooser.CANCEL_SELECTION)) {
      	System.exit(0);
	return;
      }
    }
  }
  /****************************************************************************
  * SelectButtonClicked.
  ****************************************************************************/
  private class SelectButtonClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      mainPanel.removeAll();
      loadChooser();
      mainPanel.revalidate();
      mainPanel.repaint();
    }
  }
  /****************************************************************************
  * CancelButtonClicked.
  ****************************************************************************/
  private class CancelButtonClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      // try {s.shutdownOutput();} catch (Exception e) {}
      // try {out.close();} catch (Exception e) {}
      stop = true;
    }
  }
  /****************************************************************************
  * loadChooser.
  ****************************************************************************/
  private void loadChooser () {
    chooser = new JFileChooser(pdsFile != null ? pdsFile.getParent() : ".");
    chooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
    chooser.setApproveButtonText("Select");
    chooser.addActionListener(new ChooserAction());
    mainPanel.add(new MarginPanel(10,0,new HPanel(aquaButton,GUtil.HSpace(15),terraButton),10,0),BorderLayout.NORTH);
    mainPanel.add(chooser,BorderLayout.CENTER);
  }
  /****************************************************************************
  * SendStream.
  ****************************************************************************/
  private class SendStream implements Runnable {
    private File file;
    private HostPort processor;
    private FileInputStream in = null;
    public SendStream (File file, HostPort processor) {
      this.file = file;
      this.processor = processor;
    }
    public void run () {
      try {
      	doRun();
      } catch (Exception e) {
      	// e.printStackTrace();
      	try {statusField.setText(e.toString());} catch (Exception ex) {}
      } finally {
      	try {in.close();} catch (Exception ex) {}
      	try {out.close();} catch (Exception ex) {}
      	try {s.close();} catch (Exception ex) {}
      }
      GUtil.invokeAndWait(new Runnable() {
      	public void run () {
      	  selectButton.setEnabled(true);
      	}
      });
    }
    private void doRun () throws Exception {
      long sent = 0;
      long total = file.length();
      in = new FileInputStream(file);
      s = new Socket(processor.getHost(),processor.getPort());
      out = s.getOutputStream();

      stop = false;
      byte[] buffer = new byte[642];
      while (!stop)
      {
	if (pauseCheckBox.isSelected())
        {
	  Util.sleep(100);
	  continue;
	}
      	int count = in.read(buffer,0,6);
      	if (count == -1) {
      	  if (statusField != null) {
	    statusField.setText("EOF (" + Util.encodeByteCount(sent) + " sent)");
      	  } else {
      	    System.out.println("EOF (" + Util.encodeByteCount(sent) + " sent)");
      	  }
      	  break;
      	}
        int nMore = (((buffer[4] & 0xFF) << 8) | (buffer[5] & 0xFF)) + 1;
        int pLength = 6 + nMore;
        in.read(buffer,6,nMore);
      	out.write(buffer,0,pLength);
      	out.flush();
        sent += pLength;
        if (statusField != null) {
          statusField.setText(Util.encodeByteCount(sent) + " sent (" + Long.toString((100*sent)/total) + "%)");
        } else {
          System.out.println(Util.encodeByteCount(sent) + " sent (" + Long.toString((100*sent)/total) + "%)");
        }
      }
      /*
      if (stop) {
      	if (statusField != null) {
	  statusField.setText("STOPPED (" + Util.encodeByteCount(sent) + " sent)");
      	} else {
      	  System.out.println("STOPPED (" + Util.encodeByteCount(sent) + " sent)");
      	}
      }
      */
      in.close();
      out.close();
      s.close();
    }
  }
}
