/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/UTIL
*
*  History:
*
*  13-Dec-05, J Love, GST	Original version.
*  15-Aug-06, J Love, GST	Drawing of text.  Zooming rectangle.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.util;
import gov.nasa.gsfc.drl.geo.*;
import java.awt.*;

public class BothPolesMapParms implements MapParms {
  private int width;
  private int height;
  private int xOrigNorth;
  private int yOrigNorth;
  private int xOrigSouth;
  private int yOrigSouth;
  private int radius;
  private static final int MARGIN = 10;
  /****************************************************************************
  * BothPolesMapParms.
  ****************************************************************************/
  public BothPolesMapParms (int width, int height) {
    this.width = width;
    this.height = height;
    calc();
  }
  /****************************************************************************
  * getWidth.
  ****************************************************************************/
  public int getWidth () {
    return width;
  }
  /****************************************************************************
  * getHeight.
  ****************************************************************************/
  public int getHeight () {
    return height;
  }
  /****************************************************************************
  * getViewport.
  ****************************************************************************/
  public Rectangle getViewport () {
    return new Rectangle(0,0,width,height);
  }
  /****************************************************************************
  * isZoomable.
  ****************************************************************************/
  public boolean isZoomable () {
    return false;
  }
  /****************************************************************************
  * calcXY.
  ****************************************************************************/
  private int[] calcXY (double lat, double lon) {
    if (lat >= 0.0) {
      return calcNorthXY(lat,lon);
    } else {
      return calcSouthXY(lat,lon);
    }
  }
  /****************************************************************************
  * calcNorthXY.
  ****************************************************************************/
  private int[] calcNorthXY (double lat, double lon) {
    double d = radius * Util.cos(lat);
    return new int[] {xOrigNorth+((int)Math.round(d*Util.cos(lon))),
    		      yOrigNorth-((int)Math.round(d*Util.sin(lon)))};
  }
  /****************************************************************************
  * calcSouthXY.
  ****************************************************************************/
  private int[] calcSouthXY (double lat, double lon) {
    double d = radius * Util.cos(lat);
    return new int[] {xOrigSouth+((int)Math.round(d*Util.cos(lon))),
    		      yOrigSouth+((int)Math.round(d*Util.sin(lon)))};
  }
  /****************************************************************************
  * getLatLon.
  ****************************************************************************/
  public double[] getLatLon (int x, int y) {
    return new double[] {0.0,0.0};
  }
  /****************************************************************************
  * getAreaFilter.
  ****************************************************************************/
  public String getAreaFilter () {
    return "";
  }
  /****************************************************************************
  * isVisible.
  ****************************************************************************/
  public boolean isVisible (double lat, double lon) {
    return true;
  }
  public boolean isVisible (double minLat, double maxLat, double minLon, double maxLon) {
    return true;
  }
  public boolean isVisible (Geo.Extents extents) {
    return true;
  }
  public boolean isVisible (GeoBlock geoBlock) {
    return true;
  }
  /****************************************************************************
  * resize.
  ****************************************************************************/
  public Rectangle resize (int width, int height) {
    this.width = width;
    this.height = height;
    calc();
    return null;
  }
  /****************************************************************************
  * zoom.
  ****************************************************************************/
  public Rectangle zoom (double factor, int x, int y) {
    // Not supported.
    return null;
  }
  public Rectangle zoom (int x0, int y0, int x1, int y1) {
    // Not supported.
    return null;
  }
  public Rectangle zoom (double startStbdLat, double startStbdLon,
		         double startPortLat, double startPortLon,
		         double stopStbdLat, double stopStbdLon,
		         double stopPortLat, double stopPortLon) {
    // Not supported.
    return null;
  }
  /****************************************************************************
  * calc.
  ****************************************************************************/
  private void calc () {
    if (width-MARGIN-MARGIN-MARGIN > 2*(height-MARGIN-MARGIN)) {
      radius = (height-MARGIN-MARGIN-1) / 2;
      xOrigNorth = (width/2) - (MARGIN/2) - radius;
      yOrigNorth = height / 2;
      xOrigSouth = (width/2) + (MARGIN/2) + radius;
      yOrigSouth = height / 2;
    } else {
      radius = ((width-MARGIN-MARGIN-MARGIN-1) / 2) / 2;
      xOrigNorth = (width/2) - (MARGIN/2) - radius;
      yOrigNorth = MARGIN+radius;
      xOrigSouth = (width/2) + (MARGIN/2) + radius;
      yOrigSouth = MARGIN+radius;
    }
  }
  /****************************************************************************
  * drawImage.
  ****************************************************************************/
  public void drawImage (Graphics g, Image image) {
    g.drawImage(image,0,0,null);
  }
  /****************************************************************************
  * drawText.
  ****************************************************************************/
  public void drawText (Graphics g, double lat, double lon, String text) {
    int[] pt = calcXY(lat,lon);
    FontMetrics fm = g.getFontMetrics();
    int x = pt[0] - (fm.stringWidth(text)/2);
    int y = pt[1] + (fm.getAscent()/2);
    g.drawString(text,x,y);
  }
  /****************************************************************************
  * drawPoint.
  ****************************************************************************/
  public void drawPoint (Graphics g, double lat, double lon, int size) {
    int[] pt = calcXY(lat,lon);
    g.fillRect(pt[0]-(size/2),pt[1]-(size/2),size,size);
  }
  /****************************************************************************
  * drawLine.
  ****************************************************************************/
  public void drawLine (Graphics g, double lat0, double lon0, double lat1, double lon1) {
    if ((lat0 >= 0.0 && lat1 >= 0.0) || (lat0 < 0.0 && lat1 < 0.0)) {
      int[] pt0 = calcXY(lat0,lon0);
      int[] pt1 = calcXY(lat1,lon1);
      g.drawLine(pt0[0],pt0[1],pt1[0],pt1[1]);
    }
  }
  /****************************************************************************
  * fillPoly.
  ****************************************************************************/
  public void fillPoly (Graphics g, double lat0, double lon0, double lat1,
      			double lon1, double lat2, double lon2, double lat3,
      			double lon3 /*, Color color*/) {
    if ((lat0 >= 0.0 && lat1 >= 0.0 && lat2 >= 0.0 && lat3 >= 0.0) ||
        (lat0 < 0.0 && lat1 < 0.0 && lat2 < 0.0 && lat3 < 0.0)) {
      int[] x = new int[4], y = new int[4];
      int[] pt = calcXY(lat0,lon0);
      x[0] = pt[0];
      y[0] = pt[1];
      pt = calcXY(lat1,lon1);
      x[1] = pt[0];
      y[1] = pt[1];
      pt = calcXY(lat2,lon2);
      x[2] = pt[0];
      y[2] = pt[1];
      pt = calcXY(lat3,lon3);
      x[3] = pt[0];
      y[3] = pt[1];
      // g.setColor(color);
      g.fillPolygon(x,y,4);
      g.drawPolygon(x,y,4);
    }
  }
  /****************************************************************************
  * drawGrid.
  ****************************************************************************/
  public void drawGrid (Graphics g) {
    for (double lat = 0.0; lat <= 80.0; lat += 10.0) {
       int r = calcNorthXY(lat,0.0)[0] - xOrigNorth;
       g.setColor(Color.gray);
       g.drawArc(xOrigNorth-r,yOrigNorth-r,(2*r),(2*r),0,360);
    }
    for (double lon = -180.0; lon < 180.0; lon += 22.5) {
       if (lon % 45.0 == 0.0) {
         int[] pt1 = calcNorthXY(0.0,lon);
	 g.setColor(Color.gray);
         g.drawLine(xOrigNorth,yOrigNorth,pt1[0],pt1[1]);
       } else {
         int[] pt0 = calcNorthXY(80.0,lon);
         int[] pt1 = calcNorthXY(0.0,lon);
	 g.setColor(Color.gray);
         g.drawLine(pt0[0],pt0[1],pt1[0],pt1[1]);
       }
    }
    for (double lat = 0.0; lat >= -80.0; lat -= 10.0) {
       int r = calcSouthXY(lat,0.0)[0] - xOrigSouth;
       g.setColor(Color.gray);
       g.drawArc(xOrigSouth-r,yOrigSouth-r,(2*r),(2*r),0,360);
    }
    for (double lon = -180.0; lon < 180.0; lon += 22.5) {
       if (lon % 45.0 == 0.0) {
         int[] pt1 = calcSouthXY(0.0,lon);
	 g.setColor(Color.gray);
         g.drawLine(xOrigSouth,yOrigSouth,pt1[0],pt1[1]);
       } else {
         int[] pt0 = calcSouthXY(-80.0,lon);
         int[] pt1 = calcSouthXY(0.0,lon);
	 g.setColor(Color.gray);
         g.drawLine(pt0[0],pt0[1],pt1[0],pt1[1]);
       }
    }
  }
  /****************************************************************************
  * drawBorder.
  ****************************************************************************/
  public void drawBorder (Graphics g) {

  }
}
