/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/UTIL
*
*  History:
*
*   5-Aug-04, J Love, GST	Original version.
*   8-Feb-06, J Love, GST	Java 1.5.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.util;
import java.io.*;
import java.net.*;
import java.util.*;

public class FtpSession {
  private Socket cSock = null;
  private InputStream cIn = null;
  private OutputStream cOut = null;
  private ServerSocket ss = null;
  private Socket dSock = null;
  private InputStream dIn = null;
  private OutputStream dOut = null;
  private FtpSessionListener listener = null;
  //
  private static final int UNKNOWN_OS = 0;
  public static final int DOS = 1;
  public static final int UNIX = 2;
  public static final int UNIX_ALT1 = 3;
  private int os = UNKNOWN_OS;
  //
  public static final int FILES_ONLY = 1;
  public static final int DIRECTORIES_ONLY = 2;
  public static final int DIRECTORIES_AND_FILES = 3;
  //
  private class HostPort {
    private String host;
    private int port;
    public HostPort (String host, int port) {
      this.host = host;
      this.port = port;
    }
    public String getHost () {
      return host;
    }
    public int getPort () {
      return port;
    }
  }
  /****************************************************************************
  * FtpSession.
  ****************************************************************************/
  public FtpSession (String host, int port) throws Exception {
    construct(host,port,UNKNOWN_OS);
  }
  public FtpSession (String host, int port, int os) throws Exception {
    construct(host,port,os);
  }
  public FtpSession (String host, int port, String os) throws Exception {
    if (os.equals("dos")) {
      construct(host,port,DOS);
      return;
    }
    if (os.equals("unix")) {
      construct(host,port,UNIX);
      return;
    }
    if (os.equals("unix_alt1")) {
      construct(host,port,UNIX_ALT1);
      return;
    }
    construct(host,port,UNKNOWN_OS);
  }
  private void construct (String host, int port, int os) throws Exception {
    this.os = os;
    try {
      cSock = new Socket(host,port);
      cIn = cSock.getInputStream();
      cOut = cSock.getOutputStream();
      String[] lines = readReply();
      if (problemReply(lines)) {
	throw new Exception(encodeReply(lines));
      }
    } catch (Exception e) {
      try {cOut.close();} catch (Exception e1) {}
      try {cIn.close();} catch (Exception e1) {}
      try {cSock.close();} catch (Exception e1) {}
      throw e;
    }
  }
  /****************************************************************************
  * setListener.
  ****************************************************************************/
  public void setListener (FtpSessionListener listener) {
    this.listener = listener;
  }
  /****************************************************************************
  * login.
  ****************************************************************************/
  public void login (String user, String pw) throws Exception {
    sendCommand("USER " + user);
    String[] lines = readReply();
    if (problemReply(lines)) throw new Exception(encodeReply(lines));
    sendCommand("PASS " + pw);
    lines = readReply();
    if (problemReply(lines)) throw new Exception(encodeReply(lines));
  }
  /****************************************************************************
  * cd.
  ****************************************************************************/
  public void cd (String dir) throws Exception {
    sendCommand("CWD " + dir);
    String[] lines = readReply();
    if (problemReply(lines)) throw new Exception(encodeReply(lines));
  }
  public void cd (String[] dirs) throws Exception {
    for (int i = 0; i < dirs.length; i++) {
       sendCommand("CWD " + dirs[i]);
       String[] lines = readReply();
       if (problemReply(lines)) throw new Exception(encodeReply(lines));
    }
  }
  /****************************************************************************
  * size.
  * If the name specified is a directory, the result is undefined.
  * Returns 0 if unknown because the replies aren't as expected.
  ****************************************************************************/
  public long size (String name) throws Exception {
    if (os == UNKNOWN_OS) {
      tryToDetermineOS();
    }
    switch (os) {
      case DOS: {
	return sizeDOS(name);
      }
      case UNIX: {
	return sizeUNIX(name,false);
      }
      case UNIX_ALT1: {
	return sizeUNIX(name,true);
      }
      default: {
	throw new Exception("internal error");
      }
    }
  }
  private long sizeDOS (String name) throws Exception {
    String answer = answerPASV("LIST " + name,new String[0]);
    StringTokenizer byBlanks = new StringTokenizer(answer," ");
    try {
      /*String date =*/ byBlanks.nextToken();
      /*String time =*/ byBlanks.nextToken();
      return Long.parseLong(byBlanks.nextToken());
    } catch (Exception e) {
      return 0;
    }
  }
  private long sizeUNIX (String name, boolean alt1) throws Exception {
    String answer = answerPASV("LIST " + name,new String[0]);
    StringTokenizer byBlanks = new StringTokenizer(answer," ");
    try {
      String permissions = byBlanks.nextToken();
      if (permissions.startsWith("l")) {
        int index = answer.indexOf("->");
        if (index == -1) {
	  return 0;
        }
        return sizeUNIX(answer.substring(index+2).trim(),alt1);
      }
      /*String skip1 =*/ byBlanks.nextToken();
      /*String skip2 =*/ byBlanks.nextToken();
      if (!alt1) {
	/*String skip3 =*/ byBlanks.nextToken();
      }
      return Long.parseLong(byBlanks.nextToken());
    } catch (Exception e) {
      return 0;
    }
  }
  /****************************************************************************
  * listing.
  ****************************************************************************/
  public LinkedList<String> listing () throws Exception {
    return listing(DIRECTORIES_AND_FILES);
  }
  public LinkedList<String> listing (int which) throws Exception {
    if (os == UNKNOWN_OS) {
      tryToDetermineOS();
    }
    LinkedList<String> names = new LinkedList<String>();
    String answer = answerPASV("LIST",new String[] {"550"});
    if (answer == null) {
      return names;
    }
    StringTokenizer byLine = new StringTokenizer(answer,"\r\n");
    // If UNIX, first skip the "total" line.
    switch (os) {
      case UNIX:
      case UNIX_ALT1: {
	if (byLine.hasMoreTokens()) {
	  /*String skip =*/ byLine.nextToken();
	}
	break;
      }
    }
    while (byLine.hasMoreTokens()) {
      String entry = byLine.nextToken();
      // Check length of entry because some FTP servers insert blanks lines...
      if (entry.length() > 0) {
	switch (which) {
	  case FILES_ONLY:
	    if (!isDirectory(entry)) {
	      names.add(getName(entry));
	    }
	    break;
	  case DIRECTORIES_ONLY:
	    if (isDirectory(entry)) {
	      names.add(getName(entry));
	    }
	    break;
	  case DIRECTORIES_AND_FILES:
	    names.add(getName(entry));
	    break;
	}
      }
    }
    return names;
  }
  /****************************************************************************
  * quit.
  ****************************************************************************/
  public void quit () throws Exception {
    sendCommand("QUIT");
    String[] lines = readReply();
    if (problemReply(lines)) throw new Exception(encodeReply(lines));
    cOut.close();
    cIn.close();
    cSock.close();
  }
  /****************************************************************************
  * abort.
  ****************************************************************************/
  public void abort () {
    try {cOut.close();} catch (Exception e) {}
    try {cIn.close();} catch (Exception e) {}
    try {cSock.close();} catch (Exception e) {}
    try {ss.close();} catch (Exception e) {}
    try {dSock.close();} catch (Exception e) {}
    try {dIn.close();} catch (Exception e) {}
    try {dOut.close();} catch (Exception e) {}
  }
  /****************************************************************************
  * tryToDetermineOS.
  * Only UNIX or DOS is expected...
  ****************************************************************************/
  private void tryToDetermineOS () throws Exception {
    String answer = answerPASV("LIST",new String[0]);
    StringTokenizer byLine = new StringTokenizer(answer,"\r\n");
    if (!byLine.hasMoreTokens()) {
      os = DOS;
      return;
    }
    String firstLine = byLine.nextToken();
    if (firstLine.startsWith("total")) {
      // Try to determine if ALT1
      os = UNIX;
    } else {
      os = DOS;
    }
    return;
  }
  /****************************************************************************
  * isDirectory.
  ****************************************************************************/
  private boolean isDirectory (String entry) throws Exception {
    try {
      cd(entry);
    } catch (Exception e) {
      return false;
    }
    cd("..");
    return true;
  }
  /****************************************************************************
  * getName.
  ****************************************************************************/
  private String getName (String entry) throws Exception {
    switch (os) {
      case DOS: {
	return getNameDOS(entry);
      }
      case UNIX: {
	return getNameUNIX(entry,false);
      }
      case UNIX_ALT1: {
	return getNameUNIX(entry,true);
      }
    }
    throw new Exception("internal error");
  }
  private String getNameDOS (String entry) throws Exception {
    StringTokenizer byBlanks = new StringTokenizer(entry," \t");
    if (byBlanks.countTokens() < 4) {
      throw new Exception("getNameDOS: unexpected LIST entry");
    }
    String date = byBlanks.nextToken();
    String time = byBlanks.nextToken();
    String sizeOrDIR = byBlanks.nextToken();
    String beginName = byBlanks.nextToken();
    // This looks strange...
    int index = entry.indexOf(date);
    index = entry.indexOf(time,index+date.length());
    index = entry.indexOf(sizeOrDIR,index+time.length());
    index = entry.indexOf(beginName,index+sizeOrDIR.length());
    return entry.substring(index).trim();
  }
  private String getNameUNIX (String entry, boolean alt1) throws Exception {
    StringTokenizer byBlanks = new StringTokenizer(entry," \t");
    if (byBlanks.countTokens() < 9) {
      throw new Exception("getNameUNIX: unexpected LIST entry");
    }
    String permissions = byBlanks.nextToken();
    String skip1 = byBlanks.nextToken();
    String skip2 = byBlanks.nextToken();
    String skip3 = (alt1 ? null : byBlanks.nextToken());
    String size = byBlanks.nextToken();
    String when1 = byBlanks.nextToken();
    String when2 = byBlanks.nextToken();
    String when3 = byBlanks.nextToken();
    // This looks strange...
    int index = entry.indexOf(permissions);
    index = entry.indexOf(skip1,index+permissions.length());
    index = entry.indexOf(skip2,index+skip1.length());
    if (!alt1) {
      index = entry.indexOf(skip3,index+skip2.length());
      index = entry.indexOf(size,index+skip3.length());
    } else {
      index = entry.indexOf(size,index+skip2.length());
    }
    index = entry.indexOf(when1,index+size.length());
    index = entry.indexOf(when2,index+when1.length());
    index = entry.indexOf(when3,index+when2.length());
    String name = entry.substring(index+when3.length()+1);
    index = name.indexOf(" -> ");
    return (index == -1 ? name : name.substring(0,index));
  }
  /****************************************************************************
  * answerPASV.
  * Returns null if one of the expected errors occurs.
  ****************************************************************************/
  private String answerPASV (String question,
			     String[] expectedErrors) throws Exception {
    StringBuffer answer = new StringBuffer();
    try {
      sendCommand("PASV");
      String[] lines = readReply();
      if (problemReply(lines)) {
	throw new Exception(encodeReply(lines));
      }
      HostPort hostPort = parsePASV(lines[0]);
      dSock = new Socket(hostPort.getHost(),hostPort.getPort());
      sendCommand(question);
      lines = readReply();
      if (problemReply(lines)) {
	for (int i = 0; i < expectedErrors.length; i++) {
	   if (lines[0].startsWith(expectedErrors[i])) {
	     return null;
	   }
	}
        throw new Exception(encodeReply(lines));
      }
      dIn = dSock.getInputStream();
      boolean eof = false;
      while (!eof) {
        int b = dIn.read();
	switch (b) {
	  case -1: {
	    eof = true;
	    break;
	  }
	  default: {
	    answer.append((char)b);
	    break;
	  }
        }
      }
      dIn.close();
      dSock.close();
      lines = readReply();
      if (problemReply(lines)) {
	throw new Exception(encodeReply(lines));
      }
      return answer.toString();
    } finally {
      try {dIn.close();} catch (Exception e1) {}
      try {dSock.close();} catch (Exception e1) {}
    }
  }
  /****************************************************************************
  * ascii.
  ****************************************************************************/
  public void ascii () throws Exception {
    sendCommand("TYPE A");
    String[] lines = readReply();
    if (problemReply(lines)) throw new Exception(encodeReply(lines));
  }
  /****************************************************************************
  * binary.
  ****************************************************************************/
  public void binary () throws Exception {
    sendCommand("TYPE I");
    String[] lines = readReply();
    if (problemReply(lines)) throw new Exception(encodeReply(lines));
  }
  /****************************************************************************
  * get.
  * The PASV method is tried first.  If that fails, the PORT method is tried.
  ****************************************************************************/
  public void get (String remoteFile, Object localDestination) throws Exception {
    try {
      getPASV(remoteFile,localDestination);
    } catch (Exception e) {
      getPORT(remoteFile,localDestination);
    }
  }
  /****************************************************************************
  * getBytes.
  * The PASV method is tried first.  If that fails, the PORT method is tried.
  ****************************************************************************/
  public byte[] getBytes (String remoteFile) throws Exception {
    try {
      return getPASV(remoteFile,null);
    } catch (Exception e) {
      return getPORT(remoteFile,null);
    }
  }
  /****************************************************************************
  * getPORT.
  ****************************************************************************/
  private byte[] getPORT (String remoteFile,
			  Object localDestination) throws Exception {
    byte[] bytes = null;
    try {
      ss = new ServerSocket(0,1,InetAddress.getLocalHost());
      sendCommand(buildPORT(ss.getInetAddress().getHostAddress(),
			    ss.getLocalPort()));
      String[] lines = readReply();
      if (problemReply(lines)) {
	throw new Exception(encodeReply(lines));
      }
      sendCommand("RETR " + remoteFile);
      lines = readReply();
      if (problemReply(lines)) {
	throw new Exception(encodeReply(lines));
      }
      dSock = ss.accept();
      ss.close();
      dIn = dSock.getInputStream();
      if (localDestination != null) {
      	if (localDestination instanceof File) {
	  dOut = new FileOutputStream((File)localDestination);
      	}
      	if (localDestination instanceof String) {
	  dOut = new FileOutputStream((String)localDestination);
      	}
      	if (localDestination instanceof OutputStream) {
	  dOut = (OutputStream) localDestination;
      	}
      } else {
	int nBytes = (int) size(remoteFile);
	dOut = new ByteArrayOutputStream(nBytes > 0 ? nBytes : 1024);
      }
      transferData(dIn,dOut);
      if (localDestination == null) {
	bytes = ((ByteArrayOutputStream)dOut).toByteArray();
      }
      dOut.close();
      dIn.close();
      dSock.close();
      lines = readReply();
      if (problemReply(lines)) {
	throw new Exception(encodeReply(lines));
      }
    } catch (Exception e) {
      try {dOut.close();} catch (Exception e1) {}
      try {dIn.close();} catch (Exception e1) {}
      try {dSock.close();} catch (Exception e1) {}
      try {ss.close();} catch (Exception e1) {}
      throw e;
    }
    return bytes;	// Will be null if retrieved to a local file...
  }
  /****************************************************************************
  * getPASV.
  ****************************************************************************/
  private byte[] getPASV (String remoteFile,
			  Object localDestination) throws Exception {
    byte[] bytes = null;
    try {
      sendCommand("PASV");
      String[] lines = readReply();
      if (problemReply(lines)) {
	throw new Exception(encodeReply(lines));
      }
      HostPort hostPort = parsePASV(lines[0]);
      sendCommand("RETR " + remoteFile);
      dSock = new Socket(hostPort.getHost(),hostPort.getPort());
      lines = readReply();
      if (problemReply(lines)) {
	throw new Exception(encodeReply(lines));
      }
      dIn = dSock.getInputStream();
      if (localDestination != null) {
      	if (localDestination instanceof File) {
	  dOut = new FileOutputStream((File)localDestination);
      	}
      	if (localDestination instanceof String) {
	  dOut = new FileOutputStream((String)localDestination);
      	}
      	if (localDestination instanceof OutputStream) {
      	  dOut = (OutputStream) localDestination;
      	}
      } else {
	int nBytes = (int) size(remoteFile);
	dOut = new ByteArrayOutputStream(nBytes > 0 ? nBytes : 1024);
      }
      transferData(dIn,dOut);
      if (localDestination == null) {
	bytes = ((ByteArrayOutputStream)dOut).toByteArray();
      }
      dOut.close();
      dIn.close();
      dSock.close();
      lines = readReply();
      if (problemReply(lines)) {
	throw new Exception(encodeReply(lines));
      }
    } catch (Exception e) {
      try {dOut.close();} catch (Exception e1) {}
      try {dIn.close();} catch (Exception e1) {}
      try {dSock.close();} catch (Exception e1) {}
      throw e;
    }
    return bytes;	// Will be null if retrieved to a local file...
  }
  /****************************************************************************
  * put.
  * The PASV method is tried first.  If that fails, the PORT method is tried.
  ****************************************************************************/
  public void put (Object localSource, String remoteFile) throws Exception {
    try {
      putPASV(localSource,remoteFile);
    } catch (Exception e) {
      putPORT(localSource,remoteFile);
    }
  }
  /****************************************************************************
  * putBytes.
  * The PASV method is tried first.  If that fails, the PORT method is tried.
  ****************************************************************************/
  public void putBytes (byte[] bytes, String remoteFile) throws Exception {
    try {
      putPASV(bytes,remoteFile);
    } catch (Exception e) {
      putPORT(bytes,remoteFile);
    }
  }
  /****************************************************************************
  * putPORT.
  ****************************************************************************/
  private void putPORT (Object localSource,
			String remoteFile) throws Exception {
    try {
      ss = new ServerSocket(0,1,InetAddress.getLocalHost());
      sendCommand(buildPORT(ss.getInetAddress().getHostAddress(),
			    ss.getLocalPort()));
      String[] lines = readReply();
      if (problemReply(lines)) {
	throw new Exception(encodeReply(lines));
      }
      sendCommand("STOR " + remoteFile);
      lines = readReply();
      if (problemReply(lines)) {
	throw new Exception(encodeReply(lines));
      }
      dSock = ss.accept();
      ss.close();
      if (localSource instanceof byte[]) {
	dIn = new ByteArrayInputStream((byte[])localSource);
      }
      if (localSource instanceof File) {
	dIn = new FileInputStream((File)localSource);
      }
      if (localSource instanceof String) {
	dIn = new FileInputStream((String)localSource);
      }
      if (localSource instanceof InputStream) {
      	dIn = (InputStream) localSource;
      }
      dOut = dSock.getOutputStream();
      transferData(dIn,dOut);
      dOut.close();
      dIn.close();
      dSock.close();
      lines = readReply();
      if (problemReply(lines)) {
	throw new Exception(encodeReply(lines));
      }
    } catch (Exception e) {
      try {dOut.close();} catch (Exception e1) {}
      try {dIn.close();} catch (Exception e1) {}
      try {dSock.close();} catch (Exception e1) {}
      try {ss.close();} catch (Exception e1) {}
      throw e;
    }
  }
  /****************************************************************************
  * putPASV.
  ****************************************************************************/
  private void putPASV (Object localSource,
			String remoteFile) throws Exception {
    try {
      sendCommand("PASV");
      String[] lines = readReply();
      if (problemReply(lines)) {
	throw new Exception(encodeReply(lines));
      }
      HostPort hostPort = parsePASV(lines[0]);
      sendCommand("STOR " + remoteFile);
      dSock = new Socket(hostPort.getHost(),hostPort.getPort());
      lines = readReply();
      if (problemReply(lines)) {
	throw new Exception(encodeReply(lines));
      }
      if (localSource instanceof byte[]) {
	dIn = new ByteArrayInputStream((byte[])localSource);
      }
      if (localSource instanceof File) {
	dIn = new FileInputStream((File)localSource);
      }
      if (localSource instanceof String) {
	dIn = new FileInputStream((String)localSource);
      }
      if (localSource instanceof InputStream) {
	dIn = (InputStream) localSource;
      }
      dOut = dSock.getOutputStream();
      transferData(dIn,dOut);
      dOut.close();
      dIn.close();
      dSock.close();
      lines = readReply();
      if (problemReply(lines)) {
	throw new Exception(encodeReply(lines));
      }
    } catch (Exception e) {
      try {dOut.close();} catch (Exception e1) {}
      try {dIn.close();} catch (Exception e1) {}
      try {dSock.close();} catch (Exception e1) {}
      throw e;
    }
  }
  /****************************************************************************
  * buildPORT.
  ****************************************************************************/
  private String buildPORT (String hostAddress, int localPort) {
    StringTokenizer byDot = new StringTokenizer(hostAddress,".");
    String host1 = byDot.nextToken();
    String host2 = byDot.nextToken();
    String host3 = byDot.nextToken();
    String host4 = byDot.nextToken();
    int msb = (localPort >> 8) & 0x000000FF;
    int lsb = localPort & 0x000000FF;
    return "PORT " + host1 + "," + host2 + "," + host3 + "," + host4 +
	   "," + Integer.toString(msb) + "," + Integer.toString(lsb);
  }
  /****************************************************************************
  * parsePASV.
  ****************************************************************************/
  private HostPort parsePASV (String line) throws Exception {
    StringTokenizer st = new StringTokenizer(line,"(,)");
    if (st.countTokens() < 7) {
      throw new Exception("unexpected reply to PASV: " + line);
    }
    st.nextToken();
    String host1 = st.nextToken();
    String host2 = st.nextToken();
    String host3 = st.nextToken();
    String host4 = st.nextToken();
    int msb = Integer.parseInt(st.nextToken());
    int lsb = Integer.parseInt(st.nextToken());
    String host = host1 + "." + host2 + "." + host3 + "." + host4;
    int port = (msb << 8) | lsb;
    return new HostPort(host,port);
  }
  /****************************************************************************
  * delete.
  ****************************************************************************/
  public void delete (String remoteFile) throws Exception {
    sendCommand("DELE " + remoteFile);
    String[] lines = readReply();
    if (problemReply(lines)) throw new Exception(encodeReply(lines));
  }
  /****************************************************************************
  * sendCommand.
  ****************************************************************************/
  private void sendCommand (String command) throws Exception {
    byte[] bytes = command.getBytes();
    cOut.write(bytes);
    cOut.write((int)'\n');
    cOut.flush();
  }
  /****************************************************************************
  * readReply.
  ****************************************************************************/
  private String[] readReply () throws Exception {
    String prefixNumber = null;
    LinkedList<String> lines = new LinkedList<String>();
    StringBuffer line = new StringBuffer();
    while (true) {
      int b = cIn.read();
      switch (b) {
        case -1:
	  throw new Exception("early EOF");
	case '\r':
	  break;
	case '\n':
	  lines.add(line.toString());
	  if (line.length() >= 4) {
	    if (line.charAt(3) == '-') {
	      if (prefixNumber == null) {
	        prefixNumber = line.substring(0,3);
	      }
	    } else {
	      if (prefixNumber != null) {
	        if (line.substring(0,3).equals(prefixNumber)) {
		  return lines.toArray(new String[lines.size()]);
	        }
	      } else {
		return lines.toArray(new String[lines.size()]);
	      }
	    }
	  }
	  line.setLength(0);
	  break;
	default:
	  line.append((char)b);
	  break;
      }
    }
  }
  /****************************************************************************
  * transferData.
  ****************************************************************************/
  private void transferData (InputStream in,
			     OutputStream out) throws Exception {
    long byteCount = 0;
    byte[] b = new byte[32*1024];
    while (true) {
      int count = in.read(b);
      if (count == -1) break;
      out.write(b,0,count);
      byteCount += count;
      if (listener != null) {
	listener.transferProgress(byteCount);
      }
    }
  }
  /****************************************************************************
  * problemReply.
  ****************************************************************************/
  private boolean problemReply (String[] lines) {
    for (int i = 0; i < lines.length; i++) {
       if (lines[i].startsWith("4")) return true;
       if (lines[i].startsWith("5")) return true;
    }
    return false;
  }
  /****************************************************************************
  * encodeReply.
  ****************************************************************************/
  private String encodeReply (String[] lines) {
    StringBuffer reply = new StringBuffer();
    reply.append(lines[0]);
    for (int i = 1; i < lines.length; i++) {
       reply.append(" ");
       if (reply.length() > 4) reply.append(lines[i].substring(4));
    }
    return reply.toString();
  }
  /****************************************************************************
  * finalize.
  ****************************************************************************/
  protected void finalize () {
    try {cOut.close();} catch (Exception e1) {}
    try {cIn.close();} catch (Exception e1) {}
    try {cSock.close();} catch (Exception e1) {}
    try {ss.close();} catch (Exception e) {}
    try {dSock.close();} catch (Exception e) {}
    try {dIn.close();} catch (Exception e) {}
    try {dOut.close();} catch (Exception e) {}
  }
}
