/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/UTIL
*
*  History:
*
*   5-Aug-04, J Love, GST	Original version.
*  17-Mar-05, J Love, GST	Moved "isAscending".
*  19-Dec-05, J Love, GST	Geolocation, etc.
*  17-Jan-07, J Love, GST	Support for fixing a corrupted pass file.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.util;
import gov.nasa.gsfc.drl.geo.*;
import java.io.*;

public class HeaderBlock implements Block {
  private GroundStation groundStation;
  private int platformCode;
  private int instrumentCode;
  private int directionCode;
  private int startSolarCode;
  private long startedAt;
  private Geo.Extents estExtents;
  private boolean isGeolocated;
  private boolean isDebowed;
  private int coarsestResolution;
  private int coarsestBlockRowCount;
  private int coarsestBlockColCount;
  private int nBands;
  private int[] bandIndices;
  private int[] bandResolutions;
  private int[] bandBlockRowCounts;
  private int[] bandBlockColCounts;
  private int geoResolution;
  private int geoBlockRowCount;
  private int geoBlockColCount;
  private long blockCount;
  private long offsetOfLastBlock;
  /****************************************************************************
  * HeaderBlock.
  ****************************************************************************/
  public HeaderBlock (HeaderBlock header, boolean isGeolocated, boolean isDebowed) {
    this.groundStation = header.getGroundStation();
    this.platformCode = header.getPlatformCode();
    this.instrumentCode = header.getInstrumentCode();
    this.directionCode = header.getDirectionCode();
    this.startSolarCode = header.getStartSolarCode();
    this.startedAt = header.getStartedAt();
    this.estExtents = header.getEstExtents();
    this.isGeolocated = isGeolocated;
    this.isDebowed = isDebowed;
  }
  public HeaderBlock (GroundStation groundStation, int platformCode,
  		      int instrumentCode, int directionCode,
		      int startSolarCode, long startedAt, Geo.Extents estExtents,
		      boolean isGeolocated, boolean isDebowed,
		      int coarsestResolution, int coarsestBlockRowCount,
		      int coarsestBlockColCount, /* int widthPct, int offCenterPct, */
		      int nBands, int[] bandIndices, int[] bandResolutions,
		      int[] bandBlockRowCounts, int[] bandBlockColCounts,
		      int geoResolution, int geoBlockRowCount,
		      int geoBlockColCount) {
    this.groundStation = groundStation;
    this.platformCode = platformCode;
    this.instrumentCode = instrumentCode;
    this.directionCode = directionCode;
    this.startSolarCode = startSolarCode;
    this.startedAt = startedAt;
    this.estExtents = estExtents;
    this.isGeolocated = isGeolocated;
    this.isDebowed = isDebowed;
    this.coarsestResolution = coarsestResolution;
    this.coarsestBlockRowCount = coarsestBlockRowCount;
    this.coarsestBlockColCount = coarsestBlockColCount;
    this.nBands = nBands;
    this.bandIndices = bandIndices;
    this.bandResolutions = bandResolutions;
    this.bandBlockRowCounts = bandBlockRowCounts;
    this.bandBlockColCounts = bandBlockColCounts;
    this.geoResolution = geoResolution;
    this.geoBlockRowCount = geoBlockRowCount;
    this.geoBlockColCount = geoBlockColCount;
  }
  public HeaderBlock (DataInput in) throws Exception {
    read(in);
  }
  /****************************************************************************
  * read.
  ****************************************************************************/
  public void read (DataInput in) throws Exception {
    groundStation = new GroundStation(in);
    platformCode = in.readInt();
    instrumentCode = in.readInt();
    directionCode = in.readInt();
    startSolarCode = in.readInt();
    startedAt = in.readLong();
    estExtents = new Geo.Extents(in);
    isGeolocated = in.readBoolean();
    isDebowed = in.readBoolean();
    coarsestResolution = in.readInt();
    coarsestBlockRowCount = in.readInt();
    coarsestBlockColCount = in.readInt();
    nBands = in.readInt();
    bandIndices = new int[nBands];
    bandResolutions = new int[nBands];
    bandBlockRowCounts = new int[nBands];
    bandBlockColCounts = new int[nBands];
    for (int band = 0; band < nBands; band++) {
       bandIndices[band] = in.readInt();
       bandResolutions[band] = in.readInt();
       bandBlockRowCounts[band] = in.readInt();
       bandBlockColCounts[band] = in.readInt();
    }
    geoResolution = in.readInt();
    geoBlockRowCount = in.readInt();
    geoBlockColCount = in.readInt();
    blockCount = in.readLong();
    offsetOfLastBlock = in.readLong();
  }
  /****************************************************************************
  * getGroundStation.
  ****************************************************************************/
  public GroundStation getGroundStation () {
    return groundStation;
  }
  /****************************************************************************
  * getPlatformCode.
  ****************************************************************************/
  public int getPlatformCode () {
    return platformCode;
  }
  /****************************************************************************
  * getInstrumentCode.
  ****************************************************************************/
  public int getInstrumentCode () {
    return instrumentCode;
  }
  /****************************************************************************
  * getDirectionCode.
  ****************************************************************************/
  public int getDirectionCode () {
    return directionCode;
  }
  /****************************************************************************
  * getStartSolarCode.
  ****************************************************************************/
  public int getStartSolarCode () {
    return startSolarCode;
  }
  /****************************************************************************
  * getStartedAt.
  ****************************************************************************/
  public long getStartedAt () {
    return startedAt;
  }
  /****************************************************************************
  * getEstExtents.
  ****************************************************************************/
  public Geo.Extents getEstExtents () {
    return estExtents;
  }
  /****************************************************************************
  * isGeolocated.
  ****************************************************************************/
  public boolean isGeolocated () {
    return isGeolocated;
  }
  /****************************************************************************
  * isDebowed.
  ****************************************************************************/
  public boolean isDebowed () {
    return isDebowed;
  }
  /****************************************************************************
  * getCoarsestResolution.
  ****************************************************************************/
  public int getCoarsestResolution () {
    return coarsestResolution;
  }
  /****************************************************************************
  * getCoarsestBlockRowCount.
  ****************************************************************************/
  public int getCoarsestBlockRowCount () {
    return coarsestBlockRowCount;
  }
  /****************************************************************************
  * getCoarsestBlockColCount.
  ****************************************************************************/
  public int getCoarsestBlockColCount () {
    return coarsestBlockColCount;
  }
  /****************************************************************************
  * getBandCount.
  ****************************************************************************/
  public int getBandCount () {
    return nBands;
  }
  /****************************************************************************
  * getBandIndex.
  ****************************************************************************/
  public int getBandIndex (int offset) {
    return bandIndices[offset];
  }
  /****************************************************************************
  * getBandResolution.
  ****************************************************************************/
  public int getBandResolution (int offset) {
    return bandResolutions[offset];
  }
  /****************************************************************************
  * getBlockRowCount.
  ****************************************************************************/
  public int getBlockRowCount (int offset) {
    return bandBlockRowCounts[offset];
  }
  /****************************************************************************
  * getBlockColCount.
  ****************************************************************************/
  public int getBlockColCount (int offset) {
    return bandBlockColCounts[offset];
  }
  /****************************************************************************
  * getGeoResolution.
  ****************************************************************************/
  public int getGeoResolution () {
    return geoResolution;
  }
  /****************************************************************************
  * getGeoBlockRowCount.
  ****************************************************************************/
  public int getGeoBlockRowCount () {
    return geoBlockRowCount;
  }
  /****************************************************************************
  * getGeoBlockColCount.
  ****************************************************************************/
  public int getGeoBlockColCount () {
    return geoBlockColCount;
  }
  /****************************************************************************
  * getBlockCount.
  ****************************************************************************/
  public long getBlockCount () {
    return blockCount;
  }
  /****************************************************************************
  * getOffsetOfLastBlock.
  ****************************************************************************/
  public long getOffsetOfLastBlock () {
    return offsetOfLastBlock;
  }
  /****************************************************************************
  * setCoarsestResolution.
  ****************************************************************************/
  public void setCoarsestResolution (int coarsestResolution) {
    this.coarsestResolution = coarsestResolution;
  }
  /****************************************************************************
  * setCoarsestBlockRowCount.
  ****************************************************************************/
  public void setCoarsestBlockRowCount (int coarsestBlockRowCount) {
    this.coarsestBlockRowCount = coarsestBlockRowCount;
  }
  /****************************************************************************
  * setCoarsestBlockColCount.
  ****************************************************************************/
  public void setCoarsestBlockColCount (int coarsestBlockColCount) {
    this.coarsestBlockColCount = coarsestBlockColCount;
  }
  /****************************************************************************
  * setBandCount.
  ****************************************************************************/
  public void setBandCount (int nBands) {
    this.nBands = nBands;
    bandIndices = new int[nBands];
    bandResolutions = new int[nBands];
    bandBlockRowCounts = new int[nBands];
    bandBlockColCounts = new int[nBands];
  }
  /****************************************************************************
  * setBandIndex.
  ****************************************************************************/
  public void setBandIndex (int offset, int bandIndex) {
    this.bandIndices[offset] = bandIndex;
  }
  /****************************************************************************
  * setBandResolution.
  ****************************************************************************/
  public void setBandResolution (int offset, int bandResolution) {
    this.bandResolutions[offset] = bandResolution;
  }
  /****************************************************************************
  * setBlockRowCount.
  ****************************************************************************/
  public void setBlockRowCount (int offset, int nBlockRows) {
    this.bandBlockRowCounts[offset] = nBlockRows;
  }
  /****************************************************************************
  * setBlockColCount.
  ****************************************************************************/
  public void setBlockColCount (int offset, int nBlockCols) {
    this.bandBlockColCounts[offset] = nBlockCols;
  }
  /****************************************************************************
  * setGeoResolution.
  ****************************************************************************/
  public void setGeoResolution (int geoResolution) {
    this.geoResolution = geoResolution;
  }
  /****************************************************************************
  * setGeoBlockRowCount.
  ****************************************************************************/
  public void setGeoBlockRowCount (int geoBlockRowCount) {
    this.geoBlockRowCount = geoBlockRowCount;
  }
  /****************************************************************************
  * setGeoBlockColCount.
  ****************************************************************************/
  public void setGeoBlockColCount (int geoBlockColCount) {
    this.geoBlockColCount = geoBlockColCount;
  }
  /****************************************************************************
  * setBlockCount.
  ****************************************************************************/
  public void setBlockCount (long blockCount) {
    this.blockCount = blockCount;
  }
  /****************************************************************************
  * incrBlockCount.
  ****************************************************************************/
  public void incrBlockCount () {
    incrBlockCount(1);
  }
  public void incrBlockCount (int count) {
    blockCount += count;
  }
  /****************************************************************************
  * setOffsetOfLastBlock.
  ****************************************************************************/
  public void setOffsetOfLastBlock (long offsetOfLastBlock) {
    this.offsetOfLastBlock = offsetOfLastBlock;
  }
  /****************************************************************************
  * write.
  ****************************************************************************/
  public void write (DataOutput out) throws Exception {
    out.writeInt(Simulcast.HEADER_BLOCK_CODE);
    groundStation.write(out);
    out.writeInt(platformCode);
    out.writeInt(instrumentCode);
    out.writeInt(directionCode);
    out.writeInt(startSolarCode);
    out.writeLong(startedAt);
    estExtents.write(out);
    out.writeBoolean(isGeolocated);
    out.writeBoolean(isDebowed);
    out.writeInt(coarsestResolution);
    out.writeInt(coarsestBlockRowCount);
    out.writeInt(coarsestBlockColCount);
    out.writeInt(nBands);
    for (int band = 0; band < nBands; band++) {
       out.writeInt(bandIndices[band]);
       out.writeInt(bandResolutions[band]);
       out.writeInt(bandBlockRowCounts[band]);
       out.writeInt(bandBlockColCounts[band]);
    }
    out.writeInt(geoResolution);
    out.writeInt(geoBlockRowCount);
    out.writeInt(geoBlockColCount);
    out.writeLong(blockCount);
    out.writeLong(offsetOfLastBlock);
  }
}
