/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/UTIL
*
*  History:
*
*   8-Feb-06, J Love, GST	Original version.
*  15-Aug-06, J Love, GST	Drawing of text.  Zooming rectangle.
*  21-May-07, Z Yin, GST        Between +-180, draw image by two separated polygons.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.util;
import gov.nasa.gsfc.drl.geo.*;
import java.awt.*;

public class MercatorMapParms implements MapParms {
  private int width;		// Full width including margins.
  private int height;
  private int vpWidth;		// Width of viewport.
  private int vpHeight;
  private int vsWidth;		// Width of virtual space.
  private int vsHeight;
  private double vScaleX;
  private double vScaleY;
  private int vpLocOnVSx;
  private int vpLocOnVSy;
  private double zoom;
  private static final int[] gridIntervalChoices = new int[] {1,2,3,4,5,10,15,30,45};
  private static final int X = 0;
  private static final int Y = 1;
  // private static final int MIN = 0;
  // private static final int MAX = 1;
  private static final int LAT = 0;
  private static final int LON = 1;
  private static final int MARGIN = 22;
  /****************************************************************************
  * MercatorMapParms.
  ****************************************************************************/
  public MercatorMapParms (int width, int height) {
    this.width = width;
    this.height = height;
    zoom = 1.0;
    vCalc();
    vpLocOnVSx = 0;
    vpLocOnVSy = 0;
  }
  public MercatorMapParms (XmlParser parser) throws Exception {
    this.width = Integer.parseInt(parser.getElementValue("width"));
    this.height = Integer.parseInt(parser.getElementValue("height"));
    this.vpWidth = Integer.parseInt(parser.getElementValue("vpWidth"));
    this.vpHeight = Integer.parseInt(parser.getElementValue("vpHeight"));
    this.vsWidth = Integer.parseInt(parser.getElementValue("vsWidth"));
    this.vsHeight = Integer.parseInt(parser.getElementValue("vsHeight"));
    this.vScaleX = Double.parseDouble(parser.getElementValue("vScaleX"));
    this.vScaleY = Double.parseDouble(parser.getElementValue("vScaleY"));
    this.vpLocOnVSx = Integer.parseInt(parser.getElementValue("vpLocOnVSx"));
    this.vpLocOnVSy = Integer.parseInt(parser.getElementValue("vpLocOnVSy"));
    this.zoom = Double.parseDouble(parser.getElementValue("zoom"));
  }
  /****************************************************************************
  * getWidth.
  ****************************************************************************/
  public int getWidth () {
    return width;
  }
  /****************************************************************************
  * getHeight.
  ****************************************************************************/
  public int getHeight () {
    return height;
  }
  /****************************************************************************
  * getViewport.
  ****************************************************************************/
  public Rectangle getViewport () {
    return new Rectangle(MARGIN,MARGIN,vpWidth,vpHeight);
  }
  /****************************************************************************
  * isZoomable.
  ****************************************************************************/
  public boolean isZoomable () {
    return true;
  }
  /****************************************************************************
  * calcXY.
  ****************************************************************************/
  private int[] calcXY (double lat, double lon) {
    return new int[] {calcX(lon),calcY(lat)};
  }
  /****************************************************************************
  * calcX.
  * Relative to map.
  ****************************************************************************/
  private int calcX (double lon) {
    return MARGIN + (((int) Math.round(vScaleX*(lon+180.0))) - vpLocOnVSx);
  }
  /****************************************************************************
  * calcY.
  * Relative to map.
  ****************************************************************************/
  private int calcY (double lat) {
    return MARGIN + (((int) Math.round(vScaleY*(90.0-lat))) - vpLocOnVSy);
  }
  /****************************************************************************
  * getLatLon.
  ****************************************************************************/
  public double[] getLatLon (int x, int y) {
    double lat = Util.toRange(-90.0,90.0-(((y-MARGIN)+vpLocOnVSy)/vScaleY),90.0);
    double lon = Util.toRange(-180.0,(((x-MARGIN)+vpLocOnVSx)/vScaleX)-180.0,180.0);
    return new double[] {lat,lon};
  }
  /****************************************************************************
  * getAreaFilter.
  ****************************************************************************/
  public String getAreaFilter () {
    return "<areaFilter>" + toXml() + "</areaFilter>";
  }
  private String toXml () {
    return "<mercator>" +
	   "<width>" + Integer.toString(width) + "</width>" +
	   "<height>" + Integer.toString(height) + "</height>" +
	   "<vsWidth>" + Integer.toString(vsWidth) + "</vsWidth>" +
	   "<vsHeight>" + Integer.toString(vsHeight) + "</vsHeight>" +
	   "<vpWidth>" + Integer.toString(vpWidth) + "</vpWidth>" +
	   "<vpHeight>" + Integer.toString(vpHeight) + "</vpHeight>" +
	   "<vScaleX>" + Double.toString(vScaleX) + "</vScaleX>" +
	   "<vScaleY>" + Double.toString(vScaleY) + "</vScaleY>" +
	   "<vpLocOnVSx>" + Integer.toString(vpLocOnVSx) + "</vpLocOnVSx>" +
	   "<vpLocOnVSy>" + Integer.toString(vpLocOnVSy) + "</vpLocOnVSy>" +
	   "<zoom>" + Double.toString(zoom) + "</zoom>" +
	   "</mercator>";
  }
  /****************************************************************************
  * isVisible.
  ****************************************************************************/
  public boolean isVisible (double lat, double lon) {
    int x = calcX(lon);
    int y = calcY(lat);
    return Util.inRange(MARGIN,x,MARGIN+vpWidth-1) &&
    	   Util.inRange(MARGIN,y,MARGIN+vpHeight-1);
  }
  public boolean isVisible (double minLat, double maxLat, double minLon, double maxLon) {
    int minX = calcX(minLon);
    int maxX = calcX(maxLon);
    int minY = calcY(maxLat);
    int maxY = calcY(minLat);
    return !(minX > (MARGIN+vpWidth-1) || maxX < MARGIN ||
    	     minY > (MARGIN+vpHeight-1) || maxY < MARGIN);
  }
  public boolean isVisible (Geo.Extents extents) {
    return isVisible(extents.getMinLat(),extents.getMaxLat(),
    		     extents.getMinLon(),extents.getMaxLon());
  }
  public boolean isVisible (GeoBlock geoBlock) {
    int nRows = geoBlock.getRowCount();
    int nCols = geoBlock.getColCount();
    double[][] lats = geoBlock.getLats();
    double[][] lons = geoBlock.getLons();
    for (int i = 0; i <= nCols; i++) {
       if (isVisible(lats[0][i],lons[0][i])) {
       	 return true;
       }
       if (isVisible(lats[nRows][i],lons[nRows][i])) {
       	 return true;
       }
    }
    return false;
  }
  /****************************************************************************
  * resize.
  ****************************************************************************/
  public Rectangle resize (int width, int height) {
    this.width = width;
    this.height = height;
    vCalc();
    /*return*/ zoom(1.0,width/2,height/2);	// Only works if window made smaller.
    return null;
  }
  /****************************************************************************
  * zoom.
  ****************************************************************************/
  public Rectangle zoom (double startStbdLat, double startStbdLon,
		         double startPortLat, double startPortLon,
		         double stopStbdLat, double stopStbdLon,
		         double stopPortLat, double stopPortLon) {
    return zoom(Util.min(calcX(startStbdLon),calcX(startPortLon),calcX(stopStbdLon),calcX(stopPortLon)),
	        Util.min(calcY(startStbdLat),calcY(startPortLat),calcY(stopStbdLat),calcY(stopPortLat)),
	        Util.max(calcX(startStbdLon),calcX(startPortLon),calcX(stopStbdLon),calcX(stopPortLon)),
	        Util.max(calcY(startStbdLat),calcY(startPortLat),calcY(stopStbdLat),calcY(stopPortLat)));
  }
  // x/y relative to map.
  // 0 is upper-left, 1 is lower-right.
  public Rectangle zoom (int x0, int y0, int x1, int y1) {
    return doZoom(x0-MARGIN,y0-MARGIN,x1-MARGIN,y1-MARGIN);
  }
  // x/y relative to map.
  public Rectangle zoom (double factor, int x, int y) {
    return doZoom(factor,x-MARGIN,y-MARGIN);
  }
  // x/y relative to viewport.
  private Rectangle doZoom (int x0, int y0, int x1, int y1) {
    double xFactor = (width-MARGIN-MARGIN) / ((double)(x1-x0));
    double yFactor = (height-MARGIN-MARGIN) / ((double)(y1-y0));
    return doZoom(Util.min(xFactor,yFactor),(x0+x1)/2,(y0+y1)/2);
  }
  // x/y relative to viewport.
  private Rectangle doZoom (double factor, int x, int y) {
    
    double xPctUL = (vpLocOnVSx-MARGIN) / ((double)(vsWidth-1));
    double yPctUL = (vpLocOnVSy-MARGIN) / ((double)(vsHeight-1));
    double xPctLR = ((vpLocOnVSx-MARGIN)+(width-1)) / ((double)(vsWidth-1));
    double yPctLR = ((vpLocOnVSy-MARGIN)+(height-1)) / ((double)(vsHeight-1));
    
    if (zoom * factor <= 1.0) {
      zoom = 1.0;
      vCalc();
      vpLocOnVSx = 0;
      vpLocOnVSy = 0;
      return null;
    }
    double xPct = (vpLocOnVSx+x) / ((double)(vsWidth-1));
    double yPct = (vpLocOnVSy+y) / ((double)(vsHeight-1));
    zoom *= factor;
    vCalc();
    vpLocOnVSx = ((int)Math.round(xPct*vsWidth)) - (vpWidth/2);
    vpLocOnVSy = ((int)Math.round(yPct*vsHeight)) - (vpHeight/2);
    if (vpLocOnVSx < 0) {
      vpLocOnVSx = 0;
    } else {
      if (vpLocOnVSx + vpWidth > vsWidth-1) {
       vpLocOnVSx = (vsWidth-1) - vpWidth;
      }
    }
    if (vpLocOnVSy < 0) {
      vpLocOnVSy = 0;
     } else {
       if (vpLocOnVSy + vpHeight > vsHeight-1) {
      	vpLocOnVSy = (vsHeight-1) - vpHeight;
      }
    }
    
    if (factor >= 1.0) {
      int xUL = (int) Math.round(xPctUL*(vsWidth-1));
      int yUL = (int) Math.round(yPctUL*(vsHeight-1));
      int xLR = (int) Math.round(xPctLR*(vsWidth-1));
      int yLR = (int) Math.round(yPctLR*(vsHeight-1));
      int xRel = Math.round(width * (((vpLocOnVSx-MARGIN)-xUL) / ((float)(xLR-xUL))));
      int yRel = Math.round(height * (((vpLocOnVSy-MARGIN)-yUL) / ((float)(yLR-yUL))));
      int widthRel = Math.round(width * (width / ((float)(xLR-xUL))));
      int heightRel = Math.round(height * (height / ((float)(yLR-yUL))));
      return new Rectangle(xRel,yRel,widthRel,heightRel);
    }

    return null;
  }
  /****************************************************************************
  * vCalc.
  ****************************************************************************/
  private void vCalc () {
    if (width-MARGIN-MARGIN > 2*(height-MARGIN-MARGIN)) {
      vpHeight = height-MARGIN-MARGIN;
      vsHeight = (int) (vpHeight * zoom);
      vsWidth = 2 * vsHeight;
      vpWidth = Util.min(vsWidth,width-MARGIN-MARGIN);
    } else {
      vpWidth = width-MARGIN-MARGIN;
      vsWidth = (int) (vpWidth * zoom);
      vsHeight = vsWidth / 2;
      vpHeight = Util.min(vsHeight,height-MARGIN-MARGIN);
    }
    vScaleY = (vsHeight-1) / 180.0;
    vScaleX = (vsWidth-1) / 360.0;
  }
  /****************************************************************************
  * drawImage.
  ****************************************************************************/
  public void drawImage (Graphics g, Image image) {
    g.drawImage(image,0,0,null);
  }
  /****************************************************************************
  * drawText.
  ****************************************************************************/
  public void drawText (Graphics g, double lat, double lon, String text) {
    int[] pt = calcXY(lat,lon);
    FontMetrics fm = g.getFontMetrics();
    int x = pt[X] - (fm.stringWidth(text)/2);
    int y = pt[Y] + (fm.getAscent()/2);
    g.drawString(text,x,y);
  }
  /****************************************************************************
  * drawPoint.
  ****************************************************************************/
  public void drawPoint (Graphics g, double lat, double lon, int size) {
    int[] pt = calcXY(lat,lon);
    g.fillRect(pt[X]-(size/2),pt[Y]-(size/2),size,size);
  }
  /****************************************************************************
  * drawLine.
  ****************************************************************************/
  public void drawLine (Graphics g, double lat0, double lon0, double lat1, double lon1) {
    int[] pt0 = calcXY(lat0,lon0);
    int[] pt1 = calcXY(lat1,lon1);
    g.drawLine(pt0[X],pt0[Y],pt1[X],pt1[Y]);
  }
  /****************************************************************************
  * fillPoly.
  ****************************************************************************/
  public void fillPoly (Graphics g, double lat0, double lon0, double lat1,
			double lon1, double lat2, double lon2, double lat3,
			double lon3 /*, Color color*/) {
    int[] x = new int[4], y = new int[4];
    int[] pt = calcXY(lat0,lon0);
    x[0] = pt[X];
    y[0] = pt[Y];
    pt = calcXY(lat1,lon1);
    x[1] = pt[X];
    y[1] = pt[Y];
    pt = calcXY(lat2,lon2);
    x[2] = pt[X];
    y[2] = pt[Y];
    pt = calcXY(lat3,lon3);
    x[3] = pt[X];
    y[3] = pt[Y];
    // g.setColor(color);
    // Between +-180, draw image by two separated polygons.
    if (Math.abs(x[0]-x[1])>vpWidth*0.8 ||
        Math.abs(x[0]-x[2])>vpWidth*0.8 ||
        Math.abs(x[0]-x[3])>vpWidth*0.8 ||
        Math.abs(x[1]-x[2])>vpWidth*0.8 ||
        Math.abs(x[1]-x[3])>vpWidth*0.8 ||
        Math.abs(x[2]-x[3])>vpWidth*0.8) 
    { 
      Polygon poly=new Polygon(x,y,4);
      Rectangle rect=poly.getBounds();

      g.fillRect(MARGIN-1, rect.y-1, rect.x-MARGIN+2, rect.height+2);
      g.drawRect(MARGIN-1, rect.y-1, rect.x-MARGIN+2, rect.height+2);

      g.fillRect(rect.x+rect.width-1, rect.y-1, MARGIN+vpWidth-rect.x+1, rect.height+2);
      g.drawRect(rect.x+rect.width-1, rect.y-1, MARGIN+vpWidth-rect.x+1, rect.height+2);
    }
    else
    { 
      g.fillPolygon(x,y,4);
      g.drawPolygon(x,y,4);
    }
  }
  /****************************************************************************
  * drawGrid.
  ****************************************************************************/
  public void drawGrid (Graphics g) {
    // Calculate lat/lon extents.
    double[] upperLeft = getLatLon(MARGIN,MARGIN);
    double[] lowerRight = getLatLon(MARGIN+vpWidth-1,MARGIN+vpHeight-1);
    // Calculate grid interval.
    int gridInterval = 0;
    double lonRange = lowerRight[LON] - upperLeft[LON];
    for (int i = 0; i < gridIntervalChoices.length; i++) {
       if (lonRange / gridIntervalChoices[i] < 10) {
       	 gridInterval = gridIntervalChoices[i];
       	 break;
       }
    }
    // Calculate left-most longitude line.
    int leftLon = 0;
    while (leftLon > -180) {
      leftLon -= gridInterval;
    }
    while (leftLon <= upperLeft[LON]) {
      leftLon += gridInterval;
    }
    int rightLon = leftLon;
    while (rightLon + gridInterval < lowerRight[LON]) {
      rightLon += gridInterval;
    }
    // Calculate bottom-most latitude line.
    int botLat = 0;
    while (botLat > -90) {
      botLat -= gridInterval;
    }
    while (botLat <= lowerRight[LAT]) {
      botLat += gridInterval;
    }
    int topLat = botLat;
    while (topLat + gridInterval < upperLeft[LAT]) {
      topLat += gridInterval;
    }
    FontMetrics metrics = g.getFontMetrics(g.getFont());
    int maxLatLabelWidth = metrics.stringWidth("00");
    int maxLabelHeight = metrics.getMaxAscent();
    int leftX = calcX(upperLeft[LON]);
    int rightX = calcX(lowerRight[LON]);
    int topY = calcY(upperLeft[LAT]);
    int botY = calcY(lowerRight[LAT]);
    for (int lat = botLat; lat <= topLat; lat += gridInterval) {
       String label = Integer.toString(Math.abs(lat));
       int labelWidth = metrics.stringWidth(label);
       int y = calcY(lat);
       g.setColor(Color.gray.darker());
       g.drawLine(leftX,y,rightX,y);
       g.setColor(Color.gray.brighter());
       g.drawString(label,MARGIN-maxLatLabelWidth+((maxLatLabelWidth-labelWidth)/2)-2,y+(maxLabelHeight/2));
       g.drawString(label,MARGIN+vpWidth+((maxLatLabelWidth-labelWidth)/2)+2,y+(maxLabelHeight/2));
    }
    for (int lon = leftLon; lon <= rightLon; lon += gridInterval) {
       String label = Integer.toString(Math.abs(lon));
       int labelWidth = metrics.stringWidth(label);
       int x = calcX(lon);
       g.setColor(Color.gray.darker());
       g.drawLine(x,topY,x,botY);
       g.setColor(Color.gray.brighter());
       g.drawString(label,x-(labelWidth/2),MARGIN-3);
       g.drawString(label,x-(labelWidth/2),MARGIN+vpHeight+maxLabelHeight);
    }
  }
  /****************************************************************************
  * drawBorder.
  ****************************************************************************/
  public void drawBorder (Graphics g) {
    g.drawRect(MARGIN,MARGIN,vpWidth-1,vpHeight-1);
  }
}
