/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*  26-Aug-05, J Love, GST	Original version.
*   3-May-06, J Love, GST	Geolocation, satellite names, etc.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.util;
import gov.nasa.gsfc.drl.geo.sgp4.*;
import java.awt.*;
import java.util.*;

public class OrbitsOverlay {
  private Orbit[] orbits;
  private LinkedList<LinkedList<OrbitHandler>> lists = new LinkedList<LinkedList<OrbitHandler>>();
  private MapParms mapParms;
  private OrbitsOverlayListener listener;
  private boolean stop = false;
  private static final long TRACK_INTERVAL_MS = 5000;
  private static final int NUM_ORBITS = 3;
  /****************************************************************************
  * OrbitsOverlay.
  ****************************************************************************/
  public OrbitsOverlay (Object[][] orbitParms, MapParms mapParms, OrbitsOverlayListener listener) throws Exception {
    this.mapParms = mapParms;
    this.listener = listener;
    orbits = new Orbit[orbitParms.length];
    long now = System.currentTimeMillis();
    for (int i = 0; i < orbitParms.length; i++) {
       orbits[i] = new Orbit(new TLE((String)orbitParms[i][1],
				     (String)orbitParms[i][2],
				     (String)orbitParms[i][3]));
       LinkedList<OrbitHandler> handlers = new LinkedList<OrbitHandler>();
       setOrbits(handlers,(String)orbitParms[i][1],(Color)orbitParms[i][0],orbits[i],now);
       lists.add(handlers);
    }
    Util.startThread(new UpdateOrbits());
  }
  /****************************************************************************
  * setOrbits.
  ****************************************************************************/
  private void setOrbits (LinkedList<OrbitHandler> handlers, String satelliteName, Color color, Orbit orbit, long now) {
    while (handlers.size() < NUM_ORBITS) {
      if (handlers.size() > 0) {
        OrbitHandler prevHandler = (OrbitHandler) handlers.getLast();
        long startTime = prevHandler.getEndTime() + TRACK_INTERVAL_MS;
        handlers.addLast(new OrbitHandler(satelliteName,color,orbit,startTime,TRACK_INTERVAL_MS));
      } else {
        handlers.addLast(new OrbitHandler(satelliteName,color,orbit,now,TRACK_INTERVAL_MS));
      }
    }
  }
  /****************************************************************************
  * UpdateOrbits.
  ****************************************************************************/
  private class UpdateOrbits implements Runnable {
    public void run () {
      while (true) {
      	if (stop) {
      	  return;
      	}
        long now = System.currentTimeMillis();
        for (int i = 0; i < lists.size(); i++) {
           updateOrbits(now,orbits[i],lists.get(i));
        }
        listener.orbitsUpdated();
        Util.sleep(TRACK_INTERVAL_MS);
      }
    }
  }
  /****************************************************************************
  * stop.
  ****************************************************************************/
  public void stop () {
    stop = true;
  }
  /****************************************************************************
  * updateOrbits.
  ****************************************************************************/
  private synchronized void updateOrbits (long now, Orbit orbit, LinkedList<OrbitHandler> handlers) {
    OrbitHandler first = (OrbitHandler) handlers.getFirst();
    if (first.update(now)) {
      handlers.removeFirst();
      OrbitHandler last = (OrbitHandler) handlers.getLast();
      handlers.addLast(new OrbitHandler(last.getSatelliteName(),last.getColor(),
					orbit,last.getEndTime()+TRACK_INTERVAL_MS,TRACK_INTERVAL_MS));
    }
  }
  /****************************************************************************
  * draw.
  ****************************************************************************/
  public synchronized void draw (Graphics g) {
    for (int i = 0; i < lists.size(); i++) {
       boolean current = true;
       for (OrbitHandler handler : lists.get(i)) {
          drawOrbit(g,handler,current);
          current = false;
       }
    }
  }
  /****************************************************************************
  * drawOrbit.
  ****************************************************************************/
  public void drawOrbit (Graphics g, OrbitHandler orbitHandler, boolean current) {
    g.setClip(mapParms.getViewport());
    g.setColor(orbitHandler.getColor().darker());
    Iterator it = orbitHandler.getTrack().iterator();
    while (it.hasNext()) {
      OrbitLocation loc = (OrbitLocation) it.next();
      mapParms.drawPoint(g,loc.getLat(),loc.getLon(),1);
    }
    if (current) {
      OrbitLocation loc = (OrbitLocation) orbitHandler.getTrack().getFirst();
      // mapParms.drawPoint(g,loc.getLat(),loc.getLon(),5);
      g.setColor(orbitHandler.getColor());
      mapParms.drawText(g,loc.getLat(),loc.getLon(),orbitHandler.getSatelliteName());
    }
    g.setClip(null);
  }
}
