/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*  20-Sep-06, J Love, GST	Original version.
*  10-Jan-06, J Love, GST	Added DEBUG option.
*   1-Feb-07, J Love, GST	Daemon threads for Windows service (this is
*   				used by Images which runs as a service).
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.util;
import gov.nasa.gsfc.aisb.simulcast.server.Server;
import java.net.*;
import java.io.*;
import java.util.*;

public class ServerHandler implements Runnable {
  private ServerHandlerListener listener = null;
  private HostPort serverHostPort = null;
  private HostPort nextServerHostPort = null;
  private Socket s = null;
  private DataInputStream in = null;
  private DataOutputStream out = null;
  private java.util.Timer timer;
  private boolean disconnect = false;
  private static final long TIMEOUT_INTERVAL_MS = (5 * Util.SERVER_READY_INTERVAL_SEC) * 1000;
  private static final boolean DEBUG = false;
  private long LastTime = 0;
  private long CurrentTime = 0;
  /****************************************************************************
  * ServerHandler.
  ****************************************************************************/
  public ServerHandler (HostPort serverHostPort) {
    this.serverHostPort = serverHostPort;
  }
  public ServerHandler (HostPort serverHostPort, ServerHandlerListener listener) {
    this.serverHostPort = serverHostPort;
    this.listener = listener;
  }
  /****************************************************************************
  * setListener.
  ****************************************************************************/
  public void setListener (ServerHandlerListener listener) {
    this.listener = listener;
  }
  /****************************************************************************
  * getHostPort.
  ****************************************************************************/
  public HostPort getHostPort () {
    return serverHostPort;
  }
  /****************************************************************************
  * start.
  ****************************************************************************/
  public void start () {
    Util.startThread(this,true);
  }
  /****************************************************************************
  * connect.
  ****************************************************************************/
  public void connect (HostPort nextServerHostPort) {
    this.nextServerHostPort = nextServerHostPort;
    disconnect = (serverHostPort != null);
    shutdown();
  }
  /****************************************************************************
  * disconnect.
  ****************************************************************************/
  public void disconnect () {
    disconnect = (serverHostPort != null);
    shutdown();
  }
  /****************************************************************************
  * run.
  ****************************************************************************/
  public void run () {
    while (true) {
      try {
      	if (serverHostPort != null) {
	  doConnection();
      	} else {
	  Util.sleep(0.1);
      	}
      } catch (Exception e) {
	if (disconnect) {
	  listener.disconnected();
	  serverHostPort = null;
	  disconnect = false;
	} else {
          // Every logDisplayTime m to display log.
	  CurrentTime = System.currentTimeMillis();
	  if ((CurrentTime-LastTime)>(Server.logDisplayTime*60000))
	  {
	    LastTime=CurrentTime;
	    listener.error(Util.encodeException(e),s != null);
	  }
	}
	Util.sleep(1.0);
      } finally {
      	try {timer.cancel();} catch (Exception ex) {}
      	shutdown();
      }
      if (nextServerHostPort != null) {
	serverHostPort = nextServerHostPort;
	nextServerHostPort = null;
      }
    }
  }
  /****************************************************************************
  * doConnection.
  ****************************************************************************/
  private void doConnection () throws Exception {
    //Noisy, don't use.
    //listener.info("Connecting to server " + serverHostPort.toString() + "...");
    Util.sleep(0.25);
    s = new Socket(serverHostPort.getHost(),serverHostPort.getPort());
    out = new DataOutputStream(new BufferedOutputStream(s.getOutputStream()));
    in = new DataInputStream(new BufferedInputStream(s.getInputStream()));
    startTimer();
    listener.connected(serverHostPort.toString());
    while (true) {
      String msg = in.readUTF();
      if (DEBUG) {
	System.out.println("ServerHandler.doConnection, msg: " +
			   (msg.length() > 1000 ? (msg.substring(0,1000)+"...") : msg));
      }
      restartTimer();
      listener.received(msg);
    }
  }
  /****************************************************************************
  * send.
  ****************************************************************************/
  public synchronized void send (String msg) {
    try {
      out.writeUTF(msg);
      out.flush();
      if (DEBUG) {
	System.out.println("ServerHandler.send, msg: " + msg);
      }
    } catch (Exception e) {
      shutdown();
    }
  }
  /****************************************************************************
  * restartTimer.
  ****************************************************************************/
  private void restartTimer () {
    timer.cancel();
    startTimer();
  }
  /****************************************************************************
  * startTimer.
  ****************************************************************************/
  private void startTimer () {
    timer = new java.util.Timer();
    timer.schedule(new TimeOut(),TIMEOUT_INTERVAL_MS);
  }
  /****************************************************************************
  * TimeOut.
  ****************************************************************************/
  private class TimeOut extends TimerTask {
    public void run () {
      shutdown();
    }
  }
  /****************************************************************************
  * shutdown.
  ****************************************************************************/
  private void shutdown () {
    // try {s.shutdownInput();} catch (Exception ex) {}
    // try {s.shutdownOutput();} catch (Exception ex) {}
    try {in.close();} catch (Exception ex) {}
    try {out.close();} catch (Exception ex) {}
    try {s.close();} catch (Exception ex) {}
    s = null;
    in = null;
    out = null;
  }
}
