/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*  13-Dec-05, J Love, GST	Original version.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.util;
import java.io.*;
import java.awt.*;
import java.awt.image.*;

public class ShpOverlay extends BufferedImage {
  private MapParms mapParms;
  private boolean abort = false;
  private Graphics g;
  private static final int POINT = 1;
  private static final int POLYLINE = 3;
  private static final int POLYGON = 5;
  private static final int POLYGON_M = 25;
  /****************************************************************************
  * ShpOverlay.
  ****************************************************************************/
  public ShpOverlay (MapParms mapParms) {
    super(mapParms.getWidth(),mapParms.getHeight(),TYPE_INT_ARGB);
    this.mapParms = mapParms;
    g = getGraphics();
    g.setClip(mapParms.getViewport());
    // g.setColor(new Color(0,0,0,0));
    // g.fillRect(0,0,mapParms.getWidth(),mapParms.getHeight());
  }
  /****************************************************************************
  * addShpFile.
  ****************************************************************************/
  public void addShpFile (String shpFile, Color color) throws Exception {
    g.setColor(color);
    BufferedInputStream in = null;
    try {
      in = new BufferedInputStream(Util.getInputStream(shpFile));
      readBigInt(in);
      readBigInt(in);
      readBigInt(in);
      readBigInt(in);
      readBigInt(in);
      readBigInt(in);
      readBigInt(in);
      readLittleInt(in);
      readLittleInt(in);
      readLittleDouble(in);
      readLittleDouble(in);
      readLittleDouble(in);
      readLittleDouble(in);
      readLittleDouble(in);
      readLittleDouble(in);
      readLittleDouble(in);
      readLittleDouble(in);
      while (true) {
      	if (abort) {
      	  throw new InterruptedException("aborted");
      	}
	readBigInt(in);
	readBigInt(in);
	int shapeType = readLittleInt(in);
	switch (shapeType) {
	  case POINT: {
	    double lon = readLittleDouble(in);
	    double lat = readLittleDouble(in);
	    if (mapParms.isVisible(lat,lon)) {
	      mapParms.drawPoint(g,lat,lon,1);
	    }
	    break;
	  }
	  case POLYLINE:
	  case POLYGON:
	  case POLYGON_M: {
	    double xMin = readLittleDouble(in);
	    double yMin = readLittleDouble(in);
	    double xMax = readLittleDouble(in);
	    double yMax = readLittleDouble(in);
	    int numParts = readLittleInt(in);
	    int numPoints = readLittleInt(in);
	    if (mapParms.isVisible(yMin,yMax,xMin,xMax)) {
	      int[] partOffsets = new int[numParts+1];
	      for (int i = 0; i < numParts; i++) {
	         partOffsets[i] = readLittleInt(in);
	      }
	      partOffsets[numParts] = numPoints;
	      for (int i = 0; i < numParts; i++) {
	         int count = partOffsets[i+1] - partOffsets[i];
	         double lon0 = readLittleDouble(in);
	         double lat0 = readLittleDouble(in);
	         for (int j = 1; j < count; j++) {
	            double lon1 = readLittleDouble(in);
	            double lat1 = readLittleDouble(in);
	            mapParms.drawLine(g,lat0,lon0,lat1,lon1);
	            lat0 = lat1;
	            lon0 = lon1;
	         }
	      }
	      if (shapeType == POLYGON_M) {
	        readLittleDouble(in);			// Mmin
	        readLittleDouble(in);			// Mmax
	        for (int i = 0; i < numPoints; i++) {
	           readLittleDouble(in);		// Marray value
	        }
	      }
	    } else {
	      skip(in,4*numParts);
	      skip(in,2*8*numPoints);
	      if (shapeType == POLYGON_M) {
	      	skip(in,8);
	      	skip(in,8);
	      	skip(in,8*numPoints);
	      }
	    }
	    break;
	  }
	  default:
	    throw new Exception("unknown shape type: " + shapeType);
	}
      }
    } catch (EOFException e) {
      // This is expected - all other exceptions are not.
    } finally {
      try {in.close();} catch (Exception e) {}
    }
  }
  /****************************************************************************
  * abort.
  ****************************************************************************/
  public void abort () {
    abort = true;
  }
  /****************************************************************************
  * readBigInt.
  ****************************************************************************/
  private static int readBigInt (InputStream in) throws Exception {
    byte[] bytes = new byte[4];
    for (int i = 0; i < 4; i++) {
       int b = in.read();
       if (b == -1) throw new EOFException();
       bytes[i] = (byte) b;
    }
    DataInputStream dis = new DataInputStream(new ByteArrayInputStream(bytes));
    int value = dis.readInt();
    dis.close();
    return value;
  }
  /****************************************************************************
  * readLittleInt.
  ****************************************************************************/
  private static int readLittleInt (InputStream in) throws Exception {
    byte[] bytes = new byte[4];
    for (int i = 0; i < 4; i++) {
       int b = in.read();
       if (b == -1) throw new EOFException();
       bytes[3-i] = (byte) b;
    }
    DataInputStream dis = new DataInputStream(new ByteArrayInputStream(bytes));
    int value = dis.readInt();
    dis.close();
    return value;
  }
  /****************************************************************************
  * readLittleDouble.
  ****************************************************************************/
  private static double readLittleDouble (InputStream in) throws Exception {
    byte[] bytes = new byte[8];
    for (int i = 0; i < 8; i++) {
       int b = in.read();
       if (b == -1) throw new EOFException();
       bytes[7-i] = (byte) b;
    }
    DataInputStream dis = new DataInputStream(new ByteArrayInputStream(bytes));
    double value = dis.readDouble();
    dis.close();
    return value;
  }
  /****************************************************************************
  * skip.
  ****************************************************************************/
  private static void skip (InputStream in, int count) throws Exception {
    while (count > 0) {
      count -= in.skip(count);
    }
  }
}
