/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/UTIL
* 
*  History:
* 
*  21-Jan-05, J Love, GST	Original version.
*  25-Feb-05, J Love, GST	Ascending/descending determination method.
*  17-Mar-05, J Love, GST	Ground station argument (tmp).
*  22-Mar-05, J Love, GST	NSLS logging.
*  28-Dec-05, J Love, GST	Geolocation, etc.
*   5-Dec-06, J Love, GST	Added Viewer, etc.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.util;
import java.io.*;

public class Simulcast {
  public static final String VERSION = "4.2";
  public static final int UNKNOWN = -1;
  public static final int AQUA_PLATFORM_CODE = 154;
  public static final int TERRA_PLATFORM_CODE = 42;
  public static final int NPP_PLATFORM_CODE = 0;
  public static final int MODIS_INSTRUMENT_CODE = 64;
  public static final int VIIRS_INSTRUMENT_CODE = 0;
  public static final int DAY_PASS_CODE = 1;
  public static final int NIGHT_PASS_CODE = 2;
  public static final int ASCENDING_PASS_CODE = 1;
  public static final int DESCENDING_PASS_CODE = 2;
  public static final int BLOCK_CODE_FIELD_SIZE = 4;
  // Block codes.
  public static final int HEADER_BLOCK_CODE = 1;
  public static final int SCAN_BLOCK_CODE = 2;
  public static final int GEO_BLOCK_CODE = 3;
  public static final int BOWED_DATA_BLOCK_CODE = 4;
  public static final int DEBOWED_DATA_BLOCK_CODE = 5;
  public static final int FOOTER_BLOCK_CODE = 6;
  public static final int ERROR_BLOCK_CODE = 7;
  public static final int XML_BLOCK_CODE = 8;
  /****************************************************************************
  * readBlock.
  ****************************************************************************/
  public static Object readBlock (DataInput in) throws Exception {
    int blockCode = in.readInt();
    return readBlockBody(blockCode,in);
  }
  public static Object readBlock (DataInput in, ClassList blocks) throws Exception {
    Block block = null;
    int blockCode = in.readInt();
    switch (blockCode) {
      case HEADER_BLOCK_CODE: {
      	block = (Block) blocks.get(HeaderBlock.class);
      	break;
      }
      case SCAN_BLOCK_CODE: {
      	block = (Block) blocks.get(ScanBlock.class);
      	break;
      }
      case GEO_BLOCK_CODE: {
      	block = (Block) blocks.get(GeoBlock.class);
      	break;
      }
      case BOWED_DATA_BLOCK_CODE: {
      	block = (Block) blocks.get(BowedDataBlock.class);
      	break;
      }
      case DEBOWED_DATA_BLOCK_CODE: {
      	block = (Block) blocks.get(DebowedDataBlock.class);
      	break;
      }
      case FOOTER_BLOCK_CODE: {
      	block = (Block) blocks.get(FooterBlock.class);
      	break;
      }
      case ERROR_BLOCK_CODE: {
      	block = (Block) blocks.get(ErrorBlock.class);
      	break;
      }
      case XML_BLOCK_CODE: {
      	block = (Block) blocks.get(XmlBlock.class);
      	break;
      }
      default: {
	throw new Exception("unknown block code: " + blockCode);
      }
    }
    if (block != null) {
      // System.out.println("reusing");
      block.read(in);
      return block;
    }
    block = (Block) readBlockBody(blockCode,in);
    // System.out.println("new");
    blocks.addLast(block);
    return block;
  }
  /****************************************************************************
  * readBlockBody.
  ****************************************************************************/
  public static Object readBlockBody (int blockCode, DataInput in) throws Exception {
    switch (blockCode) {
      case HEADER_BLOCK_CODE: {
      	return new HeaderBlock(in);
      }
      case SCAN_BLOCK_CODE: {
      	return new ScanBlock(in);
      }
      case GEO_BLOCK_CODE: {
      	return new GeoBlock(in);
      }
      case BOWED_DATA_BLOCK_CODE: {
      	return new BowedDataBlock(in);
      }
      case DEBOWED_DATA_BLOCK_CODE: {
      	return new DebowedDataBlock(in);
      }
      case FOOTER_BLOCK_CODE: {
      	return new FooterBlock(in);
      }
      case ERROR_BLOCK_CODE: {
      	return new ErrorBlock(in);
      }
      case XML_BLOCK_CODE: {
      	return new XmlBlock(in);
      }
      default: {
	throw new Exception("unknown block code: " + blockCode);
      }
    }
  }
  /****************************************************************************
  * skipBlock.
  ****************************************************************************/
  public static void skipBlock (DataInput in) throws Exception {
    int blockCode = in.readInt();
    switch (blockCode) {
      case HEADER_BLOCK_CODE: {
      	throw new Exception("not a skipable block");
      }
      case SCAN_BLOCK_CODE: {
      	throw new Exception("not a skipable block");
      }
      case GEO_BLOCK_CODE: {
      	GeoBlock.skip(in);
      	break;
      }
      case BOWED_DATA_BLOCK_CODE: {
      	BowedDataBlock.skip(in);
      	break;
      }
      case DEBOWED_DATA_BLOCK_CODE: {
      	DebowedDataBlock.skip(in);
      	break;
      }
      case FOOTER_BLOCK_CODE: {
      	throw new Exception("not a skipable block");
      }
      case ERROR_BLOCK_CODE: {
      	throw new Exception("not a skipable block");
      }
      case XML_BLOCK_CODE: {
      	throw new Exception("not a skipable block");
      }
      default: {
	throw new Exception("unknown block code: " + blockCode);
      }
    }
  }
  /****************************************************************************
  * getPlatformName.
  ****************************************************************************/
  public static String getPlatformName (int platformCode) {
    switch (platformCode) {
      case AQUA_PLATFORM_CODE: {
        return "AQUA";
      }
      case TERRA_PLATFORM_CODE: {
        return "TERRA";
      }
      case NPP_PLATFORM_CODE: {
        return "NPP";
      }
      default: {
      	return " ";
      }
    }
  }
  /****************************************************************************
  * getInstrumentName.
  ****************************************************************************/
  public static String getInstrumentName (int instrumentCode) {
    switch (instrumentCode) {
      case MODIS_INSTRUMENT_CODE: {
        return "MODIS";
      }
      case VIIRS_INSTRUMENT_CODE: {
        return "VIIRS";
      }
      default: {
      	return " ";
      }
    }
  }
  /****************************************************************************
  * getPassDirection.
  ****************************************************************************/
  public static String getPassDirection (int directionCode) {
    switch (directionCode) {
      case ASCENDING_PASS_CODE: {
        return "Ascending";
      }
      case DESCENDING_PASS_CODE: {
        return "Descending";
      }
      default: {
      	return " ";
      }
    }
  }
  /****************************************************************************
  * getPassSolar.
  ****************************************************************************/
  public static String getPassSolar (int solarCode) {
    switch (solarCode) {
      case DAY_PASS_CODE: {
        return "Day";
      }
      case NIGHT_PASS_CODE: {
        return "Night";
      }
      default: {
      	return " ";
      }
    }
  }
  /****************************************************************************
  * getPlatformCode.
  ****************************************************************************/
  public static int getPlatformCode (String platformName) throws Exception {
    if (platformName.toLowerCase().equals("aqua")) {
      return AQUA_PLATFORM_CODE;
    }
    if (platformName.toLowerCase().equals("terra")) {
      return TERRA_PLATFORM_CODE;
    }
    throw new Exception("unknown platform name");
  }
}
