/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/UTIL
*
*  History:
*
*  13-Dec-05, J Love, GST	Original version.
*  15-Aug-06, J Love, GST	Drawing of text.  Zooming rectangle.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.util;
import gov.nasa.gsfc.drl.geo.*;
import java.awt.*;
import java.awt.geom.*;

public class SinglePoleMapParms implements MapParms {
  private boolean north;
  private int width;		// Full width including margins.
  private int height;
  private int vpWidth;		// Width of viewport.
  private int vpHeight;
  private int vsWidth;		// Width of virtual space.
  private int vsHeight;
  private int vsRadius;		// Radius of virtual space.
  private int vsOriginX;	// Origin (pole) on virtual space.
  private int vsOriginY;
  private int vpLocOnVsX;	// Location of viewport's upperleft corner on virtual space.
  private int vpLocOnVsY;
  private double zoom;
  private static final int X = 0;
  private static final int Y = 1;
  private static final int MARGIN = 10;
  /****************************************************************************
  * SinglePoleMapParms.
  ****************************************************************************/
  public SinglePoleMapParms (boolean north, int width, int height) {
    this.north = north;
    this.width = width;
    this.height = height;
    zoom = 1.0;
    vCalc();
    vpLocOnVsX = 0;
    vpLocOnVsY = 0;
    center();
  }
  public SinglePoleMapParms (XmlParser parser) throws Exception {
    this.north = parser.hasElement("north");
    this.width = Integer.parseInt(parser.getElementValue("width"));
    this.height = Integer.parseInt(parser.getElementValue("height"));
    this.vpWidth = Integer.parseInt(parser.getElementValue("vpWidth"));
    this.vpHeight = Integer.parseInt(parser.getElementValue("vpHeight"));
    this.vsWidth = Integer.parseInt(parser.getElementValue("vsWidth"));
    this.vsHeight = Integer.parseInt(parser.getElementValue("vsHeight"));
    this.vsRadius = Integer.parseInt(parser.getElementValue("vsRadius"));
    this.vsOriginX = Integer.parseInt(parser.getElementValue("vsOriginX"));
    this.vsOriginY = Integer.parseInt(parser.getElementValue("vsOriginY"));
    this.vpLocOnVsX = Integer.parseInt(parser.getElementValue("vpLocOnVsX"));
    this.vpLocOnVsY = Integer.parseInt(parser.getElementValue("vpLocOnVsY"));
    this.zoom = Double.parseDouble(parser.getElementValue("zoom"));
  }
  /****************************************************************************
  * getWidth.
  ****************************************************************************/
  public int getWidth () {
    return width;
  }
  /****************************************************************************
  * getHeight.
  ****************************************************************************/
  public int getHeight () {
    return height;
  }
  /****************************************************************************
  * getViewport.
  ****************************************************************************/
  public Rectangle getViewport () {
    return new Rectangle(MARGIN,MARGIN,vpWidth,vpHeight);
  }
  /****************************************************************************
  * isZoomable.
  ****************************************************************************/
  public boolean isZoomable () {
    return true;
  }
  /****************************************************************************
  * calcXY.
  * Relative to map.
  ****************************************************************************/
  private int[] calcXY (double lat, double lon) {
    double d = vsRadius * Util.cos(lat);
    if (north) {
      int x = MARGIN + ((vsOriginX + ((int)Math.round(d*Util.cos(lon)))) - vpLocOnVsX);
      int y = MARGIN + ((vsOriginY - ((int)Math.round(d*Util.sin(lon)))) - vpLocOnVsY);
      return new int[] {x,y};
    } else {
      int x = MARGIN + ((vsOriginX + ((int)Math.round(d*Util.cos(lon)))) - vpLocOnVsX);
      int y = MARGIN + ((vsOriginY + ((int)Math.round(d*Util.sin(lon)))) - vpLocOnVsY);
      return new int[] {x,y};
    }
  }
  /****************************************************************************
  * getLatLon.
  ****************************************************************************/
  public double[] getLatLon (int x, int y) {
    return null;
  }
  /****************************************************************************
  * getAreaFilter.
  ****************************************************************************/
  public String getAreaFilter () {
    return "<areaFilter>" + toXml() + "</areaFilter>";
  }
  private String toXml () {
    return "<singlePole>" +
    	   (north ? "<north/>" : "<south/>") +
	   "<width>" + Integer.toString(width) + "</width>" +
	   "<height>" + Integer.toString(height) + "</height>" +
	   "<vpWidth>" + Integer.toString(vpWidth) + "</vpWidth>" +
	   "<vpHeight>" + Integer.toString(vpHeight) + "</vpHeight>" +
	   "<vsWidth>" + Integer.toString(vsWidth) + "</vsWidth>" +
	   "<vsHeight>" + Integer.toString(vsHeight) + "</vsHeight>" +
	   "<vsRadius>" + Integer.toString(vsRadius) + "</vsRadius>" +
	   "<vsOriginX>" + Integer.toString(vsOriginX) + "</vsOriginX>" +
	   "<vsOriginY>" + Integer.toString(vsOriginY) + "</vsOriginY>" +
	   "<vpLocOnVsX>" + Integer.toString(vpLocOnVsX) + "</vpLocOnVsX>" +
	   "<vpLocOnVsY>" + Integer.toString(vpLocOnVsY) + "</vpLocOnVsY>" +
	   "<zoom>" + Double.toString(zoom) + "</zoom>" +
	   "</singlePole>";
  }
  /****************************************************************************
  * isVisible.
  ****************************************************************************/
  public boolean isVisible (double lat, double lon) {
    int[] pt = calcXY(lat,lon);
    return Util.inRange(MARGIN,pt[X],MARGIN+(vpWidth-1)) &&
           Util.inRange(MARGIN,pt[Y],MARGIN+(vpHeight-1));
  }
  public boolean isVisible (double minLat, double maxLat, double minLon, double maxLon) {
    return true;
  }
  public boolean isVisible (Geo.Extents extents) {
    return isVisible(extents.getMinLat(),extents.getMaxLat(),
    		     extents.getMinLon(),extents.getMaxLon());
  }
  public boolean isVisible (GeoBlock geoBlock) {
    int nRows = geoBlock.getRowCount();
    int nCols = geoBlock.getColCount();
    double[][] lats = geoBlock.getLats();
    double[][] lons = geoBlock.getLons();
    for (int i = 0; i <= nCols; i++) {
       if (isVisible(lats[0][i],lons[0][i])) {
       	 return true;
       }
       if (isVisible(lats[nRows][i],lons[nRows][i])) {
       	 return true;
       }
    }
    return false;
  }
  /****************************************************************************
  * resize.
  ****************************************************************************/
  public Rectangle resize (int width, int height) {
    this.width = width;
    this.height = height;
    vCalc();
    return zoom(1.0,width/2,height/2);
  }
  /****************************************************************************
  * zoom.
  ****************************************************************************/
  public Rectangle zoom (double startStbdLat, double startStbdLon,
		         double startPortLat, double startPortLon,
		         double stopStbdLat, double stopStbdLon,
		         double stopPortLat, double stopPortLon) {
    int[] startStbdXY = calcXY(startStbdLat,startStbdLon);
    int[] startPortXY = calcXY(startPortLat,startPortLon);
    int[] stopStbdXY = calcXY(stopStbdLat,stopStbdLon);
    int[] stopPortXY = calcXY(stopPortLat,stopPortLon);
    return zoom(Util.min(startStbdXY[0],startPortXY[0],stopStbdXY[0],stopPortXY[0]),
                Util.min(startStbdXY[1],startPortXY[1],stopStbdXY[1],stopPortXY[1]),
	        Util.max(startStbdXY[0],startPortXY[0],stopStbdXY[0],stopPortXY[0]),
    	        Util.max(startStbdXY[1],startPortXY[1],stopStbdXY[1],stopPortXY[1]));
  }
  // x/y relative to map.
  public Rectangle zoom (int x0, int y0, int x1, int y1) {
    return doZoom(x0-MARGIN,y0-MARGIN,x1-MARGIN,y1-MARGIN);
  }
  // x/y relative to map.
  public Rectangle zoom (double factor, int x, int y) {
    return doZoom(factor,x-MARGIN,y-MARGIN);
  }
  // x/y relative to viewport.
  private Rectangle doZoom (int x0, int y0, int x1, int y1) {
    double xFactor = vpWidth / ((double)(x1-x0));
    double yFactor = vpHeight / ((double)(y1-y0));
    return doZoom(Util.min(xFactor,yFactor),(x0+x1)/2,(y0+y1)/2);
  }
  // x/y relative to viewport.
  private Rectangle doZoom (double factor, int x, int y) {
    if (zoom * factor <= 1.0) {
      zoom = 1.0;
      vCalc();
      vpLocOnVsX = 0;
      vpLocOnVsY = 0;
    } else {
      double xPct = (vpLocOnVsX+x) / ((double)(vsWidth-1));
      double yPct = (vpLocOnVsY+y) / ((double)(vsHeight-1));
      zoom *= factor;
      vCalc();
      vpLocOnVsX = ((int)Math.round(xPct*vsWidth)) - (vpWidth/2);
      vpLocOnVsY = ((int)Math.round(yPct*vsHeight)) - (vpHeight/2);
    }
    center();
    return null;
  }
  /****************************************************************************
  * vCalc.
  ****************************************************************************/
  private void vCalc () {
    if (width-MARGIN-MARGIN > height-MARGIN-MARGIN) {
      vpHeight = height-MARGIN-MARGIN;
      vsHeight = (int) ((vpHeight-20) * zoom);
      vsRadius = (vsHeight-1) / 2;
      vsWidth = vsHeight;
      vpWidth = width-MARGIN-MARGIN;
    } else {
      vpWidth = width-MARGIN-MARGIN;
      vsWidth = (int) ((vpWidth-20) * zoom);
      vsRadius = (vsWidth-1) / 2;
      vsHeight = vsWidth;
      vpHeight = height-MARGIN-MARGIN;
    }
    vsOriginX = vsWidth / 2;
    vsOriginY = vsHeight / 2;
  }
  /****************************************************************************
  * center.
  ****************************************************************************/
  private void center () {
    if (vsWidth < vpWidth) {
      vpLocOnVsX = (vsWidth - vpWidth) / 2;
    }
    if (vsHeight < vpHeight) {
      vpLocOnVsY = (vsHeight - vpHeight) / 2;
    }
  }
  /****************************************************************************
  * drawImage.
  ****************************************************************************/
  public void drawImage (Graphics g, Image image) {
    g.drawImage(image,0,0,null);
  }
  /****************************************************************************
  * drawText.
  ****************************************************************************/
  public void drawText (Graphics g, double lat, double lon, String text) {
    if (north) {
      if (lat < 0.0) {
      	return;
      }
    } else {
      if (lat > 0.0) {
      	return;
      }
    }
    int[] pt = calcXY(lat,lon);
    FontMetrics fm = g.getFontMetrics();
    int x = pt[X] - (fm.stringWidth(text)/2);
    int y = pt[Y] + (fm.getAscent()/2);
    g.drawString(text,x,y);
  }
  /****************************************************************************
  * drawPoint.
  ****************************************************************************/
  public void drawPoint (Graphics g, double lat, double lon, int size) {
    if (north) {
      if (lat < 0.0) {
      	return;
      }
    } else {
      if (lat > 0.0) {
      	return;
      }
    }
    int[] pt = calcXY(lat,lon);
    g.fillRect(pt[X]-(size/2),pt[Y]-(size/2),size,size);
  }
  /****************************************************************************
  * drawLine.
  ****************************************************************************/
  public void drawLine (Graphics g, double lat0, double lon0, double lat1, double lon1) {
    if (north) {
      if (lat0 < 0.0 || lat1 < 0.0) {
      	return;
      }
    } else {
      if (lat0 > 0.0 || lat1 > 0.0) {
      	return;
      }
    }
    int[] pt0 = calcXY(lat0,lon0);
    int[] pt1 = calcXY(lat1,lon1);
    g.drawLine(pt0[X],pt0[Y],pt1[X],pt1[Y]);
  }
  /****************************************************************************
  * fillPoly.
  ****************************************************************************/
  public void fillPoly (Graphics g, double lat0, double lon0, double lat1,
      			double lon1, double lat2, double lon2, double lat3,
      			double lon3 /*, Color color*/) {
    if (north) {
      if (lat0 < 0.0 || lat1 < 0.0 || lat2 < 0.0 || lat3 < 0.0) {
      	return;
      }
    } else {
      if (lat0 > 0.0 || lat1 > 0.0 || lat2 > 0.0 || lat3 > 0.0) {
      	return;
      }
    }
    int[] x = new int[4], y = new int[4];
    int[] pt = calcXY(lat0,lon0);
    x[0] = pt[X];
    y[0] = pt[Y];
    pt = calcXY(lat1,lon1);
    x[1] = pt[X];
    y[1] = pt[Y];
    pt = calcXY(lat2,lon2);
    x[2] = pt[X];
    y[2] = pt[Y];
    pt = calcXY(lat3,lon3);
    x[3] = pt[X];
    y[3] = pt[Y];
    // g.setColor(color);
    g.fillPolygon(x,y,4);
    g.drawPolygon(x,y,4);
  }
  /****************************************************************************
  * drawGrid.
  ****************************************************************************/
  public void drawGrid (Graphics g) {
   Graphics2D g2 = (Graphics2D) g;
    double poleLat = (north ? 90.0 : -90.0);
    double eightyLat = (north ? 80.0 : -80.0);
    int[] lats = (north ? new int[] {0,10,20,30,40,50,60,70,80} :
			  new int[] {0,-10,-20,-30,-40,-50,-60,-70,-80});
    double latLabelsLon = 0;
    int maxCount = 0;
    for (double lon = -172.5; lon < 180.0; lon += 15.0) {
       int count = 0;
       for (int i = 0; i < lats.length; i++) {
          if (isVisible(lats[i],lon)) {
            count++;
          }
       }
       if (count > maxCount) {
       	 maxCount = count;
       	 latLabelsLon = lon;
       }
    }
    g.setClip(MARGIN,MARGIN,vpWidth,vpHeight);
    for (int i = 0; i < lats.length; i++) {
       int vR = calcXY(lats[i],0.0)[X] - calcXY(poleLat,0.0)[X];
       g.setColor(Color.gray);
       // g.drawArc(MARGIN+(vsOriginX-vR)-vpLocOnVsX,MARGIN+(vsOriginY-vR)-vpLocOnVsY,(2*vR),(2*vR),0,360);
       g2.draw(new Arc2D.Double(MARGIN+(vsOriginX-vR)-vpLocOnVsX,MARGIN+(vsOriginY-vR)-vpLocOnVsY,(2*vR),(2*vR),0,360,Arc2D.OPEN));
       if (i >= 2) {
         int[] pt0 = calcXY(lats[i],latLabelsLon);
         labelDrawn(g,pt0[X],pt0[Y],Integer.toString(Math.abs(lats[i])));
       }
    }
    for (int lon = -180; lon < 180; lon += 15) {
       String label = Integer.toString(Math.abs(lon));
       if (lon % 45 == 0.0) {
       	 int[] pt0 = calcXY(poleLat,0.0);
         int[] pt1 = calcXY(0.0,lon);
	 g.setColor(Color.gray);
	 // System.out.println(pt0[X] + " " + pt0[Y] + " " + pt1[X] + " " + pt1[Y]);
         // g.drawLine(pt0[X],pt0[Y],pt1[X],pt1[Y]);
         // g2.draw(new Line2D.Double(pt0[X],pt0[Y],pt1[X],pt1[Y]));
	 drawLonLabel(g,label,pt0[X],pt0[Y],pt1[X],pt1[Y]);
       } else {
         int[] pt0 = calcXY(eightyLat,lon);
         int[] pt1 = calcXY(0.0,lon);
	 g.setColor(Color.gray);
	 // System.out.println(pt0[X] + " " + pt0[Y] + " " + pt1[X] + " " + pt1[Y]);
         // g.drawLine(pt0[X],pt0[Y],pt1[X],pt1[Y]);
         // g2.draw(new Line2D.Double(pt0[X],pt0[Y],pt1[X],pt1[Y]));
	 drawLonLabel(g,label,pt0[X],pt0[Y],pt1[X],pt1[Y]);
       }
    }
    g.setClip(null);
  }
  private void drawLonLabel (Graphics g, String label, int x0, int y0, int x1, int y1) {
    if (x0 != x1) {
      double slope = (y0-y1) / ((double)(x0-x1));
      double yInt = y1 - (slope*x1);
      if (x0 < x1) {
	int x00 = Util.max(x0,MARGIN);
	int x11 = Util.min(x1,MARGIN+vpWidth-1);
	g.drawLine(x00,(int)Math.round((slope*x00)+yInt),x11,(int)Math.round((slope*x11)+yInt));
        for (int x = x11; x >= x00; x -= 1) {
           int y = (int) Math.round((slope*x)+yInt);
           if (labelDrawn(g,x,y,label)) {
             break;
           }
        }
      } else {
	int x00 = Util.min(x0,MARGIN+vpWidth-1);
	int x11 = Util.max(x1,MARGIN);
	g.drawLine(x00,(int)Math.round((slope*x00)+yInt),x11,(int)Math.round((slope*x11)+yInt));
        for (int x = x11; x <= x00; x += 1) {
           int y = (int) Math.round((slope*x)+yInt);
           if (labelDrawn(g,x,y,label)) {
             break;
           }
        }
      }
    } else {
      if (y0 < y1) {
        g.drawLine(x0,Util.max(y0,MARGIN),x0,Util.min(y1,MARGIN+vpHeight-1));
	for (int y = y1; y >= y0; y--) {
           if (labelDrawn(g,x0,y,label)) {
             break;
           }
	}
      } else {
        g.drawLine(x0,Util.max(y1,MARGIN),x0,Util.min(y0,MARGIN+vpHeight-1));
	for (int y = y1; y <= y0; y++) {
           if (labelDrawn(g,x0,y,label)) {
             break;
           }
	}
      }
    }
  }         
  private boolean labelDrawn (Graphics g, int x, int y, String label) {
    FontMetrics metrics = g.getFontMetrics(g.getFont());
    int labelWidth = metrics.stringWidth(label);
    int labelHeight = metrics.getMaxAscent();
    int upperLeftX = x - (labelWidth/2);
    int upperLeftY = y - (labelHeight/2);
    if (Util.inRange(MARGIN+2,upperLeftX,MARGIN+vpWidth-labelWidth-2) &&
    	Util.inRange(MARGIN+2,upperLeftY,MARGIN+vpHeight-labelHeight-3)) {
      g.setColor(Color.gray.brighter());
      g.drawString(label,upperLeftX,upperLeftY+labelHeight);
      return true;
    }
    return false;
  }
  /****************************************************************************
  * drawBorder.
  ****************************************************************************/
  public void drawBorder (Graphics g) {
    g.drawRect(MARGIN,MARGIN,vpWidth-1,vpHeight-1);
  }
}
