/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/UTIL
*
*  History:
*
*   6-Dec-04, J Love, GST	Original version.
*  22-Mar-05, J Love, GST	NSLS logging.
*   9-Dec-05, J Love, GST	Multiple routes to processors.  Geolocation.
* 				Local log file handling.
*  10-Mar-06, J Love, GST	StdOut logging option.
*  18-Oct-06, J Love, GST	Added readString.
*   5-Dec-06, J Love, GST	More options in initLog.
*  10-Jan-07, J Love, GST	Added decodeTimeInterval.  Dealt with IPv6.
*  31-Jan-07, J Love, GST	Added getFile/setDirectorySeparator.  Changed
*  				initLog to use sc.home property.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.util;
import gov.nasa.gsfc.nisgs.nsls.*;
import java.text.*;
import java.util.*;
import java.io.*;
import java.net.*;

public class Util {
  private static final long ONE_KB = 1024;
  private static final long ONE_MB = 1024 * 1024;
  private static final long ONE_GB = 1024 * 1024 * 1024;
  public static final int MS_PER_SEC = 1000;
  public static final int MS_PER_MIN = 60 * MS_PER_SEC;
  public static final int MS_PER_HOUR = 60 * MS_PER_MIN;
  public static final int MS_PER_DAY = 24 * MS_PER_HOUR;
  private static SimpleDateFormat dateFormat = null;
  public static final DecimalFormat lz2 = new DecimalFormat("00");
  public static final DecimalFormat lz6 = new DecimalFormat("000000");
  public static final DecimalFormat fp1 = new DecimalFormat("0.0");
  public static final DecimalFormat fp2 = new DecimalFormat("0.00");
  public static final DecimalFormat fp3 = new DecimalFormat("0.000");
  public static final String XML_HEADER = "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>";
  public static final char[] alphabet = new char[] {
    'a','b','c','d','e','f','g','h','i','j','k','l','m','n','o','p','q','r',
    's','t','u','v','w','x','y','z'
  };
  public static final int SERVER_READY_INTERVAL_SEC = 60;
  /****************************************************************************
  * getEnv.
  ****************************************************************************/
  public static String getEnv () {
    return "JRE: "+System.getProperty("java.vendor")+" "+System.getProperty("java.version")+", OS: "+System.getProperty("os.name")+" "+System.getProperty("os.arch")+" "+System.getProperty("os.version")+", USER: "+System.getProperty("user.name");
  }
  /****************************************************************************
  * skipBytes.
  ****************************************************************************/
  public static void skipBytes (DataInput in, int nBytes) throws Exception {
    int remaining = nBytes;
    while (remaining > 0) {
      remaining -= in.skipBytes(remaining);
    }
  }
  /****************************************************************************
  * encodeThrowable.
  ****************************************************************************/
  public static String encodeThrowable (Throwable t) {
    String text = t.toString();
    int index = text.indexOf(": ");
    if (index != -1) {
      text = text.substring(index+2);
    }
    return text;
  }
  /****************************************************************************
  * adjustValue.
  ****************************************************************************/
  public static int adjustValue (int value, double contrast, int brightness) {
    return toRange(0,(int)Math.round((value*contrast)+brightness),255);
  }
  /****************************************************************************
  * fill.
  ****************************************************************************/
  public static void fill (byte[][][] array, byte value) {
    for (int x0 = 0; x0 < array.length; x0++) {
       for (int x1 = 0; x1 < array[x0].length; x1++) {
       	  Arrays.fill(array[x0][x1],value);
       }
    }
  }
  public static void fill (int[][][] array, int value) {
    for (int x0 = 0; x0 < array.length; x0++) {
       for (int x1 = 0; x1 < array[x0].length; x1++) {
       	  Arrays.fill(array[x0][x1],value);
       }
    }
  }
  /****************************************************************************
  * initLog.
  ****************************************************************************/
  public static Log initLog (XmlParser parser) throws Exception {
    if (parser.hasElement("server")) {
      HostPort logAddress = new HostPort(parser.getElement("server").getElementValue("hostPort"));
      File tmpDir = Util.getFile(parser.getElement("server").getElementValue("tmpDir"));
      Log log = new Log(logAddress.getHost(),logAddress.getPort(),tmpDir.getPath());
      if (parser.hasElement("local")) {
      	log.setToLogFile(getFiles(parser.getElement("local").getElement("logFiles").getElementValues("logFile")),
			 decodeByteCount(parser.getElement("local").getElementValue("maxLogFileSize")));
      }
      if (parser.hasElement("stdout")) {
	log.setToStdOut(true);
      }
      if (parser.hasElement("stderr")) {
	log.setToStdErr(true);
      }
      return log;
    }
    if (parser.hasElement("local")) {
      Log log = new Log(getFiles(parser.getElement("local").getElement("logFiles").getElementValues("logFile")),
		        decodeByteCount(parser.getElement("local").getElementValue("maxLogFileSize")));
      if (parser.hasElement("stdout")) {
	log.setToStdOut(true);
      }
      if (parser.hasElement("stderr")) {
	log.setToStdErr(true);
      }
      return log;
    }
    if (parser.hasElement("stdout")) {
      Log log = new Log(true);
      if (parser.hasElement("stderr")) {
	log.setToStdErr(true);
      }
      return log;
    }
    return new Log(false);
  }
  private static File[] getFiles (String[] paths) {
    File[] files = new File[paths.length];
    for (int i = 0; i < files.length; i++) {
       files[i] = Util.getFile(paths[i]);
    }
    return files;
  }
  /****************************************************************************
  * decodeTimeInterval.
  * Default to milliseconds if a unit isn't specified.
  ****************************************************************************/
  public static long decodeTimeInterval (String count) {
    if (count.toLowerCase().endsWith("ms")) {
      return Long.parseLong(count.substring(0,count.length()-2));
    }
    if (count.toLowerCase().endsWith("s")) {
      return Long.parseLong(count.substring(0,count.length()-1)) * MS_PER_SEC;
    }
    if (count.toLowerCase().endsWith("m")) {
      return Long.parseLong(count.substring(0,count.length()-1)) * MS_PER_MIN;
    }
    if (count.toLowerCase().endsWith("h")) {
      return Long.parseLong(count.substring(0,count.length()-1)) * MS_PER_HOUR;
    }
    if (count.toLowerCase().endsWith("d")) {
      return Long.parseLong(count.substring(0,count.length()-1)) * MS_PER_DAY;
    }
    return Long.parseLong(count);
  }
  /****************************************************************************
  * decodeByteCount.
  ****************************************************************************/
  public static long decodeByteCount (String count) {
    if (count.toLowerCase().endsWith("k")) {
      return Math.round(ONE_KB * Double.parseDouble(count.substring(0,count.length()-1)));
    }
    if (count.toLowerCase().endsWith("kb")) {
      return Math.round(ONE_KB * Double.parseDouble(count.substring(0,count.length()-2)));
    }
    if (count.toLowerCase().endsWith("m")) {
      return Math.round(ONE_MB * Double.parseDouble(count.substring(0,count.length()-1)));
    }
    if (count.toLowerCase().endsWith("mb")) {
      return Math.round(ONE_MB * Double.parseDouble(count.substring(0,count.length()-2)));
    }
    if (count.toLowerCase().endsWith("g")) {
      return Math.round(ONE_GB * Double.parseDouble(count.substring(0,count.length()-1)));
    }
    if (count.toLowerCase().endsWith("gb")) {
      return Math.round(ONE_GB * Double.parseDouble(count.substring(0,count.length()-2)));
    }
    return Long.parseLong(count);
  }
  /****************************************************************************
  * encodeContent.
  ****************************************************************************/
  public static String encodeContent (String xml) {
    StringBuffer encoded = new StringBuffer();
    for (int i = 0; i < xml.length(); i++) {
       char c = xml.charAt(i);
       switch (c) {
	 case '>': {
	   encoded.append("&gt;");
	   break;
	 }
	 case '<': {
	   encoded.append("&lt;");
	   break;
	 }
	 case '\"': {
	   encoded.append("&quot;");
	   break;
	 }
	 case '\'': {
	   encoded.append("&apos;");
	   break;
	 }
	 case '&': {
	   encoded.append("&amp;");
	   break;
	 }
	 default: {
	   encoded.append(new String(new char[] {(char)c}));
	   break;
	 }
       }
    }
    return encoded.toString();
  }
  /****************************************************************************
  * encodeHostPort.
  ****************************************************************************/
  public static String encodeHostPort (String addr) {
    int index = addr.lastIndexOf(':');
    try {
      InetAddress ia = InetAddress.getByName(addr.substring(0,index));
      return ia.getHostAddress() + ":" + addr.substring(index) +
	     " (" + ia.getHostName().toLowerCase() + ")";
    } catch (Exception e) {
      return addr;
    }
  }
  /****************************************************************************
  * getAcceptedLogEntry.
  ****************************************************************************/
  public static String getAcceptedLogEntry (Socket s) {
    return "accepted connection from " +
    	   s.getInetAddress().getHostAddress() +
	   ":" + Integer.toString(s.getPort()) + " on " +
	   s.getLocalAddress().getHostAddress() + ":" +
	   Integer.toString(s.getLocalPort());
  }
  /****************************************************************************
  * getInputStream.
  ****************************************************************************/
  public static InputStream getInputStream (String path) throws Exception {
    try {
      Object o = new Object();
      Class c = o.getClass();
      URL url = c.getResource("/"+path);
      return url.openStream();
    } catch (Exception e) {
    	
    }
    try {
      return new FileInputStream(path);
    } catch (Exception e) {
    	
    }
    throw new Exception("failed to find file: " + path);
  }
  /****************************************************************************
  * encodeTimeInterval.
  ****************************************************************************/
  public static String encodeTimeInterval (long ms) {
    if (ms % MS_PER_DAY == 0) {
      return Long.toString(ms/MS_PER_DAY) + " " + (ms == MS_PER_DAY ? "day" : "days");
    }
    if (ms % MS_PER_HOUR == 0) {
      return Long.toString(ms/MS_PER_HOUR) + " " + (ms == MS_PER_HOUR ? "hour" : "hours");
    }
    if (ms % MS_PER_MIN == 0) {
      return Long.toString(ms/MS_PER_MIN) + " " + (ms == MS_PER_MIN ? "minute" : "minutes");
    }
    if (ms % MS_PER_SEC == 0) {
      return Long.toString(ms/MS_PER_SEC) + " " + (ms == MS_PER_SEC ? "second" : "seconds");
    }
    return Long.toString(ms) + " " + (ms == 1 ? "millisecond" : "milliseconds");
  }
  /****************************************************************************
  * encodeByteCount.
  ****************************************************************************/
  public static String encodeByteCount (long nBytes) {
    if (nBytes < ONE_KB) {
      return Long.toString(nBytes) + " bytes";
    }
    if (nBytes < ONE_MB) {
      return fp1.format(((double)nBytes)/ONE_KB) + " KB";
    }
    if (nBytes < ONE_GB) {
      return fp2.format(((double)nBytes)/ONE_MB) + " MB";
    }
    return fp2.format(((double)nBytes)/ONE_GB) + " GB";
  }
  /****************************************************************************
  * encodeException.
  ****************************************************************************/
  public static String encodeException (Exception e) {
    String text = e.toString();
    if (text.startsWith("java.lang.Exception")) {
      int index = text.indexOf(": ");
      if (index != -1) {
      	return text.substring(index+2);
      }
    }
    return text;
  }
  /****************************************************************************
  * encodeDateTime.
  ****************************************************************************/
  public static String encodeDateTime (long ms) {
    if (dateFormat == null) {
      dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
      dateFormat.setTimeZone(TimeZone.getTimeZone("UTC"));
    }
    return dateFormat.format(new Date(ms)) + " UTC";
  }
  /****************************************************************************
  * encodeDuration.
  ****************************************************************************/
  public static String encodeDuration (int ms) {
    int minutes = (ms / 1000) / 60;
    int seconds = (ms / 1000) % 60;
    return Integer.toString(minutes) + ":" + lz2.format(seconds);
  }
  /****************************************************************************
  * leftJustify.
  ****************************************************************************/
  public static String leftJustify (String text, int width) {
    StringBuffer justified = new StringBuffer();
    int trailingCount = width - text.length();
    justified.append(text);
    for (int i = 0; i < trailingCount; i++) {
       justified.append(" ");
    }
    return justified.toString();
  }
  /****************************************************************************
  * rightJustify.
  ****************************************************************************/
  public static String rightJustify (String text, int width) {
    StringBuffer justified = new StringBuffer();
    int leadingCount = width - text.length();
    for (int i = 0; i < leadingCount; i++) {
       justified.append(" ");
    }
    justified.append(text);
    return justified.toString();
  }
  /****************************************************************************
  * sleep.
  ****************************************************************************/
  public static void sleep (double seconds) {
    sleep((long)(seconds * 1000));
  }
  public static void sleep (int ms) {
    sleep((long)ms);
  }
  public static void sleep (long ms) {
    try {
      Thread.sleep(ms);
    } catch (Exception e) {

    }
  }
  /****************************************************************************
  * fillBuffer.
  ****************************************************************************/
  public static void fillBuffer (DataInputStream in, byte[] buffer, int needed) throws Exception {
    fillBuffer(in,buffer,0,needed,false);
  }
  public static void fillBuffer (DataInputStream in, byte[] buffer, int offset, int needed) throws Exception {
    fillBuffer(in,buffer,offset,needed,false);
  }
  public static boolean fillBuffer (DataInputStream in, byte[] buffer,
  				    int needed, boolean eofOk) throws Exception {
    return fillBuffer(in,buffer,0,needed,eofOk);
  }
  public static boolean fillBuffer (DataInputStream in, byte[] buffer, int offset,
  				    int needed, boolean eofOk) throws Exception {
    int received = 0;
    while (received < needed) {
      int count = in.read(buffer,offset+received,needed-received);
      if (count == -1) {
      	if (eofOk) {
      	  return false;
      	} else {
      	  throw new EOFException("EOF");
      	}
      }
      received += count;
    }
    return true;
  }
  /****************************************************************************
  * min.
  ****************************************************************************/
  public static int min (int a, int b) {
    if (a < b) return a;
    return b;
  }
  public static int min (int a, int b, int c, int d) {
    return min(min(a,b),min(c,d));
  }
  public static long min (long a, long b) {
    if (a < b) return a;
    return b;
  }
  public static double min (double a, double b) {
    if (a < b) return a;
    return b;
  }
  public static double min (double a, double b, double c, double d) {
    return min(min(a,b),min(c,d));
  }
  /****************************************************************************
  * max.
  ****************************************************************************/
  public static int max (int a, int b) {
    if (a > b) return a;
    return b;
  }
  public static int max (int a, int b, int c, int d) {
    return max(max(a,b),max(c,d));
  }
  public static double max (double a, double b) {
    if (a > b) return a;
    return b;
  }
  public static double max (double a, double b, double c, double d) {
    return max(max(a,b),max(c,d));
  }
  /****************************************************************************
  * inRange.
  ****************************************************************************/
  public static boolean inRange (double min, double value, double max) {
    if (value < min) {
      return false;
    }
    if (value > max) {
      return false;
    }
    return true;
  }
  /****************************************************************************
  * toRange.
  ****************************************************************************/
  public static int toRange (int min, int value, int max) {
    if (value < min) {
      return min;
    }
    if (value > max) {
      return max;
    }
    return value;
  }
  public static double toRange (double min, double value, double max) {
    if (value < min) {
      return min;
    }
    if (value > max) {
      return max;
    }
    return value;
  }
  /****************************************************************************
  * modToRange.
  ****************************************************************************/
  public static double modToRange (double min, double value, double max) {
    double range = max - min;
    while (value < min) {
      value += range;
    }
    while (value > max) {
      value -= range;
    }
    return value;
  }
  /****************************************************************************
  * indexOf.
  ****************************************************************************/
  public static int indexOf (String string, String[] strings) {
    for (int i = 0; i < strings.length; i++) {
       if (strings[i].equals(string)) {
       	 return i;
       }
    }
    return -1;
  }
  public static int indexOf (int value, int[] values) {
    for (int i = 0; i < values.length; i++) {
       if (values[i] == value) {
       	 return i;
       }
    }
    return -1;
  }
  public static int indexOf (Object o, Object[] os) {
    for (int i = 0; i < os.length; i++) {
       if (os[i] == o) {
       	 return i;
       }
    }
    return -1;
  }
  /****************************************************************************
  * oneOf.
  ****************************************************************************/
  public static boolean oneOf (String string, String[] strings) {
    return (indexOf(string,strings) != -1);
  }
  public static boolean oneOf (int value, int[] values) {
    return (indexOf(value,values) != -1);
  }
  /****************************************************************************
  * toHexString.
  ****************************************************************************/
  public static String toHexString (byte b) {
    int i = ((int)b) & 0x000000FF;
    String hex = Integer.toHexString(i);
    if (hex.length() < 2) {
      return ("0" + hex);
    }
    return hex;
  }
  /****************************************************************************
  * startThread.
  ****************************************************************************/
  public static Thread startThread (Runnable runnable) {
    return startThread(runnable,false);
  }
  public static Thread startThread (Runnable runnable, boolean daemon) {
    Thread thread = new Thread(runnable);
    thread.setDaemon(daemon);
    thread.start();
    return thread;
  }
  /****************************************************************************
  * sin.
  ****************************************************************************/
  public static double sin (double degrees) {
    return Math.sin(Math.toRadians(degrees));
  }
  /****************************************************************************
  * cos.
  ****************************************************************************/
  public static double cos (double degrees) {
    return Math.cos(Math.toRadians(degrees));
  }
  /****************************************************************************
  * tan.
  ****************************************************************************/
  public static double tan (double degrees) {
    return Math.tan(Math.toRadians(degrees));
  }
  /****************************************************************************
  * asin.
  ****************************************************************************/
  public static double asin (double value) {
    return Math.toDegrees(Math.asin(value));
  }
  /****************************************************************************
  * acos.
  ****************************************************************************/
  public static double acos (double value) {
    return Math.toDegrees(Math.acos(value));
  }
  /****************************************************************************
  * atan.
  ****************************************************************************/
  public static double atan (double value) {
    return Math.toDegrees(Math.atan(value));
  }
  /****************************************************************************
  * readString.
  ****************************************************************************/
  public static String readString (File file) throws Exception {
    StringBuffer string = new StringBuffer();
    BufferedReader in = null;
    try {
      in = new BufferedReader(new FileReader(file));
      String line = in.readLine();
      while (line != null) {
	string.append(line);
	line = in.readLine();
      }
    } finally {
      try {in.close();} catch (Exception e) {}
    }
    return string.toString();
  }
  /****************************************************************************
  * getFile.
  ****************************************************************************/
  public static File getFile (String path) {
    path = setDirectorySeparator(path);
    File file = new File(path);
    if (file.isAbsolute()) {
      return file;
    }
    String scHome = System.getProperty("sc.home");
    if (scHome == null) {
      scHome = ".";
    }
    return new File(scHome,path);
  }
  /****************************************************************************
  * setDirectorySeparator.
  ****************************************************************************/
  public static String setDirectorySeparator (String path) {
    path = path.replace('/',File.separatorChar);
    path = path.replace('\\',File.separatorChar);
    return path;
  }
  /****************************************************************************
  * threadDump.
  ****************************************************************************/
  public static void threadDump () {
    System.out.println("");
    System.out.println("Threads:");
    Thread[] threads = new Thread[1000];
    int count = Thread.enumerate(threads);
    for (int i = 0; i < count; i++) {
       System.out.println(threads[i].getName() +
	   		  ", group: " + threads[i].getThreadGroup() +
	   		  ", daemon: " + threads[i].isDaemon() +
	   		  ", state: " + threads[i].getState() +
	   		  ", priority: " + threads[i].getPriority() +
	   		  ", alive: " + threads[i].isAlive() +
	   		  ", interrupted: " + threads[i].isInterrupted());
       if (threads[i].getState() == Thread.State.BLOCKED) {
	 StackTraceElement[] elements = threads[i].getStackTrace();
	 for (int j = 0; j < elements.length; j++) {
	    System.out.println("  " + elements[j].toString());
	 }
       }
    }
  }
}
