/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/UTIL
*
*  History:
*
*  16-Jun-04, J Love, GST	Original version.
*   8-Feb-06, J Love, GST	Java 1.5.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.util;
import java.io.*;
import java.util.*;
import javax.xml.parsers.*;
import org.w3c.dom.*;

public class XmlParser {
  Element baseElement;
  /****************************************************************************
  * XmlParser (constructor).
  ****************************************************************************/
  public XmlParser (String msg) throws Exception {
    ByteArrayInputStream in = new ByteArrayInputStream(msg.getBytes());
    construct(in,null);
  }
  public XmlParser (String msg, String rootElementName) throws Exception {
    ByteArrayInputStream in = new ByteArrayInputStream(msg.getBytes());
    construct(in,rootElementName);
  }
  public XmlParser (File file, String rootElementName) throws Exception {
    InputStream in = null;
    try {
      in = new FileInputStream(file);
      construct(in,rootElementName);
    } finally {
      try {in.close();} catch (Exception e) {}
    }
  }
  private void construct (InputStream in, String rootElementName) throws Exception {
    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
    factory.setValidating(false);
    factory.setNamespaceAware(false);
    DocumentBuilder builder = factory.newDocumentBuilder();
    baseElement = builder.parse(in).getDocumentElement();
    if (rootElementName != null) {
      if (!baseElement.getTagName().equals(rootElementName)) {
        throw new Exception("wrong root element tag name: " + baseElement.getTagName());
      }
    }
  }
  private XmlParser (Element baseElement) {
    this.baseElement = baseElement;
  }
  /****************************************************************************
  * getTagName.
  ****************************************************************************/
  public String getTagName () {
    return baseElement.getTagName();
  }
  /****************************************************************************
  * hasElement.
  ****************************************************************************/
  public boolean hasElement (String tag) {
    return hasElement(new String[] {tag});
  }
  public boolean hasElement (String[] tags) {
    try {
      /*XmlParser parser =*/ getElement(tags);
      return true;
    } catch (Exception e) {
      return false;
    }
  }
  /****************************************************************************
  * getElement.
  ****************************************************************************/
  public XmlParser getElement (String tag) throws Exception {
    return getElement(new String[] {tag});
  }
  public XmlParser getElement (String[] tags) throws Exception {
    XmlParser[] parsers = getElements(tags);
    if (parsers.length < 1) {
      throw new Exception("element not found: " + encodeTagPath(tags));
    }
    if (parsers.length > 1) {
      throw new Exception("multiple elements found where not expected: " +
			  encodeTagPath(tags));
    }
    return parsers[0];
  }
  /****************************************************************************
  * getElements.
  ****************************************************************************/
  public XmlParser[] getElements () throws Exception {
    return getElements(new String[] {null});
  }
  public XmlParser[] getElements (String tag) throws Exception {
    return getElements(new String[] {tag});
  }
  public XmlParser[] getElements (String[] tags) throws Exception {
    int lastTagIndex = tags.length - 1;
    Element element = baseElement;
    for (int i = 0; i < lastTagIndex; i++) {
       Element[] elements = findElements(element,tags[i]);
       if (elements.length < 1) {
	 throw new Exception("element not found: " + encodeTagPath(tags));
       }
       if (elements.length > 1) {
	 throw new Exception("multiple elements found where not expected: " +
			     encodeTagPath(tags));
       }
       element = elements[0];
    }
    Element[] elements = findElements(element,tags[lastTagIndex]);
    XmlParser[] parsers = new XmlParser[elements.length];
    for (int j = 0; j < parsers.length; j++) {
       parsers[j] = new XmlParser(elements[j]);
    }
    return parsers;
  }
  /****************************************************************************
  * getElementValue.
  ****************************************************************************/
  public String getElementValue (String tag) throws Exception {
    return getElementValue(new String[] {tag});
  }
  public String getElementValue (String[] tags) throws Exception {
    XmlParser parser = getElement(tags);
    return parser.getElementValue();
  }
  public String getElementValue () throws Exception {
    NodeList nodes = baseElement.getChildNodes();
    for (int i = 0; i < nodes.getLength(); i++) {
       Node node = nodes.item(i);
       if (node.getNodeType() == Node.TEXT_NODE) {
	 return node.getNodeValue();
       }
    }
    // throw new Exception("no textual content found for element: " + baseElement.getNodeName());
    return "";
  }
  /****************************************************************************
  * getElementValues.
  ****************************************************************************/
  public String[] getElementValues (String tag) throws Exception {
    return getElementValues(new String[] {tag});
  }
  public String[] getElementValues (String[] tags) throws Exception {
    XmlParser[] parsers = getElements(tags);
    String[] values = new String[parsers.length];
    for (int i = 0; i < parsers.length; i++) {
       values[i] = parsers[i].getElementValue();
    }
    return values;
  }
  /****************************************************************************
  * getAttributeValue.
  ****************************************************************************/
  public String getAttributeValue (String attributeName) throws Exception {
    NamedNodeMap attributes = baseElement.getAttributes();
    if (attributes != null) {
      for (int j = 0; j < attributes.getLength(); j++) {
	 Node node = attributes.item(j);
	 if (node.getNodeType() == Node.ATTRIBUTE_NODE) {
	   if (node.getNodeName().equals(attributeName)) {
	     return node.getNodeValue();
	   }
         }
       }
    }
    return null;
  }
  /****************************************************************************
  * findElements.
  ****************************************************************************/
  private Element[] findElements (Element element,
				  String tag) {		// null means all elements
    LinkedList<Element> elements = new LinkedList<Element>();
    NodeList nodes = element.getChildNodes();
    for (int i = 0; i < nodes.getLength(); i++) {
       Node node = nodes.item(i);
       if (node.getNodeType() == Node.ELEMENT_NODE) {
	 if (tag != null) {
	   if (node.getNodeName().equals(tag)) {
	     elements.add((Element)node);
	   }
	 } else {
	   elements.add((Element)node);
	 }
       }
    }
    return elements.toArray(new Element[elements.size()]);
  }
  /****************************************************************************
  * encodeTagPath.
  ****************************************************************************/
  private String encodeTagPath (String[] tags) {
    StringBuffer path = new StringBuffer(tags[0]);
    for (int i = 1; i < tags.length; i++) {
       path.append(">" + tags[i]);
    }
    return path.toString();
  }
}
