/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*   9-Nov-06, J Love, GST	Original version.
*  30-Jan-07, J Love, GST	Tweaked appearance.
*  26-Feb-07, J Love, GST	More tool tips.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.viewer;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import gov.nasa.gsfc.drl.geo.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.*;
import java.util.*;

public class MapHandler extends JPanel implements OrbitsOverlayListener {
  private int projection;
  private StringMap2<MapView> views = new StringMap2<MapView>();
  private MapParms mapParms = null;
  private GroundStationOverlay gsOverlay = null;
  private OrbitsOverlay orbitsOverlay = null;
  private ShpOverlayLoader shpLoader = null;
  private GroundStations groundStations = null;
  private Object[][] orbitParms = null;
  private int[] firstCorner = null;
  private int[] dragCorner = null;
  private int[] secondCorner = null;
  private boolean firstPaint = true;
  public static final String[] shpNames = new String[] {
    "Countries","US States","US Urban Areas","US Cities","Canadian Territories","Mexican States","Caribbean Islands"
  };
  public static final boolean[] shpDefaultStates = new boolean[] {
    true,true,false,false,true,false,false
  };
  private static final String[] shpFiles = new String[] {
    "shapes/world-esri/world.shp","shapes/states/s_12my03.shp","shapes/urban-esri/USurban.shp","shapes/cities/cities.shp","shapes/canada/PROVINCE.SHP","shapes/mexico/states.shp","shapes/caribbean/carib_wvs_geo_wgs84.shp"
  };
  private static final Color[] shpColors = new Color[] {
    new Color(0,120,0),new Color(0,120,0),new Color(0,100,0),new Color(0,80,0),new Color(0,120,0),new Color(0,120,0),new Color(0,120,0)
  };
  private ShpOverlay[] shpOverlays = new ShpOverlay[] {
    null,null,null,null,null,null,null
  };
  public static final int MERCATOR = 1;
  public static final int BOTH_POLAR = 2;
  public static final int NORTH_POLAR = 3;
  public static final int SOUTH_POLAR = 4;
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * MapHandler.
  ****************************************************************************/
  public MapHandler (int projection) {
    this.projection = projection;
    setPreferredSize(getMaximumSize());
    addMouseListener(new ActionMouse());
    addMouseMotionListener(new ActionMouseMotion());
    addComponentListener(new MapListener());
  }
  /****************************************************************************
  * MapListener.
  ****************************************************************************/
  private class MapListener extends ComponentAdapter {
    public void componentResized (ComponentEvent ce) {
      resized();
    }
  }
  private synchronized void resized () {
    Dimension dims = getSize();
    prereconfig();
    reconfig(mapParms.resize(dims.width,dims.height));
  }
  /****************************************************************************
  * addView.
  ****************************************************************************/
  public synchronized void addView (String passName) {
    if (mapParms != null) {
      views.putLast(passName,new MapView(mapParms));
    } else {
      views.putLast(passName,null);
    }
    repaint();
  }
  /****************************************************************************
  * removeView.
  ****************************************************************************/
  public synchronized void removeView (String passName) {
    views.remove(passName);
    repaint();
  }
  /****************************************************************************
  * removeAllViews.
  ****************************************************************************/
  public synchronized void removeAllViews () {
    views.clear();
    repaint();
  }
  /****************************************************************************
  * passSelected.
  ****************************************************************************/
  public synchronized void passSelected (String passName) {
    MapView view = views.remove(passName);
    views.putLast(passName,view);
    repaint();
  }
  /****************************************************************************
  * prereconfig.
  ****************************************************************************/
  private synchronized void prereconfig () {
    if (shpLoader != null) {
      shpLoader.stop();
    }
    if (orbitsOverlay != null) {
      orbitsOverlay.stop();
    }
  }
  /****************************************************************************
  * reconfig.
  ****************************************************************************/
  private synchronized void reconfig (Rectangle rect) {
    config(rect);
    revalidate();
    repaint();
  }
  /****************************************************************************
  * config.
  ****************************************************************************/
  private synchronized void config (Rectangle rect) {
    if (mapParms == null) {
      Dimension dims = getSize();
      switch (projection) {
        case MERCATOR: {
	  mapParms = new MercatorMapParms(dims.width,dims.height);
	  break;
        }
        case BOTH_POLAR: {
	  mapParms = new BothPolesMapParms(dims.width,dims.height);
	  break;
        }
        case NORTH_POLAR: {
	  mapParms = new SinglePoleMapParms(true,dims.width,dims.height);
	  break;
        }
        case SOUTH_POLAR: {
	  mapParms = new SinglePoleMapParms(false,dims.width,dims.height);
	  break;
        }
      }
    }
    for (String passName : views.getStrings()) {
       MapView view = views.get(passName);
       if (view != null) {
	 view.config(mapParms,rect);
       } else {
	 views.put(passName,new MapView(mapParms));
       }
    }
    if (groundStations != null) {
      gsOverlay = new GroundStationOverlay(groundStations,mapParms);
    } else {
      gsOverlay = null;
    }
    if (orbitParms != null) {
      try {
	orbitsOverlay = new OrbitsOverlay(orbitParms,mapParms,this);
      } catch (Exception e) {
      	e.printStackTrace();
      	orbitsOverlay = null;
      }
    } else {
      orbitsOverlay = null;
    }
    for (int i = 0; i < shpOverlays.length; i++) {
       shpOverlays[i] = null;
    }
    shpLoader = new ShpOverlayLoader();
    Util.startThread(shpLoader);
  }
  /****************************************************************************
  * setGroundStations.
  ****************************************************************************/
  public synchronized void setGroundStations (GroundStations groundStations) {
    this.groundStations = groundStations;
    if (mapParms != null) {
      gsOverlay = new GroundStationOverlay(groundStations,mapParms);
    }
    repaint();
  }
  /****************************************************************************
  * setTLEs.
  ****************************************************************************/
  public synchronized void setTLEs (String[] aquaTLE, String[] terraTLE) {
    orbitParms = new Object[][] {{new Color(160,160,255),"AQUA",aquaTLE[0],aquaTLE[1]},{Color.RED,"TERRA",terraTLE[0],terraTLE[1]}};
    if (mapParms != null) {
      try {
	orbitsOverlay = new OrbitsOverlay(orbitParms,mapParms,this);
      } catch (Exception e) {
      	e.printStackTrace();
      	orbitsOverlay = null;
      }
    }
    repaint();
  }
  /****************************************************************************
  * getAreaFilter.
  ****************************************************************************/
  public synchronized String getAreaFilter () {
    if (Viewer.optionsScreen.autoZoomCheckBox.isSelected()) {
      return "";
    }
    return mapParms.getAreaFilter();
  }
  /****************************************************************************
  * clear.
  ****************************************************************************/
  public synchronized void clear () {
    for (String passName : views.getStrings()) {
       MapView view = views.get(passName);
       if (view == null)
       {
         // System.out.println("If view is null, don't clear.");
       }
       else
       {
         view.clear();
       }
    }
    repaint();
  }
  /****************************************************************************
  * doHeader.
  ****************************************************************************/
  public synchronized void doHeader (String passName, HeaderBlock headerBlock) throws Exception {
    MapView view = (MapView) views.get(passName);
    if (Viewer.optionsScreen.eraseOnReplayCheckBox.isSelected()) {
      view.clear();
    }
    if (Viewer.optionsScreen.autoZoomCheckBox.isSelected()) {
      if (Viewer.optionsScreen.eraseOnAutoZoomCheckBox.isSelected()) {
        clear();
      }
      Geo.Extents extents = headerBlock.getEstExtents();
      prereconfig();
      reconfig(mapParms.zoom(extents.getBeginRight().getLat(),extents.getBeginRight().getLon(),
	      		     extents.getBeginLeft().getLat(),extents.getBeginLeft().getLon(),
			     extents.getEndRight().getLat(),extents.getEndRight().getLon(),
			     extents.getEndLeft().getLat(),extents.getEndLeft().getLon()));
    }
    view.doHeader(headerBlock);
    repaint();
  }
  /****************************************************************************
  * doScan.
  ****************************************************************************/
  public synchronized void doScan (String passName, GeoBlock geoBlock, Color[][] colors) {
    MapView view = (MapView) views.get(passName);
    view.doScan(geoBlock,colors);
    repaint();
  }
  /****************************************************************************
  * doFooter.
  ****************************************************************************/
  public synchronized void doFooter (String passName) {
    MapView view = (MapView) views.get(passName);
    view.doFooter();
    repaint();
  }
  /****************************************************************************
  * getBufferedImage.
  ****************************************************************************/
  public synchronized BufferedImage getBufferedImage () {
    BufferedImage bi = new BufferedImage(mapParms.getWidth(),
            				 mapParms.getHeight(),
            				 BufferedImage.TYPE_INT_RGB);
    Graphics g = bi.getGraphics();
    paint(g);
    return bi;
  }
  /****************************************************************************
  * orbitsUpdated.
  ****************************************************************************/
  public synchronized void orbitsUpdated () {
    repaint();
  }
  /****************************************************************************
  * ShpOverlayLoader.
  ****************************************************************************/
  private class ShpOverlayLoader implements Runnable {
    private ShpOverlay overlay = null;
    private boolean stop = false;
    private boolean stopped = false;
    public void run () {
      for (int i = 0; i < shpFiles.length; i++) {
         try {
           overlay = new ShpOverlay(mapParms);
	   overlay.addShpFile(shpFiles[i],shpColors[i]);
	   shpOverlays[i] = overlay;
	   repaint();
         } catch (InterruptedException e) {
           break;
         } catch (Exception e) {
           e.printStackTrace();
         }
         if (stop) {
           break;
         }
      }
      stopped = true;
    }
    public void stop () {
      stop = true;
      if (overlay != null) {
	overlay.abort();
      }
      while (!stopped) {
      	Util.sleep(1);
      }
    }
  }
  /****************************************************************************
  * ActionMouse.
  ****************************************************************************/
  private class ActionMouse extends MouseAdapter {
    public void mousePressed (MouseEvent me) {
      switch (me.getButton()) {
        case MouseEvent.BUTTON1: {
          if (mapParms.isZoomable()) {
	    firstCorner = new int[] {me.getX(),me.getY()};
          }
	  break;
        }
        case MouseEvent.BUTTON3: {
          JPopupMenu popupMenu = new JPopupMenu();
          if (mapParms.isZoomable()) {
            JMenuItem zoomInItem = new JMenuItem("Zoom In (2x)");
            zoomInItem.setToolTipText("Zoom in (2x) centered at the mouse pointer location.");
            zoomInItem.addActionListener(new ZoomSelected(2.0,me.getX(),me.getY()));
            popupMenu.add(zoomInItem);
            JMenuItem zoomOutItem = new JMenuItem("Zoom Out (2x)");
            zoomOutItem.setToolTipText("Zoom out (2x) centered at the mouse pointer location.");
            zoomOutItem.addActionListener(new ZoomSelected(0.5,me.getX(),me.getY()));
            popupMenu.add(zoomOutItem);
            JMenuItem zoomOutFullItem = new JMenuItem("Zoom Out (All)");
            zoomOutFullItem.setToolTipText("Zoom out to view all of the earth.");
            zoomOutFullItem.addActionListener(new ZoomSelected(0.000000000001,me.getX(),me.getY()));
            popupMenu.add(zoomOutFullItem);
            JMenuItem centerItem = new JMenuItem("Center");
            centerItem.setToolTipText("Recenter the map at the mouse pointer location.");
            centerItem.addActionListener(new ZoomSelected(1.0,me.getX(),me.getY()));
            popupMenu.add(centerItem);
            popupMenu.addSeparator();
          }
          JCheckBoxMenuItem showGridItem = new JCheckBoxMenuItem("Grid",Viewer.optionsScreen.showGridCheckBox.isSelected());
          showGridItem.setToolTipText("Specifies whether or not latitude/longitude grid lines are displayed.");
          showGridItem.addActionListener(new ShowGridClicked());
          popupMenu.add(showGridItem);
          JCheckBoxMenuItem showFeaturesItem = new JCheckBoxMenuItem("Features",Viewer.optionsScreen.showFeaturesCheckBox.isSelected());
          showFeaturesItem.setToolTipText("Specifies whether or not geographic/political features are displayed.");
          showFeaturesItem.addActionListener(new ShowFeaturesClicked());
          popupMenu.add(showFeaturesItem);
          JCheckBoxMenuItem showOrbitsItem = new JCheckBoxMenuItem("Orbits",Viewer.optionsScreen.showOrbitsCheckBox.isSelected());
          showOrbitsItem.setToolTipText("Specifies whether or not satellite locations/orbits are displayed.");
          showOrbitsItem.addActionListener(new ShowOrbitsClicked());
          popupMenu.add(showOrbitsItem);
          JCheckBoxMenuItem showStationsItem = new JCheckBoxMenuItem("Ground Stations",Viewer.optionsScreen.showGroundStationCheckBox.isSelected());
          showStationsItem.setToolTipText("Specifies whether or not ground stations are displayed.");
          showStationsItem.addActionListener(new ShowStationsClicked());
          popupMenu.add(showStationsItem);
          popupMenu.addSeparator();
          JRadioButtonMenuItem mercatorButton = new JRadioButtonMenuItem("Mercator Projection",projection == MERCATOR);
          mercatorButton.setToolTipText("Selects a Mercator projection.");
          mercatorButton.addActionListener(new ProjectionSelected(MERCATOR));
          popupMenu.add(mercatorButton);
          JRadioButtonMenuItem bothPolesButton = new JRadioButtonMenuItem("Polar Projections",projection == BOTH_POLAR);
          bothPolesButton.setToolTipText("Selects a Polar projection showing both poles.");
          bothPolesButton.addActionListener(new ProjectionSelected(BOTH_POLAR));
          popupMenu.add(bothPolesButton);
          JRadioButtonMenuItem northPoleButton = new JRadioButtonMenuItem("North Polar Projection",projection == NORTH_POLAR);
          northPoleButton.setToolTipText("Selects a Polar projection showing just the north pole.");
          northPoleButton.addActionListener(new ProjectionSelected(NORTH_POLAR));
          popupMenu.add(northPoleButton);
          JRadioButtonMenuItem southPoleButton = new JRadioButtonMenuItem("South Polar Projection",projection == SOUTH_POLAR);
          southPoleButton.setToolTipText("Selects a Polar projection showing just the south pole.");
          southPoleButton.addActionListener(new ProjectionSelected(SOUTH_POLAR));
          popupMenu.add(southPoleButton);
          GUtil.buttonGroup(mercatorButton,bothPolesButton,northPoleButton,southPoleButton);
          popupMenu.addSeparator();
          JMenuItem clearItem = new JMenuItem("Erase Passes");
          clearItem.setToolTipText("Erases all pass from the map.");
          clearItem.addActionListener(new ClearClicked());
          popupMenu.add(clearItem);
          JMenuItem saveAsItem = new JMenuItem("Save As...");
          saveAsItem.setToolTipText("Displays a dialog used to save the map to an image file.");
          saveAsItem.addActionListener(new SaveAsClicked());
          popupMenu.add(saveAsItem);
          popupMenu.pack();
          popupMenu.show(me.getComponent(),me.getX(),me.getY());
	  break;
        }
      }
      repaint();
    }
    public void mouseReleased (MouseEvent me) {
      switch (me.getButton()) {
        case MouseEvent.BUTTON1: {
          if (mapParms.isZoomable()) {
            secondCorner = new int[] {me.getX(),me.getY()};
            boolean zoomExtents = false;
            if (firstCorner != null && secondCorner != null) {
              if (firstCorner[0] != secondCorner[0] || firstCorner[1] != secondCorner[1]) {
      	        zoomExtents = true;
      	      }
            }
            if (zoomExtents) {
      	      if (Math.abs(firstCorner[0]-secondCorner[0]) > 10 && Math.abs(firstCorner[1]-secondCorner[1]) > 10) {
                zoom(Util.min(firstCorner[0],secondCorner[0]),Util.min(firstCorner[1],secondCorner[1]),
		     Util.max(firstCorner[0],secondCorner[0]),Util.max(firstCorner[1],secondCorner[1]));
      	      }
            } else {
	      zoom(2.0,me.getX(),me.getY());
            }
	    firstCorner = null;
	    dragCorner = null;
	    secondCorner = null;
          }
          break;
        }
        case MouseEvent.BUTTON3: {
          break;
        }
      }
      repaint();
    }
  }
  /****************************************************************************
  * ZoomSelected.
  ****************************************************************************/
  private class ZoomSelected implements ActionListener {
    private double factor;
    private int x, y;
    public ZoomSelected (double factor, int x, int y) {
      this.factor = factor;
      this.x = x;
      this.y = y;
    }
    public void actionPerformed (ActionEvent ae) {
      zoom(factor,x,y);
    }
  }
  /****************************************************************************
  * zoom.
  ****************************************************************************/
  private synchronized void zoom (double factor, int x, int y) {
    if (Viewer.optionsScreen.eraseOnZoomCheckBox.isSelected()) {
      clear();
    }
    prereconfig();
    reconfig(mapParms.zoom(factor,x,y));
  }
  private synchronized void zoom (int x0, int y0, int x1, int y1) {
    if (Viewer.optionsScreen.eraseOnZoomCheckBox.isSelected()) {
      clear();
    }
    prereconfig();
    reconfig(mapParms.zoom(x0,y0,x1,y1));
  }
  /****************************************************************************
  * ShowGridClicked.
  ****************************************************************************/
  private class ShowGridClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      toggleShowGrid();
    }
  }
  private synchronized void toggleShowGrid () {
    Viewer.optionsScreen.showGridCheckBox.setSelected(!Viewer.optionsScreen.showGridCheckBox.isSelected());
    repaint();
  }
  /****************************************************************************
  * ShowFeaturesClicked.
  ****************************************************************************/
  private class ShowFeaturesClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      toggleShowFeatures();
    }
  }
  private synchronized void toggleShowFeatures () {
    Viewer.optionsScreen.showFeaturesCheckBox.setSelected(!Viewer.optionsScreen.showFeaturesCheckBox.isSelected());
    repaint();
  }
  /****************************************************************************
  * ShowOrbitsClicked.
  ****************************************************************************/
  private class ShowOrbitsClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      toggleShowOrbits();
    }
  }
  private synchronized void toggleShowOrbits () {
    Viewer.optionsScreen.showOrbitsCheckBox.setSelected(!Viewer.optionsScreen.showOrbitsCheckBox.isSelected());
    repaint();
  }
  /****************************************************************************
  * ShowStationsClicked.
  ****************************************************************************/
  private class ShowStationsClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      toggleShowStations();
    }
  }
  private synchronized void toggleShowStations () {
    Viewer.optionsScreen.showGroundStationCheckBox.setSelected(!Viewer.optionsScreen.showGroundStationCheckBox.isSelected());
    repaint();
  }
  /****************************************************************************
  * ProjectionSelected.
  ****************************************************************************/
  private class ProjectionSelected implements ActionListener {
    private int newProjection;
    public ProjectionSelected (int newProjection) {
      this.newProjection = newProjection;
    }
    public void actionPerformed (ActionEvent ae) {
      changeProjection(newProjection);
    }
  }
  private synchronized void changeProjection (int newProjection) {
    prereconfig();
    projection = newProjection;
    mapParms = null;
    reconfig(null);
  }
  /****************************************************************************
  * ClearClicked.
  ****************************************************************************/
  private class ClearClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      clear();
    }
  }
  /****************************************************************************
  * SaveAsClicked.
  ****************************************************************************/
  private class SaveAsClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      saveAs();
    }
  }
  private synchronized void saveAs () {
    BufferedImage bi = getBufferedImage();
    SaveAs saveAs = new SaveAs("map",bi);
    saveAs.pack();
    GUtil.centerWindow(saveAs);
    saveAs.setVisible(true);
  }
  /****************************************************************************
  * ActionMouseMotion.
  ****************************************************************************/
  private class ActionMouseMotion extends MouseMotionAdapter {
    public void mouseMoved (MouseEvent me) {
      // double[] loc = mapParms.getLatLon(me.getX(),me.getY());
      // System.out.println(loc[0] + " " + loc[1]);
    }
    public void mouseDragged (MouseEvent me) {
      if (firstCorner != null) {
	dragCorner = new int[] {me.getX(),me.getY()};
	repaint();
      }
    }
  }
  /****************************************************************************
  * paint.
  ****************************************************************************/
  public synchronized void paint (Graphics g) {
    if (firstPaint) {
      config(null);
      firstPaint = false;
    }
    // Ignore paint request if the map parameters don't agree with the size
    // of this JPanel.  This happens during a resizing of the JFrame.
    if (getWidth() != mapParms.getWidth() || getHeight() != mapParms.getHeight()) {
      g.setColor(Color.black);
      g.fillRect(0,0,getWidth(),getHeight());
      return;
    }
    // Draw background.
    g.setColor(Color.black);
    g.fillRect(0,0,mapParms.getWidth(),mapParms.getHeight());
    // Draw passes.
    Iterator<MapView> it = views.iterator();
    while (it.hasNext()) {
      MapView view = it.next();
      mapParms.drawImage(g,view.getImage());
      switch (Viewer.optionsScreen.outlineCombo.getSelectedIndex()) {
        case OptionsScreen.PASS_OUTLINES_OFF: {
	  break;
        }
        case OptionsScreen.PASS_OUTLINES_ALL: {
	  mapParms.drawImage(g,view.getIndicator());
	  break;
        }
        case OptionsScreen.PASS_OUTLINES_SELECTED: {
	  if (view == views.getLast()) {
	    mapParms.drawImage(g,view.getIndicator());
	  }
	  break;
        }
      }
    }
    // Draw features.
    if (Viewer.optionsScreen.showFeaturesCheckBox.isSelected()) {
      for (int i = 0; i < shpOverlays.length; i++) {
         if (shpOverlays[i] != null) {
           if (Viewer.optionsScreen.showFeatureCheckBox[i].isSelected()) {
             mapParms.drawImage(g,shpOverlays[i]);
           }
         }
      }
    }
    // Draw orbits.
    if (Viewer.optionsScreen.showOrbitsCheckBox.isSelected()) {
      if (orbitsOverlay != null) {
      	orbitsOverlay.draw(g);
      }
    }
    // Draw ground station locations.
    if (Viewer.optionsScreen.showGroundStationCheckBox.isSelected()) {
      if (gsOverlay != null) {
      	gsOverlay.draw(g);
      }
    }
    // Draw grid.
    if (Viewer.optionsScreen.showGridCheckBox.isSelected()) {
      mapParms.drawGrid(g);
    }
    // Draw border.
    g.setColor(Color.gray.brighter());
    mapParms.drawBorder(g);
    // Zoom Extents drag.
    if (dragCorner != null) {
      g.setColor(Color.white);
      g.drawRect(Util.min(firstCorner[0],dragCorner[0]),
                 Util.min(firstCorner[1],dragCorner[1]),
                 Math.abs(firstCorner[0]-dragCorner[0]),
                 Math.abs(firstCorner[1]-dragCorner[1]));
    }
  }
}
