/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*  24-Aug-06, J Love, GST	Original version.
*  21-May-07, Z Yin, GST	Between +-180, don't drew pass outline.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.viewer;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.awt.*;
import java.awt.image.*;

public class MapView {
  private MapParms mapParms;
  private BufferedImage image;
  private BufferedImage indicator;
  private Graphics gImage;
  private Graphics gIndicator;
  // private boolean first = true;
  // private boolean done = false;
  private double[] doneLats = null;
  private double[] doneLons = null;
  private double[][] lats = null;
  private double[][] lons = null;
  private int geoBinSize;
  private int nGeoRows;
  private int nGeoCols;
  private int nScans;
  /****************************************************************************
  * MapView.
  ****************************************************************************/
  public MapView (MapParms mapParms) {
    this.mapParms = mapParms;
    create();
  }
  /****************************************************************************
  * create.
  ****************************************************************************/
  private void create () {
    image = new BufferedImage(mapParms.getWidth(),mapParms.getHeight(),BufferedImage.TYPE_INT_ARGB);
    gImage = image.getGraphics();
    gImage.setClip(mapParms.getViewport());
    indicator = new BufferedImage(mapParms.getWidth(),mapParms.getHeight(),BufferedImage.TYPE_INT_ARGB);
    gIndicator = indicator.getGraphics();
    gIndicator.setClip(mapParms.getViewport());
  }
  /****************************************************************************
  * config.
  ****************************************************************************/
  public void config (MapParms newMapParms, Rectangle rect) {
    // 
    if (rect == null) {
      mapParms = newMapParms;
      create();
      return;
    }
    // Is the subimage completely off the image?
    if (rect.x + rect.width - 1 < 0 ||
	rect.x > mapParms.getWidth() ||
	rect.y + rect.height - 1 < 0 ||
	rect.y > mapParms.getHeight()) {
      mapParms = newMapParms;
      create();
      return;
    }
    // Copy subimage to new image...
    int sx, sy, dx, dy, w, h;
    if (rect.x < 0) {
      sx = 0;
      dx = -rect.x;
      w = rect.width + rect.x;
    } else {
      if (rect.x + rect.width - 1 > mapParms.getWidth() - 1) {
	sx = rect.x;
	dx = 0;
	w = mapParms.getWidth() - rect.x;
      } else {
	sx = rect.x;
	dx = 0;
	w = rect.width;
      }
    }
    if (rect.y < 0) {
      sy = 0;
      dy = -rect.y;
      h = rect.height + rect.y;
    } else {
      if (rect.y + rect.height - 1 > mapParms.getHeight() - 1) {
	sy = rect.y;
	dy = 0;
	h = mapParms.getHeight() - rect.y;
      } else {
	sy = rect.y;
	dy = 0;
	h = rect.height;
      }
    }
    BufferedImage biImage = new BufferedImage(rect.width,rect.height,BufferedImage.TYPE_INT_ARGB);
    int[] rgb = image.getRGB(sx,sy,w,h,null,0,w);
    biImage.setRGB(dx,dy,w,h,rgb,0,w);
    BufferedImage biIndicator = new BufferedImage(rect.width,rect.height,BufferedImage.TYPE_INT_ARGB);
    rgb = indicator.getRGB(sx,sy,w,h,null,0,w);
    biIndicator.setRGB(dx,dy,w,h,rgb,0,w);
    mapParms = newMapParms;
    create();
    gImage.drawImage(biImage.getScaledInstance(mapParms.getWidth(),mapParms.getHeight(),Image.SCALE_DEFAULT),0,0,null);
    gIndicator.drawImage(biIndicator.getScaledInstance(mapParms.getWidth(),mapParms.getHeight(),Image.SCALE_DEFAULT),0,0,null);
    return;
  }
  /****************************************************************************
  * getImage.
  ****************************************************************************/
  public BufferedImage getImage () {
    return image;
  }
  /****************************************************************************
  * getIndicator.
  ****************************************************************************/
  public BufferedImage getIndicator () {
    return indicator;
  }
  /****************************************************************************
  * clear.
  ****************************************************************************/
  public void clear () {
    create();
  }
  /****************************************************************************
  * doHeader.
  ****************************************************************************/
  public void doHeader (HeaderBlock headerBlock) {
    // clear();
    geoBinSize = headerBlock.getGeoResolution() / headerBlock.getCoarsestResolution();
    lats = new double[geoBinSize+1][geoBinSize+1];
    lons = new double[geoBinSize+1][geoBinSize+1];
    nGeoRows = headerBlock.getGeoBlockRowCount();
    nGeoCols = headerBlock.getGeoBlockColCount();
    doneLats = new double[headerBlock.getGeoBlockColCount()+1];
    doneLons = new double[headerBlock.getGeoBlockColCount()+1];
    nScans = 0;
  }
  /****************************************************************************
  * doScan.
  ****************************************************************************/
  public void doScan (GeoBlock geoBlock, Color[][] colors) {
    double[][] geoLats = geoBlock.getLats();
    double[][] geoLons = geoBlock.getLons();
    nScans++;
    for (int geoRow = 0; geoRow < nGeoRows; geoRow++) {
       for (int geoCol = 0; geoCol < nGeoCols; geoCol++) {
          loadGeo(geoLats[geoRow][geoCol],geoLats[geoRow][geoCol+1],
          	  geoLats[geoRow+1][geoCol],geoLats[geoRow+1][geoCol+1],
		  geoBinSize,lats);
          loadGeo(geoLons[geoRow][geoCol],geoLons[geoRow][geoCol+1],
          	  geoLons[geoRow+1][geoCol],geoLons[geoRow+1][geoCol+1],
		  geoBinSize,lons);
          for (int rx = 0; rx < geoBinSize; rx++) {
             for (int cx = 0; cx < geoBinSize; cx++) {
		int row = (geoRow * geoBinSize) + rx;
		int col = (geoCol * geoBinSize) + cx;
		gImage.setColor(colors[row][col]);
		mapParms.fillPoly(gImage,lats[rx][cx],lons[rx][cx],
				         lats[rx][cx+1],lons[rx][cx+1],
				         lats[rx+1][cx+1],lons[rx+1][cx+1],
				         lats[rx+1][cx],lons[rx+1][cx]);
             }
          }
          if (geoRow == 0 && nScans == 1) {
	    gIndicator.setColor(Color.YELLOW);
            if (Math.abs(lons[0][0]-lons[0][geoBinSize])>350)
            {
	      // Between +-180, don't drew pass outline.
            }
            else
            {
	      mapParms.drawLine(gIndicator,lats[0][0],lons[0][0],lats[0][geoBinSize],lons[0][geoBinSize]);
            }
          }
	  if (geoCol == 0) {
	    gIndicator.setColor(Color.YELLOW);
            if (Math.abs(lons[0][0]-lons[geoBinSize][0])>350)
            {
	      // Between +-180, don't drew pass outline.
            }
            else
            {
              mapParms.drawLine(gIndicator,lats[0][0],lons[0][0],lats[geoBinSize][0],lons[geoBinSize][0]);
            }
          }
          if (geoCol == nGeoCols-1) {
	    gIndicator.setColor(Color.YELLOW);
            if (Math.abs(lons[0][geoBinSize]-lons[geoBinSize][geoBinSize])>350)
            {
	      // Between +-180, don't drew pass outline.
            }
            else
            {
	      mapParms.drawLine(gIndicator,lats[0][geoBinSize],lons[0][geoBinSize],lats[geoBinSize][geoBinSize],lons[geoBinSize][geoBinSize]);
            }
          }
          if (geoRow == nGeoRows-1) {
            doneLats[geoCol] = lats[geoBinSize][0];
            doneLats[geoCol+1] = lats[geoBinSize][geoBinSize];
            doneLons[geoCol] = lons[geoBinSize][0];
            doneLons[geoCol+1] = lons[geoBinSize][geoBinSize];
          }
       }
    }
  }
  /****************************************************************************
  * doFooter.
  ****************************************************************************/
  public void doFooter () {
    if (nScans > 0) {
      gIndicator.setColor(Color.YELLOW);
      for (int i = 0; i < doneLats.length-1; i++) {
        if (Math.abs(doneLons[i]-doneLons[i+1])>350)
        {
          // Between +-180, don't drew pass outline.
          if (doneLons[i]>170 && doneLons[i+1]<-170)
          {
            mapParms.drawLine(gIndicator,doneLats[i],doneLons[i],doneLats[i],180);
            mapParms.drawLine(gIndicator,doneLats[i+1],-180,doneLats[i+1],doneLons[i+1]);
          }
          if (doneLons[i]<-170 && doneLons[i+1]>170)
          {
            mapParms.drawLine(gIndicator,doneLats[i],doneLons[i],doneLats[i],-180);
            mapParms.drawLine(gIndicator,doneLats[i+1],180,doneLats[i+1],doneLons[i+1]);
          }
        }
        else
        {
          mapParms.drawLine(gIndicator,doneLats[i],doneLons[i],doneLats[i+1],doneLons[i+1]);
        }
      }
    }
    /*
    doneLats = null;
    doneLons = null;
    lats = null;
    lons = null;
    done = true;
    */
  }
  /****************************************************************************
  * draw.
  ****************************************************************************/
  /*
  public void draw (int nGeoDataRows, int nGeoDataCols, int geoBinSize,
  		    double[][] geoLats, double[][] geoLons, Color[][] colors) {
    if (done) {
      image = new BufferedImage(mapParms.getWidth(),mapParms.getHeight(),BufferedImage.TYPE_INT_ARGB);
      gImage = image.getGraphics();
      gImage.setClip(mapParms.getViewport());
      indicator = new BufferedImage(mapParms.getWidth(),mapParms.getHeight(),BufferedImage.TYPE_INT_ARGB);
      gIndicator = indicator.getGraphics();
      gIndicator.setClip(mapParms.getViewport());
      done = false;
      first = true;
    }
    if (first) {
      lats = new double[geoBinSize+1][geoBinSize+1];
      lons = new double[geoBinSize+1][geoBinSize+1];
      doneLats = new double[nGeoDataCols+1];
      doneLons = new double[nGeoDataCols+1];
    }
    for (int geoRow = 0; geoRow < nGeoDataRows; geoRow++) {
       for (int geoCol = 0; geoCol < nGeoDataCols; geoCol++) {
          loadGeo(geoLats[geoRow][geoCol],geoLats[geoRow][geoCol+1],
          	  geoLats[geoRow+1][geoCol],geoLats[geoRow+1][geoCol+1],
		  geoBinSize,lats);
          loadGeo(geoLons[geoRow][geoCol],geoLons[geoRow][geoCol+1],
          	  geoLons[geoRow+1][geoCol],geoLons[geoRow+1][geoCol+1],
		  geoBinSize,lons);
          for (int rx = 0; rx < geoBinSize; rx++) {
             for (int cx = 0; cx < geoBinSize; cx++) {
		int row = (geoRow * geoBinSize) + rx;
		int col = (geoCol * geoBinSize) + cx;
		gImage.setColor(colors[row][col]);
		mapParms.fillPoly(gImage,lats[rx][cx],lons[rx][cx],
				         lats[rx][cx+1],lons[rx][cx+1],
				         lats[rx+1][cx+1],lons[rx+1][cx+1],
				         lats[rx+1][cx],lons[rx+1][cx]);
             }
          }
          if (geoRow == 0 && first) {
	    gIndicator.setColor(Color.YELLOW);
	    mapParms.drawLine(gIndicator,lats[0][0],lons[0][0],lats[0][geoBinSize],lons[0][geoBinSize]);
          }
	  if (geoCol == 0) {
	    gIndicator.setColor(Color.YELLOW);
	    mapParms.drawLine(gIndicator,lats[0][0],lons[0][0],lats[geoBinSize][0],lons[geoBinSize][0]);
          }
          if (geoCol == nGeoDataCols-1) {
	    gIndicator.setColor(Color.YELLOW);
	    mapParms.drawLine(gIndicator,lats[0][geoBinSize],lons[0][geoBinSize],lats[geoBinSize][geoBinSize],lons[geoBinSize][geoBinSize]);
          }
          if (geoRow == nGeoDataRows-1) {
            doneLats[geoCol] = lats[geoBinSize][0];
            doneLats[geoCol+1] = lats[geoBinSize][geoBinSize];
            doneLons[geoCol] = lons[geoBinSize][0];
            doneLons[geoCol+1] = lons[geoBinSize][geoBinSize];
          }
       }
    }
    first = false;
  }
  */
  /****************************************************************************
  * loadGeo.
  ****************************************************************************/
  private void loadGeo (double geo00, double geo01, double geo10, double geo11,
          		int binSize, double[][] geos) {
    // Corners.
    geos[0][0] = geo00;
    geos[0][binSize] = geo01;
    geos[binSize][0] = geo10;
    geos[binSize][binSize] = geo11;
    if (binSize > 1) {
      // Top.
      double geoJump = (geos[0][binSize] - geos[0][0]) / binSize;
      for (int cx = 1; cx < binSize; cx++) {
         geos[0][cx] = geos[0][0] + (geoJump * cx);
      }
      // Bottom.
      geoJump = (geos[binSize][binSize] - geos[binSize][0]) / binSize;
      for (int cx = 1; cx < binSize; cx++) {
         geos[binSize][cx] = geos[binSize][0] + (geoJump * cx);
      }
      // Middle.
      for (int cx = 0; cx <= binSize; cx++) {
         geoJump = (geos[binSize][cx] - geos[0][cx]) / binSize;
         for (int rx = 1; rx < binSize; rx++) {
            geos[rx][cx] = geos[0][cx] + (geoJump * rx);
         }
      }
    }
  }
  /****************************************************************************
  * done.
  ****************************************************************************/
  /*
  public void done () {
    gIndicator.setColor(Color.YELLOW);
    for (int i = 0; i < doneLats.length-1; i++) {
       mapParms.drawLine(gIndicator,doneLats[i],doneLons[i],doneLats[i+1],doneLons[i+1]);
    }
    doneLats = null;
    doneLons = null;
    lats = null;
    lons = null;
    done = true;
  }
  */
}
