/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*  18-Oct-06, J Love, GST	Original version.
*  10-Jan-07, J Love, GST	Appearance tweaking.
*  26-Feb-07, J Love, GST	More tool tips.
*  15-Mar-07, Z Yin,  GST       Display image resolution choices equal or larger
*                               then data resolution.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.viewer;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;

public class ModisAutoDayNightVisual extends HPanel implements Visual {
  private int nBlockRows;
  private int nBlockCols;
  private CustomCombo resolutionCombo;
  private EnhancementAlgorithm dayAlgorithm = null;
  private CustomCombo dayEnhancementCombo;
  private PictureControls dayPictureControls;
  private CustomCombo nightBandCombo;
  private EnhancementAlgorithm nightAlgorithm = null;
  private CustomCombo nightEnhancementCombo;
  private PictureControls nightPictureControls;
  private CustomCombo solarCombo;
  private HPanel solarPanel;
  private HPanel dayPanel;
  private HPanel nightPanel;
  private Color[][][] colors;
  private int[] values = new int[3];
  private Color[] grays = GUtil.getGrays();
  private static final int DAY = 0;
  private static final int NIGHT = 1;
  private static final String[] solarChoices = new String[] {"Day","Night"};
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * ModisAutoDayNightVisual.
  ****************************************************************************/
  public ModisAutoDayNightVisual () {
    this(Modis.resolutionChoices[2],
	 Modis.dayEnhancementChoices[Modis.DAY_ENHANCEMENT_2],1.00,0,
	 Modis.INFRARED_BAND,Modis.nightEnhancementChoices[Modis.NIGHT_ENHANCEMENT_NONE],2.50,50);
  }
  public ModisAutoDayNightVisual (String resolutionChoice,
				  String dayEnhancementChoice,
				  double dayContrast, int dayBrightness,
				  String infraredBand, String nightEnhancementChoice,
				  double nightContrast, int nightBrightness) {
    resolutionCombo = new CustomCombo(Modis.resolutionChoices,resolutionChoice);
    resolutionCombo.setToolTipText("Resolution of instrument data to be displayed.");
    dayEnhancementCombo = new CustomCombo(Modis.dayEnhancementChoices,dayEnhancementChoice);
    dayEnhancementCombo.setToolTipText("Enhancement algorithm to be applied to day scans (before contrast and brightness adjustments).");
    dayEnhancementCombo.addActionListener(new DayEnhancementSelected());
    setDayEnhancementAlgorithm();
    dayPictureControls = new PictureControls(dayContrast,dayBrightness);
    nightBandCombo = new CustomCombo(Modis.emissiveBandChoices,infraredBand);
    nightBandCombo.setToolTipText("Emissive band to display.");
    nightEnhancementCombo = new CustomCombo(Modis.nightEnhancementChoices,nightEnhancementChoice);
    nightEnhancementCombo.setToolTipText("Enhancement algorithm to be applied to night scans (before contrast and brightness adjustments).");
    nightEnhancementCombo.addActionListener(new NightEnhancementSelected());
    setNightEnhancementAlgorithm();
    nightPictureControls = new PictureControls(nightContrast,nightBrightness);
    dayPanel = new HPanel(new VPanel(TOP_ALIGNMENT,new HPanel(LEFT_ALIGNMENT,GUtil.HSpace(1),new CustomLabel("Enhancement")),new HPanel(LEFT_ALIGNMENT,dayEnhancementCombo)),GUtil.HSpace(10),new VPanel(TOP_ALIGNMENT,dayPictureControls));
    nightPanel = new HPanel(new VPanel(TOP_ALIGNMENT,new HPanel(LEFT_ALIGNMENT,GUtil.HSpace(1),new CustomLabel("Enhancement")),new HPanel(LEFT_ALIGNMENT,nightEnhancementCombo)),GUtil.HSpace(10),new VPanel(TOP_ALIGNMENT,new HPanel(LEFT_ALIGNMENT,GUtil.HSpace(1),new CustomLabel("Band")),new HPanel(LEFT_ALIGNMENT,nightBandCombo)),GUtil.HSpace(10),new VPanel(TOP_ALIGNMENT,nightPictureControls));
    solarPanel = new HPanel(dayPanel);
    solarCombo = new CustomCombo(solarChoices);
    solarCombo.setToolTipText("Instrument mode (day or night).");
    solarCombo.addActionListener(new SolarSelected());
    VPanel v1 = new VPanel(TOP_ALIGNMENT,new HPanel(LEFT_ALIGNMENT,GUtil.HSpace(1),new CustomLabel("Resolution")),new HPanel(LEFT_ALIGNMENT,resolutionCombo));
    VPanel v2 = new VPanel(TOP_ALIGNMENT,new HPanel(LEFT_ALIGNMENT,GUtil.HSpace(1),new CustomLabel("Mode")),new HPanel(LEFT_ALIGNMENT,solarCombo));
    add(new HPanel(v1,GUtil.HSpace(10),new HPanel(new VPanel(TOP_ALIGNMENT,solarPanel),GUtil.HSpace(10),v2)));
  }
  /****************************************************************************
  * newInstance.
  ****************************************************************************/
  public ModisAutoDayNightVisual newInstance () {
    // System.out.println("\n Create day visual.");
    return new ModisAutoDayNightVisual(resolutionCombo.getSelection(),
				       dayEnhancementCombo.getSelection(),
				       dayPictureControls.getContrast(),
				       dayPictureControls.getBrightness(),
				       nightBandCombo.getSelection(),
				       nightEnhancementCombo.getSelection(),
				       nightPictureControls.getContrast(),
				       nightPictureControls.getBrightness());
  }
  /****************************************************************************
  * getRequestBandsElement.
  ****************************************************************************/
  public String getRequestBandsElement () {
    StringBuffer msg = new StringBuffer();
    msg.append("<bands>");
    msg.append("<band>");
    msg.append("<bandIndex>" + Integer.toString(Modis.TRUE_COLOR_RED_BAND_INDEX) + "</bandIndex>");
    msg.append("<resolution>" + Modis.resolutionValues[Modis.resolutionIndex(resolutionCombo.getSelection())] + "</resolution>");
    msg.append("</band>");
    msg.append("<band>");
    msg.append("<bandIndex>" + Integer.toString(Modis.TRUE_COLOR_GREEN_BAND_INDEX) + "</bandIndex>");
    msg.append("<resolution>" + Modis.resolutionValues[Modis.resolutionIndex(resolutionCombo.getSelection())] + "</resolution>");
    msg.append("</band>");
    msg.append("<band>");
    msg.append("<bandIndex>" + Integer.toString(Modis.TRUE_COLOR_BLUE_BAND_INDEX) + "</bandIndex>");
    msg.append("<resolution>" + Modis.resolutionValues[Modis.resolutionIndex(resolutionCombo.getSelection())] + "</resolution>");
    msg.append("</band>");
    msg.append("<band>");
    msg.append("<bandIndex>" + Integer.toString(Modis.INFRARED_BAND_INDEX) + "</bandIndex>");
    msg.append("<resolution>" + Modis.resolutionValues[Modis.resolutionIndex(resolutionCombo.getSelection())] + "</resolution>");
    msg.append("</band>");
    msg.append("</bands>");
    return msg.toString();
  }
  /****************************************************************************
  * doHeader.
  ****************************************************************************/
  public void doHeader (HeaderBlock headerBlock) throws Exception {
    if (headerBlock.getBandCount() != 4) {
      throw new Exception("unexpected number of bands");
    }
    if (headerBlock.getBandIndex(0) != Modis.TRUE_COLOR_RED_BAND_INDEX) {
      throw new Exception("unexpected red band index");
    }
    int resolution = headerBlock.getBandResolution(0);
    nBlockRows = headerBlock.getBlockRowCount(0);
    nBlockCols = headerBlock.getBlockColCount(0);
    if (headerBlock.getBandIndex(1) != Modis.TRUE_COLOR_GREEN_BAND_INDEX) {
      throw new Exception("unexpected green band index");
    }
    if (headerBlock.getBandResolution(1) != resolution) {
      throw new Exception("inconsistent resolutions");
    }
    if (headerBlock.getBlockRowCount(1) != nBlockRows) {
      throw new Exception("inconsistent block rows");
    }
    if (headerBlock.getBlockColCount(1) != nBlockCols) {
      throw new Exception("inconsistent block columns");
    }
    if (headerBlock.getBandIndex(2) != Modis.TRUE_COLOR_BLUE_BAND_INDEX) {
      throw new Exception("unexpected blue band index");
    }
    if (headerBlock.getBandResolution(2) != resolution) {
      throw new Exception("inconsistent resolutions");
    }
    if (headerBlock.getBlockRowCount(2) != nBlockRows) {
      throw new Exception("inconsistent block rows");
    }
    if (headerBlock.getBlockColCount(2) != nBlockCols) {
      throw new Exception("inconsistent block columns");
    }
    if (headerBlock.getBandIndex(3) != Modis.INFRARED_BAND_INDEX) {
      throw new Exception("unexpected infrared band index");
    }
    if (headerBlock.getBandResolution(3) != resolution) {
      throw new Exception("inconsistent resolutions");
    }
    if (headerBlock.getBlockRowCount(3) != nBlockRows) {
      throw new Exception("inconsistent block rows");
    }
    if (headerBlock.getBlockColCount(3) != nBlockCols) {
      throw new Exception("inconsistent block columns");
    }
    if (headerBlock.isGeolocated()) {
      int geoBinSize = headerBlock.getGeoResolution() / resolution;
      int nGeoDataRows = headerBlock.getGeoBlockRowCount();
      if ((nGeoDataRows * geoBinSize) != nBlockRows) {
        throw new Exception("inconsistent geolocation block rows");
      }
      int nGeoDataCols = headerBlock.getGeoBlockColCount();
      if ((nGeoDataCols * geoBinSize) != nBlockCols) {
        throw new Exception("inconsistent geolocation block columns");
      }
    }
    colors = new Color[1][nBlockRows][nBlockCols];
  }
  /****************************************************************************
  * doScan.
  ****************************************************************************/
  public Color[][][] doScan (ScanBlock scanBlock, byte[][][] data) throws Exception {
    switch (scanBlock.getSolarCode()) {
      case Simulcast.DAY_PASS_CODE: {
	if (solarCombo.getSelectedIndex() != DAY) {
	  GUtil.setCombo(solarCombo,DAY);
	}
      	return dayScan(data);
      }
      case Simulcast.NIGHT_PASS_CODE: {
	if (solarCombo.getSelectedIndex() != NIGHT) {
	  GUtil.setCombo(solarCombo,NIGHT);
	}
      	return nightScan(data);
      }
    }
    throw new Exception("unknown solar code");
  }
  private Color[][][] dayScan (byte[][][] data) throws Exception {
    for (int row = 0; row < nBlockRows; row++) {
       for (int col = 0; col < nBlockCols; col++) {
          values[0] = data[0][row][col] & 0xFF;
          values[1] = data[1][row][col] & 0xFF;
          values[2] = data[2][row][col] & 0xFF;
          if (dayAlgorithm != null) {
            dayAlgorithm.enhance(values);
          }
          values[0] = dayPictureControls.adjustValue(values[0]);
          values[1] = dayPictureControls.adjustValue(values[1]);
          values[2] = dayPictureControls.adjustValue(values[2]);
          colors[0][row][col] = new Color(values[0],values[1],values[2]);
       }
    }
    return colors;
  }
  private Color[][][] nightScan (byte[][][] data) throws Exception {
    for (int row = 0; row < nBlockRows; row++) {
       for (int col = 0; col < nBlockCols; col++) {
          int value = data[3][row][col] & 0xFF;
          if (nightAlgorithm != null) {
            nightAlgorithm.enhance(new int[] {value});
          }
          value = 255 - nightPictureControls.adjustValue(value);
	  colors[0][row][col] = grays[value];
       }
    }
    return colors;
  }
  /****************************************************************************
  * setControls.
  ****************************************************************************/
  public void setControls (boolean enabled) {
    resolutionCombo.setEnabled(enabled);
    // dayEnhancementCombo.setEnabled(enabled);
    // nightEnhancementCombo.setEnabled(enabled);
    solarCombo.setEnabled(enabled);
  }
  /****************************************************************************
  * getResolutionCombo.
  ****************************************************************************/
  public CustomCombo getResolutionCombo()
  {
    return resolutionCombo;
  }
  /****************************************************************************
  * SolarSelected.
  ****************************************************************************/
  private class SolarSelected implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      solarPanel.removeAll();
      switch (solarCombo.getSelectedIndex()) {
        case DAY: {
          solarPanel.add(dayPanel);
          break;
        }
        case NIGHT: {
          solarPanel.add(nightPanel);
          break;
        }
      }
      solarPanel.revalidate();
      solarPanel.repaint();
    }
  }
  /****************************************************************************
  * DayEnhancementSelected.
  ****************************************************************************/
  private class DayEnhancementSelected implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      setDayEnhancementAlgorithm();
    }
  }
  private void setDayEnhancementAlgorithm () {
    String className = Modis.dayEnhancementClassNames[dayEnhancementCombo.getSelectedIndex()];
    if (className == null) {
      dayAlgorithm = null;
    } else {
      try {
	dayAlgorithm = (EnhancementAlgorithm) Class.forName(className).newInstance();
      } catch (Exception e) {
        dayAlgorithm = null;
      }
    }
  }
  /****************************************************************************
  * NightEnhancementSelected.
  ****************************************************************************/
  private class NightEnhancementSelected implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      setNightEnhancementAlgorithm();
    }
  }
  private void setNightEnhancementAlgorithm () {
    String className = Modis.dayEnhancementClassNames[nightEnhancementCombo.getSelectedIndex()];
    if (className == null) {
      nightAlgorithm = null;
    } else {
      try {
	nightAlgorithm = (EnhancementAlgorithm) Class.forName(className).newInstance();
      } catch (Exception e) {
        nightAlgorithm = null;
      }
    }
  }
  /****************************************************************************
  * set.
  ****************************************************************************/
  public void set (XmlParser parser) throws Exception {
    resolutionCombo.setSelection(parser.getElementValue("resolution"));
    dayEnhancementCombo.setSelection(parser.getElement("day").getElementValue("enhancement"));
    dayPictureControls.setContrast(Double.parseDouble(parser.getElement("day").getElementValue("contrast")));
    dayPictureControls.setBrightness(Integer.parseInt(parser.getElement("day").getElementValue("brightness")));
    nightEnhancementCombo.setSelection(parser.getElement("night").getElementValue("enhancement"));
    nightPictureControls.setContrast(Double.parseDouble(parser.getElement("night").getElementValue("contrast")));
    nightPictureControls.setBrightness(Integer.parseInt(parser.getElement("night").getElementValue("brightness")));
  }
  /****************************************************************************
  * toXml.
  ****************************************************************************/
  public void toXml (PrintWriter out) throws Exception {
    out.println("  <visual name=\"" + Util.encodeContent(VisualSelector.visualChoices[VisualSelector.MODIS_AUTO_DAY_NIGHT_VISUAL]) + "\">");
    out.println("    <resolution>" + Util.encodeContent(resolutionCombo.getSelection()) + "</resolution>");
    out.println("    <day>");
    out.println("      <enhancement>" + Util.encodeContent(dayEnhancementCombo.getSelection()) + "</enhancement>");
    out.println("      <contrast>" + Double.toString(dayPictureControls.getContrast()) + "</contrast>");
    out.println("      <brightness>" + Integer.toString(dayPictureControls.getBrightness()) + "</brightness>");
    out.println("    </day>");
    out.println("    <night>");
    out.println("      <enhancement>" + Util.encodeContent(nightEnhancementCombo.getSelection()) + "</enhancement>");
    out.println("      <contrast>" + Double.toString(nightPictureControls.getContrast()) + "</contrast>");
    out.println("      <brightness>" + Integer.toString(nightPictureControls.getBrightness()) + "</brightness>");
    out.println("    </night>");
    out.println("  </visual>");
  }
}
