/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*  18-Oct-06, J Love, GST	Original version.
*  10-Jan-07, J Love, GST	Appearance tweaking.
*  26-Feb-07, J Love, GST	More tool tips.
*  15-Mar-07, Z Yin,  GST       Display image resolution choices equal or larger
*                               then data resolution.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.viewer;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.Socket;

public class ModisFalseColorVisual extends HPanel implements Visual {
  private int nBlockRows;
  private int nBlockCols;
  private CustomCombo resolutionCombo;
  private CustomCombo colorCombo;
  private CustomCombo redBandCombo;
  private CustomCombo greenBandCombo;
  private CustomCombo blueBandCombo;
  private PictureControls redPictureControls;
  private PictureControls greenPictureControls;
  private PictureControls bluePictureControls;
  private HPanel colorPanel;
  private HPanel[] colorPanels;
  private Color[][][] colors;
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * ModisFalseColorVisual.
  ****************************************************************************/
  public ModisFalseColorVisual () {
    this(Modis.resolutionChoices[2],"1",1.00,0,"1",1.00,0,"1",1.00,0);
  }
  public ModisFalseColorVisual (String resolutionChoice,
				String redBand, double redContrast, int redBrightness,
				String greenBand, double greenContrast, int greenBrightness,
				String blueBand, double blueContrast, int blueBrightness) {
    resolutionCombo = new CustomCombo(Modis.resolutionChoices,resolutionChoice);
    resolutionCombo.setToolTipText("Resolution of instrument data to be displayed.");
    colorCombo = new CustomCombo(new String[] {"Red","Green","Blue"});
    colorCombo.setToolTipText("False color component.");
    colorCombo.addActionListener(new ColorSelected());
    redBandCombo = new CustomCombo(Modis.dayBandChoices,redBand);
    redBandCombo.setToolTipText("Band for the red component of the false-color composite.");
    redBandCombo.addActionListener(new RedSelected());
    greenBandCombo = new CustomCombo(Modis.dayBandChoices,greenBand);
    greenBandCombo.setToolTipText("Band for the green component of the false-color composite.");
    greenBandCombo.addActionListener(new GreenSelected());
    blueBandCombo = new CustomCombo(Modis.dayBandChoices,blueBand);
    blueBandCombo.setToolTipText("Band for the blue component of the false-color composite.");
    blueBandCombo.addActionListener(new BlueSelected());
    redPictureControls = new PictureControls(redContrast,redBrightness);
    greenPictureControls = new PictureControls(greenContrast,greenBrightness);
    bluePictureControls = new PictureControls(blueContrast,blueBrightness);
    colorPanels = new HPanel[] {
      new HPanel(new Component[] {new VPanel(TOP_ALIGNMENT,new HPanel(LEFT_ALIGNMENT,GUtil.HSpace(1),new CustomLabel("Band")),new HPanel(LEFT_ALIGNMENT,redBandCombo)),GUtil.HSpace(15),redPictureControls}),
      new HPanel(new Component[] {new VPanel(TOP_ALIGNMENT,new HPanel(LEFT_ALIGNMENT,GUtil.HSpace(1),new CustomLabel("Band")),new HPanel(LEFT_ALIGNMENT,greenBandCombo)),GUtil.HSpace(15),greenPictureControls}),
      new HPanel(new Component[] {new VPanel(TOP_ALIGNMENT,new HPanel(LEFT_ALIGNMENT,GUtil.HSpace(1),new CustomLabel("Band")),new HPanel(LEFT_ALIGNMENT,blueBandCombo)),GUtil.HSpace(15),bluePictureControls})
    };
    colorPanel = new HPanel(colorPanels[0]);
    VPanel v1 = new VPanel(TOP_ALIGNMENT,new HPanel(LEFT_ALIGNMENT,GUtil.HSpace(1),new CustomLabel("Resolution")),new HPanel(LEFT_ALIGNMENT,resolutionCombo));
    VPanel v2 = new VPanel(TOP_ALIGNMENT,new HPanel(LEFT_ALIGNMENT,GUtil.HSpace(1),new CustomLabel("Color")),new HPanel(LEFT_ALIGNMENT,colorCombo));
    add(new HPanel(new Component[] {v1,GUtil.HSpace(10),v2,GUtil.HSpace(10),colorPanel}));
  }
  /****************************************************************************
  * newInstance.
  ****************************************************************************/
  public ModisFalseColorVisual newInstance () {
    // System.out.println("\n Create false visual.");
    return new ModisFalseColorVisual(resolutionCombo.getSelection(),
				     redBandCombo.getSelection(),
				     redPictureControls.getContrast(),
				     redPictureControls.getBrightness(),
				     greenBandCombo.getSelection(),
				     greenPictureControls.getContrast(),
				     greenPictureControls.getBrightness(),
				     blueBandCombo.getSelection(),
				     bluePictureControls.getContrast(),
				     bluePictureControls.getBrightness());
  }
  /****************************************************************************
  * getRequestBandsElement.
  ****************************************************************************/
  public String getRequestBandsElement () {
    StringBuffer xml = new StringBuffer();
    xml.append("<bands>");
    xml.append("<band>");
    xml.append("<bandIndex>" + Integer.toString(redBandCombo.getSelectedIndex()) + "</bandIndex>");
    xml.append("<resolution>" + Modis.resolutionValues[Modis.resolutionIndex(resolutionCombo.getSelection())] + "</resolution>");
    xml.append("</band>");
    xml.append("<band>");
    xml.append("<bandIndex>" + Integer.toString(greenBandCombo.getSelectedIndex()) + "</bandIndex>");
    xml.append("<resolution>" + Modis.resolutionValues[Modis.resolutionIndex(resolutionCombo.getSelection())] + "</resolution>");
    xml.append("</band>");
    xml.append("<band>");
    xml.append("<bandIndex>" + Integer.toString(blueBandCombo.getSelectedIndex()) + "</bandIndex>");
    xml.append("<resolution>" + Modis.resolutionValues[Modis.resolutionIndex(resolutionCombo.getSelection())] + "</resolution>");
    xml.append("</band>");
    xml.append("</bands>");
    return xml.toString();
  }
  /****************************************************************************
  * doHeader.
  ****************************************************************************/
  public void doHeader (HeaderBlock headerBlock) throws Exception {
    if (headerBlock.getBandCount() != 3) {
      throw new Exception("unexpected number of bands");
    }
    if (headerBlock.getBandIndex(0) != redBandCombo.getSelectedIndex()) {
      throw new Exception("unexpected red band index");
    }
    int resolution = headerBlock.getBandResolution(0);
    nBlockRows = headerBlock.getBlockRowCount(0);
    nBlockCols = headerBlock.getBlockColCount(0);
    if (headerBlock.getBandIndex(1) != greenBandCombo.getSelectedIndex()) {
      throw new Exception("unexpected green band index");
    }
    if (headerBlock.getBandResolution(1) != resolution) {
      throw new Exception("inconsistent resolutions");
    }
    if (headerBlock.getBlockRowCount(1) != nBlockRows) {
      throw new Exception("inconsistent block rows");
    }
    if (headerBlock.getBlockColCount(1) != nBlockCols) {
      throw new Exception("inconsistent block columns");
    }
    if (headerBlock.getBandIndex(2) != blueBandCombo.getSelectedIndex()) {
      throw new Exception("unexpected blue band index");
    }
    if (headerBlock.getBandResolution(2) != resolution) {
      throw new Exception("inconsistent resolutions");
    }
    if (headerBlock.getBlockRowCount(2) != nBlockRows) {
      throw new Exception("inconsistent block rows");
    }
    if (headerBlock.getBlockColCount(2) != nBlockCols) {
      throw new Exception("inconsistent block columns");
    }
    if (headerBlock.isGeolocated()) {
      int geoBinSize = headerBlock.getGeoResolution() / resolution;
      int nGeoDataRows = headerBlock.getGeoBlockRowCount();
      if ((nGeoDataRows * geoBinSize) != nBlockRows) {
        throw new Exception("inconsistent geolocation block rows");
      }
      int nGeoDataCols = headerBlock.getGeoBlockColCount();
      if ((nGeoDataCols * geoBinSize) != nBlockCols) {
        throw new Exception("inconsistent geolocation block columns");
      }
    }
    colors = new Color[1][nBlockRows][nBlockCols];
  }
  /****************************************************************************
  * doScan.
  ****************************************************************************/
  public Color[][][] doScan (ScanBlock scanBlock, byte[][][] data) throws Exception {
    for (int row = 0; row < nBlockRows; row++) {
       for (int col = 0; col < nBlockCols; col++) {
          int redValue = data[0][row][col] & 0xFF;
          int greenValue = data[1][row][col] & 0xFF;
          int blueValue = data[2][row][col] & 0xFF;
          redValue = redPictureControls.adjustValue(redValue);
          greenValue = greenPictureControls.adjustValue(greenValue);
          blueValue = bluePictureControls.adjustValue(blueValue);
          colors[0][row][col] = new Color(redValue,greenValue,blueValue);
       }
    }
    return colors;
  }
  /****************************************************************************
  * setControls.
  ****************************************************************************/
  public void setControls (boolean enabled) {
    resolutionCombo.setEnabled(enabled);
    redBandCombo.setEnabled(enabled);
    greenBandCombo.setEnabled(enabled);
    blueBandCombo.setEnabled(enabled);
  }
  /****************************************************************************
  * getRedBandCombo.
  ****************************************************************************/
  public CustomCombo getRedBandCombo()
  {
    return redBandCombo;
  }
  /****************************************************************************
  * getGreenBandCombo.
  ****************************************************************************/
  public CustomCombo getGreenBandCombo()
  {
    return greenBandCombo;
  }
  /****************************************************************************
  * getBlueBandCombo.
  ****************************************************************************/
  public CustomCombo getBlueBandCombo()
  {
    return blueBandCombo;
  }
  /****************************************************************************
  * getResolutionCombo.
  ****************************************************************************/
  public CustomCombo getResolutionCombo()
  {
    return resolutionCombo;
  }
  /****************************************************************************
  * ColorSelected.
  ****************************************************************************/
  private class ColorSelected implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      colorPanel.removeAll();
      colorPanel.add(colorPanels[colorCombo.getSelectedIndex()]);
      colorPanel.revalidate();
      colorPanel.repaint();
    }
  }
  /****************************************************************************
  * RedSelected.
  ****************************************************************************/
  private class RedSelected implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      String passName=null;

      try
      {
        if (Viewer.mapLayoutButton.isSelected())
        {
          passName=Viewer.mapLayout.getPlayerManager().getPlayerCombo().getSelection();

          // Send message to server to get BiggestInResolution from bands.
          Viewer.serverHandler.send(Util.XML_HEADER + "<simulcastMessage>" +
          "<redBand>" + "<name>" + passName + "</name>" +
          "</redBand>" + "</simulcastMessage>");
        }

        if (Viewer.instrumentLayoutButton.isSelected())
        {
          passName=Viewer.instrumentLayout.getPlayerManager().getPlayerCombo().getSelection();

          // Send message to server to get BiggestInResolution from bands.
          Viewer.serverHandler.send(Util.XML_HEADER + "<simulcastMessage>" +
          "<redBand>" + "<name>" + passName + "</name>" +
          "</redBand>" + "</simulcastMessage>");
        }
      }
      catch (NullPointerException ie)
      {
        // System.out.println("\n No passName at start of viewer");
        return;
      }
    }
  }
  /****************************************************************************
  * GreenSelected.
  ****************************************************************************/
  private class GreenSelected implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      String passName=null;

      try
      {
        if (Viewer.mapLayoutButton.isSelected())
        {
          passName=Viewer.mapLayout.getPlayerManager().getPlayerCombo().getSelection();

          // Send message to server to get BiggestInResolution from bands.
          Viewer.serverHandler.send(Util.XML_HEADER + "<simulcastMessage>" +
          "<greenBand>" + "<name>" + passName + "</name>" +
          "</greenBand>" + "</simulcastMessage>");
        }

        if (Viewer.instrumentLayoutButton.isSelected())
        {
          passName=Viewer.instrumentLayout.getPlayerManager().getPlayerCombo().getSelection();

          // Send message to server to get BiggestInResolution from bands.
          Viewer.serverHandler.send(Util.XML_HEADER + "<simulcastMessage>" +
          "<greenBand>" + "<name>" + passName + "</name>" +
          "</greenBand>" + "</simulcastMessage>");
        }
      }
      catch (NullPointerException ie)
      {
        // System.out.println("\n No passName at start of viewer");
        return;
      }
    }
  }
  /****************************************************************************
  * BlueSelected.
  ****************************************************************************/
  private class BlueSelected implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      String passName=null;

      try
      {
        if (Viewer.mapLayoutButton.isSelected())
        {
          passName=Viewer.mapLayout.getPlayerManager().getPlayerCombo().getSelection();

          // Send message to server to get BiggestInResolution from bands.
          Viewer.serverHandler.send(Util.XML_HEADER + "<simulcastMessage>" +
          "<blueBand>" + "<name>" + passName + "</name>" +
          "</blueBand>" + "</simulcastMessage>");
        }

        if (Viewer.instrumentLayoutButton.isSelected())
        {
          passName=Viewer.instrumentLayout.getPlayerManager().getPlayerCombo().getSelection();

          // Send message to server to get BiggestInResolution from bands.
          Viewer.serverHandler.send(Util.XML_HEADER + "<simulcastMessage>" +
          "<blueBand>" + "<name>" + passName + "</name>" +
          "</blueBand>" + "</simulcastMessage>");
        }
      }
      catch (NullPointerException ie)
      {
        // System.out.println("\n No passName at start of viewer");
        return;
      }
    }
  }
  /****************************************************************************
  * set.
  ****************************************************************************/
  public void set (XmlParser parser) throws Exception {
    resolutionCombo.setSelection(parser.getElementValue("resolution"));
    redBandCombo.setSelection(parser.getElement("red").getElementValue("band"));
    redPictureControls.setContrast(Double.parseDouble(parser.getElement("red").getElementValue("contrast")));
    greenPictureControls.setBrightness(Integer.parseInt(parser.getElement("green").getElementValue("brightness")));
    greenBandCombo.setSelection(parser.getElement("green").getElementValue("band"));
    greenPictureControls.setContrast(Double.parseDouble(parser.getElement("green").getElementValue("contrast")));
    greenPictureControls.setBrightness(Integer.parseInt(parser.getElement("green").getElementValue("brightness")));
    blueBandCombo.setSelection(parser.getElement("blue").getElementValue("band"));
    bluePictureControls.setContrast(Double.parseDouble(parser.getElement("blue").getElementValue("contrast")));
    bluePictureControls.setBrightness(Integer.parseInt(parser.getElement("blue").getElementValue("brightness")));
  }
  /****************************************************************************
  * toXml.
  ****************************************************************************/
  public void toXml (PrintWriter out) throws Exception {
    out.println("  <visual name=\"" + Util.encodeContent(VisualSelector.visualChoices[VisualSelector.MODIS_FALSE_COLOR_VISUAL]) + "\">");
    out.println("    <resolution>" + Util.encodeContent(resolutionCombo.getSelection()) + "</resolution>");
    out.println("    <red>");
    out.println("      <band>" + Util.encodeContent(redBandCombo.getSelection()) + "</band>");
    out.println("      <contrast>" + Double.toString(redPictureControls.getContrast()) + "</contrast>");
    out.println("      <brightness>" + Integer.toString(redPictureControls.getBrightness()) + "</brightness>");
    out.println("    </red>");
    out.println("    <green>");
    out.println("      <band>" + Util.encodeContent(greenBandCombo.getSelection()) + "</band>");
    out.println("      <contrast>" + Double.toString(greenPictureControls.getContrast()) + "</contrast>");
    out.println("      <brightness>" + Integer.toString(greenPictureControls.getBrightness()) + "</brightness>");
    out.println("    </green>");
    out.println("    <blue>");
    out.println("      <band>" + Util.encodeContent(blueBandCombo.getSelection()) + "</band>");
    out.println("      <contrast>" + Double.toString(bluePictureControls.getContrast()) + "</contrast>");
    out.println("      <brightness>" + Integer.toString(bluePictureControls.getBrightness()) + "</brightness>");
    out.println("    </blue>");
    out.println("  </visual>");
  }
}
