/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*  18-Oct-06, J Love, GST	Original version.
*  10-Jan-07, J Love, GST	Appearance tweaking.
*  26-Feb-07, J Love, GST	More tool tips.
*  15-Mar-07, Z Yin,  GST       Added for displaying image resolution choices equal or larger
*                               then data resolution.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.viewer;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.awt.*;
import java.io.*;
import java.awt.event.*;
import java.net.Socket;

public class ModisGrayScaleVisual extends HPanel implements Visual {
  private int nBlockRows;
  private int nBlockCols;
  private CustomCombo bandCombo;
  private CustomCombo resolutionCombo;
  private PictureControls pictureControls;
  private Color[][][] colors;
  private Color[] grays = GUtil.getGrays();
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * ModisGrayScaleVisual.
  ****************************************************************************/
  public ModisGrayScaleVisual () {
    this(Modis.resolutionChoices[2],"1",1.00,0);
  }
  public ModisGrayScaleVisual (String resolutionChoice, String bandChoice,
			       double contrast, int brightness) {
    bandCombo = new CustomCombo(Modis.dayBandChoices,bandChoice);
    bandCombo.setToolTipText("Band to display.");
    bandCombo.setSelectedIndex(0);
    bandCombo.addActionListener(new GraySelected());
    resolutionCombo = new CustomCombo(Modis.resolutionChoices,resolutionChoice);
    resolutionCombo.setToolTipText("Resolution of instrument data to be displayed.");
    pictureControls = new PictureControls(contrast,brightness);
    VPanel v1 = new VPanel(TOP_ALIGNMENT,new HPanel(LEFT_ALIGNMENT,GUtil.HSpace(1),new CustomLabel("Resolution")),new HPanel(LEFT_ALIGNMENT,resolutionCombo));
    VPanel v2 = new VPanel(TOP_ALIGNMENT,new HPanel(LEFT_ALIGNMENT,GUtil.HSpace(1),new CustomLabel("Band")),new HPanel(LEFT_ALIGNMENT,bandCombo));
    add(new HPanel(v1,GUtil.HSpace(10),v2,GUtil.HSpace(15),pictureControls));
  }
  /****************************************************************************
  * newInstance.
  ****************************************************************************/
  public ModisGrayScaleVisual newInstance () {
    // System.out.println("\n Create gray visual.");
    return new ModisGrayScaleVisual(resolutionCombo.getSelection(),
				    bandCombo.getSelection(),
				    pictureControls.getContrast(),
				    pictureControls.getBrightness());
  }
  /****************************************************************************
  * getRequestBandsElement.
  ****************************************************************************/
  public String getRequestBandsElement () {
    StringBuffer xml = new StringBuffer();
    xml.append("<bands>");
    xml.append("<band>");
    xml.append("<bandIndex>" + Integer.toString(Modis.dayBandIndices[bandCombo.getSelectedIndex()]) + "</bandIndex>");
    xml.append("<resolution>" + Modis.resolutionValues[Modis.resolutionIndex(resolutionCombo.getSelection())] + "</resolution>");
    xml.append("</band>");
    xml.append("</bands>");
    return xml.toString();
  }
  /****************************************************************************
  * doHeader.
  ****************************************************************************/
  public void doHeader (HeaderBlock headerBlock) throws Exception {
    if (headerBlock.getBandCount() != 1) {
      throw new Exception("unexpected number of bands");
    }
    if (headerBlock.getBandIndex(0) != Modis.dayBandIndices[bandCombo.getSelectedIndex()]) {
      throw new Exception("unexpected band index");
    }
    int resolution = headerBlock.getBandResolution(0);
    nBlockRows = headerBlock.getBlockRowCount(0);
    nBlockCols = headerBlock.getBlockColCount(0);
    if (headerBlock.isGeolocated()) {
      int geoBinSize = headerBlock.getGeoResolution() / resolution;
      int nGeoDataRows = headerBlock.getGeoBlockRowCount();
      if ((nGeoDataRows * geoBinSize) != nBlockRows) {
        throw new Exception("inconsistent geolocation block rows");
      }
      int nGeoDataCols = headerBlock.getGeoBlockColCount();
      if ((nGeoDataCols * geoBinSize) != nBlockCols) {
        throw new Exception("inconsistent geolocation block columns");
      }
    }
    colors = new Color[1][nBlockRows][nBlockCols];
  }
  /****************************************************************************
  * doScan.
  ****************************************************************************/
  public Color[][][] doScan (ScanBlock scanBlock, byte[][][] data) throws Exception {
    for (int row = 0; row < nBlockRows; row++) {
       for (int col = 0; col < nBlockCols; col++) {
          int value = data[0][row][col] & 0xFF;
          value = pictureControls.adjustValue(value);
	  colors[0][row][col] = grays[value];
       }
    }
    return colors;
  }
  /****************************************************************************
  * setControls.
  ****************************************************************************/
  public void setControls (boolean enabled) {
    resolutionCombo.setEnabled(enabled);
    bandCombo.setEnabled(enabled);
  }
  /****************************************************************************
  * getBandCombo.
  ****************************************************************************/
  public CustomCombo getBandCombo()
  {
    return bandCombo;
  }
  /****************************************************************************
  * getResolutionCombo.
  ****************************************************************************/
  public CustomCombo getResolutionCombo()
  {
    return resolutionCombo;
  }
  /****************************************************************************
  * GraySelected.
  ****************************************************************************/
  private class GraySelected implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      String passName=null;

      try
      {
        if (Viewer.mapLayoutButton.isSelected())
        {
          passName=Viewer.mapLayout.getPlayerManager().getPlayerCombo().getSelection();

          // Send message to server to get BiggestInResolution from bands.
          Viewer.serverHandler.send(Util.XML_HEADER + "<simulcastMessage>" +
          "<grayBand>" + "<name>" + passName + "</name>" +
          "</grayBand>" + "</simulcastMessage>");
        }

        if (Viewer.instrumentLayoutButton.isSelected())
        {
          passName=Viewer.instrumentLayout.getPlayerManager().getPlayerCombo().getSelection();

          // Send message to server to get BiggestInResolution from bands.
          Viewer.serverHandler.send(Util.XML_HEADER + "<simulcastMessage>" +
          "<grayBand>" + "<name>" + passName + "</name>" +
          "</grayBand>" + "</simulcastMessage>");
        }
      }
      catch (NullPointerException ie)
      {
        // System.out.println("\n No passName at start of viewer");
        return;
      }
    }
  }
  /****************************************************************************
  * set.
  ****************************************************************************/
  public void set (XmlParser parser) throws Exception {
    resolutionCombo.setSelection(parser.getElementValue("resolution"));
    bandCombo.setSelection(parser.getElementValue("band"));
    pictureControls.setContrast(Double.parseDouble(parser.getElementValue("contrast")));
    pictureControls.setBrightness(Integer.parseInt(parser.getElementValue("brightness")));
  }
  /****************************************************************************
  * toXml.
  ****************************************************************************/
  public void toXml (PrintWriter out) throws Exception {
    out.println("  <visual name=\"" + VisualSelector.visualChoices[VisualSelector.MODIS_GRAY_SCALE_VISUAL] + "\">");
    out.println("    <resolution>" + Util.encodeContent(resolutionCombo.getSelection()) + "</resolution>");
    out.println("    <band>" + Util.encodeContent(bandCombo.getSelection()) + "</band>");
    out.println("    <contrast>" + Double.toString(pictureControls.getContrast()) + "</contrast>");
    out.println("    <brightness>" + Integer.toString(pictureControls.getBrightness()) + "</brightness>");
    out.println("  </visual>");
  }
}
