/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*  18-Oct-06, J Love, GST	Original version.
*  10-Jan-07, J Love, GST	Appearance tweaking.
*  26-Feb-07, J Love, GST	More tool tips.
*  15-Mar-07, Z Yin,  GST       Display image resolution choices equal or larger
*                               then data resolution.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.viewer;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.Socket;

public class ModisInfraredVisual extends HPanel implements Visual {
  private int nBlockRows;
  private int nBlockCols;
  private CustomCombo resolutionCombo;
  private EnhancementAlgorithm algorithm = null;
  private CustomCombo enhancementCombo;
  private CustomCombo bandCombo;
  private PictureControls pictureControls;
  private Color[][][] colors;
  private Color[] grays = GUtil.getGrays();
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * InfraredMapViewHandler.
  ****************************************************************************/
  public ModisInfraredVisual () {
    this(Modis.resolutionChoices[2],Modis.nightEnhancementChoices[Modis.NIGHT_ENHANCEMENT_NONE],Modis.INFRARED_BAND,2.50,50);
  }
  public ModisInfraredVisual (String resolutionChoice, String enhancementChoice, 
			      String bandChoice, double contrast, int brightness) {
    resolutionCombo = new CustomCombo(Modis.resolutionChoices,resolutionChoice);
    resolutionCombo.setToolTipText("Resolution of instrument data to be displayed.");
    enhancementCombo = new CustomCombo(Modis.nightEnhancementChoices,enhancementChoice);
    enhancementCombo.setToolTipText("Enhancement algorithm to be applied (before contrast and brightness adjustments).");
    enhancementCombo.addActionListener(new EnhancementSelected());
    setEnhancementAlgorithm();
    bandCombo = new CustomCombo(Modis.emissiveBandChoices,bandChoice);
    bandCombo.addActionListener(new InfraredSelected());
    bandCombo.setToolTipText("Emissive band to display.");
    pictureControls = new PictureControls(contrast,brightness);
    VPanel v1 = new VPanel(TOP_ALIGNMENT,new HPanel(LEFT_ALIGNMENT,GUtil.HSpace(1),new CustomLabel("Resolution")),new HPanel(LEFT_ALIGNMENT,resolutionCombo));
    VPanel v2 = new VPanel(TOP_ALIGNMENT,new HPanel(LEFT_ALIGNMENT,GUtil.HSpace(1),new CustomLabel("Enhancement")),new HPanel(LEFT_ALIGNMENT,enhancementCombo));
    VPanel v3 = new VPanel(TOP_ALIGNMENT,new HPanel(LEFT_ALIGNMENT,GUtil.HSpace(1),new CustomLabel("Band")),new HPanel(LEFT_ALIGNMENT,bandCombo));
    add(new HPanel(v1,GUtil.HSpace(10),v2,GUtil.HSpace(10),v3,GUtil.HSpace(15),pictureControls));
  }
  /****************************************************************************
  * newInstance.
  ****************************************************************************/
  public ModisInfraredVisual newInstance () {
    // System.out.println("\n Create infrared visual.");
    return new ModisInfraredVisual(resolutionCombo.getSelection(),
				   enhancementCombo.getSelection(),
				   bandCombo.getSelection(),
				   pictureControls.getContrast(),
				   pictureControls.getBrightness());
  }
  /****************************************************************************
  * getRequestBandsElement.
  ****************************************************************************/
  public String getRequestBandsElement () {
    StringBuffer msg = new StringBuffer();
    msg.append("<bands>");
    msg.append("<band>");
    msg.append("<bandIndex>" + Integer.toString(/*Modis.INFRARED_BAND_INDEX*/Modis.emissiveBandIndices[bandCombo.getSelectedIndex()]) + "</bandIndex>");
    msg.append("<resolution>" + Modis.resolutionValues[Modis.resolutionIndex(resolutionCombo.getSelection())] + "</resolution>");
    msg.append("</band>");
    msg.append("</bands>");
    return msg.toString();
  }
  /****************************************************************************
  * doHeader.
  ****************************************************************************/
  public void doHeader (HeaderBlock headerBlock) throws Exception {
    if (headerBlock.getBandCount() != 1) {
      throw new Exception("unexpected number of bands");
    }
    if (headerBlock.getBandIndex(0) != /*Modis.INFRARED_BAND_INDEX*/Modis.emissiveBandIndices[bandCombo.getSelectedIndex()]) {
      throw new Exception("unexpected band index");
    }
    int resolution = headerBlock.getBandResolution(0);
    nBlockRows = headerBlock.getBlockRowCount(0);
    nBlockCols = headerBlock.getBlockColCount(0);
    if (headerBlock.isGeolocated()) {
      int geoBinSize = headerBlock.getGeoResolution() / resolution;
      int nGeoDataRows = headerBlock.getGeoBlockRowCount();
      if ((nGeoDataRows * geoBinSize) != nBlockRows) {
        throw new Exception("inconsistent geolocation block rows");
      }
      int nGeoDataCols = headerBlock.getGeoBlockColCount();
      if ((nGeoDataCols * geoBinSize) != nBlockCols) {
        throw new Exception("inconsistent geolocation block columns");
      }
    }
    colors = new Color[1][nBlockRows][nBlockCols];
    // mapHandler.headerBlock(headerBlock);
  }
  /****************************************************************************
  * doScan.
  ****************************************************************************/
  public Color[][][] doScan (ScanBlock scanBlock, byte[][][] data) throws Exception {
    for (int row = 0; row < nBlockRows; row++) {
       for (int col = 0; col < nBlockCols; col++) {
          int value = data[0][row][col] & 0xFF;
          if (algorithm != null) {
            algorithm.enhance(new int[] {value});
          }
          value = 255 - pictureControls.adjustValue(value);
	  colors[0][row][col] = grays[value];
       }
    }
    return colors;
  }
  /****************************************************************************
  * setControls.
  ****************************************************************************/
  public void setControls (boolean enabled) {
    resolutionCombo.setEnabled(enabled);
  }
  /****************************************************************************
  * getBandCombo.
  ****************************************************************************/
  public CustomCombo getBandCombo()
  {
    return bandCombo;
  }
  /****************************************************************************
  * getResolutionCombo.
  ****************************************************************************/
  public CustomCombo getResolutionCombo()
  {
    return resolutionCombo;
  }
  /****************************************************************************
  * InfraredSelected.
  ****************************************************************************/
  private class InfraredSelected implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      String passName=null;

      try
      {
        if (Viewer.mapLayoutButton.isSelected())
        {
          passName=Viewer.mapLayout.getPlayerManager().getPlayerCombo().getSelection();
          // Send message to server to get BiggestInResolution from bands.
          Viewer.serverHandler.send(Util.XML_HEADER + "<simulcastMessage>" +
          "<infraredBand>" + "<name>" + passName + "</name>" +
          "</infraredBand>" + "</simulcastMessage>");
        }

        if (Viewer.instrumentLayoutButton.isSelected())
        {
          passName=Viewer.instrumentLayout.getPlayerManager().getPlayerCombo().getSelection();

          // Send message to server to get BiggestInResolution from bands.
          Viewer.serverHandler.send(Util.XML_HEADER + "<simulcastMessage>" +
          "<infraredBand>" + "<name>" + passName + "</name>" +
          "</infraredBand>" + "</simulcastMessage>");
        }
      }
      catch (NullPointerException me)
      {
        // System.out.println("\n No passName at start of viewer");
      }
    }
  }
  /****************************************************************************
  * EnhancementSelected.
  ****************************************************************************/
  private class EnhancementSelected implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      setEnhancementAlgorithm();
    }
  }
  private void setEnhancementAlgorithm () {
    String className = Modis.nightEnhancementClassNames[enhancementCombo.getSelectedIndex()];
    if (className == null) {
      algorithm = null;
    } else {
      try {
	algorithm = (EnhancementAlgorithm) Class.forName(className).newInstance();
      } catch (Exception e) {
        algorithm = null;
      }
    }
  }
  /****************************************************************************
  * set.
  ****************************************************************************/
  public void set (XmlParser parser) throws Exception {
    resolutionCombo.setSelection(parser.getElementValue("resolution"));
    enhancementCombo.setSelection(parser.getElementValue("enhancement"));
    bandCombo.setSelection(parser.getElementValue("band"));
    pictureControls.setContrast(Double.parseDouble(parser.getElementValue("contrast")));
    pictureControls.setBrightness(Integer.parseInt(parser.getElementValue("brightness")));
  }
  /****************************************************************************
  * toXml.
  ****************************************************************************/
  public void toXml (PrintWriter out) throws Exception {
    out.println("  <visual name=\"" + VisualSelector.visualChoices[VisualSelector.MODIS_INFRARED_VISUAL] + "\">");
    out.println("    <resolution>" + Util.encodeContent(resolutionCombo.getSelection()) + "</resolution>");
    out.println("    <enhancement>" + Util.encodeContent(enhancementCombo.getSelection()) + "</enhancement>");
    out.println("    <band>" + Util.encodeContent(bandCombo.getSelection()) + "</band>");
    out.println("    <contrast>" + Double.toString(pictureControls.getContrast()) + "</contrast>");
    out.println("    <brightness>" + Integer.toString(pictureControls.getBrightness()) + "</brightness>");
    out.println("  </visual>");
  }
}
