/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*  18-Oct-06, J Love, GST	Original version.
*  10-Jan-07, J Love, GST	Appearance tweaking.
*  26-Feb-07, J Love, GST	More tool tips.
*  15-Mar-07, Z Yin,  GST       Display image resolution choices equal or larger
*                               then data resolution.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.viewer;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;

public class ModisTrueColorVisual extends HPanel implements Visual {
  private int nBlockRows;
  private int nBlockCols;
  private CustomCombo resolutionCombo;
  private CustomCombo enhancementCombo;
  private PictureControls pictureControls;
  private EnhancementAlgorithm algorithm;
  private Color[][][] colors;
  private int[] values = new int[3];
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * ModisTrueColorVisual.
  ****************************************************************************/
  public ModisTrueColorVisual () {
    this(Modis.resolutionChoices[2],Modis.dayEnhancementChoices[Modis.DAY_ENHANCEMENT_2],1.00,0);
  }
  public ModisTrueColorVisual (String resolutionChoice, String enhancementChoice,
			       double contrast, int brightness) {
    resolutionCombo = new CustomCombo(Modis.resolutionChoices,resolutionChoice);
    resolutionCombo.setToolTipText("Resolution of instrument data to be displayed.");
    enhancementCombo = new CustomCombo(Modis.dayEnhancementChoices,enhancementChoice);
    enhancementCombo.setToolTipText("Enhancement algorithm to be applied (before contrast and brightness adjustments).");
    enhancementCombo.addActionListener(new EnhancementSelected());
    setEnhancementAlgorithm();
    pictureControls = new PictureControls(contrast,brightness);
    VPanel v1 = new VPanel(TOP_ALIGNMENT,new HPanel(LEFT_ALIGNMENT,GUtil.HSpace(1),new CustomLabel("Resolution")),new HPanel(LEFT_ALIGNMENT,resolutionCombo));
    VPanel v2 = new VPanel(TOP_ALIGNMENT,new HPanel(LEFT_ALIGNMENT,GUtil.HSpace(1),new CustomLabel("Enhancement")),new HPanel(LEFT_ALIGNMENT,enhancementCombo));
    add(new HPanel(v1,GUtil.HSpace(10),v2,GUtil.HSpace(10),pictureControls));
  }
  /****************************************************************************
  * newInstance.
  ****************************************************************************/
  public ModisTrueColorVisual newInstance () {
    // System.out.println("\n Create true visual." );
    return new ModisTrueColorVisual(resolutionCombo.getSelection(),
				    enhancementCombo.getSelection(),
				    pictureControls.getContrast(),
				    pictureControls.getBrightness());
  }
  /****************************************************************************
  * getRequestBandsElement.
  ****************************************************************************/
  public String getRequestBandsElement () {
    StringBuffer xml = new StringBuffer();
    xml.append("<bands>");
    xml.append("<band>");
    xml.append("<bandIndex>" + Integer.toString(Modis.TRUE_COLOR_RED_BAND_INDEX) + "</bandIndex>");
    xml.append("<resolution>" + Modis.resolutionValues[Modis.resolutionIndex(resolutionCombo.getSelection())] + "</resolution>");
    xml.append("</band>");
    xml.append("<band>");
    xml.append("<bandIndex>" + Integer.toString(Modis.TRUE_COLOR_GREEN_BAND_INDEX) + "</bandIndex>");
    xml.append("<resolution>" + Modis.resolutionValues[Modis.resolutionIndex(resolutionCombo.getSelection())] + "</resolution>");
    xml.append("</band>");
    xml.append("<band>");
    xml.append("<bandIndex>" + Integer.toString(Modis.TRUE_COLOR_BLUE_BAND_INDEX) + "</bandIndex>");
    xml.append("<resolution>" + Modis.resolutionValues[Modis.resolutionIndex(resolutionCombo.getSelection())] + "</resolution>");
    xml.append("</band>");
    xml.append("</bands>");
    return xml.toString();
  }
  /****************************************************************************
  * doHeader.
  ****************************************************************************/
  public void doHeader (HeaderBlock headerBlock) throws Exception {
    if (headerBlock.getBandCount() != 3) {
      throw new Exception("unexpected number of bands");
    }
    if (headerBlock.getBandIndex(0) != Modis.TRUE_COLOR_RED_BAND_INDEX) {
      throw new Exception("unexpected red band index");
    }
    int resolution = headerBlock.getBandResolution(0);
    nBlockRows = headerBlock.getBlockRowCount(0);
    nBlockCols = headerBlock.getBlockColCount(0);
    if (headerBlock.getBandIndex(1) != Modis.TRUE_COLOR_GREEN_BAND_INDEX) {
      throw new Exception("unexpected green band index");
    }
    if (headerBlock.getBandResolution(1) != resolution) {
      throw new Exception("inconsistent resolutions");
    }
    if (headerBlock.getBlockRowCount(1) != nBlockRows) {
      throw new Exception("inconsistent block rows");
    }
    if (headerBlock.getBlockColCount(1) != nBlockCols) {
      throw new Exception("inconsistent block columns");
    }
    if (headerBlock.getBandIndex(2) != Modis.TRUE_COLOR_BLUE_BAND_INDEX) {
      throw new Exception("unexpected blue band index");
    }
    if (headerBlock.getBandResolution(2) != resolution) {
      throw new Exception("inconsistent resolutions");
    }
    if (headerBlock.getBlockRowCount(2) != nBlockRows) {
      throw new Exception("inconsistent block rows");
    }
    if (headerBlock.getBlockColCount(2) != nBlockCols) {
      throw new Exception("inconsistent block columns");
    }
    if (headerBlock.isGeolocated()) {
      int geoBinSize = headerBlock.getGeoResolution() / resolution;
      int nGeoDataRows = headerBlock.getGeoBlockRowCount();
      if ((nGeoDataRows * geoBinSize) != nBlockRows) {
        throw new Exception("inconsistent geolocation block rows");
      }
      int nGeoDataCols = headerBlock.getGeoBlockColCount();
      if ((nGeoDataCols * geoBinSize) != nBlockCols) {
        throw new Exception("inconsistent geolocation block columns");
      }
    }
    colors = new Color[1][nBlockRows][nBlockCols];
  }
  /****************************************************************************
  * doScan.
  ****************************************************************************/
  public Color[][][] doScan (ScanBlock scanBlock, byte[][][] data) throws Exception {
    for (int row = 0; row < nBlockRows; row++) {
       for (int col = 0; col < nBlockCols; col++) {
          values[0] = data[0][row][col] & 0xFF;
          values[1] = data[1][row][col] & 0xFF;
          values[2] = data[2][row][col] & 0xFF;
          if (algorithm != null) {
            algorithm.enhance(values);
          }
          values[0] = pictureControls.adjustValue(values[0]);
          values[1] = pictureControls.adjustValue(values[1]);
          values[2] = pictureControls.adjustValue(values[2]);
          colors[0][row][col] = new Color(values[0],values[1],values[2]);
       }
    }
    return colors;
  }
  /****************************************************************************
  * setControls.
  ****************************************************************************/
  public void setControls (boolean enabled) {
    resolutionCombo.setEnabled(enabled);
  }
  /****************************************************************************
  * getResolutionCombo.
  ****************************************************************************/
  public CustomCombo getResolutionCombo()
  {
    return resolutionCombo;
  }
  /****************************************************************************
  * EnhancementSelected.
  ****************************************************************************/
  private class EnhancementSelected implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      setEnhancementAlgorithm();
    }
  }
  private void setEnhancementAlgorithm () {
    String className = Modis.dayEnhancementClassNames[enhancementCombo.getSelectedIndex()];
    if (className == null) {
      algorithm = null;
    } else {
      try {
	algorithm = (EnhancementAlgorithm) Class.forName(className).newInstance();
      } catch (Exception e) {
        algorithm = null;
      }
    }
  }
  /****************************************************************************
  * set.
  ****************************************************************************/
  public void set (XmlParser parser) throws Exception {
    resolutionCombo.setSelection(parser.getElementValue("resolution"));
    enhancementCombo.setSelection(parser.getElementValue("enhancement"));
    pictureControls.setContrast(Double.parseDouble(parser.getElementValue("contrast")));
    pictureControls.setBrightness(Integer.parseInt(parser.getElementValue("brightness")));
  }
  /****************************************************************************
  * toXml.
  ****************************************************************************/
  public void toXml (PrintWriter out) throws Exception {
    out.println("  <visual name=\"" + VisualSelector.visualChoices[VisualSelector.MODIS_TRUE_COLOR_VISUAL] + "\">");
    out.println("    <resolution>" + Util.encodeContent(resolutionCombo.getSelection()) + "</resolution>");
    out.println("    <enhancement>" + Util.encodeContent(enhancementCombo.getSelection()) + "</enhancement>");
    out.println("    <contrast>" + Double.toString(pictureControls.getContrast()) + "</contrast>");
    out.println("    <brightness>" + Integer.toString(pictureControls.getBrightness()) + "</brightness>");
    out.println("  </visual>");
  }
}
