/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*  19-Oct-06, J Love, GST	Original version.
*  30-Jan-07, J Love, GST	Tweaked layout.
*  26-Feb-07, J Love, GST	More/better tool tips.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.viewer;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.awt.event.*;
import javax.swing.*;
import java.awt.*;
import javax.swing.border.*;
import java.io.*;

public class OptionsScreen extends VPanel {
  protected JRadioButton sortByPassNameButton;
  protected JRadioButton sortByStartTimeButton;
  protected JCheckBox instrumentViewAutoPlayCheckBox;
  protected JCheckBox mapViewAutoPlayCheckBox;
  protected JCheckBox autoZoomCheckBox;
  protected JCheckBox eraseOnAutoPlayCheckBox;
  protected JCheckBox eraseOnAutoZoomCheckBox;
  protected JCheckBox eraseOnZoomCheckBox;
  protected JCheckBox eraseOnReplayCheckBox;
  protected JCheckBox showOrbitsCheckBox;
  protected JCheckBox showGridCheckBox;
  protected JCheckBox showFeaturesCheckBox;
  protected JCheckBox showGroundStationCheckBox;
  protected JCheckBox[] showFeatureCheckBox;
  protected CustomCombo outlineCombo;
  protected JRadioButton unlimitedPassesButton;
  protected JRadioButton maxTotalPassesButton;
  protected JRadioButton maxPassesPerStationButton;
  protected JRadioButton maxPassesPerSatelliteButton;
  protected CustomTextField maxTotalPassesField;
  protected CustomTextField maxPassesPerStationField;
  protected CustomTextField maxPassesPerSatelliteField;
  protected JCheckBox autoScrollCheckBox;
  protected JCheckBox limitViewCheckBox;
  protected JCheckBox compressCheckBox;
  protected CustomCombo loopCombo;
  protected CustomCombo throttleCombo;
  private HPanel visualHolder;
  protected VisualSelector visualSelector;
  protected static final int PASS_OUTLINES_OFF = 0;
  protected static final int PASS_OUTLINES_SELECTED = 1;
  protected static final int PASS_OUTLINES_ALL = 2;
  private static final String[] outlineChoices = new String[] {
    "Off                      ","Selected","All"
  };
  protected static final int LOOPING_OFF = 0;
  protected static final int LOOPING_ON = 1;
  protected static final int LOOPING_ON_STATION = 2;
  protected static final int LOOPING_ON_SATELLITE = 3;
  protected static final int LOOPING_ON_STATION_SATELLITE = 4;
  protected static final int LOOPING_ON_STATION_SATELLITE_MODE = 5;
  private static final String[] loopChoices = new String[] {
    "Off","On","On Station","On Satellite","On Station/Satellite","On Station/Satellite/Mode"
  };
  protected static final int THROTTLE_OFF = 0;
  private static final String[] throttleChoices = new String[] {
    "Off               ","10X","5X","3X","2X","1X","1/2X","1/4X"
  };
  protected static final double[] throttleValues = new double[] {
    0.0,10.0,5.0,3.0,2.0,1.0,0.5,0.25
  };
  private static final File preferencesFile = new File(System.getProperty("user.home") + File.separator + ".sc-preferences");
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * OptionsScreen.
  ****************************************************************************/
  public OptionsScreen (String autoPlay, boolean mapAutoZoom) throws Exception {
    setBorder(new LineBorder(Color.black,1));
    // Pass manager.
    sortByPassNameButton = new JRadioButton("Pass Name",false);
    sortByPassNameButton.setToolTipText("If selected, passes are sorted by pass name");
    sortByPassNameButton.addActionListener(new SortOrderChanged());
    sortByStartTimeButton = new JRadioButton("Start Time",true);
    sortByStartTimeButton.setToolTipText("If selected, passes are sorted by start time");
    sortByStartTimeButton.addActionListener(new SortOrderChanged());
    GUtil.buttonGroup(sortByPassNameButton,sortByStartTimeButton);
    VPanel pmPanel = new VPanel(
      new HPanel(GUtil.HSpace(3),new CustomLabel("Sort By"),GUtil.HGlue()),
      new HPanel(sortByStartTimeButton,GUtil.HGlue()),
      new HPanel(sortByPassNameButton,GUtil.HGlue()),
      GUtil.VGlue()
    );
    pmPanel.setBorder(new TitledBorder(new LineBorder(Color.black,1)," Pass Manager "));
    // Map layout.
    mapViewAutoPlayCheckBox = new JCheckBox("Auto Play",autoPlay.equals("map"));
    mapViewAutoPlayCheckBox.setToolTipText("If checked, new passes are automatically loaded/played on the Map Layout.");
    autoZoomCheckBox = new JCheckBox("Auto Zoom",mapAutoZoom);
    autoZoomCheckBox.setToolTipText("If checked, the map is automatically zoomed to cover the estimated extents of the pass.");
    eraseOnAutoZoomCheckBox = new JCheckBox("Erase Passes On Auto Zoom",false);
    eraseOnAutoZoomCheckBox.setToolTipText("If checked, passes are erased when the map is auto-zoomed.");
    eraseOnAutoPlayCheckBox = new JCheckBox("Erase Passes On Auto Play",false);
    eraseOnAutoPlayCheckBox.setToolTipText("If checked, passes are erased when a pass is auto-played.");
    eraseOnZoomCheckBox = new JCheckBox("Erase Passes On Zoom",false);
    eraseOnZoomCheckBox.setToolTipText("If checked, passes are erased when the map is zoomed.");
    eraseOnReplayCheckBox = new JCheckBox("Erase Pass On Replay",true);
    eraseOnReplayCheckBox.setToolTipText("If checked, an existing pass is erased when the pass is replayed.");
    showOrbitsCheckBox = new JCheckBox("Show Orbits",true);
    showOrbitsCheckBox.setToolTipText("If checked, satellite positions/orbits are shown on the map.");
    showOrbitsCheckBox.addActionListener(new MapOptionChanged());
    showGridCheckBox = new JCheckBox("Show Grid",true);
    showGridCheckBox.setToolTipText("If checked, latitude/longitude grid lines are shown on the map.");
    showGridCheckBox.addActionListener(new MapOptionChanged());
    showGroundStationCheckBox = new JCheckBox("Show Ground Station",true);
    showGroundStationCheckBox.setToolTipText("If checked, ground stations are shown on the map.");
    showGroundStationCheckBox.addActionListener(new MapOptionChanged());
    showFeaturesCheckBox = new JCheckBox("Show Features",true);
    showFeaturesCheckBox.setToolTipText("If checked, geographic/political features are shown on the map.");
    showFeaturesCheckBox.addActionListener(new MapOptionChanged());
    outlineCombo = new CustomCombo(outlineChoices,PASS_OUTLINES_OFF);
    outlineCombo.setToolTipText("Specifies if outlines are displayed for all passes, just the \"selected\" pass, or for no passes.");
    VPanel mapViewPanel = new VPanel(new Component[] {new HPanel(mapViewAutoPlayCheckBox,GUtil.HGlue()),new HPanel(autoZoomCheckBox,GUtil.HGlue()),new HPanel(eraseOnAutoPlayCheckBox,GUtil.HGlue()),new HPanel(eraseOnAutoZoomCheckBox,GUtil.HGlue()),new HPanel(eraseOnZoomCheckBox,GUtil.HGlue()),new HPanel(eraseOnReplayCheckBox,GUtil.HGlue()),new HPanel(showOrbitsCheckBox,GUtil.HGlue()),new HPanel(showGridCheckBox,GUtil.HGlue()),new HPanel(showGroundStationCheckBox,GUtil.HGlue()),new HPanel(showFeaturesCheckBox,GUtil.HGlue()),GUtil.VSpace(10),new HPanel(GUtil.HSpace(3),new CustomLabel("Pass Outlines"),GUtil.HGlue()),GUtil.VSpace(2),new HPanel(GUtil.HSpace(3),outlineCombo,GUtil.HGlue()),GUtil.VGlue()});
    mapViewPanel.setBorder(new TitledBorder(new LineBorder(Color.black,1)," Map Layout "));
    // Map features.
    VPanel featuresPanel = new VPanel();
    showFeatureCheckBox = new JCheckBox[MapHandler.shpNames.length];
    for (int i = 0; i < MapHandler.shpNames.length; i++) {
       showFeatureCheckBox[i] = new JCheckBox("Show "+MapHandler.shpNames[i],MapHandler.shpDefaultStates[i]);
       showFeatureCheckBox[i].setToolTipText("If checked, " + MapHandler.shpNames[i] + " are shown on the map.");
       showFeatureCheckBox[i].addActionListener(new MapOptionChanged());
       featuresPanel.add(showFeatureCheckBox[i]);
    }
    featuresPanel.add(GUtil.VGlue());
    featuresPanel.setBorder(new TitledBorder(new LineBorder(Color.black,1)," Map Features "));
    // Instrument layout.
    instrumentViewAutoPlayCheckBox = new JCheckBox("Auto Play",autoPlay.equals("instrument"));
    instrumentViewAutoPlayCheckBox.setToolTipText("If checked, new passes are automatically loaded/played on the Instrument Layout.");
    autoScrollCheckBox = new JCheckBox("Auto Scroll",true);
    autoScrollCheckBox.setToolTipText("If checked, the image is automatically scrolled as needed.");
    limitViewCheckBox = new JCheckBox("Limit View",false);
    limitViewCheckBox.setToolTipText("If checked, size of view is limited (no scrolling back).");
    VPanel instrumentViewPanel = new VPanel(new HPanel(instrumentViewAutoPlayCheckBox,GUtil.HGlue()),new HPanel(autoScrollCheckBox,GUtil.HGlue()),new HPanel(limitViewCheckBox,GUtil.HGlue()),GUtil.VGlue());
    instrumentViewPanel.setBorder(new TitledBorder(new LineBorder(Color.black,1)," Instrument Layout "));
    // Player manager.
    compressCheckBox = new JCheckBox("Compress",false);
    compressCheckBox.setToolTipText("If checked, uses a compressed data stream during playback (helps slow connections).");
    unlimitedPassesButton = new JRadioButton("Unlimited",false);
    unlimitedPassesButton.setToolTipText("If selected, an unlimited number of passes can be displayed simultaneously.");
    maxTotalPassesButton = new JRadioButton("Maximum Total:",true);
    maxTotalPassesButton.setToolTipText("If selected, limits the total number of passes that can be displayed simultaneously.");
    maxTotalPassesField = new CustomTextField("5",3);
    maxPassesPerStationButton = new JRadioButton("Maximum Per Station:",false);
    maxPassesPerStationButton.setToolTipText("If selected, limits the number of passes per station that can be displayed simultaneously.");
    maxPassesPerStationField = new CustomTextField("1",3);
    maxPassesPerSatelliteButton = new JRadioButton("Maximum Per Satellite:",false);
    maxPassesPerSatelliteButton.setToolTipText("If selected, limits the number of passes per satellite that can be displayed simultaneously.");
    maxPassesPerSatelliteField = new CustomTextField("1",3);
    GUtil.buttonGroup(unlimitedPassesButton,maxTotalPassesButton,maxPassesPerStationButton,maxPassesPerSatelliteButton);
    loopCombo = new CustomCombo(loopChoices,LOOPING_OFF);
    loopCombo.setToolTipText("Specifies the type of playback looping.");
    throttleCombo = new CustomCombo(throttleChoices,THROTTLE_OFF);
    throttleCombo.setToolTipText("Specifies if playing should be throttled (rate limited).");
    VPanel playerPanel = new VPanel(new Component[] {
      new HPanel(compressCheckBox,GUtil.HGlue()),
      GUtil.VSpace(15),
      new HPanel(GUtil.HSpace(3),new CustomLabel("Passes Per Layout"),GUtil.HGlue()),
      new HPanel(unlimitedPassesButton,GUtil.HGlue()),
      new HPanel(maxTotalPassesButton,maxTotalPassesField,GUtil.HGlue()),
      new HPanel(maxPassesPerStationButton,maxPassesPerStationField,GUtil.HGlue()),
      new HPanel(maxPassesPerSatelliteButton,maxPassesPerSatelliteField,GUtil.HGlue()),
      GUtil.VSpace(15),
      new HPanel(GUtil.HSpace(3),new CustomLabel("Looping"),GUtil.HGlue()),
      new HPanel(GUtil.HSpace(3),loopCombo,GUtil.HGlue()),
      GUtil.VSpace(15),
      new HPanel(GUtil.HSpace(3),new CustomLabel("Throttle"),GUtil.HGlue()),
      new HPanel(GUtil.HSpace(3),throttleCombo,GUtil.HGlue()),
      GUtil.VGlue()
    });
    playerPanel.setBorder(new TitledBorder(new LineBorder(Color.black,1)," Player Manager "));
    // Visuals.
    visualSelector = new VisualSelector();
    visualHolder = new HPanel(visualSelector);
    HPanel visualsPanel = new HPanel(new MarginPanel(5,5,visualHolder,5,5));
    visualsPanel.setBorder(new TitledBorder(new LineBorder(Color.black,1)," Default Visuals "));
    visualsPanel.setToolTipText("The default visuals to be used when a pass is loaded on a layout.");
    // Layout.
    JPanel p1 = new JPanel(new GridLayout(0,4,10,10));
    p1.add(new VPanel(pmPanel,instrumentViewPanel));
    p1.add(mapViewPanel);
    p1.add(featuresPanel);
    p1.add(playerPanel);
    JPanel p2 = new VPanel(new MarginPanel(10,10,p1,10,10),new MarginPanel(0,10,visualsPanel,10,10));
    p2.setPreferredSize(p2.getMaximumSize());
    add(p2);
    readPreferences();
    writePreferences();
    Runtime.getRuntime().addShutdownHook(new ShutdownHook());
  }
  /****************************************************************************
  * MapOptionChanged.
  ****************************************************************************/
  private class MapOptionChanged implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      Viewer.mapLayout.optionChanged();
    }
  }
  /****************************************************************************
  * SortOrderChanged.
  ****************************************************************************/
  private class SortOrderChanged implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      Viewer.passManager.sortOrderChanged();
    }
  }
  /****************************************************************************
  * setDefaultVisuals.
  ****************************************************************************/
  public void setDefaultVisuals (VisualSelector visualSelector) {
    this.visualSelector = visualSelector;
    visualHolder.removeAll();
    visualHolder.add(visualSelector);
    revalidate();
    repaint();
  }
  /****************************************************************************
  * readPreferences.
  ****************************************************************************/
  private void readPreferences () {
    if (!preferencesFile.exists()) {
      return;
    }
    try {
      XmlParser parser = new XmlParser(Util.readString(preferencesFile));
      sortByPassNameButton.setSelected(parser.getElement("passManager").getElementValue("sortBy").equals("passName"));	
      sortByStartTimeButton.setSelected(parser.getElement("passManager").getElementValue("sortBy").equals("startTime"));	
      mapViewAutoPlayCheckBox.setSelected(parser.getElement("mapLayout").hasElement("autoPlay"));
      autoZoomCheckBox.setSelected(parser.getElement("mapLayout").hasElement("autoZoom"));
      eraseOnAutoPlayCheckBox.setSelected(parser.getElement("mapLayout").hasElement("eraseOnAutoPlay"));
      eraseOnAutoZoomCheckBox.setSelected(parser.getElement("mapLayout").hasElement("eraseOnAutoZoom"));
      eraseOnZoomCheckBox.setSelected(parser.getElement("mapLayout").hasElement("eraseOnZoom"));
      eraseOnReplayCheckBox.setSelected(parser.getElement("mapLayout").hasElement("eraseOnReplay"));
      showOrbitsCheckBox.setSelected(parser.getElement("mapLayout").hasElement("showOrbits"));
      showGridCheckBox.setSelected(parser.getElement("mapLayout").hasElement("showGrid"));
      showFeaturesCheckBox.setSelected(parser.getElement("mapLayout").hasElement("showFeatures"));
      showGroundStationCheckBox.setSelected(parser.getElement("mapLayout").hasElement("showGroundStation"));
      outlineCombo.setSelection(parser.getElement("mapLayout").getElementValue("passOutlines"));
      String[] featureNames = parser.getElement("mapFeatures").getElementValues("mapFeature");
      for (int i = 0; i < MapHandler.shpNames.length; i++) {
         showFeatureCheckBox[i].setSelected(Util.oneOf(MapHandler.shpNames[i],featureNames));
      }
      instrumentViewAutoPlayCheckBox.setSelected(parser.getElement("instrumentLayout").hasElement("autoPlay"));
      autoScrollCheckBox.setSelected(parser.getElement("instrumentLayout").hasElement("autoScroll"));
      limitViewCheckBox.setSelected(parser.getElement("instrumentLayout").hasElement("limitView"));
      compressCheckBox.setSelected(parser.getElement("playerManager").hasElement("compress"));
      if (parser.getElement("playerManager").getElement("passesPerLayout").hasElement("unlimited")) {
	unlimitedPassesButton.setSelected(true);
      }
      if (parser.getElement("playerManager").getElement("passesPerLayout").hasElement("maxTotal")) {
	maxTotalPassesButton.setSelected(true);
	maxTotalPassesField.setText(parser.getElement("playerManager").getElement("passesPerLayout").getElementValue("maxTotal"));
      }
      if (parser.getElement("playerManager").getElement("passesPerLayout").hasElement("maxPerStation")) {
	maxPassesPerStationButton.setSelected(true);
	maxPassesPerStationField.setText(parser.getElement("playerManager").getElement("passesPerLayout").getElementValue("maxPerStation"));
      }
      if (parser.getElement("playerManager").getElement("passesPerLayout").hasElement("maxPerSatellite")) {
	maxPassesPerSatelliteButton.setSelected(true);
	maxPassesPerSatelliteField.setText(parser.getElement("playerManager").getElement("passesPerLayout").getElementValue("maxPerSatellite"));
      }
      loopCombo.setSelection(parser.getElement("playerManager").getElementValue("looping"));
      throttleCombo.setSelection(parser.getElement("playerManager").getElementValue("throttle"));
      visualSelector.set(parser.getElement("visuals"));
    } catch (Exception e) {
      e.printStackTrace();
    }
  }
  /****************************************************************************
  * writePreferences.
  ****************************************************************************/
  private void writePreferences () {
    PrintWriter out = null;
    try {
      out = new PrintWriter(preferencesFile);
      out.println(Util.XML_HEADER);
      out.println("<simulcastPreferences>");
      out.println("  <passManager>");
      out.print("    <sortBy>");
      out.print(sortByPassNameButton.isSelected() ? "passName" : "");
      out.print(sortByStartTimeButton.isSelected() ? "startTime" : "");
      out.println("</sortBy>");
      out.println("  </passManager>");
      out.println("  <mapLayout>");
      out.println(mapViewAutoPlayCheckBox.isSelected() ? "    <autoPlay/>" : "");
      out.println(autoZoomCheckBox.isSelected() ? "    <autoZoom/>" : "");
      out.println(eraseOnAutoPlayCheckBox.isSelected() ? "    <eraseOnAutoPlay/>" : "");
      out.println(eraseOnAutoZoomCheckBox.isSelected() ? "    <eraseOnAutoZoom/>" : "");
      out.println(eraseOnZoomCheckBox.isSelected() ? "    <eraseOnZoom/>" : "");
      out.println(eraseOnReplayCheckBox.isSelected() ? "    <eraseOnReplay/>" : "");
      out.println(showOrbitsCheckBox.isSelected() ? "    <showOrbits/>" : "");
      out.println(showGridCheckBox.isSelected() ? "    <showGrid/>" : "");
      out.println(showFeaturesCheckBox.isSelected() ? "    <showFeatures/>" : "");
      out.println(showGroundStationCheckBox.isSelected() ? "    <showGroundStation/>" : "");
      out.println("    <passOutlines>" + Util.encodeContent(outlineCombo.getSelection()) + "</passOutlines>");
      out.println("  </mapLayout>");
      out.println("  <mapFeatures>");
      for (int i = 0; i < MapHandler.shpNames.length; i++) {
         out.println(showFeatureCheckBox[i].isSelected() ? "    <mapFeature>" + Util.encodeContent(MapHandler.shpNames[i]) + "</mapFeature>" : "");
      }
      out.println("  </mapFeatures>");
      out.println("  <instrumentLayout>");
      out.println(instrumentViewAutoPlayCheckBox.isSelected() ? "    <autoPlay/>" : "");
      out.println(autoScrollCheckBox.isSelected() ? "    <autoScroll/>" : "");
      out.println(limitViewCheckBox.isSelected() ? "    <limitView/>" : "");
      out.println("  </instrumentLayout>");
      out.println("  <playerManager>");
      out.println(compressCheckBox.isSelected() ? "    <compress/>" : "");
      out.print("    <passesPerLayout>");
      out.print(unlimitedPassesButton.isSelected() ? "<unlimited/>" : "");
      out.print(maxTotalPassesButton.isSelected() ? ("<maxTotal>" + maxTotalPassesField.getText() + "</maxTotal>") : "");
      out.print(maxPassesPerStationButton.isSelected() ? ("<maxPerStation>" + maxPassesPerStationField.getText() + "</maxPerStation>") : "");
      out.print(maxPassesPerSatelliteButton.isSelected() ? ("<maxPerSatellite>" + maxPassesPerSatelliteField.getText() + "</maxPerSatellite>") : "");
      out.println("</passesPerLayout>");
      out.println("    <looping>" + Util.encodeContent(loopCombo.getSelection()) + "</looping>");
      out.println("    <throttle>" + Util.encodeContent(throttleCombo.getSelection()) + "</throttle>");
      out.println("  </playerManager>");
      visualSelector.toXml(out);
      out.println("</simulcastPreferences>");
      out.close();
    } catch (Exception e) {
      e.printStackTrace();
      try {out.close();} catch (Exception ex) {}
    }
  }
  /****************************************************************************
  * ShutdownHook.
  ****************************************************************************/
  private class ShutdownHook extends Thread {
    public void run () {
      writePreferences();
    }
  }
}
