/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*  29-Sep-06, J Love, GST	Original version.
*   2-Jan-07, J Love, GST	On layouts, handle passes being removed from
*   				server.
*  30-Jan-07, J Love, GST	Tweaked layout.
*  15-Mar-07, Z Yin,  GST       Display image resolution choices equal or larger
*                               then data resolution.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.viewer;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.awt.*;
import javax.swing.*;
import java.util.*;

public class PassManager extends JPanel {
  private StringMap handlers = new StringMap();
  private VPanel handlersPanel;
  private JScrollBar vScrollBar;
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * PassManager.
  ****************************************************************************/
  public PassManager () {
    setLayout(new BorderLayout());
    handlersPanel = new VPanel();
    JScrollPane scrollPane = new JScrollPane(new MarginPanel(0,0,handlersPanel,5,0));
    scrollPane.setPreferredSize(scrollPane.getMaximumSize());
    vScrollBar = scrollPane.getVerticalScrollBar();
    add(scrollPane,BorderLayout.CENTER);
    HPanel foot = new HPanel(Viewer.serverListener,GUtil.HGlue());
    add(new MarginPanel(3,2,foot,3,2),BorderLayout.SOUTH);
  }
  /****************************************************************************
  * clear.
  ****************************************************************************/
  public void clear () {
    handlers.clear();
    handlersPanel.removeAll();
    revalidate();
    repaint();
  }
  /****************************************************************************
  * loadSelectedPasses.
  ****************************************************************************/
  public void loadSelectedPasses (Layout layout) {
    Iterator it = handlers.iterator();
    while (it.hasNext()) {
      PassHandler handler = (PassHandler) it.next();
      handler.loadIfSelected(layout);
    }
  }
  /****************************************************************************
  * handlePassListing.
  ****************************************************************************/
  public void handlePassListing (XmlParser parser) throws Exception {
    LinkedList<PassHandler> noHeaders = new LinkedList<PassHandler>();
    handlers.clear();
    handlersPanel.removeAll();
    XmlParser[] parsers = parser.getElements("passDetails");
    for (int i = 0; i < parsers.length; i++) {
       PassHandler handler = new PassHandler(parsers[i]);
       handlers.put(handler.getPassName(),handler);
       if (handler.hasHeader()) {
	 insert(handler);
       } else {
	 noHeaders.add(handler);
       }
    }
    for (PassHandler handler : noHeaders) {
       insertNoHeader(handler);
    }
    revalidate();
    repaint();
    SwingUtilities.invokeLater(new DoAutoScroll());
  }
  /****************************************************************************
  * handlePassDetails.
  ****************************************************************************/
  public void handlePassDetails (XmlParser parser) throws Exception {
    String passName = parser.getElementValue("name");
    PassHandler handler = (PassHandler) handlers.get(passName);
    handler.passDetails(parser);
    handlersPanel.remove(handler);
    insert(handler);
    revalidate();
    repaint();
  }
  /****************************************************************************
  * handlePassStarted.
  * Always put a pass just starting at the bottom.
  ****************************************************************************/
  public void handlePassStarted (XmlParser parser) throws Exception {
    PassHandler handler = new PassHandler(parser);
    handlers.put(handler.getPassName(),handler);
    handlersPanel.add(handler);
    revalidate();
    repaint();
    SwingUtilities.invokeLater(new DoAutoScroll());
  }
  /****************************************************************************
  * handlePassUpdated.
  ****************************************************************************/
  public void handlePassUpdated (XmlParser parser) throws Exception {
    String passName = parser.getElementValue("name");
    PassHandler handler = (PassHandler) handlers.get(passName);
    handler.passUpdated(parser);
    handlersPanel.remove(handler);
    insert(handler);
    revalidate();
    repaint();
  }
  /****************************************************************************
  * handlePassEnded.
  ****************************************************************************/
  public void handlePassEnded (XmlParser parser) throws Exception {
    String passName = parser.getElementValue("name");
    PassHandler handler = (PassHandler) handlers.get(passName);
    handler.passEnded(parser);
    handlersPanel.remove(handler);
    insert(handler);
    revalidate();
    repaint();
  }
  /****************************************************************************
  * handlePassRemoved.
  ****************************************************************************/
  public void handlePassRemoved (XmlParser parser) throws Exception {
    String name = parser.getElementValue("name");
    PassHandler handler = (PassHandler) handlers.remove(name);
    handlersPanel.remove(handler);
    Viewer.mapLayout.passRemoved(name);
    Viewer.instrumentLayout.passRemoved(name);
    revalidate();
    repaint();
  }
  /****************************************************************************
  * handleVisualBand.
  ****************************************************************************/
  public void handleVisualBand (XmlParser parser) throws Exception {
    String passName = parser.getElementValue("name");
    PassHandler handler = (PassHandler) handlers.get(passName);
    handler.visualBand(parser);
  }
  /****************************************************************************
  * handleInfraredBand.
  ****************************************************************************/
  public void handleInfraredBand (XmlParser parser) throws Exception {
    String passName = parser.getElementValue("name");
    PassHandler handler = (PassHandler) handlers.get(passName);
    handler.infraredBand(parser);
  }
  /****************************************************************************
  * handleGrayBand.
  ****************************************************************************/
  public void handleGrayBand (XmlParser parser) throws Exception {
    String passName = parser.getElementValue("name");
    PassHandler handler = (PassHandler) handlers.get(passName);
    handler.grayBand(parser);
  }
  /****************************************************************************
  * handleColorBand.
  ****************************************************************************/
  public void handleColorBand (XmlParser parser) throws Exception {
    String passName = parser.getElementValue("name");
    PassHandler handler = (PassHandler) handlers.get(passName);
    handler.colorBand(parser);
  }
  /****************************************************************************
  * playNext.
  ****************************************************************************/
  public void playNext (String passName, Layout layout) {
    Component[] components = handlersPanel.getComponents();
    LinkedList<PassHandler> list = new LinkedList<PassHandler>();
    PassHandler current = (PassHandler) handlers.get(passName);
    if (current != null) {
      int index = Util.indexOf(current,components);
      for (int i = index+1; i < components.length; i++) {
         list.add((PassHandler)components[i]);
      }
      for (int i = 0; i <= index; i++) {
         list.add((PassHandler)components[i]);
      }
    } else {
      for (int i = 0; i < components.length; i++) {
         list.add((PassHandler)components[i]);
      }
    }
    Iterator<PassHandler> it = list.iterator();
    while (it.hasNext()) {
      PassHandler handler = it.next();
      if (!handler.hasHeader()) {
	it.remove();
	continue;
      }
      switch (Viewer.optionsScreen.loopCombo.getSelectedIndex()) {
        case OptionsScreen.LOOPING_ON: {
	  break;
        }
        case OptionsScreen.LOOPING_ON_STATION: {
	  if (!handler.getStationId().equals(current.getStationId())) {
	    it.remove();
	  }
	  break;
        }
        case OptionsScreen.LOOPING_ON_SATELLITE: {
	  if (handler.getSatelliteCode() != current.getSatelliteCode()) {
	    it.remove();
	  }
	  break;
        }
        case OptionsScreen.LOOPING_ON_STATION_SATELLITE: {
	  if (!handler.getStationId().equals(current.getStationId()) ||
	      handler.getSatelliteCode() != current.getSatelliteCode()) {
	    it.remove();
	  }
	  break;
        }
        case OptionsScreen.LOOPING_ON_STATION_SATELLITE_MODE: {
	  if (!handler.getStationId().equals(current.getStationId()) ||
	      handler.getSatelliteCode() != current.getSatelliteCode() ||
	      handler.getStartSolarCode() != current.getStartSolarCode()) {
	    it.remove();
	  }
	  break;
        }
      }
    }
    if (list.size() > 0) {
      list.getFirst().loopLoad(layout);
    }
  }
  /****************************************************************************
  * sortOrderChanged.
  ****************************************************************************/
  public void sortOrderChanged () {
    LinkedList<PassHandler> noHeaders = new LinkedList<PassHandler>();
    handlersPanel.removeAll();
    Iterator it = handlers.iterator();
    while (it.hasNext()) {
      PassHandler handler = (PassHandler) it.next();
      if (handler.hasHeader()) {
        insert(handler);
      } else {
	noHeaders.add(handler);
      }
    }
    for (PassHandler handler : noHeaders) {
       insertNoHeader(handler);
    }
    revalidate();
    repaint();
  }
  /****************************************************************************
  * insert.
  ****************************************************************************/
  private void insert (PassHandler newHandler) {
    if (!newHandler.hasHeader()) {
      insertNoHeader(newHandler);
      return;
    }
    if (Viewer.optionsScreen.sortByPassNameButton.isSelected()) {
      insertByPassName(newHandler);
      return;
    }
    if (Viewer.optionsScreen.sortByStartTimeButton.isSelected()) {
      insertByStartTime(newHandler);
      return;
    }
    return;
  }
  /****************************************************************************
  * insertByPassName.
  ****************************************************************************/
  private void insertByPassName (PassHandler newHandler) {
    Component[] components = handlersPanel.getComponents();
    for (int i = 0; i < components.length; i++) {
       PassHandler handler = (PassHandler) components[i];
       if (newHandler.getPassName().compareTo(handler.getPassName()) < 0) {
 	 handlersPanel.add(newHandler,i);
	 return;
       }
    }
    handlersPanel.add(newHandler);
    return;
  }
  /****************************************************************************
  * insertByStartTime.
  ****************************************************************************/
  private void insertByStartTime (PassHandler newHandler) {
    Component[] components = handlersPanel.getComponents();
    for (int i = 0; i < components.length; i++) {
       PassHandler handler = (PassHandler) components[i];
       if (newHandler.getStartedAt() < handler.getStartedAt()) {
 	 handlersPanel.add(newHandler,i);
	 return;
       }
    }
    handlersPanel.add(newHandler);
    return;
  }
  /****************************************************************************
  * insertNoHeader.
  ****************************************************************************/
  private void insertNoHeader (PassHandler newHandler) {
    Component[] components = handlersPanel.getComponents();
    for (int i = 0; i < components.length; i++) {
       PassHandler handler = (PassHandler) components[i];
       if (newHandler.getStationId().equals(handler.getStationId())) {
         if (newHandler.getPassName().compareTo(handler.getPassName()) < 0) {
	   handlersPanel.add(newHandler,i);
	   return;
         }
       }
    }
    handlersPanel.add(newHandler);
    return;
  }
  /****************************************************************************
  * DoAutoScroll.
  ****************************************************************************/
  private class DoAutoScroll implements Runnable {
    public void run () {
      vScrollBar.setValue(vScrollBar.getMaximum());
    }
  }
}
