/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*  18-Oct-06, J Love, GST	Original version.
*   5-Dec-06, J Love, GST	Stop/close while pass has stalled.
*   5-Jan-07, J Love, GST	Fixed error display so it no longer causes the
*   				layout to resize.
*  26-Feb-07, J Love, GST	More tool tips.
*   2-Apr-07, Z Yin,  GST       Don't display error message on player panel when loop.
*                               Also more meaningful message when not loop.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.viewer;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.io.*;
import java.net.Socket;
import java.util.zip.*;

public class Player extends VPanel {
  private String passName;
  private String stationId;
  private String platformName;
  private Layout layout;
  private PlayerManager manager;
  private HPanel contentPanel;
  private HPanel playPanel;
  private IconButton playButton;
  private IconButton pauseButton;
  private IconButton stopButton;
  private IconButton closeButton;
  private CustomLabel dateTimeField;
  private CustomLabel rateField;
  private CustomLabel nScansField;
  private VisualSelector visualSelector;
  private Play play = null;
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * PassPlayer.
  ****************************************************************************/
  public Player (String passName, int platformCode, int instrumentCode,
		 int directionCode, int startSolarCode, Layout layout,
		 PlayerManager manager) {
    this.passName = passName;
    this.layout = layout;
    this.manager = manager;
    stationId = passName.split("\\.")[0];
    platformName = Simulcast.getPlatformName(platformCode);
    contentPanel = new HPanel();
    dateTimeField = new CustomLabel(" ");
    dateTimeField.setToolTipText("Time of the last scan (GMT) received.");
    nScansField = new CustomLabel(" ");
    nScansField.setToolTipText("Number of scans received since the start of the pass.");
    rateField = new CustomLabel(" ");
    rateField.setToolTipText("Rate at which scans are being received.");
    playButton = new IconButton(GUtil.playIcon);
    playButton.setToolTipText("Play.");
    playButton.addActionListener(new PlayButtonClicked());
    pauseButton = new IconButton(GUtil.pauseIcon);
    pauseButton.setToolTipText("Pause.");
    pauseButton.addActionListener(new PauseButtonClicked());
    stopButton = new IconButton(GUtil.stopIcon);
    stopButton.setToolTipText("Stop.");
    stopButton.addActionListener(new StopButtonClicked());
    closeButton = new IconButton(GUtil.closeIcon);
    closeButton.setToolTipText("Close this pass (removing it from this layout).");
    closeButton.addActionListener(new CloseButtonClicked());
    CustomLabel infoField = new CustomLabel(Simulcast.getPlatformName(platformCode)+"/"+Simulcast.getInstrumentName(instrumentCode)+"/"+Simulcast.getPassDirection(directionCode)+"/"+Simulcast.getPassSolar(startSolarCode));
    infoField.setToolTipText("Satellite/Instrument/Direction/Mode.");
    playPanel = new HPanel(new Component[] {playButton,GUtil.HSpace(2),pauseButton,GUtil.HSpace(2),stopButton,GUtil.HSpace(10),infoField,GUtil.HSpace(25),dateTimeField,GUtil.HSpace(25),nScansField,GUtil.HGlue(),rateField,GUtil.HSpace(10)/*,visibleButton,GUtil.HSpace(5)*/,closeButton});
    contentPanel.add(playPanel);
    visualSelector = Viewer.optionsScreen.visualSelector.newInstance();
    add(visualSelector);
    add(GUtil.VSpace(2));
    add(contentPanel);

    // Send message to server to get BiggestInResolution from bands.
    Viewer.serverHandler.send(Util.XML_HEADER + "<simulcastMessage>" +
    "<visualBand>" + "<name>" + passName + "</name>" +
    "</visualBand>" + "</simulcastMessage>");
  }
  /****************************************************************************
  * getVisualSelector.
  ****************************************************************************/
  public VisualSelector getVisualSelector () {
    return visualSelector;
  }
  /****************************************************************************
  * isPlaying.
  ****************************************************************************/
  public boolean isPlaying () {
    return (play != null);
  }
  /****************************************************************************
  * getPassName.
  ****************************************************************************/
  public String getPassName () {
    return passName;
  }
  /****************************************************************************
  * getStationId.
  ****************************************************************************/
  public String getStationId () {
    return stationId;
  }
  /****************************************************************************
  * getPlatformName.
  ****************************************************************************/
  public String getPlatformName () {
    return platformName;
  }
  /****************************************************************************
  * PlayButtonClicked.
  ****************************************************************************/
  private class PlayButtonClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      if (((ae.getModifiers() & ActionEvent.CTRL_MASK) == 0)) {
	play();
      } else {
	manager.playAll();
      }
    }
  }
  /****************************************************************************
  * PauseButtonClicked.
  ****************************************************************************/
  private class PauseButtonClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      if (((ae.getModifiers() & ActionEvent.CTRL_MASK) == 0)) {
	pause();
      } else {
	manager.pauseAll();
      }
    }
  }
  /****************************************************************************
  * StopButtonClicked.
  ****************************************************************************/
  private class StopButtonClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      if (((ae.getModifiers() & ActionEvent.CTRL_MASK) == 0)) {
	stop();
      } else {
	manager.stopAll();
      }
    }
  }
  /****************************************************************************
  * CloseButtonClicked.
  ****************************************************************************/
  private class CloseButtonClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      if (((ae.getModifiers() & ActionEvent.CTRL_MASK) == 0)) {
	close();
      } else {
	manager.closeAll();
      }
    }
  }
  /****************************************************************************
  * play.
  ****************************************************************************/
  public void play () {
    if (play == null) {
      play = new Play();
      play.start();
    }
  }
  /****************************************************************************
  * pause.
  ****************************************************************************/
  public synchronized void pause () {
    if (play != null) {
      play.pause();
    }
  }
  /****************************************************************************
  * stop.
  ****************************************************************************/
  public synchronized void stop () {
    if (play != null) {
      play.stop();
    }
  }
  /****************************************************************************
  * close.
  ****************************************************************************/
  public synchronized void close () {
    if (play != null) {
      play.close();
    } else {
      remove();
    }
  }
  /****************************************************************************
  * remove.
  ****************************************************************************/
  private void remove () {
    manager.removePlayer(this);
  }
  /****************************************************************************
  * Play.
  ****************************************************************************/
  private class Play implements Runnable {
    private Socket s = null;
    private DataInputStream in = null;
    private DataOutputStream out = null;
    private Visual visual = visualSelector.getVisual();
    private boolean pause = false;
    private boolean stop = false;
    private boolean close = false;
    private long wallStart;
    private RateHandler rateHandler = new RateHandler();
    private double throttleValue = OptionsScreen.throttleValues[Viewer.optionsScreen.throttleCombo.getSelectedIndex()];
    public void start () {
      Util.startThread(this);
    }
    public void run () {
      visualSelector.setEnabled(false);
      repaint();
      try {
      	play();
      } catch (Exception e) {
      	// e.printStackTrace();
	if ((!stop) && (!close)) {
	  error(Util.encodeException(e));
	}
      } finally {
      	try {out.close();} catch (Exception ex) {}
      	try {in.close();} catch (Exception ex) {}
      	try {s.close();} catch (Exception ex) {}
      }
      layout.doFooter(passName);
      GUtil.invokeAndWait(new UpdateLabel(dateTimeField," "));
      GUtil.invokeAndWait(new UpdateLabel(nScansField," "));
      update(rateHandler.toText(true));
      visualSelector.setEnabled(true);
      repaint();
      if ((!stop) && (!close)) {
	if (Viewer.optionsScreen.loopCombo.getSelectedIndex() != OptionsScreen.LOOPING_OFF) {
	  SwingUtilities.invokeLater(new PlayNext());
	}
      }
      if (close) {
	remove();
      }
      playEnding();
    }
    private class PlayNext implements Runnable {
      public void run () {
	Viewer.passManager.playNext(passName,layout);
      }
    }
    private void play () throws Exception {
      GUtil.invokeAndWait(new UpdateLabel(dateTimeField,"Requesting pass data..."));
      GUtil.invokeAndWait(new UpdateLabel(nScansField," "));
      GUtil.invokeAndWait(new UpdateLabel(rateField," "));
      Util.sleep(0.5);
      s = new Socket(Viewer.serverHandler.getHostPort().getHost(),Viewer.serverHandler.getHostPort().getPort());
      out = new DataOutputStream(new BufferedOutputStream(s.getOutputStream()));
      send(Util.XML_HEADER + "<simulcastMessage>" + "<playMode clientId=\"" +
      	   Viewer.clientId + "\"/>" + "</simulcastMessage>");
      send(getRequestMessage());
      if (Viewer.optionsScreen.compressCheckBox.isSelected()) {
	in = new DataInputStream(new BufferedInputStream(new GZIPInputStream(s.getInputStream())));
      } else {
	in = new DataInputStream(new BufferedInputStream(s.getInputStream()));
      }
      read(in);
      in.close();
      out.close();
      s.close();
    }
    public void pause () {
      pause = !pause;
    }
    public void stop () {
      stop = true;
      abort();
    }
    public void close () {
      close = true;
      abort();
    }
    private void abort () {
      // try {s.shutdownOutput();} catch (Exception ex) {}
      // try {s.shutdownInput();} catch (Exception ex) {}
      try {out.close();} catch (Exception ex) {}
      try {in.close();} catch (Exception ex) {}
      try {s.close();} catch (Exception ex) {}
    }
    private String getRequestMessage () {
      StringBuffer msg = new StringBuffer();
      msg.append(Util.XML_HEADER);
      msg.append("<simulcastMessage>");
      msg.append("<playPass>");
      msg.append("<name>" + passName + "</name>");
      msg.append("<partial>");
      if (layout instanceof MapLayout) {
	MapLayout mapLayout = (MapLayout) layout;
        msg.append(mapLayout.getAreaFilter());
	msg.append("<send><geolocated/></send>");
      }
      if (layout instanceof InstrumentLayout) {
	msg.append("<send><debowed/></send>");
      }
      msg.append(visual.getRequestBandsElement());
      msg.append("</partial>");
      if (Viewer.optionsScreen.compressCheckBox.isSelected()) {
	msg.append("<compress>" + "gzip" + "</compress>");
      }
      msg.append("</playPass>");
      msg.append("</simulcastMessage>");
      return msg.toString();
    }
    private void send (String msg) throws Exception {
      out.writeUTF(msg);
      out.flush();
    }
    private void read (DataInputStream in) throws Exception {
      HeaderBlock headerBlock = null;
      ClassList blocks = new ClassList();
      while (true) {
        if (stop || close) {
          return;
        }
        if (pause) {
          Util.sleep(0.1);
          continue;
        }
        // System.out.println("1");
        Object block = Simulcast.readBlock(in,blocks);
        // System.out.println("2");
        if (block instanceof HeaderBlock) {
	  headerBlock = (HeaderBlock) block;
	  visual.doHeader(headerBlock);
	  layout.doHeader(passName,headerBlock);
	  wallStart = System.currentTimeMillis();
          continue;
        }
        if (block instanceof ScanBlock) {
          ScanBlock scanBlock = (ScanBlock) block;
          if (throttleValue > 0.0) {
            long passElapsed = scanBlock.getScanTime() - headerBlock.getStartedAt();
            long waitUntil = wallStart + Math.round(passElapsed/throttleValue);
            long now = System.currentTimeMillis();
            if (waitUntil > now) {
	      Util.sleep(waitUntil-now);
            }
          }
          if (!scanBlock.isNullScan()) {
            GeoBlock geoBlock = (GeoBlock) (headerBlock.isGeolocated() ? Simulcast.readBlock(in,blocks) : null);
            BowedDataBlock bowedBlock = (BowedDataBlock) (headerBlock.isGeolocated() ? Simulcast.readBlock(in,blocks) : null);
            DebowedDataBlock debowedBlock = (DebowedDataBlock) (headerBlock.isDebowed() ? Simulcast.readBlock(in,blocks) : null);
            ScanGroup scanGroup = new ScanGroup(scanBlock,geoBlock,bowedBlock,debowedBlock);
	    rateHandler.scanGroup(scanGroup);
	    byte[][][] data = (bowedBlock != null ? bowedBlock.getData() : debowedBlock.getData());
	    layout.doScan(passName,geoBlock,visual.doScan(scanBlock,data));
          }
	  update(scanBlock.getScanTime(),rateHandler.toText(false),scanBlock.getScanNumber());
	  continue;
        }
        if (block instanceof FooterBlock) {
	  return;
        }
        if (block instanceof ErrorBlock) {
          ErrorBlock errorBlock = (ErrorBlock) block;
	  throw new Exception(errorBlock.getText());
        }
        throw new Exception("unknown type of block");
      }
    }
    public void update (String rate) {
      GUtil.invokeAndWait(new UpdateLabel(rateField,rate));
    }
    public void update (long dateTime, String rate, int nScans) {
      GUtil.invokeAndWait(new UpdateLabel(dateTimeField,Util.encodeDateTime(dateTime)));
      GUtil.invokeAndWait(new UpdateLabel(nScansField,"Scan"+" "+Integer.toString(nScans)));
      rate = (rate != null ? rate : "");
      if (play != null) {
        if (pause) {
          rate = (!rate.equals("") ? (rate+", ") : "") + "PAUSED";
        } else {
          rate = (!rate.equals("") ? (rate+", ") : "") + "PLAYING";
        }
      }
      GUtil.invokeAndWait(new UpdateLabel(rateField,rate));
    }
  }
  /****************************************************************************
  * playEnding.
  ****************************************************************************/
  private synchronized void playEnding () {
    play = null;
  }
  /****************************************************************************
  * error.
  ****************************************************************************/
  public void error (String text) {
    GUtil.invokeAndWait(new DoError(text));
  }
  private class DoError implements Runnable {
    private String text;
    public DoError (String text) {
      this.text = text;
    }
    public void run () {
      /*
      Don't display error message when loop.
      */
      if (Viewer.optionsScreen.loopCombo.getSelectedIndex() == OptionsScreen.LOOPING_OFF)
      {
        contentPanel.removeAll();
        JButton okButton = new JButton("Ok");
        okButton.setMargin(new Insets(0,5,0,5));
        okButton.setForeground(Color.red);
        okButton.addActionListener(new OkButtonClicked());
        if (text.startsWith("java.lang.ClassCastException"))
        {
          text="Network or upstream server problem occured during acquisition of this pass.";
          contentPanel.add(new HPanel(new CustomLabel("Error: "+text,Color.black),GUtil.HGlue(),okButton));
        }
        else if (text.startsWith("not a skipable block"))
        {
          text="Network or upstream server problem occured during acquisition of this pass.";
          contentPanel.add(new HPanel(new CustomLabel("Error: "+text,Color.black),GUtil.HGlue(),okButton));
        }
        else if (text.startsWith("timeout while"))
        {
          text="Network or upstream server problem occured during acquisition of this pass.";
          contentPanel.add(new HPanel(new CustomLabel("Error: "+text,Color.black),GUtil.HGlue(),okButton));
        }
        else if (text.startsWith("java.net.SocketException"))
        {
          text="Network or upstream server problem occured during acquisition of this pass.";
          contentPanel.add(new HPanel(new CustomLabel("Error: "+text,Color.black),GUtil.HGlue(),okButton));
        }
        else
        {
          contentPanel.add(new HPanel(new CustomLabel("Error: "+text,Color.black),GUtil.HGlue(),okButton));
        }
        contentPanel.revalidate();
        contentPanel.repaint();
      }
    }
    private class OkButtonClicked implements ActionListener {
      public void actionPerformed (ActionEvent ae) {
        contentPanel.removeAll();
	dateTimeField.setText(" ");
	nScansField.setText(" ");
	rateField.setText(" ");
        contentPanel.add(playPanel);
        contentPanel.revalidate();
        contentPanel.repaint();
      }
    }
  }
}
