/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*  18-Oct-06, J Love, GST	Original version.
*   2-Jan-07, J Love, GST	Handle passes being removed from server.
*   2-Feb-07, J Love, GST	Tweaked layout.
*  26-Feb-07, J Love, GST	More tool tips.
*  15-Mar-07, Z Yin,  GST       Displaying image resolution choices equal or larger
*                               then data resolution.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.viewer;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.awt.event.*;
import java.util.*;

public class PlayerManager extends HPanel {
  private Layout layout;
  private StringMap2<Player> players = new StringMap2<Player>();
  private CustomCombo playerCombo;
  private HPanel comboPanel;
  private HPanel playerPanel;
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * PlayerManager.
  ****************************************************************************/
  public PlayerManager (Layout layout) {
    this.layout = layout;
    comboPanel = new HPanel();
    comboPanel.setAlignmentY(TOP_ALIGNMENT);
    playerPanel = new HPanel();
    playerPanel.setAlignmentY(TOP_ALIGNMENT);
    add(comboPanel);
    add(GUtil.HSpace(10));
    add(playerPanel);
  }
  /****************************************************************************
  * getPlayer.
  ****************************************************************************/
  public synchronized Player getPlayer (String passName) {
    return players.get(passName);
  }
  /****************************************************************************
  * getPlayerCombo.
  ****************************************************************************/
  public synchronized CustomCombo getPlayerCombo () {
    return playerCombo;
  }
  /****************************************************************************
  * addPlayer.
  ****************************************************************************/
  public synchronized void addPlayer (Player newPlayer) {
    purge(newPlayer);
    players.put(newPlayer.getPassName(),newPlayer);
    playerCombo = new CustomCombo(players.getStrings(),players.indexOf(newPlayer.getPassName()));
    playerCombo.setToolTipText("The \"selected\" pass.");
    playerCombo.addActionListener(new PlayerSelected());
    IconButton upButton = new IconButton(GUtil.upIcon,playerCombo.getPreferredSize().width);
    upButton.setToolTipText("Makes the previous pass in the list the \"selected\" pass.");
    upButton.addActionListener(new UpButtonClicked());
    upButton.setAlignmentX(CENTER_ALIGNMENT);
    IconButton downButton = new IconButton(GUtil.downIcon,playerCombo.getPreferredSize().width);
    downButton.setToolTipText("Makes the next pass in the list the \"selected\" pass.");
    downButton.addActionListener(new DownButtonClicked());
    downButton.setAlignmentX(CENTER_ALIGNMENT);
    comboPanel.removeAll();
    comboPanel.add(new VPanel(upButton,GUtil.VSpace(2),playerCombo,GUtil.VSpace(2),downButton));
    playerPanel.removeAll();
    playerPanel.add(newPlayer);
    revalidate();
    repaint();
  }
  /****************************************************************************
  * purge.
  ****************************************************************************/
  private void purge (Player newPlayer) {
    if (Viewer.optionsScreen.maxTotalPassesButton.isSelected()) {
      int max = getMax(Viewer.optionsScreen.maxTotalPassesField.getText());
      LinkedList<Player> list = new LinkedList<Player>();
      Iterator<Player> it = players.iterator();
      while (it.hasNext()) {
	Player player = it.next();
	list.add(player);
      }
      while (list.size() >= max) {
	Player player = list.removeFirst();
	player.close();
      }
      return;
    }
    if (Viewer.optionsScreen.maxPassesPerStationButton.isSelected()) {
      int max = getMax(Viewer.optionsScreen.maxPassesPerStationField.getText());
      LinkedList<Player> list = new LinkedList<Player>();
      Iterator<Player> it = players.iterator();
      while (it.hasNext()) {
	Player player = it.next();
	if (player.getStationId().equals(newPlayer.getStationId())) {
	  list.add(player);
	}
      }
      while (list.size() >= max) {
	Player player = list.removeFirst();
	player.close();
      }
      return;
    }
    if (Viewer.optionsScreen.maxPassesPerSatelliteButton.isSelected()) {
      int max = getMax(Viewer.optionsScreen.maxPassesPerSatelliteField.getText());
      LinkedList<Player> list = new LinkedList<Player>();
      Iterator<Player> it = players.iterator();
      while (it.hasNext()) {
	Player player = it.next();
	if (player.getPlatformName().equals(newPlayer.getPlatformName())) {
	  list.add(player);
	}
      }
      while (list.size() >= max) {
	Player player = list.removeFirst();
	player.close();
      }
      return;
    }
  }
  private int getMax (String text) {
    try {
      return Util.max(1,Integer.parseInt(text));
    } catch (Exception e) {
      return 1;
    }
  }
  /****************************************************************************
  * setSelectedPlayer.
  ****************************************************************************/
  public synchronized void setSelectedPlayer (String passName) {
    int index = players.indexOf(passName);
    playerCombo.setSelectedIndex(index);
  }
  /****************************************************************************
  * passRemoved.
  ****************************************************************************/
  public synchronized void passRemoved (String name) {
    if (containsPlayer(name)) {
      players.get(name).close();
    }
  }
  /****************************************************************************
  * removePlayer.
  ****************************************************************************/
  public synchronized void removePlayer (Player player) {
    String selected = playerCombo.getSelection();
    comboPanel.removeAll();
    playerPanel.removeAll();
    int index = players.indexOf(player.getPassName());
    players.remove(index);
    if (players.size() > 0) {
      playerCombo = new CustomCombo(players.getStrings(),/*index*/selected);
      playerCombo.addActionListener(new PlayerSelected());
      IconButton upButton = new IconButton(GUtil.upIcon,playerCombo.getPreferredSize().width);
      upButton.addActionListener(new UpButtonClicked());
      upButton.setAlignmentX(CENTER_ALIGNMENT);
      IconButton downButton = new IconButton(GUtil.downIcon,playerCombo.getPreferredSize().width);
      downButton.addActionListener(new DownButtonClicked());
      downButton.setAlignmentX(CENTER_ALIGNMENT);
      comboPanel.add(new VPanel(upButton,GUtil.VSpace(2),playerCombo,GUtil.VSpace(2),downButton));
      playerPanel.add(players.get(/*index*/playerCombo.getSelectedIndex()));
      layout.playerRemoved(player.getPassName(),players.get(/*index*/playerCombo.getSelectedIndex()).getPassName());
    } else {
      layout.playerRemoved(player.getPassName(),null);
    }
    revalidate();
    repaint();
  }
  /****************************************************************************
  * closeAll.
  ****************************************************************************/
  public synchronized void closeAll () {
    for (String passName : players.getStrings()) {
       players.get(passName).close();
    }
  }
  /****************************************************************************
  * containsPlayer.
  ****************************************************************************/
  public synchronized boolean containsPlayer (String passName) {
    return players.contains(passName);
  }
  /****************************************************************************
  * getVisualSelector.
  ****************************************************************************/
  public synchronized VisualSelector getVisualSelector () {
    if (players.size() > 0) {
      return players.get(playerCombo.getSelection()).getVisualSelector();
    }
    return null;
  }
  /****************************************************************************
  * clear.
  ****************************************************************************/
  public synchronized void clear () {
    players.clear();
    playerPanel.removeAll();
    revalidate();
    repaint();
  }
  /****************************************************************************
  * playAll.
  ****************************************************************************/
  public synchronized void playAll () {
    Iterator<Player> it = players.iterator();
    while (it.hasNext()) {
      it.next().play();
    }
  }
  /****************************************************************************
  * pauseAll.
  ****************************************************************************/
  public synchronized void pauseAll () {
    Iterator<Player> it = players.iterator();
    while (it.hasNext()) {
      it.next().pause();
    }
  }
  /****************************************************************************
  * stopAll.
  ****************************************************************************/
  public synchronized void stopAll () {
    Iterator<Player> it = players.iterator();
    while (it.hasNext()) {
      it.next().stop();
    }
  }
  /****************************************************************************
  * DownButtonClicked.
  ****************************************************************************/
  private class DownButtonClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      int index = (playerCombo.getSelectedIndex()+1) % playerCombo.getItemCount();
      playerCombo.setSelectedIndex(index);
    }
  }
  /****************************************************************************
  * UpButtonClicked.
  ****************************************************************************/
  private class UpButtonClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      int index = (playerCombo.getSelectedIndex()-1+playerCombo.getItemCount()) % playerCombo.getItemCount();
      playerCombo.setSelectedIndex(index);
    }
  }
  /****************************************************************************
  * PlayerSelected.
  ****************************************************************************/
  private class PlayerSelected implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      Player player = (Player) players.get(playerCombo.getSelectedIndex());
      playerPanel.removeAll();
      playerPanel.add(player);
      playerPanel.revalidate();
      playerPanel.repaint();
      layout.playerSelected(player.getPassName());
    }
  }
}
