/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*  14-Nov-06, J Love, GST	Original version.
*
*  The code to create the movie was taken from an example from Sun
*  Microsystems, Inc. that created Quicktime movies from JPEG image files.
*  The copyright notice for that example follows.  It has been modified to
*  create both Quicktime and AVI movies from BufferedImages.
*
******************************************************************************/

/*
 * @(#)JpegImagesToMovie.java	1.3 01/03/13
 *
 * Copyright (c) 1999-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

package gov.nasa.gsfc.aisb.simulcast.viewer;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.awt.*;
import java.io.*;
import java.awt.image.*;
import javax.imageio.*;
import javax.imageio.stream.*;
import javax.imageio.plugins.jpeg.*;
import javax.media.*;
import javax.media.control.*;
import javax.media.protocol.*;
import javax.media.datasink.*;
import javax.media.format.*;

public class VideoRecorder implements Runnable, ControllerListener, DataSinkListener {
  private Component component;
  private int type;
  private String path;
  private int rate;
  private int interval;
  private Object waitSync = new Object();
  private boolean stateTransitionOK = true;
  private Object waitFileSync = new Object();
  private boolean fileDone = false;
  private boolean fileSuccess = true;
  private boolean stop = false;
  private String error = null;
  public static final int AVI = 1;
  public static final int QT = 2;
  public static interface Listener {
    public void status (String text);
    public void completed (String text);
    public void error (String text);
  }
  private Listener listener;
  /****************************************************************************
  * LayoutRecorder.
  ****************************************************************************/
  public VideoRecorder (Component component, int type, String path, int rate, Listener listener) {
    this.component = component;
    this.type = type;
    this.path = path;
    this.rate = rate;
    this.listener = listener;
    interval = 1000 / rate;
  }
  /****************************************************************************
  * start.
  ****************************************************************************/
  public void start () {
    Util.startThread(this);
  }
  /****************************************************************************
  * run.
  ****************************************************************************/
  public void run () {
    try {
      listener.status("RECORDING");
      record();
      if (error != null) {
	listener.error(error);
      } else {
	listener.completed("COMPLETED");
      }
    } catch (Exception e) {
      listener.error(Util.encodeThrowable(e));
    }
  }
  /****************************************************************************
  * record.
  ****************************************************************************/
  private void record () throws Exception {
    ImageDataSource ids = new ImageDataSource(component.getWidth(),component.getHeight());
    Processor p = Manager.createProcessor(ids);
    p.addControllerListener(this);
    p.configure();
    if (!waitForState(p,Processor.Configured)) {
      throw new Exception("Failed to configure the processor.");
    }
    switch (type) {
      case AVI: {
	p.setContentDescriptor(new ContentDescriptor(FileTypeDescriptor.MSVIDEO));
	break;
      }
      case QT: {
	p.setContentDescriptor(new ContentDescriptor(FileTypeDescriptor.QUICKTIME));
	break;
      }
    }
    TrackControl tcs[] = p.getTrackControls();
    Format f[] = tcs[0].getSupportedFormats();
    if (f == null || f.length <= 0) {
      throw new Exception("The mux does not support the input format: " + tcs[0].getFormat());
    }
    tcs[0].setFormat(f[0]);
    p.realize();
    if (!waitForState(p,Processor.Realized)) {
      throw new Exception("Failed to realize the processor.");
    }
    DataSink dsink = null;
    switch (type) {
      case AVI: {
        dsink = createDataSink(p,new MediaLocator("file://"+path));
        break;
      }
      case QT: {
        dsink = createDataSink(p,new MediaLocator("file://"+path));
	break;
      }
    }
    dsink.addDataSinkListener(this);
    // fileDone = false;			// IS THIS NEEDED?
    try {
      p.start();
      dsink.start();
    } catch (Exception e) {
      throw new Exception("error during processing: " + e);
    }
    waitForFileDone();
    try {
      dsink.close();
    } catch (Exception e) {
      throw new Exception("error closing data sink: " + e);
    }
    p.removeControllerListener(this);	// IS THIS NEEDED
  }
  /****************************************************************************
  * controllerUpdate.
  ****************************************************************************/
  public void controllerUpdate (ControllerEvent evt) {
    if (evt instanceof ConfigureCompleteEvent ||
	evt instanceof RealizeCompleteEvent ||
	evt instanceof PrefetchCompleteEvent) {
      synchronized (waitSync) {
	stateTransitionOK = true;
	waitSync.notifyAll();
      }
      return;
    }
    if (evt instanceof ResourceUnavailableEvent) {
      synchronized (waitSync) {
	stateTransitionOK = false;
	waitSync.notifyAll();
      }
      return;
    }
    if (evt instanceof EndOfMediaEvent) {
      evt.getSourceController().stop();
      evt.getSourceController().close();
      return;
    }
  }
  /****************************************************************************
  * dataSinkUpdate.
  ****************************************************************************/
  public void dataSinkUpdate (DataSinkEvent evt) {
    if (evt instanceof EndOfStreamEvent) {
      synchronized (waitFileSync) {
	fileDone = true;
	waitFileSync.notifyAll();
      }
      return;
    }
    if (evt instanceof DataSinkErrorEvent) {
      synchronized (waitFileSync) {
	fileDone = true;
	fileSuccess = false;
	waitFileSync.notifyAll();
      }
      return;
    }
  }
  /****************************************************************************
  * waitForState.
  * SHOULD THE TRY-CATCH BE INSIDE THE WHILE LOOP?
  ****************************************************************************/
  private boolean waitForState(Processor p, int state) {
    synchronized (waitSync) {
      try {
	while ((p.getState() < state) && stateTransitionOK) {
	  waitSync.wait();
	}
      } catch (Exception e) {
	
      }
      return stateTransitionOK;
    }
  }
  /****************************************************************************
  * waitForFileDone.
  * SHOULD THE TRY-CATCH BE INSIDE THE WHILE LOOP?
  ****************************************************************************/
  private boolean waitForFileDone() {
    synchronized (waitFileSync) {
      try {
	while (!fileDone) {
	  waitFileSync.wait();
	}
      } catch (Exception e) {
	  
      }
      return fileSuccess;
    }
  }
  /****************************************************************************
  * createDataSink.
  ****************************************************************************/
  private DataSink createDataSink (Processor p, MediaLocator outML) throws Exception {
    DataSource ds = p.getDataOutput();
    if (ds == null) {
      throw new Exception("the processor does not have an output DataSource");
    }
    DataSink dsink;
    try {
      dsink = Manager.createDataSink(ds,outML);
      dsink.open();
    } catch (Exception e) {
      throw new Exception("Cannot create the DataSink: " + e);
    }
    return dsink;
  }
  /****************************************************************************
  * ImageDataSource.
  ****************************************************************************/
  private class ImageDataSource extends PullBufferDataSource {
    private ImageSourceStream streams[];
    public ImageDataSource (int width, int height) {
      streams = new ImageSourceStream[1];
      streams[0] = new ImageSourceStream(width,height);
    }
    public void setLocator (MediaLocator source) {
    }
    public MediaLocator getLocator () {
      return null;
    }
    public String getContentType () {
      return ContentDescriptor.RAW;
    }
    public void connect () {
    }
    public void disconnect () {
    }
    public void start () {
    }
    public void stop () {
    }
    public PullBufferStream[] getStreams () {
      return streams;
    }
    public Time getDuration() {
      return DURATION_UNKNOWN;
    }
    public Object[] getControls () {
      return new Object[0];
    }
    public Object getControl (String type) {
      return null;
    }
  }
  /****************************************************************************
  * ImageSourceStream.
  ****************************************************************************/
  private class ImageSourceStream implements PullBufferStream {
    private int iWidth, iHeight;
    private int oWidth, oHeight;
    private VideoFormat format;
    private long startedAt;
    private long prevAt;
    private long nextAt;
    private long seqNo = 0;
    private boolean ended = false;
    public ImageSourceStream (int width, int height) {
      iWidth = width;
      iHeight = height;
      switch (type) {
        case AVI: {
	  if (iWidth % 4 == 0) {
	    oWidth = iWidth;
	  } else {
	    oWidth = iWidth + (4 - (iWidth%4));
	  }
	  if (iHeight % 4 == 0) {
	    oHeight = iHeight;
	  } else {
	    oHeight = iHeight + (4 - (iHeight%4));
	  }
	  break;
        }
        case QT: {
	  oWidth = iWidth;
	  oHeight = iHeight;
	  break;
        }
      }
      switch (type) {
        case AVI: {
          format = new VideoFormat(VideoFormat.RGB,new Dimension(oWidth,oHeight),Format.NOT_SPECIFIED,Format.byteArray,(float)rate);
          break;
        }
        case QT: {
	  format = new VideoFormat(VideoFormat.JPEG,new Dimension(oWidth,oHeight),Format.NOT_SPECIFIED,Format.byteArray,(float)rate);
	  break;
        }
      }
    }
    public boolean willReadBlock() {
      return false;
    }
    public void read (Buffer buf) throws IOException {
      try {
	doRead(buf);
      } catch (Exception e) {
	stop = true;
	error = Util.encodeThrowable(e);
      }
    }
    private void doRead (Buffer buf) throws IOException {
      if (stop) {
	buf.setEOM(true);
	buf.setOffset(0);
	buf.setLength(0);
	ended = true;
	return;
      }
      if (seqNo == 0) {
	startedAt = System.currentTimeMillis();
	prevAt = startedAt;
	nextAt = startedAt + interval;
      }	
      long at = System.currentTimeMillis();
      if (at < nextAt) {
	Util.sleep(nextAt-at);
	at = nextAt;
      }
      BufferedImage bi = new BufferedImage(iWidth,iHeight,BufferedImage.TYPE_INT_RGB);
      GUtil.invokeAndWait(new Paint(component,bi));
      switch (type) {
        case AVI: {
	  byte[] data = new byte[3*oWidth*oHeight];
	  buf.setData(data);
	  int i = 0;
	  for (int y = iHeight-1; y >= 0; y--) {
	     for (int x = 0; x < iWidth; x++) {
	        int v = bi.getRGB(x,y);
	        data[i++] = (byte) ((v & 0x000000FF));
	        data[i++] = (byte) ((v & 0x0000FF00) >> 8);
	        data[i++] = (byte) ((v & 0x00FF0000) >> 16);
	     }
	     for (int x = iWidth; x < oWidth; x++) {
	        data[i++] = (byte) 0;
	        data[i++] = (byte) 0;
	        data[i++] = (byte) 0;
	     }
          }
	  for (int y = oHeight-1; y >= iHeight; y--) {
	     for (int x = 0; x < oWidth; x++) {
	        data[i++] = (byte) 0;
	        data[i++] = (byte) 0;
	        data[i++] = (byte) 0;
	     }
	  }
	  buf.setLength(3*oWidth*oHeight);
	  break;
        }
        case QT: {
          ByteArrayOutputStream out = new ByteArrayOutputStream();
          ImageWriter writer = ImageIO.getImageWritersByFormatName("jpeg").next();
          MemoryCacheImageOutputStream out2 = new MemoryCacheImageOutputStream(out);
          writer.setOutput(out2);
          JPEGImageWriteParam param = new JPEGImageWriteParam(null);
          param.setCompressionMode(ImageWriteParam.MODE_EXPLICIT);
          param.setCompressionQuality((float)1.0);
          writer.write(null,new IIOImage(bi,null,null),param);
          out2.close();
          out.close();
          buf.setData(out.toByteArray());
          buf.setLength(out.size());
          break;
        }
      }
      buf.setOffset(0);
      buf.setFormat(format);
      buf.setFlags(buf.getFlags() | Buffer.FLAG_KEY_FRAME);
      buf.setTimeStamp((at-startedAt)*1000*1000);
      buf.setDuration((at-prevAt)*1000*1000);
      buf.setSequenceNumber(seqNo++);
      long secs = (at-startedAt) / 1000;
      listener.status("RECORDING  " + Long.toString(secs/60) + ":" + Util.lz2.format(secs%60) + "  [" + Long.toString(seqNo) + " frames]");
      prevAt = at;
      nextAt = at + interval;
    }
    public Format getFormat () {
      return format;
    }
    public ContentDescriptor getContentDescriptor() {
      return new ContentDescriptor(ContentDescriptor.RAW);
    }
    public long getContentLength() {
      return 0;
    }
    public boolean endOfStream() {
      return ended;
    }
    public Object[] getControls() {
      return new Object[0];
    }
    public Object getControl(String type) {
      return null;
    }
  }
  /****************************************************************************
  * stop.
  ****************************************************************************/
  public void stop () {
    stop = true;
  }
  /****************************************************************************
  * Paint.
  ****************************************************************************/
  private class Paint implements Runnable {
    private Component component;
    private BufferedImage bi;
    public Paint (Component component, BufferedImage bi) {
      this.component = component;
      this.bi = bi;
    }
    public void run () {
      component.paint(bi.getGraphics());
    }
  }
}
