/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*  11-Nov-06, J Love, GST	Original version.
*  10-Feb-07, J Love, GST	Use event thread to update labels.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.viewer;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import java.io.*;

public class VideoRecorderControls extends JFrame implements VideoRecorder.Listener {
  private static VideoRecorderControls controls = null;
  private JRadioButton viewerButton, mapLayoutButton, mapButton, instrumentLayoutButton;
  private JRadioButton aviButton, qtButton;
  private IntegerSlider rateSlider;
  private JTextField pathField;
  private CustomLabel statusField;
  private VideoRecorder recorder = null;
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * create.
  ****************************************************************************/
  public static void create () {
    if (controls == null) {
      controls = new VideoRecorderControls();
    }
  }
  /****************************************************************************
  * VideoRecorderControls.
  ****************************************************************************/
  public VideoRecorderControls () {
    setTitle("Simulcast Video Recorder");
    getContentPane().setLayout(new BorderLayout());
    addWindowListener(new Disposed());
    
    viewerButton = new JRadioButton("Viewer",true);
    viewerButton.setToolTipText("Entire Viewer Window");
    mapLayoutButton = new JRadioButton("Map Layout",false);
    mapLayoutButton.setToolTipText("Map Layout (including player controls)");
    mapButton = new JRadioButton("Map",false);
    mapButton.setToolTipText("Map (excluding player controls)");
    instrumentLayoutButton = new JRadioButton("Instrument Layout",false);
    instrumentLayoutButton.setToolTipText("Instrument Layout (including player controls)");
    GUtil.buttonGroup(viewerButton,mapLayoutButton,mapButton,instrumentLayoutButton);
    HPanel sourcePanel = new HPanel(viewerButton,GUtil.HSpace(10),mapLayoutButton,GUtil.HSpace(10),mapButton,GUtil.HSpace(10),instrumentLayoutButton,GUtil.HGlue());
    sourcePanel.setBorder(new TitledBorder(new LineBorder(Color.black,1)," Source "));

    aviButton = new JRadioButton("AVI",true);
    aviButton.addActionListener(new FormatSelected(".avi"));
    aviButton.setToolTipText("AVI Movie");
    qtButton = new JRadioButton("QUICKTIME",false);
    qtButton.addActionListener(new FormatSelected(".mov"));
    qtButton.setToolTipText("Quicktime Movie");
    GUtil.buttonGroup(aviButton,qtButton);
    HPanel formatPanel = new HPanel(aviButton,GUtil.HSpace(10),qtButton,GUtil.HGlue());
    formatPanel.setBorder(new TitledBorder(new LineBorder(Color.black,1)," Format "));
    rateSlider = new IntegerSlider(1,1,60,"Frame Rate (frames/second)");
    HPanel ratePanel = new HPanel(new MarginPanel(0,5,rateSlider,3,0));
    ratePanel.setBorder(new TitledBorder(new LineBorder(Color.black,1)," Frame Rate "));
    
    pathField = new JTextField(System.getProperty("user.home")+File.separator+"sc-viewer.avi",50);
    pathField.setToolTipText("Location and name of the image file to create.");
    pathField.setMaximumSize(new Dimension(pathField.getMaximumSize().width,pathField.getPreferredSize().height));
    JButton browseButton = new JButton("Browse");
    browseButton.setToolTipText("Displays a file chooser dialog.");
    browseButton.addActionListener(new BrowseButtonClicked());
    browseButton.setMargin(new Insets(0,5,0,5));
    HPanel pathPanel = new HPanel(new MarginPanel(0,5,new HPanel(pathField,GUtil.HSpace(5),browseButton),3,5));
    pathPanel.setBorder(new TitledBorder(new LineBorder(Color.black,1)," Save As "));
    
    JButton startButton = new JButton("Start");
    startButton.setToolTipText("Start recording.");
    startButton.addActionListener(new StartButtonClicked());
    statusField = new CustomLabel("");
    JButton stopButton = new JButton("Stop");
    stopButton.setToolTipText("Stop recording.");
    stopButton.addActionListener(new StopButtonClicked());
    MarginPanel buttonsPanel = new MarginPanel(5,5,new HPanel(startButton,GUtil.HGlue(),statusField,GUtil.HGlue(),stopButton),5,5);
    
    getContentPane().add(new VPanel(sourcePanel,formatPanel,ratePanel,pathPanel,GUtil.VGlue(),buttonsPanel));
    
    pack();
    setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
    GUtil.configureWindow(this,0.40,0.30);
    setVisible(true);
  }
  /****************************************************************************
  * status.
  ****************************************************************************/
  public void status (String text) {
    GUtil.invokeAndWait(new UpdateLabel(statusField,text,Color.BLACK));
  }
  /****************************************************************************
  * completed.
  ****************************************************************************/
  public void completed (String text) {
    recorder = null;
    GUtil.invokeAndWait(new UpdateLabel(statusField,text,Color.BLACK));
  }
  /****************************************************************************
  * error.
  ****************************************************************************/
  public void error (String text) {
    recorder = null;
    GUtil.invokeAndWait(new UpdateLabel(statusField,text,Color.RED));
  }
  /****************************************************************************
  * FormatSelected.
  ****************************************************************************/
  private class FormatSelected implements ActionListener {
    private String extension;
    public FormatSelected (String extension) {
      this.extension = extension;
    }
    public void actionPerformed (ActionEvent ae) {
      String path = pathField.getText();
      int index = path.lastIndexOf('.');
      if (index != -1) {
	path = path.substring(0,index) + extension;
      }
      pathField.setText(path);
    }
  }
  /****************************************************************************
  * BrowseButtonClicked.
  ****************************************************************************/
  private class BrowseButtonClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      browse();
    }
  }
  private void browse () {
    JFileChooser chooser = new JFileChooser();
    File file = new File(pathField.getText());
    chooser.setCurrentDirectory(file.exists() ? file : new File(file.getParent()));
    chooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
    chooser.setApproveButtonText("Select");
    chooser.setSelectedFile(file);
    switch (chooser.showDialog(this,"Select")) {
      case JFileChooser.APPROVE_OPTION: {
	pathField.setText(chooser.getSelectedFile().getAbsolutePath());
	break;
      }
    }
  }
  /****************************************************************************
  * StartButtonClicked.
  ****************************************************************************/
  private class StartButtonClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      start();
    }
  }
  private void start () {
    if (recorder == null) {
      int type = 0;
      if (aviButton.isSelected()) {
	type = VideoRecorder.AVI;
      }
      if (qtButton.isSelected()) {
	type = VideoRecorder.QT;
      }
      Component component = null;
      if (viewerButton.isSelected()) {
	component = Viewer.frame.getContentPane();
      }
      if (mapLayoutButton.isSelected()) {
	component = Viewer.mapLayout;
      }
      if (mapButton.isSelected()) {
	component = Viewer.mapLayout.map;
      }
      if (instrumentLayoutButton.isSelected()) {
	component = Viewer.instrumentLayout;
      }
      recorder = new VideoRecorder(component,type,pathField.getText(),rateSlider.getValue(),this);
      recorder.start();
    }
  }
  /****************************************************************************
  * StopButtonClicked.
  ****************************************************************************/
  private class StopButtonClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      if (recorder != null) {
	recorder.stop();
      }
    }
  }
  /****************************************************************************
  * Disposed.
  ****************************************************************************/
  private class Disposed extends WindowAdapter {
    public void windowClosed (WindowEvent we) {
      if (recorder != null) {
	recorder.stop();
	recorder = null;
      }
      controls = null;
    }
  }
}
