/*
Copyright (c) 1999-2007, United States Government, as represented by
the Administrator for The National Aeronautics and Space Administration.
All rights reserved.
*/
/******************************************************************************
*
*  SIMULCAST/CLIENT
*
*  History:
*
*   9-Nov-06, J Love, GST	Original version.
*  11-Feb-07, J Love, GST	Tweaked layout.
*  26-Feb-07, J Love, GST	More tooltips.
*  24-Apr-07, Z Yin, GST	Make resolution change intelligently.
*
******************************************************************************/

package gov.nasa.gsfc.aisb.simulcast.viewer;
import gov.nasa.gsfc.aisb.simulcast.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;

public class Viewer extends JFrame {
  protected static JFrame frame;
  protected static String clientId = "";
  protected static String serverSimulcastVersion = null;
  private static JRadioButton passManagerButton;
  protected static JRadioButton instrumentLayoutButton;
  protected static JRadioButton mapLayoutButton;
  private static JRadioButton optionsScreenButton;
  protected static PassManager passManager;
  protected static InstrumentLayout instrumentLayout;
  protected static MapLayout mapLayout;
  protected static boolean playerBand;
  protected static OptionsScreen optionsScreen;
  protected static ServerHandler serverHandler;
  protected static ServerListener serverListener;
  private static boolean maximize = false;
  private static String autoPlay = "map";
  private static boolean mapAutoZoom = false;
  protected static final Color DARK_GREEN = new Color(0,200,0);
  private static JPanel mainPanel;
  private MenuBarClock clock;
  private static String[] serverChoices = new String[] {
    "nisds1.sci.gsfc.nasa.gov:3502",
    "localhost:3502"
  };
  private static final String USAGE = "Arguments: [-maximize] [-auto_play {instrument|map}] [-map_auto_zoom] {-server_choice <host:port>}* [<host:port>]";
  private static final long serialVersionUID = 1L;
  /****************************************************************************
  * main.
  ****************************************************************************/
  public static void main (String[] args) throws Exception {
    HostPort serverHostPort = null;
    try {
      serverHostPort = parseArgs(args);
    } catch (Exception e) {
      System.out.println("Error: " + Util.encodeThrowable(e));
      System.out.println(USAGE);
      System.exit(1);
    }
    Viewer viewer = new Viewer(serverHostPort);
    viewer.start(serverHostPort == null);
    Thread.setDefaultUncaughtExceptionHandler(new Uncaught());
    /*    
    while (true) {
      Util.sleep(60.0);
      Util.threadDump();
    }
    */
  }
  /****************************************************************************
  * parseArgs.
  ****************************************************************************/
  private static HostPort parseArgs (String[] args) throws Exception {
    HostPort serverHostPort = null;
    LinkedList<String> servers = new LinkedList<String>();
    int x = 0;
    while (x < args.length) {
      String arg = args[x++];
      if (arg.equals("-maximize")) {
      	maximize = true;
      	continue;
      }
      if (arg.equals("-auto_play")) {
      	autoPlay = args[x++];
      	continue;
      }
      if (arg.equals("-map_auto_zoom")) {
      	mapAutoZoom = true;
      	continue;
      }
      if (arg.equals("-server_choice")) {
      	servers.add(args[x++]);
      	continue;
      }
      if (serverHostPort == null) {
      	serverHostPort = new HostPort(arg);
      	continue;
      }
      throw new Exception("unexpected argument: " + arg);
    }
    if (servers.size() > 0) {
      serverChoices = (String[]) servers.toArray(new String[servers.size()]);
    }
    return serverHostPort;
  }
  /****************************************************************************
  * Viewer.
  ****************************************************************************/
  public Viewer (HostPort serverHostPort) throws Exception {
    frame = this;
    // UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
    UIManager.getDefaults().put("ComboBox.disabledForeground",Color.black);
    setTitle("Simulcast Viewer");
    getContentPane().setLayout(new BorderLayout());
    JMenuBar menuBar = new JMenuBar();
    JMenu fileMenu = new JMenu("File");
    JMenuItem connectItem = new JMenuItem("Connect...");
    connectItem.setToolTipText("Connect to a Simulcast Server.");
    connectItem.addActionListener(new ConnectButtonClicked());
    fileMenu.add(connectItem);
    JMenuItem disconnectItem = new JMenuItem("Disconnect");
    disconnectItem.setToolTipText("Disconnect from the current Simulcast Server.");
    disconnectItem.addActionListener(new DisconnectButtonClicked());
    fileMenu.add(disconnectItem);
    fileMenu.addSeparator();
    JMenuItem saveVisualsItem = new JMenuItem("Save Visuals as Defaults");
    saveVisualsItem.setToolTipText("Saves the current visuals as the defaults on the Options screen.");
    saveVisualsItem.addActionListener(new SaveVisualsButtonClicked());
    fileMenu.add(saveVisualsItem);
    JMenuItem videoRecorderItem = new JMenuItem("Show Video Recorder");
    videoRecorderItem.setToolTipText("Displays the Video Recorder window.");
    videoRecorderItem.addActionListener(new ShowVideoRecorderButtonClicked());
    fileMenu.add(videoRecorderItem);
    fileMenu.addSeparator();
    JMenuItem exitItem = new JMenuItem("Exit");
    exitItem.setToolTipText("Exit the Simulcast Viewer.");
    exitItem.addActionListener(new ExitButtonClicked());
    fileMenu.add(exitItem);
    menuBar.add(fileMenu);
    JMenu helpMenu = new JMenu("Help");
    JMenuItem aboutItem = new JMenuItem("About Simulcast...");
    aboutItem.setToolTipText("Displays Simulcast version information.");
    aboutItem.addActionListener(new AboutButtonClicked(this));
    helpMenu.add(aboutItem);
    menuBar.add(helpMenu);
    menuBar.add(Box.createHorizontalGlue());
    clock = new MenuBarClock(true);
    menuBar.add(clock);
    mainPanel = new JPanel(new BorderLayout());
    serverListener = new ServerListener();
    serverHandler = new ServerHandler(serverHostPort,serverListener);
    passManager = new PassManager();
    mapLayout = new MapLayout(MapHandler.MERCATOR);
    instrumentLayout = new InstrumentLayout();
    optionsScreen = new OptionsScreen(autoPlay,mapAutoZoom);
    passManagerButton = new JRadioButton("Pass Manager",false);
    passManagerButton.setToolTipText("Switches to the Pass Manager screen.");
    passManagerButton.addActionListener(new PassManagerButtonClicked());
    mapLayoutButton = new JRadioButton("Map Layout",false);
    mapLayoutButton.setToolTipText("Switches to the Map Layout screen.");
    mapLayoutButton.addActionListener(new MapLayoutButtonClicked());
    instrumentLayoutButton = new JRadioButton("Instrument Layout",false);
    instrumentLayoutButton.setToolTipText("Switches to the Instrument Layout screen.");
    instrumentLayoutButton.addActionListener(new InstrumentLayoutButtonClicked());
    optionsScreenButton = new JRadioButton("Options",false);
    optionsScreenButton.setToolTipText("Switches to the Options screen.");
    optionsScreenButton.addActionListener(new OptionsButtonClicked());
    GUtil.buttonGroup(passManagerButton,mapLayoutButton,instrumentLayoutButton,optionsScreenButton);
    HPanel buttonPanel = new HPanel(GUtil.HGlue(),passManagerButton,GUtil.HSpace(40),mapLayoutButton,GUtil.HSpace(40),instrumentLayoutButton,GUtil.HSpace(40),optionsScreenButton,GUtil.HGlue());
    getContentPane().add(new VPanel(menuBar,new MarginPanel(3,3,buttonPanel,3,3),GUtil.VSpace(5),new MarginPanel(3,3,mainPanel,3,3)),BorderLayout.CENTER);
    pack();
    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    if (maximize) {
      setExtendedState(MAXIMIZED_BOTH);
    } else {
      // GUtil.configureWindow(this,0.95,0.90);
      // GUtil.configureWindow(this,1270,960);
      GUtil.configureWindow(this,1000,700);
    }
    
    /*
      setUndecorated(true);
      GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice().setFullScreenWindow(frame);
    */
    
    setVisible(true);
    passManagerButton.doClick();
  }
  /****************************************************************************
  * start.
  ****************************************************************************/
  public void start (boolean promptForServer) throws Exception {
    clock.start();
    serverHandler.start();
    if (promptForServer) {
      Util.sleep(0.5);
      SwingUtilities.invokeLater(new PromptForServer());
    }
  }
  /****************************************************************************
  * switchMainPanel.
  ****************************************************************************/
  private static void switchMainPanel (Component component) {
    mainPanel.removeAll();
    mainPanel.add(component);
    mainPanel.revalidate();
    mainPanel.repaint();
  }
  /****************************************************************************
  * switchToPassManager.
  ****************************************************************************/
  public static void switchToPassManager () {
    SwingUtilities.invokeLater(new SwitchToPassManager());
  }
  private static class SwitchToPassManager implements Runnable {
    public void run () {
      passManagerButton.setSelected(true);
      switchMainPanel(passManager);
    }
  }
  /****************************************************************************
  * switchToInstrumentLayout.
  ****************************************************************************/
  public static void switchToInstrumentLayout () {
    SwingUtilities.invokeLater(new SwitchToInstrumentLayout());
  }
  private static class SwitchToInstrumentLayout implements Runnable {
    public void run () {
      instrumentLayoutButton.setSelected(true);
      switchMainPanel(instrumentLayout);
    }
  }
  /****************************************************************************
  * switchToMapLayout.
  ****************************************************************************/
  public static void switchToMapLayout () {
    SwingUtilities.invokeLater(new SwitchToMapLayout());
  }
  private static class SwitchToMapLayout implements Runnable {
    public void run () {
      mapLayoutButton.setSelected(true);
      switchMainPanel(mapLayout);
    }
  }
  /****************************************************************************
  * PromptForServer.
  ****************************************************************************/
  private class PromptForServer implements Runnable {
    private JDialog dialog;
    private JComboBox combo;
    private boolean done = false;
    public void run () {
      dialog = new JDialog(frame,"Simulcast Server Selection",true);
      dialog.setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
      dialog.setResizable(false);
      Container content = dialog.getContentPane();
      combo = new JComboBox(serverChoices);
      combo.setToolTipText("Enter the Simulcast Server (in the form host:port) or select one from the list.");
      combo.setEditable(true);
      combo.setMaximumSize(combo.getPreferredSize());
      JButton connectButton = new JButton("Connect");
      connectButton.setToolTipText("Connects to the specified Simulcast Server.");
      connectButton.addActionListener(new SelectButtonClicked());
      JButton cancelButton = new JButton("Cancel");
      cancelButton.setToolTipText("Cancels without connecting.");
      cancelButton.addActionListener(new CancelButtonClicked());
      content.add(new MarginPanel(10,10,new VPanel(
        new HPanel(GUtil.HGlue(),new JLabel("Select the Simulcast Server host:port..."),GUtil.HGlue()),
        GUtil.VSpace(10),
        new HPanel(GUtil.HGlue(),combo,GUtil.HGlue()),
        GUtil.VSpace(10),
        new HPanel(GUtil.HGlue(),connectButton,GUtil.HSpace(10),cancelButton,GUtil.HGlue()),
        GUtil.VGlue()
      ),10,10));
      dialog.pack();
      GUtil.centerWindow(dialog);
      dialog.setVisible(true);
      while (!done) {
      	Util.sleep(0.25);
      }
    }
    private class SelectButtonClicked implements ActionListener {
      public void actionPerformed (ActionEvent ae) {
        try {
          HostPort serverHostPort = new HostPort((String)combo.getSelectedItem());
          serverHandler.connect(serverHostPort);
          dialog.dispose();
          done = true;
        } catch (Exception e) {
          dialog.dispose();
          JOptionPane.showMessageDialog(frame,e.toString(),"Error",
            				  JOptionPane.ERROR_MESSAGE);
          dialog.setVisible(true);
        }
      }
    }
    private class CancelButtonClicked implements ActionListener {
      public void actionPerformed (ActionEvent ae) {
        dialog.dispose();
        done = true;
      }
    }
  }
  /****************************************************************************
  * PassManagerButtonClicked.
  ****************************************************************************/
  private class PassManagerButtonClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      switchMainPanel(passManager);
    }
  }
  /****************************************************************************
  * MapButtonClicked.
  ****************************************************************************/
  private class MapLayoutButtonClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      switchMainPanel(mapLayout);
      passManager.loadSelectedPasses(mapLayout);
    }
  }
  /****************************************************************************
  * InstrumentButtonClicked.
  ****************************************************************************/
  private class InstrumentLayoutButtonClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      switchMainPanel(instrumentLayout);
      passManager.loadSelectedPasses(instrumentLayout);
    }
  }
  /****************************************************************************
  * OptionsButtonClicked.
  ****************************************************************************/
  private class OptionsButtonClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      switchMainPanel(optionsScreen);
    }
  }
  /****************************************************************************
  * ConnectButtonClicked.
  ****************************************************************************/
  private class ConnectButtonClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      SwingUtilities.invokeLater(new PromptForServer());
    }
  }
  /****************************************************************************
  * DisconnectButtonClicked.
  ****************************************************************************/
  private class DisconnectButtonClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      serverHandler.disconnect();
    }
  }
  /****************************************************************************
  * SaveVisualsButtonClicked.
  ****************************************************************************/
  private class SaveVisualsButtonClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      if (mapLayoutButton.isSelected()) {
	VisualSelector visualSelector = mapLayout.getVisualSelector();
	if (visualSelector != null) {
	  optionsScreen.setDefaultVisuals(visualSelector.newInstance());
	}
      }
      if (instrumentLayoutButton.isSelected()) {
	VisualSelector visualSelector = instrumentLayout.getVisualSelector();
	if (visualSelector != null) {
	  optionsScreen.setDefaultVisuals(visualSelector.newInstance());
	}
      }
    }
  }
  /****************************************************************************
  * ShowVideoRecorderButtonClicked.
  ****************************************************************************/
  private class ShowVideoRecorderButtonClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      VideoRecorderControls.create();
    }
  }
  /****************************************************************************
  * ExitButtonClicked.
  ****************************************************************************/
  private class ExitButtonClicked implements ActionListener {
    public void actionPerformed (ActionEvent ae) {
      System.exit(0);
    }
  }
  /****************************************************************************
  * AboutButtonClicked.
  ****************************************************************************/
  private class AboutButtonClicked implements ActionListener {
    private JFrame frame;
    public AboutButtonClicked (JFrame frame) {
      this.frame = frame;
    }
    public void actionPerformed (ActionEvent ae) {
      String text = "Simulcast Viewer V" + Simulcast.VERSION +
		    (serverSimulcastVersion != null ? ("\nSimulcast Server V" + serverSimulcastVersion) : "");
      JOptionPane.showMessageDialog(frame,text,"About Simulcast...",
				    JOptionPane.INFORMATION_MESSAGE);
    }
  }
  /****************************************************************************
  * Uncaught.
  ****************************************************************************/
  private static class Uncaught implements Thread.UncaughtExceptionHandler {
    public void uncaughtException (Thread thread, Throwable throwable) {
      System.out.println("UNCAUGHT EXCEPTION, Thread: " + thread.getName() + ", throwable: " + throwable.toString());
      throwable.printStackTrace();
    }
  }
}
